/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Class that's used to draw a game entity (a character, the Foul, an animal) on the screen.
 */
public class Entity {
	/** Game applet */
	private MidnightApplet applet;

	/** image width (in tiles) */
	byte width;
	
	/** image height (in tiles) */
	byte height;
	
	/** characters in the <code>MidnightMidlet.charFontData</code> that make up the image */
	short [] characters;
	
	/** colour attributes of the character image */
	short [] attributes;
	
	Entity(MidnightApplet applet, int width, int height, short [] characters, short[] attributes) {
		this.applet = applet;
		this.width = (byte) width;
		this.height = (byte) height;
		this.characters = characters;
		this.attributes = attributes;
	}

	/**
	 * Draws the appropriate image of the character (depending on whether he's riding
	 * or walking) with the baseline at the given row and column. The image is made up of 8x8 
	 * represented as tiles in the <code>MidnightApplet.characterFont</code> "font".
	 */
	public void paint(Graphics g, int column, int row) {
		paint(g, column, row, false);
	}
	
	/**
	 * Draws the appropriate image of the character (depending on whether he's riding
	 * or walking) with the baseline at the given row and column. The image is made up of 8x8 
	 * represented as tiles in the <code>MidnightApplet.characterFont</code> "font".
	 */
	public void paint(Graphics g, int column, int row, boolean isNight) {
		row = row - height + 1;
		int windowAttribute = isNight? 0x40 : 0x7f;
		int printInk = isNight? 0x41 : 0;
		int i = 0;
		for (int dy = 0; dy < height; dy++) {
			for (int dx = 0; dx < width; dx++) {
				short ch = characters[i];
				byte attribute = (byte)attributes[i];
				// force blue and black at night time
				attribute &= windowAttribute;
				attribute |= printInk;
				drawChar(g, (column + dx) * 8, (row + dy) * 8, ch, attribute);
				i++;
			}
		}
	}
	
	/**
	 * Draws a single 8x8 character from <code>MidnightApplet.characterFont</code> font
	 * in the given colour at a specified (x, y) coordinate.
	 * 
	 * an attribute byte is broken down as follows
	 * 7		6		5		4		3		2		1		0
	 * --------------------------------------------------------------
	 * F		B		PAP_G	PAP_R	PAP_B	INK_G	INK_R	INK_B
	 *
	 * where the 3bits for INK and PAPER are RGB values
	 * B is the BRIGHT bit, if it is a '1' then the colours (except black) appear brighter
	 * F is the FLASH bit, if it is a '1' then the ULA swaps the PAPER and INK values.
	 * 
	 * Colours are:
	 * 	0 - black
	 *  1 - blue
	 *  2 - red
	 *  3 - magenta
	 *  4 - green
	 *  5 - cyan
	 *  6 - yellow
	 *  7 - white
	 */
	protected void drawChar(Graphics g, int x, int y, short ch, byte attribute) {
		Color fg = color[attribute & 0x07];
		Color bg = color[(attribute >> 3) & 0x07];
		for (int dy = 0; dy < 8; dy++, y++) {
			byte data = applet.entityFont[ch][dy];
			for (int dx = 7; dx >= 0; dx--, x++) {
				if ((data & (1<<dx)) != 0) {
					g.setColor(fg);
				}
				else {
					g.setColor(bg);
				}
				g.drawLine(x, y, x, y);
			}
			x -= 8;
		}
	}
	
	/** Mapping from Spectrum's to AWT colours */
	private Color [] color = {
		Color.BLACK, Color.BLUE, Color.RED, Color.MAGENTA,
		Color.GREEN, Color.CYAN, Color.YELLOW, Color.WHITE,
	};
}
