/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Iterator;

import midnight.Army;
import midnight.Character;
import midnight.Location;
import midnight.Map;
import midnight.Midnight;

/**
 * User interface for the Midnight engine implemented as a Java applet.
 */
public class MidnightApplet extends Applet {
	protected Midnight game;
	protected Map map;
	protected Character character;
	protected GameScreen screen;
	protected MainScreen mainScreen;
	protected CharacterSelectionScreen characterSelectionScreen;
	protected ChooseActionScreen chooseActionScreen;
	protected MapScreen mapScreen;
	protected ThinkScreen thinkScreen;
	protected NightScreen nightScreen;
	
	/** Images of characters in the applet font */
	protected byte [][] font;
	
	/** Tiles making up game entities (characters and enemies)  as characters in a font */
	protected byte [][] entityFont;
	
	/** Tiles making up character's shields */
	protected byte [][] shieldFont;
	
	/**
	 * Called when the applet is (re)loaded.
	 * 
	 * @see java.applet.Applet#init
	 */
    public void init() {
		setupGame();
		setupView();
    }

	protected void setupGame() {
		game = Midnight.getInstance();
		map = game.getMap();
		character = game.LUXOR;
	}
	
	protected void setupView() {
		setLayout(new BorderLayout());
		removeAll();
    	
		try {
			createFont();
			createEntityFont();
			createShieldFont();
		} catch (IOException ignored) {}
	}
		
	protected void save() {
		FileDialog dialog = new FileDialog(new Frame(), "Save Game", FileDialog.SAVE);
		dialog.setFilenameFilter(new FilenameFilter() {
			/**
			 * @see java.io.FilenameFilter#accept(File, String)
			 */
			public boolean accept(File dir, String name) {
				return name.endsWith(".jlom");
			}
		});
		dialog.show();
		String filename = dialog.getFile();
		if (filename == null) return;
		if (!filename.endsWith(".jlom")) filename += ".jlom";
		File f = new File(dialog.getDirectory(), filename);
		try {
			DataOutputStream out = new DataOutputStream(new FileOutputStream(f));
			game.save(out);
			out.writeUTF(character.getName());
			out.close();
		} catch (IOException e) {
		}
		mainScreen();
	}
	
	protected void load() {
		FileDialog dialog = new FileDialog(new Frame(), "Load Game", FileDialog.LOAD);
		dialog.setFilenameFilter(new FilenameFilter() {
			/**
			 * @see java.io.FilenameFilter#accept(File, String)
			 */
			public boolean accept(File dir, String name) {
				return name.endsWith(".jlom");
			}
		});
		dialog.show();
		String filename = dialog.getFile();
		if (filename == null) return;
		if (!filename.endsWith(".jlom")) filename += ".jlom";
		File f = new File(dialog.getDirectory(), filename);
		if (!f.canRead()) return;
		try {
			DataInputStream in = new DataInputStream(new FileInputStream(f));
			game.load(in);
			String current = in.readUTF();
			for (Iterator it = game.characters(); it.hasNext();) {
				Character character = (Character) it.next();
				if (character.getName().equals(current)) {
					this.character = character;
					break;
				}
			}
			in.close();
		} catch (IOException e) {
		}
		mainScreen();
	}
	/**
	 * Called to start the applet's execution.
	 * 
	 * @see java.applet.Applet#load
	 */
    public void start() {
    	mainScreen = new MainScreen(this);
    	characterSelectionScreen = new CharacterSelectionScreen(this);
    	chooseActionScreen = new ChooseActionScreen(this);
    	thinkScreen = new ThinkScreen(this);
    	mapScreen = new MapScreen(this);
    	nightScreen = new NightScreen(this);
    	setScreen(mainScreen);
    }
	
	/**
	 * Sets the current game screen (landscape/thinking/action/character-selection/night).
	 */
	private void setScreen(GameScreen screen) {
		if (this.screen != screen) {
			if (this.screen != null) {
				removeKeyListener(this.screen);
				remove(this.screen);
			}
			this.screen = screen;
			add("Center", screen);
			addKeyListener(screen);
			requestFocus();
		}
		repaint();
	}
	
	/**
	 * Sets the current character.
	 */
	protected void setCharacter(Character character) {
		if (game.isControllable(character)) {
			this.character = character;
		}
		else {
			this.character = game.LUXOR;
		}
		mainScreen();
	}

	/**
	 * Activates character selection screen.
	 */
	protected void chooseCharacter() {
		setScreen(characterSelectionScreen);
	}

	/**
	 * Moves the character forward or puts up a think screen if he cannot move.
	 */
	protected void walk() {
		character.walkForward();
		mainScreen();
	}
	
	/**
	 * Current character seeks for location's object.
	 */
	protected void characterSeek() {
		character.seek();
		think();
	}
	
	/**
	 * Hides/unhides the current character.
	 */
	protected void characterHide() {
		character.setHidden(!character.isHidden());
		think();
	}
	
	/**
	 * Current character fights the location's beasts.
	 */
	protected void characterFight() {
		character.fight();
		think();
	}
	
	/**
	 * Current character recruits another one.
	 */
	protected void characterRecruit(Character recruiting) {
		character.recruit(recruiting);
		setCharacter(recruiting);
		mainScreen();
	}
	
	/**
	 * Current character recruits men from the keep/citadel.
	 */
	protected void characterRecruitMen() {
		character.recruitMen();
		think();
	}
	
	/**
	 * Current character adds men to the keep/citadel.
	 */
	protected void characterStandOnGuard() {
		character.standOnGuard();
		think();
	}
	
	/**
	 * Current character attacks the enemy in the neighbouring location.
	 */
	protected void characterAttack() {
		character.walkForward();
		mainScreen();
	}
	
	/**
	 * Activates main (landscape) screen.
	 */
	protected void mainScreen() {
		setScreen(mainScreen);
	}
	
	/**
	 * Activates action selection screen.
	 */
	protected void chooseAction() {
		setScreen(chooseActionScreen);
	}
	
	/**
	 * Activates "thinking" screen.
	 */
	protected void think() {
		setScreen(thinkScreen);
	}

	/**
	 * Activates the map screen.
	 */
	protected void viewMap() {
		addMouseListener(mapScreen);
		setScreen(mapScreen);
	}
	
	/**
	 * End of turn screen (night time activities).
	 */
	protected void night() {
		game.night();
		setScreen(nightScreen);
	}
	
	/**
	 * Starts the new day.
	 */
	protected void dawn() {
		game.dawn();
		mainScreen();
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	/**
	 * Called to refresh the applet's screen.
	 * 
	 * @see java.awt.Container#paint
	 */
    public void paint(Graphics g) {
    	screen.setSize(getSize());
    	super.paint(g);
    }
    
    /**
     * Creates images for each of the letters with ASCII codes 32-127 and
     * stores them into <code>font</code> array. Images are loaded from file
     * <emph>fontData</emph>, which contains them as 8x8 1-deep bitmaps.
     */
    protected void createFont() throws IOException {
    	DataInputStream in = new DataInputStream(this.getClass().getResourceAsStream("/fontData"));
    	font = new byte[96][8];
    	for (int i = 0; i < 96; i++) {
    		for (int j = 0; j < 8; j++) {
	    		font[i][j] = in.readByte();
    		}
    	}
    }
    
    /**
     * Creates images for each of the glyphs to draw characters, the Foul, and animals
     * stores them into <code>font</code> array. Images are loaded from file
     * <emph>fontData</emph>, which contains them as 8x8 1-deep bitmaps.
     */
    protected void createEntityFont() throws IOException {
    	DataInputStream in = new DataInputStream(this.getClass().getResourceAsStream("/entityFontData"));
    	entityFont = new byte[195][8];
    	for (int i = 0; i < 195; i++) {
    		for (int j = 0; j < 8; j++) {
	    		entityFont[i][j] = in.readByte();
    		}
    	}
    }
    
    /**
     * Creates images for each of the glyphs to draw shields
     * stores them into <code>shieldFont</code> array. Images are loaded from file
     * <emph>shieldFontData</emph>, which contains them as 8x8 1-deep bitmaps.
     */
    protected void createShieldFont() throws IOException {
    	DataInputStream in = new DataInputStream(this.getClass().getResourceAsStream("/shieldFontData"));
    	shieldFont = new byte[37][8];
    	for (int i = 0; i < 37; i++) {
    		for (int j = 0; j < 8; j++) {
	    		shieldFont[i][j] = in.readByte();
    		}
    	}
    }
    
    /** painters for character images */
    protected Entity [] entityPainter = {
    	// Foul
    	new Entity(this, 3, 5, 
    		new short [] {
    			0x79,0x7A,0x7B,0x7C,0x7D,0x7E,0x7F,0x80,
    			0x81,0x82,0x83,0x84,0x85,0x86,0x87}, 
    		new short [] {
    			0x78,0x78,0x78,0x79,0x61,0x72,0x79,0x61,0x72,0x79,0x79,0x7A,0x7B,0x7B,0x7B
    		}),
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x1F,0x20,0x21,0x03,0x1D,0x22,0x06,
				0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,
				0x0F,0x10,0x11,0x12,0x00,0x13,0x14,0x15,
				0x16,0x17,0x18,0x19
    		}, 
    		new short [] {
    			0x78,0x78,0x78,0x78,0x78,0x78,0x30,0x7A,0x78,0x78,0x60,0x7A,0x78,0x78,0x58,0x78,
				0x78,0x78,0x68,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78
    		}),
    	// Free
    	new Entity(this, 3, 5, 
    		new short [] {
    			0x6B,0x6C,0x6D,0x6E,0x6F,0x70,0x71,0x72,
				0x73,0x74,0x75,0x76,0x00,0x77,0x78
    		}, 
    		new short [] {
    			0x7E,0x7A,0x7E,0x7D,0x72,0x79,0x7D,0x72,0x79,0x7D,0x72,0x79,0x78,0x79,0x79
    		}),
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x1A,0x1B,0x1C,0x03,0x1D,0x1E,0x06,
				0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,
				0x0F,0x10,0x11,0x12,0x00,0x13,0x14,0x15,
				0x16,0x17,0x18,0x19
    		}, 
    		new short [] {
    			0x78,0x7E,0x7A,0x7E,0x78,0x78,0x79,0x7D,0x78,0x78,0x72,0x7D,0x78,0x78,0x58,0x78,
				0x78,0x78,0x68,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78
    		}),
    	// Fey
    	new Entity(this, 3, 5,
    		new short [] {
    			0x6B,0x6C,0x6D,0x6E,0x6F,0x70,0x71,0x72,
				0x73,0x74,0x75,0x76,0x00,0x77,0x78
    		}, 
    		new short [] {
    			0x7D,0x7A,0x7D,0x7B,0x69,0x79,0x73,0x69,0x79,0x7B,0x69,0x79,0x78,0x7A,0x79
    		}),
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x1A,0x1B,0x1C,0x03,0x1D,0x1E,0x06,
				0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,
				0x0F,0x10,0x11,0x12,0x00,0x13,0x14,0x15,
				0x16,0x17,0x18,0x19
			}, 
    		new short [] {
    			0x78,0x7D,0x79,0x7D,0x78,0x78,0x79,0x7C,0x78,0x78,0x69,0x7C,0x78,0x78,0x58,0x78,
				0x78,0x78,0x60,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78
    		}),
    	// Targ
    	new Entity(this, 3, 5, 
    		new short [] {
    			0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,
				0xBC,0xBD,0xBE,0xBF,0xC0,0xC1,0xC2
    		},
    		new short [] {
    			0x78,0x7A,0x78,0x79,0x79,0x7B,0x79,0x72,0x7B,0x78,0x7C,0x78,0x7A,0x78,0x7A
    		}),
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x01,0x02,0x00,0x03,0x04,0x05,0x06,
				0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,
				0x0F,0x10,0x11,0x12,0x00,0x13,0x14,0x15,
				0x16,0x17,0x18,0x19
    		}, 
    		new short [] {
    			0x78,0x78,0x7A,0x78,0x78,0x78,0x3A,0x7B,0x78,0x78,0x69,0x7B,0x78,0x78,0x60,0x78,
				0x78,0x78,0x70,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78
    		}),
    	// Wise
    	new Entity(this, 4, 6, 
    		new short [] {
    			0x97,0x00,0x00,0x00,0x98,0x2C,0x2D,0x00,
				0x99,0x9A,0x9B,0x00,0x9C,0x9D,0x9E,0x00,
				0x9F,0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6
    		}, 
    		new short [] {
    			0x78,0x78,0x78,0x78,0x78,0x7E,0x7E,0x78,0x79,0x7A,0x7E,0x78,0x79,0x69,0x79,0x78,
				0x79,0x79,0x79,0x79,0x7B,0x7B,0x7B,0x7B
    		}),
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x23,0x24,0x25,0x03,0x04,0x26,0x27,
				0x07,0x08,0x28,0x29,0x0B,0x0C,0x2A,0x2B,
				0x0F,0x10,0x11,0x12,0x00,0x13,0x14,0x15,
				0x16,0x17,0x18,0x19
    		},
    		new short [] {
    			0x78,0x78,0x7B,0x7B,0x7A,0x7A,0x7B,0x7B,0x7A,0x7A,0x7B,0x7B,0x7A,0x7A,0x72,0x7A,
				0x7A,0x7A,0x42,0x7A,0x78,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A
    		}),
    	// Morkin
    	new Entity(this, 4, 6, 
    		new short [] {
    			0x97,0x00,0x00,0x00,0x98,0x2C,0x2D,0x00,
				0x99,0x9A,0x9B,0x00,0x9C,0x9D,0x9E,0x00,
				0x9F,0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6
    		},
    		new short [] {
    			0x78,0x78,0x78,0x78,0x78,0x7E,0x7E,0x78,0x79,0x7A,0x7E,0x78,0x79,0x69,0x79,0x78,
				0x79,0x79,0x79,0x79,0x7B,0x7B,0x7B,0x7B
    		}),
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x00,0x2C,0x2D,0x03,0x2E,0x2F,0x30,
				0x07,0x31,0x32,0x33,0x0B,0x0C,0x34,0x35,
				0x0F,0x10,0x36,0x12,0x00,0x13,0x14,0x15,
				0x16,0x17,0x18,0x19
    		},
    		new short [] {
    			0x78,0x78,0x7E,0x7E,0x7A,0x7A,0x7A,0x7E,0x7A,0x7A,0x69,0x78,0x7A,0x7A,0x62,0x7A,
				0x7A,0x7A,0x72,0x7A,0x78,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A
    		}),
    	// Skulkrin
    	new Entity(this, 3, 4, 
    		new short [] {
    			0x00,0xA7,0x00,0xA8,0xA9,0xAA,0xAB,0xAC,
				0xAD,0xAE,0xAF,0xB0
    		}, 
    		new short [] {
    			0x78,0x7A,0x78,0x7A,0x72,0x7A,0x7A,0x4A,0x7A,0x7B,0x7B,0x7B
    		}),
    	null,
    	// Dragon
    	new Entity(this, 4, 6, 
    		new short [] {
    			0x00,0x40,0x41,0x00,0x42,0x43,0x44,0x45,
				0x46,0x47,0x48,0x49,0x4A,0x3A,0x4B,0x4C,
				0x4D,0x4E,0x4F,0x50,0x51,0x52,0x53,0x54
    		}, 
    		new short [] {
    			0x78,0x7A,0x7A,0x78,0x7A,0x7A,0x7A,0x7E,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,
				0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A,0x7A
    		}),
    	null,
    	// Wolf
    	new Entity(this, 4, 3, 
    		new short [] {
    			0x37,0x38,0x00,0x00,0x39,0x3A,0x3B,0x3C,
				0x3D,0x3E,0x3F,0x00
    		}, 
    		new short [] {
    			0x78,0x7D,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78
    		}),
    	// Ice troll
    	new Entity(this, 4, 7, 
    		new short [] {
    			0x00,0x55,0x56,0x00,0x57,0x58,0x59,0x5A,
				0x5B,0x5C,0x5D,0x5E,0x5F,0x60,0x61,0x62,
				0x63,0x64,0x65,0x66,0x00,0x67,0x68,0x00,
				0x00,0x69,0x6A,0x00
    		}, 
    		new short [] {
				0x78,0x7B,0x7B,0x78,0x79,0x6B,0x6B,0x79,0x79,0x73,0x73,0x79,0x79,0x71,0x71,0x79,
				0x78,0x71,0x71,0x78,0x78,0x7C,0x7C,0x78,0x78,0x78,0x78,0x78
    		}),
    	// Horse
    	new Entity(this, 4, 6, 
    		new short [] {
    			0x03,0x2E,0x00,0x00,0x07,0xB1,0xB2,0xB3,
				0x0B,0x0C,0x3A,0x35,0x0F,0x10,0x3A,0x12,
				0x00,0x13,0x14,0x15,0x16,0x17,0x18,0x19
    		}, 
    		new short [] {
    			0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78,
				0x78,0x78,0x78,0x78,0x78,0x78,0x78,0x78
    		}),
    };

	/**
	 * Returns the painter that can draw the game character.
	 */
    protected Entity getEntity(Character character) {
		int imgIndex = character.getRace().getIndex() * 2;
		if (character.isOnHorse()) imgIndex++;
		return entityPainter[imgIndex];
    }
    
    /**
     * Returns the painter that can draw the enemy army.
     */
    protected Entity getEntity(Army army) {
		int imgIndex = army.getRace().getIndex() * 2;
		if (army.getType() == Army.Type.RIDERS) imgIndex++;
		return entityPainter[imgIndex];
    }
    
    /** mapping from LocationObject ids to entity painters */
    protected int [] objectPainter = {16, 14, 17, 12, 18};
    
    /**
     * Returns the painter that can draw the location object.
     * This currently assumes object's index 1-5 (horses plus the four nasties).
     */
    protected Entity getEntity(Location.Object object) {
    	return entityPainter[objectPainter[object.getIndex()-1]];
    }

	protected Shield [] shieldPainter = {
		new Shield(this, new short [] {0x03,0x29,0x0B,0x32,0x11}),
		new Shield(this, new short [] {0x02,0x21,0x09,0x33,0x14,0x33, 0x23}),
		new Shield(this, new short [] {0x02,0x35,0x09}),
		new Shield(this, new short [] {0x05,0x0C,0x09,0x3D,0x19}),
		new Shield(this, new short [] {0x03,0x3E,0x09,0x32,0x22}),
		new Shield(this, new short [] {0x02,0x36,0x09,0x3A,0x22}),
		new Shield(this, new short [] {0x03,0x36,0x09,0x2A,0x22}),
		new Shield(this, new short [] {0x00,0x16,0x11}),
		new Shield(this, new short [] {0x05,0x3E,0x11}),
		new Shield(this, new short [] {0x04,0x3E,0x11}),
		new Shield(this, new short [] {0x02,0x3E,0x11}),
		new Shield(this, new short [] {0x06,0x16,0x11}),
		new Shield(this, new short [] {0x02,0x36,0x09,0x2D,0x21}),
		new Shield(this, new short [] {0x03,0x36,0x09,0x3D,0x21}),
		new Shield(this, new short [] {0x04,0x3D,0x09}),
		new Shield(this, new short [] {0x05,0x15,0x09}),
		new Shield(this, new short [] {0x00,0x1D,0x09}),
		new Shield(this, new short [] {0x03,0x35,0x09}),
		new Shield(this, new short [] {0x07,0x15,0x09}),
		new Shield(this, new short [] {0x07,0x25,0x09}),
		new Shield(this, new short [] {0x02,0x31,0x0A}),
		new Shield(this, new short [] {0x05,0x39,0x0A}),
		new Shield(this, new short [] {0x07,0x19,0x0A}),
		new Shield(this, new short [] {0x02,0x32,0x0A}),
		new Shield(this, new short [] {0x00,0x22,0x0A}),
		new Shield(this, new short [] {0x00,0x11,0x0A}),
		new Shield(this, new short [] {0x04,0x39,0x0A}),
		new Shield(this, new short [] {0x03,0x3A,0x0A}),
		new Shield(this, new short [] {0x00,0x1E,0x09,0x24,0x22}),
		new Shield(this, new short [] {0x00,0x32,0x09,0x1B,0x1B}),
		new Shield(this, new short [] {0x06,0x14,0x09,0x0B,0x1B}),
		new Shield(this, new short [] {0x02,0x33,0x09,0x33,0x14,0x33,0x22})
	};
	  
    /**
     * Returns the painter that can draw the character's shield.
     */
    protected Shield getShield(Character c) {
    	return shieldPainter[c.getId()];
    }
    
    protected short[][] shieldPartData = {
    	{6, 7,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
			0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x01,0x05,
			0x03,0x01,0x01,0x01,0x01,0x06,0x00,0x04,0x01,0x01,0x07,0x00,
			0x00,0x00,0x04,0x07,0x00,0x00},
		{2, 3,0x08,0x09,0x0A,0x0B,0x0C,0x0D},
		{2, 2,0x0E,0x10,0x0F,0x11},
		{1, 2,0x12,0x13},
		{3, 1,0x14,0x15,0x16},
		{4, 2,0x17,0x18,0x19,0x1A,0x00,0x1B,0x1C,0x00},
		{4, 2,0x1D,0x1E,0x1F,0x20,0x21,0x22,0x23,0x24},
    };
    
    /**
     * Returns the data needed to draw a shield part.
     */
    protected short [] getShieldPartData(int i) {
    	return shieldPartData[i];
    }
    
    public static void main(String [] args) {
		final MidnightApplet applet = new MidnightApplet();
		Frame frame = new Frame("jLOM");
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				applet.stop();
				applet.destroy();
				System.exit(0);
			}
		});
		frame.add("Center", applet);
		frame.show();
		applet.init();
		applet.start();
		applet.setSize(512, 384);
		frame.setSize(512, 425); // TODO: figure out why this needs to be bigger
    }
}
