/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * Class representing an army.
 */
public class Army extends Unit {
	private int howMany;
	private final Type type;
	private int successChance;
	private int casualties;
	
	public Army(Midnight game, Race race, int howMany, Type type) {
		super(game, race, 88);
		
		this.howMany = howMany;
		this.type = type;
	}
	
	/**
	 * Returns how many men are in the army.
	 */
	public int getHowMany() {
		return howMany;
	}
	
	/**
	 * Sets the number of men in the army.
	 */
	public void setHowMany(int howMany) {
		this.howMany = howMany;
	}
	
	/**
	 * Returns the type of the army (warriors or riders).
	 */
	public Type getType() {
		return type;
	}

	/**
	 * Increases the number of men in the army by a given amount.
	 */
	public void increaseNumbers(int increase) {
		howMany += increase;
	}
	
	/**
	 * Decreases the number of men in the army by a given amount.
	 */
	public void decreaseNumbers(int decrease) {
		if (decrease > howMany) decrease = howMany;
		howMany -= decrease;
	}
	
	/**
	 * Loses the specified number of men from the army, keeping
	 * track of the total.
	 */
	public void addCasualties(int number) {
		decreaseNumbers(number);
		casualties += number;
	}
	
	/**
	 * Returns the number of personnel lost in the
	 * nighttime battle.
	 */
	public int getCasualties() {
		return casualties;
	}

	/**
	 * Sets the number of personnel lost in the nighttime battle.
	 */
	public void setCasualties(int lost) {
		this.casualties = lost;
	}

	/**
	 * Resets the numbers of enemy killed and personnel lost in
	 * the nighttime battle.
	 */	
	public void dawn() {
//		enemyKilled = 0;
//		casualties = 0;
	}
	
	/**
	 * Returns the army's success chance in battle.
	 */
	public int getSuccessChance() {
		return successChance;
	}

	/**
	 * Sets the army's success chance in battle.
	 */
	public void setSuccessChance(int successChance) {
		this.successChance = successChance;
	}

	/**
	 * Increment's the unit's energy by the given amount, increased
	 * by the unit's type bonus.
	 */
	public void incrementEnergy(int increment) {
		if (type == Type.RIDERS) {
			super.incrementEnergy(6+ increment);
		}
		else {
			super.incrementEnergy(4 + increment);
		}
	}

	/**
	 * Sets the unit on guard at the location.
	 */
	public void guard(int x, int y) {
		Location location = getGame().getMap().getLocation(x, y);
		guard(location);
	}

	/**
	 * Sets the unit on guard at the location.
	 */
	public void guard(Location location) {
		setLocation(location);
		getLocation().setGuard(this);
	}

	/**
	 * Makes the unit switch sides.
	 */
	public void switchSides() {
		if (getRace() == Race.FOUL) {
			setRace(Race.FREE);
			howMany = 200;
		}
		else {
			setRace(Race.FOUL);
			howMany = 250;
		}
	}
	
	/**
	 * Returns human-readable description.
	 */
	public String toString() {
		if (howMany != 0) {
			return howMany + " " + type;
		}
		else {
			return "no " + type;
		}
	}

	/**
	 * Saves army data to an output stream in the standard jLOM format.
	 */
	public void save(DataOutputStream out) throws IOException {
		int data = (getType() == Type.RIDERS ? 0x04 : 0) | getRace().getIndex();
		out.writeByte(data);
		data = getHowMany() / 5;
		out.writeByte(data);
	}		
	
	/**
	 * Creates a unit from an input stream written in the standard jLOM format.
	 */
	public static Army load(DataInputStream in, Midnight game) throws IOException {
		int data = in.readUnsignedByte();
		Race race = Race.getRace(data & 0x03);
		Army.Type type = (data & 0x04) != 0 ? Army.Type.RIDERS : Army.Type.WARRIORS;
		int howMany = in.readUnsignedByte() * 5;
		return new Army(game, race, howMany, type);
	}
	
	/**
	 * Enum class for describing army type.
	 */
	public static class Type {
		public static final Type WARRIORS = new Type("warriors");
		public static final Type RIDERS = new Type("riders");
	
		private String description;
	
		private Type(String description) {
			this.description = description;
		}
		
		public String toString() {
			return description;
		}
	}
}
