/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Represents a full scale battle in a single location.
 */
public class Battle {
	private Location location;
	private Midnight game;
	private Race winner;
	
	private HashSet characters = new HashSet();
	private ArrayList free = new ArrayList();
	private ArrayList foul = new ArrayList();
	
	/**
	 * Battle constructor.
	 */
	public Battle(Location location) {
		this.location = location;
		this.game = location.getGame();

		addGuard();
		
		for (Iterator it = location.getCharacters().iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			
			if (character.isAlive() && !character.isHidden()) {
				addCharacter(character);
			}
		}
		
		for (Iterator it = location.getArmies().iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			addFoulArmy(army);
		}
	}
	
	/**
	 * Constructor used when restoring a saved game.
	 */
	protected Battle(Location location, Race winner) {
		this.location = location;
		this.winner = winner;
	}
	
	/**
	 * Adds whoever is on guard to the battle.
	 */
	private void addGuard() {
		Army guard = location.getGuard();
		if (guard == null || guard.getHowMany() == 0) return;
		
		if (guard.getRace() == Race.FOUL) {
			addFoulArmy(guard);
		}
		else {
			/*
			 * Riders on guard have a better success chance.
			 * Doesn't really make much sense, but that's the way
			 * LoM calculates it.
			 */
			if (guard.getType() == Army.Type.RIDERS) {
				guard.setSuccessChance(0x60);
			}
			else {
				guard.setSuccessChance(0x40);
			}
			
			free.add(guard);
		}
	}
	
	/**
	 * Adds a character to the battle.
	 */
	private void addCharacter(Character character) {
		characters.add(character);
		character.setBattle(this);
		if (character.getRiders().getHowMany() > 0) {
			addFreeArmy(character.getRiders(), character);
		}
		if (character.getWarriors().getHowMany() > 0) {
			addFreeArmy(character.getWarriors(), character);
		}
	}
	
	/**
	 * Adds a Doomdark's army to the battle.
	 */
	private void addFoulArmy(Army army) {
		int fearFactor;
		if (army.getType() == Army.Type.RIDERS) {
			fearFactor = location.getIceFear() / 4;
		}
		else {
			fearFactor = location.getIceFear() / 5;
		}
		int successChance = fearFactor;
		
		/*
		 * If fighting at a friendly keep or citadel, increase the success chance
		 */
		if (location.getGuard() != null && location.getGuard().getRace() == Race.FOUL) {
			if (location.getFeature() == Location.Feature.CITADEL) {
				successChance += 0x20;
			}
			else {
				successChance += 0x10;
			}
		}

		army.setSuccessChance(successChance);
		foul.add(army);
	}
	
	/**
	 * Adds a character's army to the battle.
	 */
	private void addFreeArmy(Army army, Character character) {
		if (army == null || army.getHowMany() == 0) return;

		int successChance = army.getEnergy();
		
		/*
		 * If fighting at a friendly keep or citadel, increase the success chance
		 */
		if (location.getGuard() != null && location.getGuard().getRace() != Race.FOUL) {
			if (location.getFeature() == Location.Feature.CITADEL) {
				successChance += 0x20;
			}
			else {
				successChance += 0x10;
			}
		}
		
		/*
		 * Riders have a better success chance in battle
		 */
		if (army.getType() == Army.Type.RIDERS) {
			successChance += location.ridersBattleBonus();
		}
		
		/*
		 * Large success chance increase if led by a mounted Fey in the forest
		 */
		if (location.getFeature() == Location.Feature.FOREST && 
			character.getRace() == Race.FEY && character.isOnHorse()) {
			successChance += 0x40;
		}
		
		successChance = successChance/2 + 0x18;
		
		army.setSuccessChance(successChance);
		free.add(army);
	}
	
	/**
	 * Runs the battle.
	 */
	public void run() {
		for (Iterator it = characters.iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			
			character.setEnemyKilled(skirmish(character.getStrength(), character.getEnergy() + 0x80, foul));
		}
		
		for (Iterator it = free.iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			
			army.setEnemyKilled(skirmish(army.getHowMany() / 5, army.getSuccessChance(), foul));
		}
		
		for (Iterator it = foul.iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			
			army.setEnemyKilled(skirmish(army.getHowMany() / 5, army.getSuccessChance(), free));
			System.out.println(army + " killed " + army.getEnemyKilled() + " of the free.");
		}
		
		for (Iterator it = characters.iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			
			System.out.println(character + " himself killed " + character.getEnemyKilled() + " of the enemy.");
			if (character.getRiders().getEnemyKilled() > 0) {
				System.out.println("His riders killed " + character.getRiders().getEnemyKilled() + " of the enemy.");
			}
			if (character.getWarriors().getEnemyKilled() > 0) {
				System.out.println("His warriors killed " + character.getWarriors().getEnemyKilled() + " of the enemy.");
			}
			if (character.getRiders().getCasualties() > 0) {
				System.out.println("He lost " + character.getRiders().getCasualties() + " riders.");
			}
			if (character.getWarriors().getCasualties() > 0) {
				System.out.println("He lost " + character.getWarriors().getCasualties() + " warriors.");
			}
		}
		
		determineResult();

		if (winner != null) {
			System.out.println("Victory went to the " + winner);
		}
		else {
			System.out.println("The battle continues...");
		}
	}
	
	/**
	 * Executes attacks for a single unit.
	 * 
	 * @param hits unit's number of attacks in this battle
	 * @param successChance unit's chance of scoring a hit
	 * @param enemies list from which to chose an enemy the unit is atacking
	 * 
	 * @return number of enemy killed
	 */
	private int skirmish(int hits, int successChance, List enemies) {
		int enemyKilled = 0;
		
		for (int i = 0; enemies.size() > 0 && i < hits; i++) {
			if (game.random(256) < successChance) {
				int enemyIndex = game.random(enemies.size());
				Army enemy = (Army) enemies.get(enemyIndex);
				
				if (game.random(256) > enemy.getSuccessChance()) {
					enemyKilled += 5;
					
					enemy.addCasualties(5);
					if (enemy.getHowMany() == 0) {
						enemies.remove(enemyIndex);
						if (enemy instanceof Doomguard) {
							System.out.println(enemy + " was destroyed.");
						}
					}
				}
			}
		}
		return enemyKilled;
	}
	
	/**
	 * Determines the result of the battle.
	 */
	private void determineResult() {
		if (foul.size() == 0) {
			winner = Race.FREE;
		}
		else if (free.size() == 0) {
			winner = Race.FOUL;
		}
		else {
			winner = null;
		}
		
		for (Iterator it = free.iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			army.decrementEnergy(0x18);
		}
		
		if (location.getGuard() != null) {
			if (winner != null) {
				if ((winner == Race.FOUL && location.getGuard().getRace() != Race.FOUL) ||
					(winner != Race.FOUL && location.getGuard().getRace() == Race.FOUL)) {
					location.getGuard().switchSides();
				}
			}
			else if (location.getGuard().getHowMany() == 0) {
				location.getGuard().increaseNumbers(20);
			}
		}

		for (Iterator it = characters.iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			character.decrementEnergy(0x14);
		}		
		if (winner == Race.FOUL) {
			whatHappenedToFreeLords();
		}
	}
	
	/**
	 * Scatters about the Free lords that lost the battle, possibly killing some of them.
	 */
	private void whatHappenedToFreeLords() {
		for (Iterator it = characters.iterator(); it.hasNext();) {
			Character character = (Character) it.next();

			character.maybeLose();
			if (character.isAlive()) {
				Map.Direction direction;
				Location destination;
				do {
					direction = Map.Direction.getDirection(game.random(8));
					destination = location.getMap().getInFront(character.getLocation(), direction);
				} while (destination.getFeature() == Location.Feature.FROZEN_WASTE);
				character.setLocation(destination);
				System.out.println(character + " retreats to " + destination);
			}
			else {
				System.out.println(character + " died");
			}
		}
	}
	
	/**
	 * Returns battle's location.
	 */
	public Location getLocation() {
		return location;
	}
	
	/**
	 * Returns battle's winner or <code>null</code> if it's undecided.
	 */
	public Race getWinner() {
		return winner;
	}
	
	public String toString() {
		return "A battle in the domain of " + location.getDomain();
	}
}
