/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Class representing a game character.
 */
public class Character extends Unit implements Comparable {
	/** character's unique id */
	private final int id;
	
	/** character's name */
	private final String name;
	
	/** character's title */
	private final String title;
	
	/** character's life force (constant while alive, 0 when dead) */
	private int life;
	
	/** character's physical strength */
	private int strength;
	
	/** character's base for calculating courage */
	private int courageBase;
	
	/** character's courage against the Ice Fear */
	private Courage courage;
	
	/** the direction in which the character is currently looking */
	private Map.Direction direction;
	
	/** game object the character is carrying */
	private Location.Object carrying;

    /** object the character just found */
    private Location.Object found;
	
	/** warriors accompanying the character */
	private Army warriors;
	
	/** riders accompanying the character */
	private Army riders;
	
	/** time remaining in the day in half-hour increments (16 is dawn, 0 is night) */
	private Time time;
	
	/** 
	 * What the character killed, or what killed him (Location.Object.NOTHING if died in a battle) 
	 * The variable is cleared on movement and dawn, if the character is alive.
	 */
	private Location.Object killed;
	
	/** battle that the character is taking part in */
	private Battle battle;
	
	/** whether the character has a horse */
	private boolean onHorse;
	
	/** key for the character id's that the character can recruit */
	private final int recruitingKey;

	/** key for the character id's that can recruit the character */
	private final int recruitedByKey;
	
	/** whether the character has been recruited */
	private boolean recruited;
	
	/** whether the character is hidden */
	private boolean hidden;
	
	/** game object the character is carrying */
	private Location.Object object;
	
	/**
	 * Character constructor.
	 */
	public Character(Midnight game, int id, String name, String title, Race race, int x, int y, int life, int energy, int strength, int courageBase, int recruitingKey, int recruitedByKey, int riders, int warriors) {
		super(game, race, energy);
		this.strength = strength;
		
		this.id = id;
		this.name = name;
		this.title = title;

		this.life = life;

		this.courageBase = courageBase;
		
		this.recruitingKey = recruitingKey;
		this.recruitedByKey = recruitedByKey;
		
		this.warriors = new Army(game, race, warriors, Army.Type.WARRIORS);
		this.riders = new Army(game, race, riders, Army.Type.RIDERS);
		
		setLocation(getGame().getMap().getLocation(x, y));
		this.direction = Map.Direction.NORTH;

		time = new Time();
		object = Location.Object.NOTHING;
		
		if (race == Race.DRAGON || race == Race.SKULKRIN) {
			onHorse = false;
		}
		else {
			onHorse = true;
		}
	}

	/**
	 * Returns the character's id.
	 * @return int
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Returns character's name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns character's full name and title
	 */
	public String getFullTitle() {
		return title;
	}

	/**
	 * Returns the life.
	 */
	protected int getLife() {
		return life;
	}
	
	/**
	 * Returns the courageBase.
	 * @return int
	 */
	protected int getCourageBase() {
		return courageBase;
	}

	/**
	 * Returns character's courage.
	 */
	public Courage getCourage() {
		calculateCourage();
		return courage;
	}
	
	protected void calculateCourage() {
		int fear = (courageBase - getLocation().getIceFear() / 7);
		courage = Courage.getCourage(fear/8);
	}

	/**
	 * Sets character's current location.
	 */
	public void setLocation(Location location) {
		if (getLocation() != null) {
			getLocation().removeCharacter(this);
		}
		super.setLocation(location);
		getLocation().addCharacter(this);
	}
		
	public Map.Direction getDirection() {
		return direction;
	}
	
	public void setDirection(Map.Direction direction) {
		this.direction = direction;
	}
	
	/**
	 * Returns <code>true</code> if the character is alive.
	 */
	public boolean isAlive() {
		return life > 0;
	}
	
	/**
	 * Marks the character as dead.
	 */
	public void die() {
		life = 0;
	}
	
	/**
	 * Returns character's strength.
	 */
	public int getStrength() {
		return strength;
	}

	/**
	 * Sets the character's strength.
	 */
	public void setStrength(int strength) {
		this.strength = strength;
	}

	/**
	 * Returns <code>true</code> if the character is hidden.
	 */
	public boolean isHidden() {
		return hidden;
	}

	/**
	 * Returns <code>true</code> if the character can hide.
	 */
	public boolean canHide() {
		return this != getGame().MORKIN && 
			getWarriors().getHowMany() == 0 &&
			getRiders().getHowMany() == 0;
	}
	
	/**
	 * Hides/unhides the character.
	 */
	public void setHidden(boolean hidden) {
		this.hidden = hidden;
	}

	/**
	 * Returns <code>true</code> if the character can move to the location he is facing.
	 * 
	 * A character can move iff he is all of the following:
	 * <ul>
	 *   <li>he can leave the current location</li>
	 *   <li>not utterly tired</li>
	 *   <li>it's not night</li>
	 *   <li>the destination is not the Frozen Wastes</li>
	 *   <li>the destination contains less than 29 characters</li>
	 *   <li>the destination does not contain an enemy army</li>
	 * </ul>
	 * 
	 * @see #canLeave()
	 */
	public boolean canWalkForward() {
		Location destination = getGame().getMap().getInFront(getLocation(), direction);

		return canLeave() && !time.isNight() && 
			getCondition() != Condition.UTTERLY_TIRED && 
			destination.getFeature() != Location.Feature.FROZEN_WASTE &&
			destination.getCharacters().size() < 29 && destination.getArmies().isEmpty() &&
			(destination.getGuard() == null || destination.getGuard().getRace() != Race.FOUL);
	}
	
	/**
	 * Returns <code>true</code> if the character can leave the current location.
	 * 
	 * A character can leave the current location iff all of the following is true:
	 * <ul>
	 *   <li>alive</li>
	 *   <li>not hidden</li>
	 *   <li>his current location doesn't contain an enemy army or it is dawn</li>
	 *   <li></li>
	 * </ul>
	 */
	protected boolean canLeave() {
		Location.Object object = getLocation().getObject();
		Army guard = getLocation().getGuard();
		return isAlive() && !isHidden() &&
			(time.isDawn() || 
				(getLocation().getArmies().size() == 0 && 
					(guard == null || guard.getRace() != Race.FOUL))) &&
			(object != Location.Object.DRAGONS &&
			object != Location.Object.ICE_TROLLS &&
			object != Location.Object.SKULKRIN &&
			object != Location.Object.WOLVES);
	}
	
	/**
	 * Moves character to the location he is currently facing.
	 * No checks are done about having sufficient energy or time for moving there or
	 * if the location is impassable. This is the responsibility of the caller and
	 * should be done by a preceding call to <code>canWalkForward</code>.
	 */
	public void walkForward() {
		Location destination = getGame().getMap().getInFront(getLocation(), direction);
		setLocation(destination);
		
		int drain = 2;
		if (direction.isDiagonal()) {
			drain++;
		}
		
		if (!onHorse) {
			drain *= 2;
		}
		
		if (destination.getFeature() == Location.Feature.DOWNS) {
			drain++;
		}
		else if (destination.getFeature() == Location.Feature.MOUNTAIN) {
			drain += 4;
		}
		else if (destination.getFeature() == Location.Feature.FOREST && getRace() != Race.FEY) {
			drain += 3;
		}
		
		if (this == getGame().FARFLAME) {
			drain = 1;
		}
		
		time.decrease(drain);
		
		setEnergy(getEnergy() - drain);
		riders.setEnergy(riders.getEnergy() - drain);
		warriors.setEnergy(warriors.getEnergy() - drain);
		
		setBattle(null);
		clearKilled();
		clearFound();
	}
	
	/**
	 * Returns the recruitedByKey.
	 */
	protected int getRecruitedByKey() {
		return recruitedByKey;
	}

	/**
	 * Returns the recruitingKey.
	 */
	protected int getRecruitingKey() {
		return recruitingKey;
	}

	/**
	 * Returns <code>true</code> if a character can be recruited by this one.
	 * 
	 * A character can be recruited iff all of the following are true:
	 * <ul>
	 *   <li>he is not already recruited</li>
	 *   <li>he is in the same location</li>
	 *   <li>the character's recruitmentMask allows it</li>
	 *   <li>there isn't an enemy army in the location or the character is Morkin</li>
	 * </ul>
	 */
	public boolean canRecruit(Character c) {
		return !c.isRecruited() && c.getLocation() == getLocation() &&
			(recruitingKey & c.recruitedByKey) != 0 &&
			(getLocation().getArmies().isEmpty() || this == getGame().MORKIN);
	}
	
	/**
	 * Recruits a character.
	 * 
	 * @return <code>true</code> if the recruitment was successful.
	 */
	public boolean recruit(Character c) {
		if (canRecruit(c)) {
			c.setRecruited(true);
			return true;
		}
		return false;
	}
	
	/**
	 * Returns <code>true</code> if the character has been recruited.
	 */
	public boolean isRecruited() {
		return recruited;
	}
	
	/**
	 * Sets the character's recruited flag.
	 */
	protected void setRecruited(boolean recruited) {
		this.recruited = recruited;
	}
	
	/**
	 * Returns character's riders.
	 */
	public Army getRiders() {
		return riders;
	}

	/**
	 * Returns character's warriors.
	 */
	public Army getWarriors() {
		return warriors;
	}

	/**
	 * Returns <code>true</code> if the character can recruit men at the location.
	 * 
	 * A character can recruit men iff all of the following are true:
	 * <ul>
	 *   <li>there is a garrison in the location</li>
	 *   <li>the garrison is of the same race as the character</li>
	 *   <li>the garrison has more than 125 men</li>
	 *   <li>recruiting 100 men from the garrison would not push the number of men of
	 *     that type that the character has over 1200</li>
	 *   <li>there is not an enemy army in the location or the character is Morkin</li>
	 */
	public boolean canRecruitMen() {
		Army guards = getLocation().getGuard();
		return guards != null && guards.getRace() == getRace() &&
			guards.getHowMany() > 125 &&
			((guards.getType() == Army.Type.RIDERS && getRiders().getHowMany() < 1175) ||
				(guards.getType() == Army.Type.WARRIORS && getWarriors().getHowMany() < 1175)) &&
			(getLocation().getArmies().isEmpty() || this == getGame().MORKIN);
	}
	
	/**
	 * Recruits men from the location.
	 * 
	 * @return <code>true</code> if the recruitment was successful.
	 */
	public boolean recruitMen() {
		if (!canRecruitMen()) return false;
		
		Army guards = getLocation().getGuard();
		guards.decreaseNumbers(100);
		if (guards.getType() == Army.Type.RIDERS) {
			getRiders().increaseNumbers(100);
		}
		else if (guards.getType() == Army.Type.WARRIORS) {
			getWarriors().increaseNumbers(100);
		}

		return true;
	}
	
	/**
	 * Returns <code>true</code> if the character can add men to the location's guard.
	 * 
	 * A character can add men to the location's guard iff all of the following are true:
	 * <ul>
	 *   <li>there is a garrison in the location</li>
	 *   <li>the garrison is of the same race as the character</li>
	 *   <li>the character has more than 100 men of the same type as the garrison</li>
	 *   <li>there is not an enemy army in the location or the character is Morkin</li>
	 */
	public boolean canStandOnGuard() {
		Army guards = getLocation().getGuard();
		return guards != null && guards.getRace() == getRace() &&
			guards.getHowMany() < 1175 &&
			((guards.getType() == Army.Type.RIDERS && getRiders().getHowMany() >= 100) ||
				(guards.getType() == Army.Type.WARRIORS && getWarriors().getHowMany() >= 100)) &&
			(getLocation().getArmies().isEmpty() || this == getGame().MORKIN);
	}
	
	/**
	 * Adds one hundred of the character's soldiers to the location's guard.
	 * 
	 * @return <code>true</code> if the action was successful.
	 */
	public boolean standOnGuard() {
		if (!canStandOnGuard()) return false;
		
		Army guards = getLocation().getGuard();
		guards.increaseNumbers(100);
		if (guards.getType() == Army.Type.RIDERS) {
			getRiders().decreaseNumbers(100);
		}
		else if (guards.getType() == Army.Type.WARRIORS) {
			getWarriors().decreaseNumbers(100);
		}

		return true;
	}
		
	/**
	 * Returns the time of day for the character.
	 */
	public Time getTime() {
		return time;
	}
	
	/**
	 * Sets the time of day for the character.
	 */
	protected void setTime(Time time) {
		this.time = time;
	}
	
	/**
	 * Returns <code>true</code> if the character is on horse.
	 */
	public boolean isOnHorse() {
		return onHorse;
	}
	
	/**
	 * Sets whether the character is on horse or not.
	 */
	public void setOnHorse(boolean onHorse) {
		this.onHorse = onHorse;
	}
	
	/**
	 * Returns <code>true</code> if the character can attack the location he is facing.

	 * A character can attack iff all of the following is true:
	 * <ul>
	 *   <li>he can leave the current location</li>
	 *   <li>there is an enemy army or garrison to attack</li>
	 *   <li>he is not utterly affraid</li>
	 * </ul>
	 * 
	 * @see #canLeave()
	 */
	public boolean canAttack() {
		Location destination = getGame().getMap().getInFront(getLocation(), direction);
		
		return canLeave() && (!destination.getArmies().isEmpty() ||
			(destination.getGuard() != null && destination.getGuard().getRace() == Race.FOUL)) &&
			getCourage() != Courage.UTTERLY_AFRAID;
	}
	
	/**
	 * Returns the battle that the character participated in last night, or
	 * <code>null</code> if he wasn't in one.
	 */
	public Battle getBattle() {
		return battle;
	}
	
	/**
	 * Sets the battle that the character participated in last night.
	 */
	protected void setBattle(Battle battle) {
		this.battle = battle;
	}

	/**
	 * Returns a description of the battle that the character took part in the
	 * night before. This includes the number of casualties and the number of enemy 
	 * killed by the character and his soldiers.
	 */
	public String describeBattle() {
		StringBuffer sb = new StringBuffer("In the battle of " );
		sb.append(battle.getLocation().getDomain());
		sb.append(" ");
		if (riders.getCasualties() != 0 || warriors.getCasualties() != 0) {
			sb.append(getName());
			sb.append(" lost ");
			if (riders.getCasualties() != 0) {
				sb.append(riders.getCasualties() + " riders");
			}
			if (riders.getCasualties() != 0 && warriors.getCasualties() != 0) {
				sb.append(" and ");
			}
			if (warriors.getCasualties() != 0) {
				sb.append(warriors.getCasualties() + " warriors");
			}
			sb.append(". ");
		}

		sb.append(getName());
		sb.append(" alone slew ");
		sb.append(enemyKilled).append(" of the Enemy. ");
		if (riders.getEnemyKilled() != 0) {
			sb.append("His riders killed ");
			sb.append(riders.getEnemyKilled());
			sb.append(" of the enemy. ");
		}
		if (warriors.getEnemyKilled() != 0) {
			sb.append("His warriors killed ");
			sb.append(warriors.getEnemyKilled());
			sb.append(" of the enemy. ");
		}
		
		if (battle.getWinner() != null) {
			sb.append("Victory went to the ");
			sb.append(battle.getWinner());
			sb.append("!");
		} else {
			sb.append("The battle continues!");
		}

		return sb.toString();
	}
	
	/**
	 * Increments character's energy (and additional 9 units for being a hero).
	 * 
	 * @see Unit#incrementEnergy(int)
	 */
	public void incrementEnergy(int increment) {
		super.incrementEnergy(9 + increment);
		if (warriors != null) warriors.incrementEnergy(increment);
		if (riders != null) riders.incrementEnergy(increment);
	}
	
	/**
	 * Starts the character off for a new day
	 */
	public void dawn() {
		time.dawn();
		if (isAlive()) {
			if (riders != null) riders.dawn();
			if (warriors != null) warriors.dawn();
		}
		clearFound();
		clearKilled();
	}

	/**
	 * Returns the game object that the character is currently carrying.
	 */
	public Location.Object getObject() {
		return object;
	}
	
	/**
	 * Sets the game object that the character is carrying.
	 */
	protected void setObject(Location.Object object) {
		this.object = object;
	}
	
	/**
	 * Returns the game object that the character is currently carrying.
	 */
	public Location.Object getFound() {
		return found;
	}
	
	/**
	 * Clears the game object that the character has just found.
	 */
	public void clearFound() {
		found = null;
	}
	
	/**
	 * Returns the game object at character's location, executing
	 * any of the object's side-effects if they exist.
	 */
	public Location.Object seek() {
		Location.Object object = getLocation().getObject();
		found = object;
		
		if (object == Location.Object.DRAGONSLAYER ||
			object == Location.Object.WOLFSLAYER) {
			if (getObject() != Location.Object.ICE_CROWN && getObject() != Location.Object.MOON_RING) {
				getLocation().setObject(getObject());
				setObject(object);
			}
		}
		else if (object == Location.Object.WILD_HORSES) {
			if (getRace() == Race.FREE || getRace() == Race.FEY ||
				getRace() == Race.TARG || getRace() == Race.WISE) {
				setOnHorse(true);
			}
		}
		else if (object == Location.Object.SHELTER) {
			incrementEnergy(0x10);
			getLocation().setObject(Location.Object.NOTHING);
		}
		else if (object == Location.Object.HAND_OF_DARK) {
			time.night();
			getLocation().setObject(Location.Object.NOTHING);
		}
		else if (object == Location.Object.CUP_OF_DREAMS) {
			time.dawn();
			getLocation().setObject(Location.Object.NOTHING);
		}
		else if (object == Location.Object.WATERS_OF_LIFE) {
			setEnergy(0x78);
			warriors.setEnergy(0x78);
			riders.setEnergy(0x78);
			getLocation().setObject(Location.Object.NOTHING);
		}
		else if (object == Location.Object.SHADOWS_OF_DEATH) {
			setEnergy(0);
			warriors.setEnergy(0);
			riders.setEnergy(0);
			getLocation().setObject(Location.Object.NOTHING);
		}
		else if (object == Location.Object.ICE_CROWN || object == Location.Object.MOON_RING) {
			if (this == getGame().MORKIN) {
				getLocation().setObject(getObject());
				setObject(object);
			}
			else {
				return Location.Object.NOTHING;
			}
		}
		return object;
	}
	
	/**
	 * Drops the object the character is carrying.
	 */
	public void dropObject() {
		getLocation().setObject(getObject());
		setObject(Location.Object.NOTHING);
	}
	
	/**
	 * Returns <code>true</code> if the character can fight something in his current location.

	 * A character can fight iff all of the following is true:
	 * <ul>
	 *   <li>he is not hidden</li>
	 *   <li>there is not an enemy army or garrison in the location or the character is Morkin</li>
	 *   <li>there is a beast to fight</li>
	 * </ul>
	 * 
	 * @see #canLeave()
	 */
	public boolean canFight() {
		Location.Object object = getLocation().getObject();
		return !isHidden() && (object == Location.Object.DRAGONS ||
				object == Location.Object.ICE_TROLLS ||
				object == Location.Object.SKULKRIN ||
				object == Location.Object.WOLVES) &&
			(getLocation().getArmies().isEmpty() || this == getGame().MORKIN);
	}
	
	/**
	 * Starts a fight with the location's nasty. If there is a free army
	 * in the location, characters automatically wins. Character with a Wolfslayer
	 * automatically wins against wolves. Character with a Dragonslayer
	 * automatically wins against dragons. The nasty that was fought is
	 * be placed in <code>killed</code> field. (If character <code>isAlive</code>,
	 * that he killed the nasty, otherwise the nasty killed him. :-)
	 */
	public void fight() {
		Location.Object object = getLocation().getObject();
		
		killed = object;
		
		for (Iterator it = getLocation().getCharacters().iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			if (character.getWarriors().getHowMany() != 0 || character.getRiders().getHowMany() != 0) {
				getLocation().setObject(Location.Object.NOTHING);
				return;
			}
		}
		if ((object == Location.Object.WOLVES && getObject() == Location.Object.WOLFSLAYER) ||
			(object == Location.Object.DRAGONS && getObject() == Location.Object.DRAGONSLAYER)) {
			getLocation().setObject(Location.Object.NOTHING);
			return;
		}
		
		maybeLose();
		
		/*
		 * Does it make sense to remove the nasties even if the character died?
		 * (Original LoM limitation)
		 */
		getLocation().setObject(Location.Object.NOTHING);
	}
	
	/**
	 * Gives the character a chance to lose in the fight the horse and/or his life.
	 */
	public void maybeLose() {
		if (isOnHorse()) {
			setOnHorse(getGame().random(2) == 0);
		}
		if ((getEnergy()/2 - 0x40 + life) < getGame().random(256)) {
			die();
		}
	}
	
	/**
	 * Returns what the character just killed, or what he was killed by.
	 * (The variable is cleared on movement and dawn, if the character is
	 * alive.)
	 */
	public Location.Object getKilled() {
		return killed;
	}
	
	/**
	 * Sets what the character just killed or was killed by.
	 * (Used to load a saved game.)
	 */
	protected void setKilled(Location.Object killed) {
		this.killed = killed;
	}
	
	/**
	 * Clears what the character has just killed or what he was killed by.
	 * Should be called once it's been reported and the character made a move.
	 */
	public void clearKilled() {
		if (isAlive()) {
			killed = null;
		}
	}
	
	public String toString() {
		return getFullTitle();
	}
	
	public int compareTo(Object o) {
		Character c = (Character) o;
		return Double.compare(id, c.id);
	}
	
	/**
	 * Saves character data to an output stream in the standard jLOM format.
	 */
	public void save(DataOutputStream out) throws IOException {
		out.writeByte(getLocation().getX());
		out.writeByte(getLocation().getY());
		out.writeByte(getDirection().getIndex());
		out.writeByte(getTime().getTime());
		out.writeUTF(getName());
		out.writeUTF(getFullTitle());
		byte data = (byte) ((isOnHorse() ? 0x80 : 0) | (isHidden() ? 0x40 : 0) | 
			(getGame().isControllable(this) ? 0x20 : 0) | (isRecruited() ? 0x10 : 0) |
			getRace().getIndex());
		out.writeByte(data);

		out.writeByte(getRiders().getHowMany() / 5);
		out.writeByte(getRiders().getEnergy());
		out.writeByte(getWarriors().getHowMany() / 5 );
		out.writeByte(getWarriors().getEnergy());

		out.writeByte(getRiders().getCasualties() / 5);
		out.writeByte(getWarriors().getCasualties() / 5);
		out.writeByte(getEnemyKilled() / 5);
		out.writeByte(getRiders().getEnemyKilled() / 5);
		out.writeByte(getWarriors().getEnemyKilled() / 5);

		Battle battle = getBattle();
		out.writeByte(battle != null ? (battle.getWinner() != null ? battle.getWinner().getIndex() | 0x08 : 0) | 0x80 : 0);
		
		out.writeByte(getLife());
		out.writeByte(getEnergy());
		out.writeByte(getStrength());
		out.writeByte(getCourageBase());
		out.writeByte(getRecruitingKey());
		out.writeByte(getRecruitedByKey());
		out.writeByte(getCourage().getIndex());

		out.writeByte(getObject() != null ? getObject().getIndex() : 0xff);
		out.writeByte(getKilled() != null ? getKilled().getIndex() : 0xff);
	}
	
	/**
	 * Creates a character from an input stream written in the standard jLOM format.
	 */
	public static Character load(DataInputStream in, Midnight game, int id) throws IOException {
		int x = in.readUnsignedByte();
		int y = in.readUnsignedByte();
		Map.Direction direction = Map.Direction.getDirection(in.readUnsignedByte());
		Time time = new Time(in.readUnsignedByte());
		String name = in.readUTF();
		String fullTitle = in.readUTF();
		
		int data = in.readUnsignedByte();
		boolean isOnHorse = (data & 0x80) != 0;
		boolean isHidden = (data & 0x40) != 0;
		boolean isControllable = (data & 0x20) != 0;
		boolean isRecruited = (data & 0x10) != 0;
		Race race = Race.getRace(data & 0x07);
		
		int riders = in.readUnsignedByte() * 5;
		int ridersEnergy = in.readUnsignedByte();
		int warriors = in.readUnsignedByte() * 5;
		int warriorsEnergy = in.readUnsignedByte();
		int ridersLost = in.readUnsignedByte() * 5;
		int warriorsLost = in.readUnsignedByte() * 5;
		int enemyKilled = in.readUnsignedByte() * 5;
		int ridersKilled = in.readUnsignedByte() * 5;
		int warriorsKilled = in.readUnsignedByte() * 5;
		
		data = in.readUnsignedByte();
		Race winner = null;
		boolean wasBattle = (data & 0x80) != 0;
		if ((data & 0x08) != 0) {
			winner = Race.getRace(data & 0x07);
		}

		int life = in.readUnsignedByte();			
		int energy = in.readUnsignedByte();
		int strength = in.readUnsignedByte();
		int courageBase = in.readUnsignedByte();
		int recruitingKey = in.readUnsignedByte();
		int recruitedBy = in.readUnsignedByte();
		Character.Courage courage = Character.Courage.getCourage(in.readUnsignedByte());
		
		data = in.readUnsignedByte();
		Location.Object object = null;
		if (data != 0xff) {
			object = Location.Object.getObject(data);
		}

		data = in.readUnsignedByte();
		Location.Object killed = null;
		if (data != 0xff) {
			killed = Location.Object.getObject(data);
		}
		
		Character character = new Character(game, id, name, fullTitle, race, x, y, life, energy, strength, courageBase, recruitingKey, recruitedBy, riders, warriors);
		character.setDirection(direction);
		character.setOnHorse(isOnHorse);
		character.setHidden(isHidden);
		character.setRecruited(isRecruited);
		character.setTime(time);
		character.setObject(object);
		character.setKilled(killed);
		
		character.setEnemyKilled(enemyKilled);
		character.getRiders().setCasualties(ridersLost);
		character.getRiders().setEnemyKilled(ridersKilled);
		character.getRiders().setEnergy(ridersEnergy);
		character.getWarriors().setCasualties(warriorsLost);
		character.getWarriors().setEnemyKilled(warriorsKilled);
		character.getWarriors().setEnergy(warriorsEnergy);
		
		if (wasBattle) {
			Battle battle = new Battle(game.getMap().getLocation(x, y), winner);
			character.setBattle(battle);
		}
		
		return character;
	}
	
	public void update(DataInputStream in) throws IOException {
		int x = in.readUnsignedByte();
		int y = in.readUnsignedByte();
		Map.Direction direction = Map.Direction.getDirection(in.readUnsignedByte());
		Time time = new Time(in.readUnsignedByte());
		String name = in.readUTF();
		if (!this.name.equals(name))
			throw new IOException("Trying to update " + this.name + " with data belonging to " + name); 
		String fullTitle = in.readUTF();
		if (!this.title.equals(fullTitle))
			throw new IOException("Trying to update " + this.title + " with data belonging to " + fullTitle); 
		
		int data = in.readUnsignedByte();
		boolean isOnHorse = (data & 0x80) != 0;
		boolean isHidden = (data & 0x40) != 0;
		boolean isControllable = (data & 0x20) != 0;
		boolean isRecruited = (data & 0x10) != 0;
		Race race = Race.getRace(data & 0x07);
		if (!getRace().equals(race))
			throw new IOException("Cannot change " + this.name + "'s race from " + getRace() + " to " + race); 
		
		int riders = in.readUnsignedByte() * 5;
		int ridersEnergy = in.readUnsignedByte();
		int warriors = in.readUnsignedByte() * 5;
		int warriorsEnergy = in.readUnsignedByte();
		int ridersLost = in.readUnsignedByte() * 5;
		int warriorsLost = in.readUnsignedByte() * 5;
		int enemyKilled = in.readUnsignedByte() * 5;
		int ridersKilled = in.readUnsignedByte() * 5;
		int warriorsKilled = in.readUnsignedByte() * 5;
		
		data = in.readUnsignedByte();
		Race winner = null;
		boolean wasBattle = (data & 0x80) != 0;
		if ((data & 0x08) != 0) {
			winner = Race.getRace(data & 0x07);
		}

		int life = in.readUnsignedByte();			
		int energy = in.readUnsignedByte();
		int strength = in.readUnsignedByte();
		if (this.strength != strength)
			throw new IOException("Cannot change " + this.name + "'s strength from " + this.strength + " to " + strength); 
		int courageBase = in.readUnsignedByte();
		if (this.courageBase != courageBase)
			throw new IOException("Cannot change " + this.name + "'s courage base from " + this.courageBase + " to " + courageBase); 
		int recruitingKey = in.readUnsignedByte();
		if (this.recruitingKey != recruitingKey)
			throw new IOException("Cannot change " + this.name + "'s recruiting key from " + this.recruitingKey + " to " + recruitingKey); 
		int recruitedBy = in.readUnsignedByte();
		if (this.recruitedByKey != recruitedBy)
			throw new IOException("Cannot change " + this.name + "'s recruitedBy key from " + this.recruitedByKey + " to " + recruitedBy); 
		Character.Courage courage = Character.Courage.getCourage(in.readUnsignedByte());
		
		data = in.readUnsignedByte();
		Location.Object object = null;
		if (data != 0xff) {
			object = Location.Object.getObject(data);
		}

		data = in.readUnsignedByte();
		Location.Object killed = null;
		if (data != 0xff) {
			killed = Location.Object.getObject(data);
		}
		
		setLocation(getGame().getMap().getLocation(x, y));
		this.life = life;
		setEnergy(energy);
		this.courage = courage;
		setDirection(direction);
		setOnHorse(isOnHorse);
		setHidden(isHidden);
		setRecruited(isRecruited);
		setTime(time);
		setObject(object);
		setKilled(killed);
		
		setEnemyKilled(enemyKilled);
		getRiders().setHowMany(riders);
		getRiders().setCasualties(ridersLost);
		getRiders().setEnemyKilled(ridersKilled);
		getRiders().setEnergy(ridersEnergy);
		getWarriors().setHowMany(warriors);
		getWarriors().setCasualties(warriorsLost);
		getWarriors().setEnemyKilled(warriorsKilled);
		getWarriors().setEnergy(warriorsEnergy);
		
		if (wasBattle) {
			Battle battle = new Battle(getGame().getMap().getLocation(x, y), winner);
			setBattle(battle);
		}
	}
	
	/**
	 * Enum class for describing character condition.
	 */
	public static class Courage implements Comparable {
		private static ArrayList values = new ArrayList();
		private int ordinal;
		
		public static final Courage UTTERLY_AFRAID = new Courage("utterly afraid");
		public static final Courage VERY_AFRAID = new Courage("very afraid");
		public static final Courage AFRAID = new Courage("afraid");
		public static final Courage QUITE_AFRAID = new Courage("quite afraid");
		public static final Courage SLIGHTLY_AFRAID = new Courage("slightly afraid");

		public static final Courage BOLD = new Courage("bold");
		public static final Courage VERY_BOLD = new Courage("very bold");
		public static final Courage UTTERLY_BOLD = new Courage("utterly bold");

		private String description;

		private Courage(String description) {
			this.description = description;
			ordinal = values.size();
			values.add(this);
		}
		
		public String toString() {
			return description;
		}
		
		public static Courage getCourage(int index) {
			if (index < 0) index = 0;
			else if (index > values.size() - 1) index = values.size() - 1;
			return (Courage) values.get(index);
		}
		
		public int getIndex() {
			return ordinal;
		}
		
		public int compareTo(Object o) {
			return ordinal - ((Courage)o).ordinal;
		}
	}
}
