/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Iterator;

/**
 * Doomdark's roaming army.
 */
public class Doomguard extends Army {
	public static int MAX_MOVE_COUNT = 6;
	
	/** Unit's orders */
	private Orders orders;
	
	/** Unit's target (character for FOLLOW, location for GOTO) */
	private Object target;
	
	/** Unit's moves executed in this turn */
	private int moveCount;
	
	/** Unit's id */
	private int id;

	/** Next unit id to assign */	
	private static int nextId = 1;
	
	public Doomguard(Midnight game, int energy, int howMany, Type type, Orders orders, Object target) {
		super(game, Race.FOUL, howMany, type);
		this.orders = orders;
		this.target = target;
		this.id = nextId++;
	}
	
	public String toString() {
		return id + " Doomguard (" + super.toString() + " at " + getLocation() + "): " + orders + " " + (target != null ? target.toString() : "");
	}
	
	/**
	 * Sets the unit's location.
	 */
	public void setLocation(int x, int y) {
		Location location = getGame().getMap().getLocation(x, y);
		setLocation(location);
	}
	
	/**
	 * Sets the unit's location.
	 */
	public void setLocation(Location location) {
		if (getLocation() != null) {
			getLocation().removeArmy(this);
		}
		super.setLocation(location);
		getLocation().addArmy(this);
	}

	/**
	 * Returns the unit's orders.
	 */
	public Orders getOrders() {
		return orders;
	}

	/**
	 * Returns the unit's target (Character for FOLLOW, Location for GOTO).
	 */
	public Object getTarget() {
		return target;
	}

	/**
	 * Decreases the number of men in the unit by a given amount.
	 * If it gets to zero, the unit is discarded.
	 */
	public void decreaseNumbers(int decrease) {
		super.decreaseNumbers(decrease);
		if (getHowMany() == 0) {
			getLocation().removeArmy(this);
			getGame().removeDoomguard(this);
		}
	}
	
	/**
	 * Stays at a location if it's interesting, moves towards one its neighbours
	 * if something interesting is there, or follows unit's orders.
	 */
	public void executeMove() {
		/*
		 * If there is something special about this location, just stay here.
		 */
		if (getLocation().isSpecial()) {
			System.out.println(this + " - stay");
			stopMoving();
			return;
		}
		
		/*
		 * Look around whether there is something interesting in the neighbourhood.
		 * This always looks around in the same direction order. (Original LoM limitation)
		 * 
		 * ?? LoM bug: the original code actually only tests whether there is something
		 * interesting in an adjacent location, not up to three away as described in the
		 * Web pages (because it checks what's in <code>DesirableLocation</code> after it
		 * returns from <code>FindLookingTowards</code>, not <code>WorkingLocation</code>.
		 */
		Map.Direction direction = Map.Direction.NORTH;
		for (int i = 0; i < 8; direction = direction.turnRight(), i++) {
			Location location = getGame().getMap().getLookingTowards(getLocation(), direction);
			if (location.isSpecial()) {
				System.out.println(this + " - checkout " + location);
				moveTo(getGame().getMap().getInFront(getLocation(), direction));
				return;
			}
		}
		
		/*
		 * Otherwise, follow unit's orders
		 */
		if (orders == Orders.FOLLOW) {
			followCharacter();
		}
		else if (orders == Orders.GOTO) {
			followGoto();
		}
		else if (orders == Orders.ROUTE) {
			followRoute();
		}
		else if (orders == Orders.WANDER) {
			wander();
		}
	}

	/**
	 * Moves towards a target character, or if he's dead, towards
	 * Luxor, or if he's dead, towards Morkin.
	 */
	private void followCharacter() {
		Character character = (Character) target;
		if (!character.isAlive()) {
			if (getGame().LUXOR.isAlive()) {
				character = getGame().LUXOR;
			}
			else {
				character = getGame().MORKIN;
			}
			target = character;
		}
		moveTowards(character.getLocation());
	}
	
	/**
	 * Moves towards the target location if there its <code>special</code>
	 * flag is set. Otherwise stays where it is.
	 */
	private void followGoto() {
		Location location = (Location) target;
		if (location.isSpecial()) {
			moveTowards((Location)target);
		}
		else {
			System.out.println(this + " - no special");
			stopMoving();
		}
	}
	
	/**
	 * Goes to the target location. Once there, picks as the unit's
	 * next destination one of the location's two next route nodes.
	 */
	private void followRoute() {
		Location destination = (Location) target;
		if (getLocation() == destination) {
			if (getGame().random(2) == 0) {
				destination = getGame().getMap().getNextNodeA(destination);
			}
			else {
				destination = getGame().getMap().getNextNodeB(destination);
			}
			target = destination;
		}
		moveTowards(destination);
	}
	
	/**
	 * Moves to a random neighbouring location.
	 */
	private void wander() {
		Location location;
		do {
			location = getGame().getMap().getInFront(getLocation(), 
				Map.Direction.getDirection(getGame().random(8)));
		} while (location.getFeature() == Location.Feature.FROZEN_WASTE);
		System.out.println(this + " - wander to " + location);
		moveTo(location);
	}
	
	/**
	 * Tries to move in the general direction of a location, with less
	 * preference given to moving through forests and mountain.
	 * 
	 * Could potentially get stuck, but this is the way it was originally
	 * implemented in LoM
	 */
	private void moveTowards(Location location) {
		if (getLocation() != location) {
			Map.Direction direction = Map.calcDirection(getLocation(), location);
			Location destination = null;
			
			/*
			 * Try to move in the general direction of the target location.
			 * Attempt up to 8 times before accepting a move into a forest or the mountains.
			 */
			for (int i = 0; i < 8; i++) {
				switch (getGame().random(4)) {
					case 0:
					case 1:
						destination = getGame().getMap().getInFront(getLocation(), direction);
						break;
					case 2:
						destination = getGame().getMap().getInFront(getLocation(), direction.turnLeft());
						break;
					case 3:
						destination = getGame().getMap().getInFront(getLocation(), direction.turnRight());
						break;
				}
				if (destination.getFeature() != Location.Feature.FOREST && 
					destination.getFeature() != Location.Feature.MOUNTAIN &&
					destination.getFeature() != Location.Feature.FROZEN_WASTE) {
						
					break;
				}
			}
			if (destination.getFeature() != Location.Feature.FROZEN_WASTE) {
				System.out.println(this + " - move to " + destination);
				moveTo(destination);
			}
			else {
				System.out.println(this + " - stuck for now");
				stopMoving();
			}
		}
		else {
			System.out.println(this + " - reached destination");
			stopMoving();
		}
	}
	
	/**
	 * Stops moving for the given turn.
	 */
	private void stopMoving() {
		moveCount = MAX_MOVE_COUNT;
	}
	
	/**
	 * Returns the moveCount.
	 */
	public int getMoveCount() {
		return moveCount;
	}

	/**
	 * Resets the moveCount to zero.
	 */
	public void resetMoveCount() {
		moveCount = 0;
	}
	
	/**
	 * Moves the unit to another location, adding to its <code>moveCount</code>
	 * as necessary.
	 */
	private void moveTo(Location location) {
		/*
		 * Don't allow more than 31 Doomdark's army in a location.
		 * (Original LoM limitation)
		 */
		if (location.getArmies().size() > 0x1f) {
			stopMoving();
			return;
		}
		
		int cost;
		if (location.getFeature() == Location.Feature.FOREST ||
			location.getFeature() == Location.Feature.MOUNTAIN) {
			
			cost = 8;
		}
		else {
			cost = 2;
		}
		
		if (getType() == Type.RIDERS) {
			cost /= 2;
		}
		
		moveCount += cost;
		setLocation(location);
	}
	
	/**
	 * Saves unit data to an output stream in the standard jLOM format.
	 */
	public void save(DataOutputStream out) throws IOException {
		Doomguard.Orders orders = getOrders();
		int data = getLocation().getX() | (orders.getIndex() << 6);
		out.writeByte(data);
		data = getLocation().getY() | (getType() == Type.RIDERS ? 0x80 : 0);
		out.writeByte(data);
		data = getHowMany()/5;
		out.writeByte(data);
		if (orders == orders.FOLLOW) {
			Character target = (Character) getTarget();
			data = target.getId();
		}
		else if (orders == orders.ROUTE || orders == orders.GOTO) {
			Location target = (Location) getTarget();
			data = getLocation().getMap().getNodeIndex(target);
		}
		else {
			data = 0;
		}
		out.writeByte(data);
	}
	
	/**
	 * Creates a unit from an input stream written in the standard jLOM format.
	 */
	public static Army load(DataInputStream in, Midnight game) throws IOException {
		int data = in.readUnsignedByte();
		int x = data & 0x3f;
		Doomguard.Orders orders = Doomguard.Orders.getOrders(data >> 6);
		data = in.readUnsignedByte();
		int y = data & 0x3f;
		Army.Type type = (data & 0x80) != 0? Army.Type.RIDERS : Army.Type.WARRIORS;
		int howMany = in.readUnsignedByte() * 5;
		data = in.readUnsignedByte();
		Object target = null;
		if (orders == orders.FOLLOW) {
			int i = 0;
			for (Iterator it = game.characters(); it.hasNext(); i++) {
				Character character = (Character) it.next();
				if (i == data) {
					target = character;
					break;
				}
			}
		}
		else if (orders == orders.ROUTE || orders == orders.GOTO) {
			target = game.getMap().getRouteNode(data);
		}
		Doomguard army = new Doomguard(game, 0, howMany, type, orders, target);
		army.setLocation(x, y);
		return army;
	}
	
	/**
	 * Enum class for describing Doomguard unit's orders.
	 */
	public static class Orders {
		public static final Orders FOLLOW = new Orders("Follow");
		public static final Orders GOTO = new Orders("Go to");
		public static final Orders ROUTE = new Orders("Route");
		public static final Orders WANDER = new Orders("Wander");
	
		private String description;
	
		private Orders(String description) {
			this.description = description;
		}
		
		public String toString() {
			return description;
		}

		private static final Orders [] values = {GOTO, WANDER, FOLLOW, ROUTE};
	
		/**
		 * Returns the order represented by the index.
		 */
		public int getIndex() {
			for (int i = 0; i < values.length; i++) {
				Orders o = values[i];
				if (o == this) return i;
			}
			return -1;
		}
		
		/**
		 * Returns the order's index in Orders enum.
		 */
		public static Orders getOrders(int index) {
			return values[index];
		}
	}
}
