/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

/**
 * Class representing a single location on the game map.
 */
public class Location {
	/** game instance that the location is part of */
	private final Midnight game;
	
	/** X-coordinate */
	private final int x;
	
	/** Y-coordinate */
	private final int y;
	
	/** location's geographical feature */
	private Feature feature;
	
	/** game object at the location */
	private Location.Object object;
	
	/** geographical area of the location */
	private final Map.Area area;
	
	/** flag whether to simply name the location "a <i>feature</i> in the domain of <i>domain</i>" */
	private final boolean domain;
	
	private boolean special;
	
	/** Unit guarding a keep or citadel */
	private Army guard;
	
	/** Doomdark's armies in the location */
	private HashSet armies = new HashSet();
	
	/** Characters in the location */
	private TreeSet characters = new TreeSet();
	
	/** Ice fear at the location */
	private int iceFear;
	
	protected Location(Midnight game, int x, int y, Feature feature, Location.Object object, Map.Area area, boolean domain, boolean special) {
		this.game = game;
		this.x = x;
		this.y = y;
		this.feature = feature;
		this.object = object;
		this.area = area;
		this.domain = domain;
		this.special = special;
	}

	/**
	 * Returns the text of location coordinates, useful for printing it out 
	 * while debugging.
	 */	
	private String getCoordinates() {
		return " [" + x + ", " + y + "]";
	}
	
	/**
	 * Returns location's human-readable name.
	 */
	public String toString() {
		if (domain) {
			return getArticle(feature) + feature + " in the Domain of " + area;
		}
		else {
			if (feature == Feature.HENGE) {
				return (area + "Henge");
			}
			else if (feature == Feature.LAKE) {
				return "Lake " + area;
			}
			else if (feature == Feature.FROZEN_WASTE) {
				return "the Frozen Wastes";
			}
			else {
				StringBuffer name = new StringBuffer("the ");
				name.append(capitalize(feature)).append(" of ");
				name.append(area);
				return name.toString();
			}
		}
	}
	
	/**
	 * Returns location's X-coordinate.
	 */
	public int getX() {
		return x;
	}
	
	/**
	 * Returns location's Y-coordinate.
	 */
	public int getY() {
		return y;
	}
	
	/**
	 * Returns the map instance of this location.
	 */
	public Map getMap() {
		return game.getMap();
	}
	
	/**
	 * Returns the game instance of this location.
	 */
	protected Midnight getGame() {
		return game;
	}
	
	/**
	 * Returns the article "a" if appropriate for the feature.
	 * Always includes a blank space, since the feature itself will be concatenated
	 * immediately afterwards.
	 */
	private String getArticle(Feature feature) {
		if (feature == Feature.MOUNTAIN || feature == Feature.DOWNS || 
			feature == Feature.FROZEN_WASTE || feature == Feature.ARMY ||
			feature == Feature.PLAINS) {
			return " ";
		}
		else {
			return "a ";
		}
	}
				
	/**
	 * Returns the geographic feature at this location.
	 */
	public Feature getFeature() {
		return feature;
	}
	
	/**
	 * Returns the domain of the location.
	 */
	public Map.Area getDomain() {
		return area;
	}
	
	/**
	 * Returns the location's domain flag.
	 */
	public boolean getDomainFlag() {
		return domain;
	}
	
	/**
	 * Returns the game object at the location.
	 */
	public Location.Object getObject() {
		return object;
	}

	/**
	 * Sets the game object at the location.
	 */
	public void setObject(Location.Object object) {
		this.object = object;
	}
	
	/**
	 * Returns the special flag for the location.
	 * @return boolean
	 */
	public boolean isSpecial() {
		return special;
	}

	/**
	 * Sets the special flag.
	 * @param special 
	 */
	public void setSpecial(boolean special) {
		this.special = special;
	}

	/**
	 * Gets the unit guarding the place.
	 */
	public Army getGuard() {
		return guard;
	}
	
	/**
	 * Sets the unit guarding the place.
	 * Only keeps and citadels can be guarded.
	 */
	public void setGuard(Army guard) {
		if (feature == Feature.KEEP || feature == Feature.CITADEL) {
			this.guard = guard;
		}
	}

	/**
	 * Gets Doomdark's armies currently in the location.
	 */
	public Set getArmies() {
		return armies;
	}
	
	/**
	 * Adds a Doomdark's army to the location. If it's a plains location, sets the
	 * feature to Feature.ARMY.
	 */
	public void addArmy(Army army) {
		armies.add(army);
		if (feature == Feature.PLAINS) {
			feature = Feature.ARMY;
		}
	}
	
	/**
	 * Removes a Doomdark's army from the location.  If there are no more armies in
	 * the plains location, sets the feature back to Feature.PLAINS.
	 */
	public void removeArmy(Army army) {
		armies.remove(army);
		if (feature == Feature.ARMY && armies.isEmpty()) {
			feature = Feature.PLAINS;
		}
	}
	
	/**
	 * Gets the characters currently in the location.
	 */
	public Set getCharacters() {
		return characters;
	}
	
	/**
	 * Adds a character to the location. If the character has an army with him and
	 * the location is plains, sets the location's feature to Feature.ARMY.
	 */
	public void addCharacter(Character character) {
		characters.add(character);
		if (feature == Feature.PLAINS && (character.getRiders().getHowMany() > 0 ||
			character.getWarriors().getHowMany() >0)) {
			feature = Feature.ARMY;
		}
	}
	
	/**
	 * Removes a character from the location. If there are no more armies in
	 * the plains location, sets the feature back to Feature.PLAINS.
	 */
	public void removeCharacter(Character character) {
		characters.remove(character);
		if (feature == Feature.ARMY) {
			for (Iterator it = characters.iterator(); it.hasNext();) {
				Character c = (Character) it.next();
				if (c.getRiders().getHowMany() > 0 || c.getWarriors().getHowMany() > 0) {
					return;
				}
			}
			feature = Feature.PLAINS;
		}
	}
	
	/**
	 * Returns bonus riders enjoy when calculating their success chance
	 * in battle. The bonus is halved in mountains.
	 */
	public int ridersBattleBonus() {
		if (feature == Feature.MOUNTAIN) {
			return 0x20;
		}
		else {
			return 0x40;
		}
	}

	/**
	 * Returns the capitalized version of a string.
	 */
	public static StringBuffer capitalize(java.lang.Object o) {
		StringBuffer sb = new StringBuffer(o.toString());
		sb.setCharAt(0, java.lang.Character.toUpperCase(sb.charAt(0)));
		return sb;
	}
	
	/**
	 * Stores the location information to a save file.
	 * The location is stored in two bytes:
	 * <ul>
	 * <li>the first byte contains feature index in the low 4 bits and
	 * object index in the high 4 bits</li>
	 * <li>the second byte contains area index in the low
	 * 6 bits, domain flag in bit 6 and special flag in bit 7</li>
	 * </ul>
	 */
	public void save(DataOutputStream out) throws IOException {
		Location.Feature feature = getFeature();
		Location.Object object = getObject();
		int data = (object.getIndex() << 4) | feature.getIndex();
		out.writeByte(data);
		
		Map.Area area = getDomain();
		boolean domainFlag = getDomainFlag();
		boolean special = isSpecial();
		data = area.getIndex() | (domainFlag? 0x40 : 0) | (special? 0x80: 0);
		out.writeByte(data);
	}
	
	/**
	 * Loads a location information from the saved file and creates the Location object.
	 * Each location is stored in two bytes:
	 * <ul>
	 * <li>the first byte contains feature index in the low 4 bits and
	 * object index in the high 4 bits</li>
	 * <li>the second byte contains area index in the low
	 * 6 bits, domain flag in bit 6 and special flag in bit 7</li>
	 * </ul>	 */
	public static Location load(DataInputStream in, int x, int y, Midnight game) throws IOException {
		int data = in.readUnsignedByte();
		
		Feature feature = Feature.getFeature(data & 0x0f);
		Object object = Object.getObject(data >> 4);
		
		data = in.readUnsignedByte();
		Map.Area area = Map.Area.getArea(data & 0x3f);
		boolean domain = (data & 0x40) != 0;
		boolean special = (data & 0x80) != 0;
		return new Location(game, x, y, feature, object, area, domain, special);
	}
	
	/**
	 * Enum class for describing location's feature.
	 */
	public static class Feature {
		public static final Feature MOUNTAIN = new Feature("mountains");
		public static final Feature CITADEL = new Feature("citadel");
		public static final Feature FOREST = new Feature("forest");
		public static final Feature HENGE = new Feature("henge");
		public static final Feature TOWER = new Feature("tower");
		public static final Feature VILLAGE = new Feature("village");
		public static final Feature DOWNS = new Feature("downs");
		public static final Feature KEEP = new Feature("keep");
		public static final Feature SNOWHALL = new Feature("snowhall");
		public static final Feature LAKE = new Feature("lake");
		public static final Feature FROZEN_WASTE = new Feature("frozen wastes");
		public static final Feature RUIN = new Feature("ruin");
		public static final Feature LITH = new Feature("lith");
		public static final Feature CAVERN = new Feature("cavern");
		public static final Feature ARMY = new Feature("plains");
		public static final Feature PLAINS = new Feature("plains");
		
		private String description;
		private Feature(String description) {
			this.description = description;
		}
		
		private static final Feature [] values = {MOUNTAIN,
			CITADEL, FOREST, HENGE, TOWER, VILLAGE, DOWNS,
			KEEP, SNOWHALL, LAKE, FROZEN_WASTE, RUIN, LITH,
			CAVERN, ARMY, PLAINS};

		/**
		 * Returns a feature represented by the index.
		 */
		public static Feature getFeature(int index) {
			return values[index];
		}
		
		/**
		 * Returns the feature's index in the array of existing features.
		 */
		public int getIndex() {
			for (int i = 0; i < values.length; i++) {
				Feature f = values[i];
				if (f == this) return i;
			}
			return -1;
		}
		
		public String toString() {
			return description;
		}
	}

	/**
	 * Enum class for describing a game object at a location or carried by a character.
	 */
	public static class Object {
		public static final Object NOTHING = new Object("nothing");
		public static final Object WOLVES = new Object("wolves");
		public static final Object DRAGONS = new Object("dragons");
		public static final Object ICE_TROLLS = new Object("ice trolls");
		public static final Object SKULKRIN = new Object("Skulkrin");
		public static final Object WILD_HORSES = new Object("wild horses");
		public static final Object SHELTER = new Object("shelter and is refreshed");
		public static final Object GUIDANCE = new Object("guidance. A voice calls, ");
		public static final Object SHADOWS_OF_DEATH = new Object("the Shadows of Death which drain him of vigour");
		public static final Object WATERS_OF_LIFE = new Object("the Waters of Life which fill him with vigour");
		public static final Object HAND_OF_DARK = new Object("the Hand of Dark which brings death to the day");
		public static final Object CUP_OF_DREAMS = new Object("the Cup of Dreams which brings new welcome");
		public static final Object WOLFSLAYER = new Object("the sword Wolfslayer");
		public static final Object DRAGONSLAYER = new Object("the sword Dragonslayer");
		public static final Object ICE_CROWN = new Object("the Ice Crown");
		public static final Object MOON_RING = new Object("the Moon Ring");
		public static final Object FAWKRIN = new Object("Fawkrin the Skulkrin");
		public static final Object FARFLAME = new Object("Farflame the Dragonlord");
		public static final Object LAKE_MIRROW = new Object("Lake Mirrow");
		public static final Object LORGRIM = new Object("Lorgrim the Wise");

		
		private String description;
		private Object(String description) {
			this.description = description;
		}
		
		private static final Object [] values = {
			NOTHING, WOLVES, DRAGONS, ICE_TROLLS, SKULKRIN, WILD_HORSES, SHELTER,
			GUIDANCE, SHADOWS_OF_DEATH, WATERS_OF_LIFE, HAND_OF_DARK, CUP_OF_DREAMS,
			WOLFSLAYER, DRAGONSLAYER, ICE_CROWN, MOON_RING, FAWKRIN, FARFLAME,
			LAKE_MIRROW, LORGRIM,
		};
		
		/**
		 * Returns <code>true</code> if the object is a beast that a character has to fight.
		 */
		public boolean isBeast() {
			return this == WOLVES || this == DRAGONS || this == SKULKRIN || this == ICE_TROLLS;
		}
		
		/**
		 * Returns object type represented by the index.
		 */
		public static Object getObject(int index) {
			return values[index];
		}
		
		/**
		 * Returns the object's index in the array of existing object types.
		 */
		public int getIndex() {
			for (int i = 0; i < values.length; i++) {
				Object o = values[i];
				if (o == this) return i;
			}
			return -1;
		}
		
		/**
		 * Human-readable description of what was found.
		 * Guidance is randomly generated, so this method should be called
		 * once and result stored.
		 */
		public String toString(Location location) {
			if (this == GUIDANCE) {
				StringBuffer message = new StringBuffer("guidance. A voice says: '");
				int rnd = location.getGame().random(32);
				if (rnd >= 4) {
					int i = 0;
					Character character = null;
					for (Iterator it = location.game.characters(); (i <= rnd) && it.hasNext(); i++) {
						character = (Character) it.next();
					}
					message.append("Looking for ");
					message.append(character);
					message.append(" you must seek ");
					message.append(character.getLocation());
					message.append("'");
				}
				else {
					Object who = getObject(rnd + 16);
					message.append(who.toString(location));
					message.append(" can destroy the Ice Crown'");
				}
				return message.toString();
			}
			else {
				return description;
			}
		}
	}

	/**
	 * Calculates and returns the ice fear at the location.
	 */
	public int getIceFear() {
		int fear;
		if (game.MORKIN.isAlive()) {
			if (Map.calcDistance(this, game.MORKIN.getLocation()) == 0) {
				iceFear = 0x1ff - Map.calcDistance(this, getMap().TOWER_OF_DESPAIR) * 4;
				return iceFear;
			}
			else {
				fear = Map.calcDistance(game.MORKIN.getLocation(), getMap().TOWER_OF_DESPAIR);
			}
		}
		else {
			fear = 0x7f;
		}
		
		if (game.LUXOR.isAlive()) {
			fear += Map.calcDistance(this, game.LUXOR.getLocation());
		}
		else {
			fear += 0x7f;
		}
		
		fear += 0x30;
		fear += game.getDoomDarksCitadels();
		
		iceFear = fear;
		return iceFear;
	}

	/**
	 * Returns text description of ice fear at the location.
	 */
	public Fear describeIceFear() {
		return Fear.getFear(7 - iceFear / 0x40);
	}
	
	/**
	 * Enum class for describing ice fear at a location.
	 */
	public static class Fear implements Comparable {
		private static ArrayList values = new ArrayList();
		private int ordinal;
		
		public static final Fear UTTERLY_COLD = new Fear("utterly cold");
		public static final Fear VERY_COLD = new Fear("very cold");
		public static final Fear COLD = new Fear("cold");
		public static final Fear QUITE_COLD = new Fear("quite cold");
		public static final Fear SLIGHTLY_COLD = new Fear("slightly cold");

		public static final Fear MILD = new Fear("mild");
		public static final Fear VERY_MILD = new Fear("very mild");
		public static final Fear UTTERLY_MILD = new Fear("utterly mild");

		private String description;

		private Fear(String description) {
			this.description = description;
			ordinal = values.size();
			values.add(this);
		}
		
		public String toString() {
			return description;
		}
		
		public static Fear getFear(int index) {
			if (index < 0) index = 0;
			else if (index > values.size() - 1) index = values.size() - 1;
			return (Fear) values.get(index);
		}
		
		public int compareTo(java.lang.Object o) {
			return ordinal - ((Fear)o).ordinal;
		}
	}
}
