/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;

/**
 * Class representing the game map.
 */
public class Map {
	/** game instance that the character is part of */
	protected final Midnight game;

	private Location [][] locations;
	private HashMap routeNodes;
	
	public final Location TOWER_OF_DESPAIR;
	public final Location XAJORKITH;
	public final Location USHGARAK;
	public final Location LAKE_MIRROW;
	
	/**
	 * Returns map's width.
	 */
	public int width() {
		return locations.length;
	}
	
	/**
	 * Returns map's height.
	 */
	public int height() {
		return locations[0].length;
	}
		
	/**
	 * Returns the location at the specified coordinates.
	 * 
	 * @param x X-coordinate (starting at 0, increasing to East)
	 * @param y Y-coordinate (starting at 0, increasing to South)
	 */
	public Location getLocation(int x, int y) {
		if (x < 0 || y < 0 || x >= locations.length || y >= locations[locations.length-1].length) {
			return FrozenWaste.getInstance();
		}
		return locations[x][y];
	}
	
	/**
	 * Sets the location in the map.
	 */
	protected void setLocation(Location l) {
		locations[l.getX()][l.getY()] = l;
	}
	
	/**
	 * Returns the location directly adjacent to the specified location,
	 * when looking in the specified direction.
	 * 
	 * @param location the location from which to look
	 * @param direction the direction in which to look (North is 0, East is 3)
	 */
	public Location getInFront(Location location, Direction direction) {
		int x = location.getX() + direction.getXAdjustment();
		int y = location.getY() + direction.getYAdjustment();
		if (x < 0 || x >= locations.length || y < 0 || y >= locations[0].length) {
			return FrozenWaste.getInstance();
		}
		else {
			return locations[x][y];
		}
	}
	
	/**
	 * Returns the location that the character can see when looking in the 
	 * specified direction. This is up to three squares far when in plains.
	 * Anything else (downs, mountains, forest, a character, or an army) stops
	 * from looking further.
	 * 
	 * @param location the location from which to look
	 * @param direction the direction in which to look (North is 0, East is 3)
	 */
	public Location getLookingTowards(Location location, Direction direction) {
		for (int i = 0; i < 3; i++) {
			location = getInFront(location, direction);
			if (location.getFeature() != Location.Feature.PLAINS || location.isSpecial()) {
				break;
			}
		}
		return location;
	}
	
	/**
	 * Returns the distance between two locations. The distance is not mathematically
	 * correct, but this is the formula that the original game was using.
	 */
	public static int calcDistance(Location a, Location b) {
		return Math.abs(a.getX() - b.getX()) + Math.abs(a.getY() - b.getY());
	}
	
	/**
	 * Returns the direction from one location to another or <code>null</code> if
	 * the origin and target locations are the same.
	 * 
	 * The algorithm is not the best, as it will favour going diagonally even if
	 * the direction to the target is just slightly off the 90 degree line, but
	 * that's what LoM used.
	 */
	public static Direction calcDirection(Location origin, Location target) {
		if (origin.getX() > target.getX()) {
			if (origin.getY() > target.getY()) {
				return Direction.NORTHWEST;
			}
			else if (origin.getY() < target.getY()) {
				return Direction.SOUTHWEST;
			}
			else {
				return Direction.WEST;
			}
		}
		else if (origin.getX() < target.getX()) {
			if (origin.getY() > target.getY()) {
				return Direction.NORTHEAST;
			}
			else if (origin.getY() < target.getY()) {
				return Direction.SOUTHEAST;
			}
			else {
				return Direction.EAST;
			}
		}
		else {
			if (origin.getY() > target.getY()) {
				return Direction.NORTH;
			}
			else if (origin.getY() < target.getY()) {
				return Direction.SOUTH;
			}
			else {
				return null;
			}
		}
	}
	
	/**
	 * Constructor creates the map from the original game data.
	 */
	protected Map(Midnight game) {
		this.game = game;
		locations = new Location[64][61];
		
		int i = 0;
		for (int y = 0; y < height(); y++) {
			for (int x = 0; x < width(); x++) {
				Location.Feature feature = Location.Feature.getFeature(mainMap[i] & 0x0f);
				Location.Object object = Location.Object.getObject(mainMap[i] >> 4);
				
				Area area = Area.getArea(referenceDescriptionMap[i] & 0x3f);
				boolean domain = (referenceDescriptionMap[i] & 0x40) != 0;
				boolean special = (referenceDescriptionMap[i] & 0x80) != 0;
				locations[x][y] = new Location(game, x, y, feature, object, area, domain, special);

				i++;
			}
		}
		
		TOWER_OF_DESPAIR = getLocation(26, 4);
		XAJORKITH = getLocation(45, 59);
		USHGARAK = getLocation(29, 7);
		LAKE_MIRROW = getLocation(9, 17);

		routeNodes = new HashMap();
		for (i = 0; i < routes.length; i++) {
			routeNodes.put(locations[routes[i][0]][routes[i][1]], new Integer(i));
		}
	}

	/**
	 * Loads the game map from an InputStream. The format is as follows:
	 * <ul>
	 * <li>(byte) width</li>
	 * <li>(byte) height</li>
	 * <li>map locations by rows, each location encoded in two bytes:
	 * <ul>
	 * <li>the first byte contains feature index in the low 4 bits and
	 * object index in the high 4 bits</li>
	 * <li>the second byte contains area index in the low
	 * 6 bits, domain flag in bit 6 and special flag in bit 7</li>
	 * </ul></li>
	 * <li>the number of special places</li>
	 * <li>for each special place, encoded in four bytes per place:
	 * <ul>
	 * <li>x coordinate</li>
	 * <li>y coordinate</li>
	 * <li>next special place index</li>
	 * <li>next special place index right branch</li>
	 * </ul>
	 * <li>
	 * </ul>
	 */
	protected void load(DataInputStream in) throws IOException {
		int width = in.readUnsignedByte();
		int height = in.readUnsignedByte();
		locations = new Location[width][height];
		for (int y = 0; y < height(); y++) {
			for (int x = 0; x < width(); x++) {
				locations[x][y] = Location.load(in, x, y, game);
			}
		}

		int specialPlaces = in.readUnsignedByte();
		routes = new int[specialPlaces][4];
		for (int i = 0; i < specialPlaces; i++) {
			int x = in.readUnsignedByte();
			int y = in.readUnsignedByte();
			int left = in.readUnsignedByte();
			int right = in.readUnsignedByte();
			routes[i][0] = x;
			routes[i][1] = y;
			routes[i][2] = left;
			routes[i][3] = right;
			routeNodes.put(locations[x][y], new Integer(i));
		}
	}

	/**
	 * Saves the game map to an OutputStream. The format is as follows:
	 * <ul>
	 * <li>(byte) width</li>
	 * <li>(byte) height</li>
	 * <li>map locations by rows, each location encoded in two bytes:
	 * <ul>
	 * <li>the first byte contains feature index in the low 4 bits and
	 * object index in the high 4 bits</li>
	 * <li>the second byte contains area index in the low
	 * 6 bits, domain flag in bit 6 and special flag in bit 7</li>
	 * </ul></li>
	 * <li>the number of special places</li>
	 * <li>for each special place, encoded in four bytes per place:
	 * <ul>
	 * <li>x coordinate</li>
	 * <li>y coordinate</li>
	 * <li>next special place index</li>
	 * <li>next special place index right branch</li>
	 * </ul>
	 * <li>
	 * </ul>
	 */
	protected void save(DataOutputStream out) throws IOException {
		out.writeByte(width());
		out.writeByte(height());
		for (int y = 0; y < height(); y++) {
			for (int x = 0; x < width(); x++) {
				locations[x][y].save(out);
			}
		}
		out.writeByte(routes.length);
		for (int i = 0; i < routes.length; i++) {
			out.writeByte(routes[i][0]);
			out.writeByte(routes[i][1]);
			out.writeByte(routes[i][2]);
			out.writeByte(routes[i][3]);
		}
	}

	public Location getRouteNode(int index) {
		return locations[routes[index][0]][routes[index][1]];
	}
	
	public int getNodeIndex(Location node) {
		return ((Integer)routeNodes.get(node)).intValue();
	}
	
	public Location getNextNodeA(Location location) {
		int nextNode = routes[getNodeIndex(location)][2];
		return locations[routes[nextNode][0]][routes[nextNode][1]];
	}
	
	public Location getNextNodeB(Location location) {
		int nextNode = routes[getNodeIndex(location)][3];
		return locations[routes[nextNode][0]][routes[nextNode][1]];
	}
	
	/**
	 * Singleton class representing a location outside the map.
	 */	
	private static class FrozenWaste extends Location {
		private static FrozenWaste instance;
		private FrozenWaste() {
			super(null, -1, -1, Location.Feature.FROZEN_WASTE, Location.Object.NOTHING, Area.NOTHING, false, false);
		}
		
		public synchronized static FrozenWaste getInstance() {
			if (instance == null) {
				instance = new FrozenWaste();
			}
			return instance;
		}
	}
	
	/**
	 * Enum class for representing a direction on the map
	 */
	public static class Direction {
		public static final Direction NORTH = new Direction("North", 0, -1);
		public static final Direction NORTHEAST = new Direction("Northeast", 1, -1);
		public static final Direction EAST = new Direction("East", 1, 0);
		public static final Direction SOUTHEAST = new Direction("Southeast", 1, 1);
		public static final Direction SOUTH = new Direction("South", 0, 1);
		public static final Direction SOUTHWEST = new Direction("Southwest", -1, 1);
		public static final Direction WEST = new Direction("West", -1, 0);
		public static final Direction NORTHWEST = new Direction("Northwest", -1, -1);
		
		private String description;
		private int xAdjustment;
		private int yAdjustment;
		
		private static int nextOrdinal = 0;
		private final int ordinal = nextOrdinal++;
		
		private static Direction [] values = {
			NORTH, NORTHEAST, EAST, SOUTHEAST, SOUTH, SOUTHWEST, WEST, NORTHWEST
		};
			
		private Direction(String description, int xAdjustment, int yAdjustment) {
			this.description = description;
			this.xAdjustment = xAdjustment;
			this.yAdjustment = yAdjustment;
		}
		
		public String toString() {
			return description;
		}
		
		public int getXAdjustment() {
			return xAdjustment;
		}
		
		public int getYAdjustment() {
			return yAdjustment;
		}
		
		public int getIndex() {
			for (int i = 0; i < values.length; i++) {
				if (values[i] == this) return i;
			}
			return -1;
		}
		
		public static Direction getDirection(int index) {
			return values[index];
		}
		
		public Direction turnRight() {
			return values[(ordinal+1)%values.length];
		}
		
		public Direction turnLeft() {
			return values[(values.length+ordinal-1)%values.length];
		}
		
		public boolean isDiagonal() {
			return xAdjustment != 0 && yAdjustment != 0;
		}
		
	}
	
	/**
	 * Enum class for describing a map area.
	 */
	public static class Area {
		public static final Area NOTHING = new Area("Nothing");
		public static final Area LOTHORIL = new Area("Lothoril");
		public static final Area GLOOM = new Area("Gloom");
		public static final Area MOON = new Area("Moon");
		public static final Area MIRROW = new Area("Mirrow");
		public static final Area GLORIM = new Area("Glorim");
		public static final Area KORKITH = new Area("Korkith");
		public static final Area LOST = new Area("the Lost");
		public static final Area DEAD = new Area("Dead");
		public static final Area WEIRD = new Area("Weird");
		public static final Area UGRAK = new Area("Ugrak");
		public static final Area DEATH = new Area("Death");
		public static final Area DOOM = new Area("Doom");
		public static final Area DESPAIR = new Area("Despair");
		public static final Area VORGATH = new Area("Vorgath");
		public static final Area USHGARAK = new Area("Ushgarak");
		public static final Area UGRORN = new Area("Ugrorn");
		public static final Area KOR = new Area("Kor");
		public static final Area TOOMOG = new Area("Toomog");
		public static final Area OGRIM = new Area("Ogrim");
		public static final Area DODRAK = new Area("Dodrak");
		public static final Area GORGRATH = new Area("Gorgrath");
		public static final Area VALETHOR = new Area("Valethor");
		public static final Area COROTH = new Area("Coroth");
		public static final Area ASHIMAR = new Area("Ashimar");
		public static final Area ITHRIL = new Area("Ithril");
		public static final Area SHADOWS = new Area("Shadows");
		public static final Area BLOOD = new Area("Blood");
		public static final Area THRALL = new Area("Thrall");
		public static final Area TORKREN = new Area("Torkren");
		public static final Area GARD = new Area("Gard");
		public static final Area MITHARG = new Area("Mitharg");
		public static final Area MOON2 = new Area("the Moon");
		public static final Area ISERATH = new Area("Iserath");
		public static final Area SHIMERIL = new Area("Shimeril");
		public static final Area ODRARK = new Area("Odrark");
		public static final Area ISHMALAY = new Area("Ishmalay");
		public static final Area BRITH = new Area("Brith");
		public static final Area SILENCE = new Area("Silence");
		public static final Area ELENIL = new Area("Elenil");
		public static final Area RORATH = new Area("Rorath");
		public static final Area MORNING = new Area("Morning");
		public static final Area THIMRATH = new Area("Thimrath");
		public static final Area CORELAY = new Area("Corelay");
		public static final Area RATHORN = new Area("Rathorn");
		public static final Area LORGRIM = new Area("Lorgrim");
		public static final Area LOR = new Area("Lor");
		public static final Area FADRATH = new Area("Fadrath");
		public static final Area DROON = new Area("Droon");
		public static final Area GRARG = new Area("Grarg");
		public static final Area DREAMS = new Area("Dreams");
		public static final Area ITHRORN = new Area("Ithrorn");
		public static final Area WHISPERS = new Area("Whispers");
		public static final Area XAJORKITH = new Area("Xajorkith");
		public static final Area HERATH = new Area("Herath");
		public static final Area KUMAR = new Area("Kumar");
		public static final Area MARAKITH = new Area("Marakith");
		public static final Area TARG = new Area("the Targ");
		public static final Area UTARG = new Area("Utarg");
		public static final Area ATHORIL = new Area("Athoril");
		public static final Area DREGRIM = new Area("Dregrim");
		public static final Area DAWN = new Area("Dawn");
		public static final Area TRORN = new Area("Trorn");
		public static final Area COOM = new Area("Coom");
		
		private String description;
		private Area(String description) {
			this.description = description;
		}
		
		private static final Area [] values = {
			NOTHING, LOTHORIL, GLOOM, MOON, MIRROW, GLORIM, KORKITH, LOST,
			DEAD, WEIRD, UGRAK, DEATH, DOOM, DESPAIR, VORGATH, USHGARAK,
			UGRORN, KOR, TOOMOG, OGRIM, DODRAK, GORGRATH, VALETHOR, COROTH,
			ASHIMAR, ITHRIL, SHADOWS, BLOOD, THRALL, TORKREN, GARD, MITHARG,
			MOON2, ISERATH, SHIMERIL, ODRARK, ISHMALAY, BRITH, SILENCE, ELENIL,
			RORATH, MORNING, THIMRATH, CORELAY, RATHORN, LORGRIM, LOR, FADRATH,
			DROON, GRARG, DREAMS, ITHRORN, WHISPERS, XAJORKITH, HERATH,
			KUMAR, MARAKITH, TARG, UTARG, ATHORIL, DREGRIM, DAWN, TRORN, COOM
		};

		public static Area getArea(int index) {
			return values[index];
		}
		
		/**
		 * Returns the area's index in the array of existing area types.
		 */
		public int getIndex() {
			for (int i = 0; i < values.length; i++) {
				Area a = values[i];
				if (a == this) return i;
			}
			return -1;
		}
		
		public String toString() {
			return description;
		}
	}

	/**
	 * Feature and object data extracted from the game source.
	 * Starts in the north-west corner of the map, by rows. 64x61
	 */
	private static int mainMap[] = {
		0x0a, 0x0a, 0x6b, 0x0f, 0xc8, 0x0f, 0x00, 0x00,
		0x61, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x68, 0x0f, 0x0f, 0x0f,
		0x0a, 0x0a, 0x0c, 0x5f, 0x0f, 0x0f, 0x73, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x1f, 0x0a, 0x0a,
		0xbc, 0x0f, 0x0f, 0x0b, 0x0f, 0x68, 0x0f, 0x0f,
		0x0f, 0x0a, 0x0f, 0x68, 0x0f, 0x0f, 0x0f, 0x68,
		0x0f, 0x0f, 0x0a, 0x0a, 0x0a, 0x0f, 0x74, 0x0f,
		0x0a, 0x0f, 0x0f, 0x0f, 0x46, 0x30, 0x00, 0x00,
		0x00, 0x00, 0x0f, 0x0f, 0x0f, 0xa8, 0x0f, 0x02,
		0x02, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0a,
		0x0a, 0x0a, 0x0a, 0x06, 0x56, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0xa8, 0x0f, 0x0f, 0x0f, 0x5f, 0x68,
		0x0f, 0x0f, 0x0f, 0x06, 0x06, 0x06, 0x99, 0x06,
		0x26, 0x0f, 0x5f, 0x0f, 0x93, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x3d, 0x0a, 0x0a, 0x99, 0x0f, 0xa8,
		0x0f, 0x0f, 0x0f, 0x06, 0x00, 0x30, 0x00, 0x00,
		0x00, 0x00, 0x40, 0x0f, 0x0f, 0x1f, 0x02, 0x02,
		0x02, 0x12, 0x0f, 0x0f, 0xbc, 0x5f, 0x0a, 0x0a,
		0x00, 0x00, 0x00, 0x20, 0x20, 0x06, 0x06, 0x06,
		0x0f, 0x0f, 0x5f, 0x1f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x06, 0x46, 0x06, 0x56, 0x36, 0x36, 0x16,
		0x06, 0x26, 0x06, 0x3d, 0x0f, 0x5f, 0x0c, 0x0f,
		0x06, 0x26, 0x0f, 0x0a, 0x0a, 0x68, 0x0f, 0x0a,
		0x0f, 0x02, 0x16, 0x06, 0x2d, 0x40, 0x00, 0x00,
		0x00, 0x00, 0x02, 0x02, 0x02, 0x0f, 0x02, 0x02,
		0x02, 0x02, 0xb3, 0x0f, 0x0f, 0xdd, 0x00, 0x00,
		0x00, 0x20, 0x00, 0x00, 0x00, 0xcb, 0x20, 0x00,
		0x0f, 0x0f, 0x0f, 0x99, 0x0f, 0x68, 0x0f, 0x0f,
		0x06, 0x06, 0x06, 0x56, 0x06, 0x06, 0x67, 0x06,
		0x06, 0x16, 0x06, 0x06, 0x36, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0a, 0x0f, 0x0a, 0x0a,
		0x0f, 0x02, 0x02, 0x99, 0x0f, 0x0f, 0x00, 0x00,
		0x00, 0x40, 0x02, 0x12, 0x02, 0x0f, 0x0f, 0x02,
		0x02, 0x0f, 0x0f, 0x06, 0x06, 0x00, 0x30, 0x30,
		0x6d, 0x42, 0xe4, 0x02, 0x67, 0x00, 0x00, 0x00,
		0x00, 0xcb, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x06, 0x68, 0x06, 0x06, 0x06, 0x06,
		0x06, 0x6d, 0x36, 0x06, 0x06, 0x0f, 0x0f, 0x06,
		0x5f, 0xb8, 0x0f, 0x0f, 0x0c, 0x0a, 0x0a, 0x0a,
		0x0f, 0x0f, 0x12, 0x02, 0x02, 0x1f, 0x0f, 0x20,
		0x42, 0x02, 0x02, 0x02, 0x02, 0x0f, 0x0f, 0x0f,
		0x7c, 0x0f, 0x0f, 0x20, 0x00, 0x0f, 0x61, 0x02,
		0x42, 0x02, 0x02, 0x12, 0x5f, 0x0f, 0x65, 0x00,
		0x00, 0x00, 0x40, 0x00, 0x20, 0x30, 0x0f, 0x0f,
		0x0f, 0xcc, 0x0f, 0x06, 0x46, 0x06, 0x56, 0x06,
		0x56, 0x06, 0x06, 0x06, 0x26, 0x06, 0x1f, 0x0f,
		0x1f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0a, 0x0a,
		0x0f, 0x0f, 0x02, 0x02, 0x02, 0x02, 0x0f, 0x0f,
		0x02, 0x02, 0x02, 0x02, 0x12, 0x02, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0xcb, 0x00, 0x30, 0x06, 0x06,
		0x6b, 0x0f, 0x0f, 0x0f, 0x1f, 0x0f, 0x0f, 0x5f,
		0x67, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00,
		0x0f, 0x0f, 0x0f, 0x0f, 0x2d, 0x06, 0x06, 0x16,
		0x06, 0x06, 0x26, 0xdb, 0x0f, 0x0f, 0x0f, 0xdc,
		0x0f, 0x0f, 0x0f, 0x06, 0x0f, 0x0f, 0x0f, 0x0a,
		0x0f, 0xb8, 0x0f, 0x12, 0x02, 0x12, 0x0f, 0x0f,
		0x02, 0x02, 0x02, 0x02, 0x02, 0x12, 0x02, 0x02,
		0x0f, 0x0f, 0x0f, 0x0f, 0xcc, 0x00, 0x00, 0x67,
		0x06, 0x06, 0x0f, 0x65, 0x0f, 0x61, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xd1, 0x0f, 0x00,
		0x20, 0xcb, 0x02, 0x5f, 0x0f, 0x0f, 0x0f, 0x1f,
		0x06, 0x06, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x30,
		0x20, 0x00, 0x0f, 0xcd, 0x0f, 0x74, 0x99, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0xb8, 0x0f, 0x0f, 0x02,
		0x02, 0x02, 0x02, 0xb3, 0x40, 0x02, 0x42, 0x02,
		0x12, 0x0f, 0x0f, 0x0f, 0x1f, 0x99, 0x00, 0x00,
		0x00, 0x20, 0x02, 0xbc, 0x1f, 0x5f, 0x00, 0x00,
		0x00, 0x40, 0x20, 0x20, 0x40, 0x00, 0x06, 0x06,
		0x67, 0x0f, 0x1f, 0x0f, 0x68, 0x02, 0x99, 0x0f,
		0x99, 0x0f, 0x73, 0x0f, 0x0f, 0x3d, 0x00, 0x00,
		0x20, 0x67, 0x40, 0x00, 0x00, 0x30, 0x0f, 0x0f,
		0x5f, 0x0f, 0x0c, 0x9c, 0xbc, 0xcc, 0x7c, 0x02,
		0x02, 0x02, 0x6d, 0x65, 0x74, 0x0f, 0x65, 0x12,
		0x12, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x06, 0x0f, 0x00, 0x00,
		0x40, 0x40, 0x00, 0x40, 0x74, 0x00, 0x00, 0x67,
		0x1f, 0x7c, 0x0f, 0x0f, 0x42, 0x02, 0x02, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0xa8, 0x0f, 0x0f, 0x40,
		0x00, 0x00, 0x00, 0x99, 0x00, 0x00, 0x99, 0x1f,
		0x0f, 0x83, 0x0f, 0x1f, 0x0f, 0x0f, 0x0f, 0xad,
		0x02, 0x02, 0x06, 0x67, 0x99, 0xdc, 0x8b, 0x02,
		0x02, 0x0f, 0xcb, 0x0f, 0x0f, 0x0f, 0x0f, 0x02,
		0x00, 0x00, 0x00, 0x00, 0x40, 0x0f, 0x00, 0x00,
		0x00, 0x00, 0x99, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x6b, 0x1f, 0x0f, 0x02, 0x42, 0x12, 0x02, 0x02,
		0x0f, 0x0c, 0x0f, 0x7c, 0x1f, 0x0f, 0x02, 0xad,
		0x00, 0x00, 0x99, 0x00, 0x00, 0x99, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0c, 0x0c, 0xbc, 0xbc, 0xdc, 0x0f,
		0x02, 0x12, 0x12, 0x56, 0x06, 0x5f, 0x02, 0x02,
		0x02, 0x5f, 0x0f, 0x0f, 0x0f, 0x67, 0x0f, 0x0f,
		0x0f, 0x67, 0x30, 0x20, 0x30, 0x0f, 0x40, 0x00,
		0x00, 0x06, 0x36, 0x46, 0x46, 0x06, 0x30, 0x40,
		0x06, 0x46, 0x6d, 0x42, 0x02, 0x74, 0x02, 0x12,
		0x42, 0x02, 0x5f, 0x0f, 0x02, 0x0f, 0x68, 0x0f,
		0x00, 0x00, 0x00, 0x00, 0x0f, 0x68, 0x0f, 0x0f,
		0x1f, 0x0f, 0x0f, 0x0f, 0x16, 0x36, 0x0f, 0x02,
		0x42, 0x02, 0x02, 0x02, 0x02, 0x02, 0x12, 0x42,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x20, 0x0f,
		0x0f, 0x06, 0x06, 0x40, 0x40, 0x61, 0x00, 0x20,
		0x5f, 0x0f, 0x0f, 0x06, 0x67, 0x26, 0x06, 0x26,
		0x06, 0x42, 0x02, 0x42, 0x02, 0x02, 0x02, 0x02,
		0x02, 0x02, 0x12, 0x67, 0x1f, 0x0f, 0x1f, 0x06,
		0x00, 0x40, 0x00, 0x0f, 0x0f, 0x0f, 0x67, 0x0f,
		0x0f, 0x0f, 0x0f, 0x06, 0x06, 0x56, 0x06, 0x06,
		0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x0f, 0x0f,
		0x67, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x56, 0x06,
		0x36, 0x0f, 0x06, 0x06, 0x0f, 0x0f, 0x0f, 0x1f,
		0x0f, 0x0f, 0xdc, 0x0f, 0x0f, 0x0f, 0xdc, 0x0f,
		0x02, 0x02, 0x02, 0x99, 0x12, 0x02, 0x42, 0x02,
		0x12, 0x02, 0x12, 0x0f, 0x0f, 0x0f, 0x06, 0x67,
		0x99, 0x0f, 0x0f, 0x16, 0x5f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x06, 0x46, 0x06, 0x7c, 0x26,
		0x16, 0xab, 0x0f, 0x0f, 0x0f, 0x0f, 0x68, 0x0f,
		0x0f, 0x0f, 0x00, 0x00, 0x00, 0x06, 0x36, 0xcb,
		0x1f, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0x0f,
		0x0f, 0x0f, 0x02, 0x12, 0x02, 0x42, 0x02, 0x42,
		0x02, 0x02, 0x02, 0x02, 0x12, 0x02, 0x02, 0x0f,
		0x65, 0x02, 0x42, 0x0f, 0x0f, 0x26, 0x06, 0x1f,
		0x0f, 0x0f, 0x06, 0x0f, 0x65, 0x65, 0x0f, 0x0f,
		0x0f, 0x0f, 0x16, 0xdb, 0x06, 0x06, 0x06, 0x06,
		0x26, 0x0f, 0x2d, 0x0f, 0x9c, 0x0f, 0x0f, 0x0f,
		0x0f, 0x20, 0x00, 0x00, 0x02, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x5f,
		0x0f, 0x0f, 0x02, 0x02, 0x42, 0xdb, 0x02, 0x02,
		0x12, 0x02, 0x02, 0x02, 0x0c, 0x02, 0x02, 0x6b,
		0x0f, 0x0f, 0x0f, 0x0f, 0x12, 0x0f, 0x5f, 0x7c,
		0x0f, 0xc1, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x6d,
		0x1f, 0x0f, 0x06, 0x06, 0x06, 0x6d, 0x06, 0x06,
		0x06, 0x06, 0x0f, 0x0f, 0x0f, 0x1f, 0x67, 0x0f,
		0x00, 0x65, 0x00, 0x0f, 0x0f, 0x9c, 0x0f, 0x0f,
		0x0f, 0x99, 0x0f, 0x67, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x67, 0x02, 0x12, 0x02, 0x02, 0x65,
		0x02, 0x02, 0x61, 0x12, 0x02, 0x02, 0x02, 0x02,
		0x5f, 0x6b, 0x0f, 0x0f, 0x67, 0x65, 0x0f, 0x0f,
		0x0f, 0x0f, 0x65, 0xdb, 0x42, 0x02, 0x68, 0x0f,
		0xc8, 0x0f, 0x56, 0x16, 0x06, 0x06, 0x06, 0x65,
		0x06, 0x99, 0x5f, 0x5f, 0x0f, 0x0f, 0x0f, 0x00,
		0x00, 0x20, 0x00, 0x67, 0x1f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x12, 0x12, 0x02, 0x02, 0x12,
		0x02, 0x02, 0x02, 0x02, 0x42, 0x2d, 0x02, 0x02,
		0x65, 0x0f, 0x0f, 0x0f, 0x0f, 0x65, 0x65, 0x0f,
		0x99, 0x0f, 0x0f, 0x12, 0x02, 0x02, 0x02, 0x0f,
		0x0f, 0x0f, 0x06, 0x06, 0x06, 0x56, 0x26, 0x06,
		0x06, 0x06, 0x06, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x30, 0x30, 0x00, 0x06, 0x1f, 0x0f, 0x67, 0x0f,
		0x0f, 0x12, 0x42, 0x5f, 0x5f, 0x0f, 0x0f, 0x0f,
		0x0f, 0xdc, 0x0f, 0x02, 0x42, 0x02, 0x02, 0x02,
		0x00, 0x00, 0x02, 0x02, 0x12, 0x02, 0x12, 0x42,
		0x02, 0x0f, 0x99, 0xdc, 0x7c, 0x1f, 0x67, 0xdb,
		0x02, 0x12, 0x42, 0xbc, 0x02, 0x02, 0x02, 0x02,
		0x0f, 0x00, 0x00, 0x06, 0x56, 0x0f, 0xcb, 0x06,
		0x06, 0x0f, 0x0f, 0x0f, 0x02, 0x02, 0x0f, 0x0f,
		0x99, 0x00, 0x00, 0x06, 0x0f, 0x0f, 0x02, 0x42,
		0x42, 0x02, 0x12, 0x12, 0x0f, 0x0c, 0x0f, 0x0f,
		0xbc, 0x1f, 0x0c, 0x0f, 0x0f, 0x0f, 0x42, 0x02,
		0x02, 0x12, 0x02, 0x12, 0x65, 0x99, 0x42, 0x02,
		0x02, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00,
		0x00, 0x12, 0x12, 0x02, 0x0f, 0x02, 0x02, 0x02,
		0x0f, 0x00, 0x00, 0x00, 0x16, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x67, 0x0f,
		0x0f, 0x0f, 0x0f, 0x06, 0x06, 0x02, 0x42, 0x42,
		0x12, 0x99, 0x12, 0x02, 0x02, 0x0f, 0x0f, 0x0f,
		0x0f, 0x9c, 0x1f, 0xbc, 0x06, 0x6b, 0x02, 0x02,
		0x02, 0x12, 0x02, 0x0f, 0x0f, 0x99, 0x0f, 0x0f,
		0x1f, 0x67, 0x0f, 0x00, 0x00, 0x30, 0x00, 0x00,
		0x02, 0xc7, 0x02, 0x02, 0x65, 0x42, 0x12, 0x02,
		0x1f, 0xdb, 0x00, 0x20, 0x00, 0x0f, 0x0f, 0xcc,
		0x0f, 0x0f, 0x00, 0x30, 0x20, 0x30, 0x30, 0x00,
		0x0f, 0x0f, 0x61, 0x0f, 0x06, 0x12, 0x12, 0x65,
		0x12, 0x02, 0x02, 0x02, 0x02, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0xdc, 0x0f, 0x7c, 0x06, 0x5f, 0x02,
		0x02, 0x02, 0x67, 0x0f, 0x73, 0x5f, 0x0f, 0xcb,
		0x5f, 0x40, 0x20, 0x00, 0x20, 0x00, 0x20, 0x02,
		0x02, 0x02, 0x02, 0x02, 0x42, 0x0f, 0x02, 0x02,
		0x0f, 0x0f, 0x00, 0x30, 0x30, 0x0f, 0x5f, 0x0f,
		0x0f, 0x00, 0x40, 0x00, 0x00, 0x20, 0x20, 0x00,
		0x30, 0x0f, 0x0f, 0x0f, 0x42, 0x02, 0x02, 0x02,
		0x02, 0x02, 0x12, 0x02, 0x0f, 0x0f, 0x0f, 0x67,
		0x65, 0x0f, 0x26, 0xbc, 0x0f, 0x56, 0x0f, 0x0f,
		0x16, 0x06, 0x06, 0x46, 0x26, 0x06, 0x67, 0x00,
		0x00, 0x00, 0x20, 0x00, 0x20, 0x0f, 0x73, 0x02,
		0x02, 0x02, 0x74, 0x02, 0x02, 0xcc, 0x02, 0x02,
		0x0f, 0x0f, 0xa3, 0x00, 0x00, 0x1f, 0x0f, 0x0f,
		0x0f, 0x40, 0x00, 0x00, 0xdb, 0x65, 0x00, 0x20,
		0x00, 0x20, 0x0f, 0x99, 0x0f, 0x02, 0x02, 0x42,
		0x02, 0x12, 0x02, 0x0f, 0x0f, 0x0f, 0x1f, 0x0f,
		0x1f, 0x0f, 0x26, 0x0f, 0xbc, 0x0f, 0x0f, 0x67,
		0x0f, 0x0f, 0x0f, 0x65, 0x00, 0x00, 0x00, 0x00,
		0x20, 0x40, 0x20, 0x0f, 0x99, 0x1f, 0x0f, 0x0f,
		0x02, 0x42, 0x02, 0x02, 0x02, 0x02, 0x1f, 0x65,
		0x0f, 0x0f, 0x00, 0x00, 0x02, 0x1f, 0x0f, 0x99,
		0x0f, 0x40, 0x00, 0x00, 0x74, 0x99, 0x30, 0x00,
		0x20, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x1f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x06, 0xcd, 0x0f, 0xcc, 0x0f, 0x0f, 0x0f,
		0x1f, 0xcc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x0f, 0x6b, 0x02, 0x0f, 0x0f, 0x0f, 0x65, 0x0f,
		0x67, 0x02, 0x02, 0x12, 0x65, 0x0f, 0x8b, 0x1f,
		0x0f, 0x02, 0x00, 0x30, 0x99, 0x0f, 0x0f, 0x00,
		0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00,
		0x0f, 0x0f, 0x5f, 0x65, 0x20, 0x40, 0x30, 0x00,
		0x40, 0x00, 0x40, 0x00, 0x30, 0x00, 0x0f, 0x0f,
		0x67, 0x00, 0x30, 0x0f, 0xdb, 0xcb, 0x0f, 0x0f,
		0x99, 0x30, 0x00, 0x00, 0x00, 0x20, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x65, 0x0f, 0xbc, 0x1f, 0x65,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xad, 0x0f, 0x1f,
		0x0f, 0x20, 0x00, 0x30, 0x5f, 0x0f, 0x02, 0x00,
		0x30, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x30, 0x02,
		0x7c, 0x06, 0x00, 0x40, 0x00, 0x00, 0x40, 0x40,
		0x00, 0x00, 0x20, 0x00, 0x40, 0x40, 0x00, 0x00,
		0x00, 0x30, 0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x00,
		0x00, 0x00, 0x40, 0x20, 0x0f, 0x67, 0x00, 0x00,
		0x00, 0x30, 0x00, 0x40, 0x0f, 0x5f, 0x67, 0x0f,
		0xdb, 0x1f, 0x0f, 0x0c, 0x0f, 0x0f, 0xcb, 0x02,
		0x0f, 0x00, 0x00, 0x30, 0x0f, 0x0f, 0x20, 0x74,
		0x00, 0x40, 0x0f, 0x42, 0x00, 0x30, 0x00, 0x20,
		0x06, 0x0f, 0x40, 0x20, 0x00, 0x00, 0x02, 0x6b,
		0x65, 0x06, 0x20, 0x00, 0x00, 0x00, 0x30, 0x20,
		0x00, 0x00, 0x67, 0x0f, 0x0f, 0x74, 0x30, 0x00,
		0x20, 0x00, 0x00, 0x0f, 0x65, 0x0f, 0x99, 0x00,
		0x30, 0x30, 0x30, 0x00, 0x00, 0x00, 0x00, 0x20,
		0x00, 0x20, 0x65, 0x1f, 0x06, 0x06, 0x02, 0x02,
		0x0f, 0x0f, 0x20, 0x30, 0x0f, 0x5f, 0x00, 0x30,
		0x00, 0x30, 0x0f, 0x02, 0x00, 0x40, 0x00, 0x6b,
		0x0f, 0x67, 0x0f, 0x0f, 0x0f, 0x0c, 0x06, 0x06,
		0x0f, 0x0f, 0x99, 0x06, 0x00, 0x00, 0x00, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x40,
		0x00, 0x00, 0x67, 0x0f, 0x74, 0x0f, 0x0f, 0x0f,
		0x0f, 0x65, 0x40, 0x00, 0x00, 0x30, 0x40, 0x00,
		0x00, 0x00, 0x65, 0x0f, 0x6b, 0x06, 0x46, 0x42,
		0x0f, 0xa8, 0x02, 0x02, 0x0f, 0x0f, 0x20, 0x00,
		0x00, 0x00, 0x1f, 0x02, 0x00, 0x00, 0x00, 0x1f,
		0x0f, 0x0f, 0x00, 0x40, 0x0f, 0x06, 0x0f, 0x0f,
		0x61, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x67, 0x0f,
		0x0f, 0x0f, 0x00, 0x20, 0x00, 0x30, 0x00, 0x00,
		0xdb, 0x0f, 0x0f, 0x06, 0x30, 0x00, 0x02, 0x12,
		0x46, 0x0f, 0x0f, 0x67, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x61, 0x1f, 0x0f, 0x0f, 0x0f, 0x5f, 0xdc,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x20, 0x2d,
		0x20, 0x00, 0x0f, 0x0f, 0x30, 0x00, 0x20, 0x99,
		0xbc, 0x00, 0x20, 0x40, 0x40, 0x20, 0xdb, 0x0f,
		0x0f, 0x8b, 0x00, 0x00, 0x30, 0x40, 0x30, 0x00,
		0x00, 0x00, 0x40, 0x20, 0x00, 0x00, 0x00, 0x0f,
		0x0f, 0x65, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x40,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x0f, 0x0f, 0x0f, 0x0f, 0x74, 0x74, 0x1f, 0x1f,
		0x0f, 0x0f, 0x0f, 0x00, 0x1f, 0x0f, 0x00, 0x20,
		0x00, 0x30, 0x5f, 0x1f, 0x00, 0x00, 0x00, 0x00,
		0x20, 0x00, 0x74, 0x20, 0x00, 0x40, 0x0f, 0x0f,
		0x0f, 0x30, 0x00, 0x40, 0x00, 0x00, 0x00, 0x30,
		0x00, 0x00, 0x40, 0x00, 0x20, 0x00, 0x3d, 0x0f,
		0x99, 0x46, 0x46, 0xcb, 0x00, 0x40, 0x00, 0x30,
		0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x67,
		0x0f, 0x0f, 0xab, 0x0f, 0x1f, 0x0f, 0x0c, 0x99,
		0x0f, 0x0f, 0x0f, 0x00, 0x40, 0x0f, 0x00, 0x00,
		0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x40, 0x67, 0x5f, 0x67,
		0x00, 0x00, 0x20, 0x00, 0x40, 0x40, 0x00, 0x00,
		0x00, 0x00, 0x30, 0x00, 0x6b, 0x0f, 0x0f, 0x5f,
		0x0f, 0x0f, 0x06, 0xc1, 0x0f, 0x1f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x65, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x02, 0x65, 0x0f, 0x99, 0x0f,
		0x0f, 0x0f, 0x68, 0x00, 0x00, 0x0f, 0x00, 0x00,
		0x40, 0x00, 0x00, 0x1f, 0x1f, 0x67, 0x20, 0x00,
		0x20, 0x40, 0x30, 0x0f, 0x0f, 0x5f, 0x0f, 0x0f,
		0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x0f,
		0x0f, 0x0f, 0x67, 0x0f, 0x0f, 0x0f, 0x0f, 0x02,
		0x0f, 0x0f, 0x0f, 0xcc, 0x0f, 0x0f, 0x1f, 0x65,
		0x0f, 0x7c, 0x0f, 0x0f, 0xad, 0x0f, 0x0f, 0x65,
		0x0f, 0x7c, 0x0f, 0x0f, 0x0f, 0x0f, 0x02, 0x0f,
		0x5f, 0x0f, 0x20, 0x00, 0x00, 0x0f, 0x0f, 0x20,
		0x30, 0x0f, 0xdb, 0x0f, 0x5f, 0x0f, 0x0f, 0xab,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x93, 0x0f, 0x0f, 0x67, 0x0f,
		0x0f, 0x0f, 0x1f, 0x9c, 0x0b, 0x0f, 0x0f, 0x0f,
		0x0f, 0xbc, 0x1f, 0x0f, 0x16, 0x06, 0x0f, 0xb3,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x5f, 0xd7, 0x0f, 0x0f, 0x1f, 0x0f,
		0x0f, 0x1f, 0x30, 0x00, 0x20, 0x00, 0x0f, 0x00,
		0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x5f, 0x0f, 0x0f,
		0x1f, 0x9c, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x1f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x02, 0x02, 0x02, 0x0f, 0x0f, 0x0f, 0x65,
		0x0f, 0x99, 0x0f, 0x0f, 0x65, 0x06, 0x06, 0x06,
		0x06, 0x06, 0x0f, 0x65, 0x0f, 0x0f, 0x99, 0x0f,
		0x5f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x65, 0x0f,
		0x0f, 0x0f, 0x00, 0x40, 0x40, 0x00, 0x0f, 0x0f,
		0x5f, 0x0f, 0x02, 0x02, 0x02, 0x42, 0x0f, 0x30,
		0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x67, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x1f, 0x0f, 0x0f, 0x0f, 0x02,
		0x02, 0x02, 0x02, 0x42, 0x42, 0x02, 0x02, 0x02,
		0x02, 0x0f, 0xcb, 0x0f, 0x56, 0x06, 0x46, 0x06,
		0x06, 0x06, 0xbc, 0x36, 0x06, 0x0b, 0x0f, 0x0f,
		0x5f, 0x65, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f,
		0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x0f, 0xdc, 0x5f,
		0x0f, 0x02, 0x02, 0x02, 0x02, 0x12, 0x02, 0x02,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x99, 0x0f, 0x0f, 0x0f, 0x42, 0x02,
		0x02, 0x02, 0x02, 0x12, 0x02, 0x02, 0x12, 0x02,
		0x02, 0x0f, 0x0f, 0x0f, 0x36, 0x06, 0xcd, 0x06,
		0x65, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 0x0f,
		0x0a, 0x0f, 0x0f, 0x26, 0x06, 0x5f, 0x0f, 0x1f,
		0x42, 0x02, 0x02, 0x42, 0x12, 0x12, 0x02, 0x02,
		0x02, 0x0f, 0x5f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x65, 0x02, 0x12,
		0x02, 0x65, 0x99, 0x12, 0x12, 0x12, 0x12, 0x1f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x26, 0x56, 0x26, 0x06,
		0x06, 0x02, 0x06, 0x99, 0x06, 0x46, 0x67, 0x06,
		0x46, 0x06, 0x0f, 0x0f, 0x0f, 0xab, 0x02, 0x02,
		0x0a, 0x0f, 0x99, 0x0f, 0x06, 0x46, 0x0f, 0x02,
		0x02, 0x12, 0x02, 0x02, 0x06, 0x02, 0x12, 0x12,
		0x02, 0x0f, 0x0f, 0xcb, 0x0f, 0x0f, 0x0f, 0x0f,
		0x1f, 0x0f, 0x0f, 0x67, 0x0f, 0x02, 0x42, 0x02,
		0x02, 0x26, 0x02, 0x3d, 0x12, 0x02, 0x0f, 0x0f,
		0x42, 0x0f, 0x12, 0x0f, 0x0f, 0x0f, 0x26, 0x0c,
		0x06, 0x46, 0x06, 0x06, 0x06, 0x65, 0x26, 0x06,
		0x06, 0x0f, 0x9c, 0x02, 0x12, 0x02, 0x02, 0x02,
		0x0a, 0x0f, 0x0f, 0x0f, 0x06, 0x06, 0x06, 0xdb,
		0x02, 0x02, 0x02, 0x06, 0x74, 0x0f, 0x0f, 0x02,
		0x0f, 0x5f, 0x0f, 0x0f, 0x0f, 0x0f, 0x67, 0x0f,
		0x0f, 0x67, 0x0f, 0x0f, 0x0f, 0x02, 0x12, 0x02,
		0x02, 0x42, 0x42, 0x02, 0x02, 0x0b, 0x0f, 0x65,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x67, 0x26, 0x36, 0x06, 0x26, 0x06, 0x06, 0x06,
		0x0f, 0x0f, 0x12, 0x42, 0x02, 0x02, 0x02, 0x02,
		0x0f, 0x0f, 0x0f, 0x20, 0x20, 0x0f, 0x26, 0x1f,
		0x12, 0x02, 0x12, 0x0f, 0x26, 0x06, 0x02, 0x02,
		0x12, 0x0f, 0xcc, 0x0f, 0x65, 0x06, 0x06, 0x56,
		0x0f, 0x0f, 0x1f, 0x0f, 0x0f, 0xcb, 0x12, 0x02,
		0x42, 0x02, 0x12, 0x65, 0x42, 0x0f, 0x0f, 0x1f,
		0xb3, 0x0f, 0xd7, 0x1f, 0x2d, 0x65, 0x5f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x06, 0x8b, 0x06, 0x06, 0xc7,
		0x02, 0x02, 0x02, 0x42, 0x02, 0x02, 0x42, 0x02,
		0x0f, 0x0f, 0x00, 0x00, 0x00, 0xad, 0x0f, 0x0f,
		0x5f, 0x12, 0x02, 0x02, 0x42, 0x02, 0x02, 0x02,
		0x1f, 0x67, 0x0f, 0x06, 0x46, 0x36, 0x06, 0x06,
		0x06, 0x16, 0x8b, 0x0f, 0xc1, 0x0f, 0x0f, 0x0f,
		0x02, 0x02, 0x02, 0x02, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0xcc, 0x0f, 0x0f, 0x0f, 0x06, 0x0f, 0x06,
		0x0f, 0x65, 0x0f, 0x0f, 0x06, 0x06, 0x0f, 0x02,
		0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
		0x1f, 0x0f, 0x00, 0x00, 0x20, 0x30, 0x0f, 0x0f,
		0x0f, 0x0f, 0x02, 0x02, 0x02, 0x02, 0x02, 0x3d,
		0x5f, 0x06, 0x16, 0x02, 0x06, 0x56, 0x99, 0x46,
		0x65, 0x46, 0x26, 0x06, 0x42, 0x0f, 0x65, 0x0f,
		0x1f, 0xbc, 0x0f, 0x5f, 0x0f, 0x67, 0x0f, 0x0f,
		0x99, 0x5f, 0x46, 0x0f, 0x06, 0x0f, 0xdc, 0x0f,
		0x6b, 0x0f, 0x1f, 0x0f, 0x0f, 0x0f, 0x0f, 0x12,
		0x12, 0x02, 0x02, 0xc1, 0x02, 0xad, 0x02, 0x42,
		0x5f, 0x0f, 0x20, 0x20, 0x30, 0x00, 0x20, 0x0f,
		0x06, 0x0f, 0x0f, 0x42, 0x02, 0x42, 0x0f, 0x0f,
		0x06, 0x06, 0x26, 0x06, 0x06, 0x06, 0x06, 0x06,
		0x06, 0x56, 0x06, 0x06, 0x06, 0xcc, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x06, 0x0f,
		0x0f, 0x65, 0x5f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x65, 0x06, 0x0f, 0x42, 0x02, 0x02,
		0x12, 0x02, 0x12, 0x02, 0x06, 0x02, 0x02, 0x02,
		0x0f, 0x6b, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x20,
		0x0f, 0x06, 0x06, 0x1f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x0f, 0x0f, 0x0f, 0x06, 0x26, 0x16, 0x16, 0x06,
		0x06, 0x26, 0x65, 0x26, 0x46, 0x0f, 0x99, 0x65,
		0x0f, 0x0f, 0x65, 0x0f, 0x0f, 0x0f, 0x0f, 0x8b,
		0x0f, 0x0f, 0x0f, 0x00, 0x61, 0x00, 0x99, 0x0f,
		0x06, 0x0f, 0x0f, 0x0f, 0x0f, 0x02, 0x02, 0x12,
		0x12, 0x02, 0x02, 0x02, 0x7c, 0x02, 0x02, 0x02,
		0x0f, 0x0f, 0xa8, 0x0f, 0x30, 0x40, 0x00, 0x00,
		0x00, 0x00, 0x0f, 0x0f, 0xab, 0x0f, 0xbc, 0x0f,
		0x0f, 0x40, 0x00, 0x0c, 0x5f, 0x0f, 0x65, 0x46,
		0x06, 0x06, 0x06, 0x06, 0x1f, 0x67, 0x40, 0x00,
		0x5f, 0x0f, 0x5f, 0x0f, 0xdc, 0x0f, 0x0f, 0x0f,
		0x20, 0x0f, 0x67, 0x0f, 0x00, 0x00, 0x40, 0x00,
		0x00, 0x06, 0x0f, 0x0f, 0xbc, 0x0f, 0x65, 0x42,
		0x02, 0x02, 0x42, 0x02, 0x99, 0x02, 0x12, 0x02,
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x30, 0x40, 0x67,
		0x40, 0x20, 0x67, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0xb3, 0x00, 0x20, 0x0f, 0x5f, 0x0f, 0x0f, 0x0f,
		0x06, 0x46, 0x06, 0x0f, 0x0f, 0x00, 0x00, 0x00,
		0x0f, 0xcb, 0x0f, 0x0f, 0x0f, 0x65, 0x99, 0x00,
		0x40, 0x0f, 0x00, 0x0f, 0x8b, 0x30, 0x0f, 0x00,
		0x99, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x02, 0x02,
		0x12, 0x02, 0x02, 0x02, 0x02, 0x74, 0x02, 0x02,
		0x0f, 0x0f, 0x0f, 0x0f, 0x99, 0x30, 0x40, 0x00,
		0x00, 0x00, 0x00, 0x5f, 0x0f, 0x0f, 0x7c, 0x0f,
		0x0f, 0x00, 0x00, 0x02, 0x5f, 0x0f, 0x0f, 0x0f,
		0x0f, 0xcc, 0x5f, 0x1f, 0x00, 0x40, 0x00, 0x42,
		0x46, 0x46, 0x0f, 0x65, 0x99, 0x20, 0x00, 0x00,
		0x00, 0x0f, 0x00, 0x00, 0x0f, 0x40, 0xcb, 0x0f,
		0x67, 0x0f, 0x5f, 0x99, 0x0f, 0x42, 0x02, 0x42,
		0x12, 0x42, 0x02, 0x02, 0x42, 0x65, 0x42, 0x02,
		0x0f, 0x1f, 0x0f, 0xcb, 0x40, 0x00, 0x00, 0x00,
		0x40, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f,
		0xcb, 0x00, 0x30, 0x02, 0x02, 0x67, 0x0f, 0x65,
		0x06, 0x0f, 0x02, 0x00, 0x00, 0x40, 0x02, 0x0f,
		0x0f, 0x0f, 0x6d, 0x40, 0x00, 0x00, 0x74, 0x30,
		0x30, 0x0f, 0x40, 0x00, 0x0f, 0x67, 0x0f, 0x00,
		0x00, 0xcc, 0x5f, 0x36, 0x65, 0x02, 0x02, 0x02,
		0x0f, 0x0f, 0x02, 0x02, 0x12, 0x02, 0x0f, 0x42,
		0x0f, 0x6d, 0x0f, 0x30, 0x40, 0x40, 0x40, 0x00,
		0x00, 0x20, 0x20, 0x00, 0x99, 0x0f, 0xbc, 0x0f,
		0x20, 0x00, 0x00, 0x3d, 0x36, 0x26, 0x56, 0x06,
		0x0f, 0x02, 0x00, 0x00, 0x00, 0x06, 0x0f, 0x5f,
		0x0f, 0x20, 0x20, 0x40, 0x40, 0x00, 0x00, 0x00,
		0xdb, 0x0f, 0x00, 0x00, 0x00, 0x0f, 0x30, 0x99,
		0x00, 0x00, 0x0f, 0x5f, 0xdb, 0x0f, 0x67, 0x1f,
		0x0f, 0x65, 0x0f, 0x02, 0x12, 0x02, 0x0f, 0x02,
		0x0f, 0x0f, 0x00, 0x00, 0x20, 0x00, 0x12, 0x02,
		0x02, 0x00, 0x40, 0x00, 0x40, 0x0f, 0x0f, 0x0f,
		0x00, 0x00, 0x20, 0x06, 0x06, 0x65, 0x0f, 0x0f,
		0x0f, 0x02, 0x00, 0x40, 0x06, 0x0f, 0x65, 0x0f,
		0x00, 0x20, 0x40, 0x00, 0x40, 0x02, 0x12, 0x67,
		0x06, 0x0f, 0x67, 0x00, 0x00, 0x0f, 0x20, 0x30,
		0x00, 0x40, 0x67, 0x65, 0xdc, 0x56, 0x06, 0x5f,
		0x0c, 0x0f, 0x12, 0x12, 0x12, 0x83, 0x02, 0x12,
		0x0f, 0x5f, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x65,
		0x0f, 0x99, 0x00, 0x00, 0x40, 0x0f, 0xbc, 0x0f,
		0x30, 0x00, 0x65, 0x06, 0x1f, 0x0f, 0x0f, 0x0f,
		0x65, 0x00, 0x00, 0x00, 0x56, 0x0f, 0x0f, 0x00,
		0x00, 0x20, 0x00, 0x99, 0x0f, 0xdb, 0x99, 0x65,
		0x0f, 0x00, 0x20, 0x00, 0x00, 0x0f, 0x67, 0x00,
		0x20, 0x00, 0x30, 0x40, 0x20, 0x74, 0x0f, 0x5f,
		0x06, 0x06, 0x02, 0x12, 0x0f, 0x02, 0x12, 0x12,
		0x0f, 0x5f, 0x40, 0x00, 0x00, 0x65, 0x0f, 0x99,
		0x0f, 0x5f, 0x40, 0x20, 0x00, 0x0f, 0x0f, 0x0f,
		0x00, 0x00, 0x06, 0x56, 0x0f, 0x0f, 0x0f, 0x0f,
		0x40, 0x00, 0x00, 0x06, 0x0f, 0x1f, 0x0f, 0x0f,
		0x1f, 0x1f, 0x0f, 0xdc, 0x06, 0x0f, 0x00, 0x00,
		0x30, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x00,
		0x65, 0x20, 0x00, 0x00, 0x00, 0x00, 0x30, 0x00,
		0x30, 0x5f, 0x0f, 0xbc, 0x0f, 0x0f, 0x02, 0x02,
		0x26, 0x0f, 0x30, 0x00, 0x00, 0x0f, 0x74, 0x16,
		0x0f, 0x65, 0x00, 0x00, 0x67, 0x0f, 0x7c, 0x0f,
		0x30, 0x00, 0x06, 0x0f, 0x0f, 0x0f, 0x02, 0x20,
		0x00, 0x67, 0x06, 0x0f, 0x65, 0x0f, 0x0f, 0x0f,
		0x0f, 0x5f, 0x00, 0x00, 0x00, 0x20, 0x00, 0x40,
		0x00, 0x00, 0x00, 0x30, 0x67, 0x0f, 0x40, 0x00,
		0x00, 0x99, 0x00, 0x00, 0x30, 0x00, 0x00, 0xd7,
		0x00, 0x30, 0x00, 0x20, 0x0f, 0x65, 0xad, 0x0f,
		0x26, 0x0f, 0x20, 0x30, 0x20, 0x65, 0x0f, 0x67,
		0x0f, 0x0f, 0xc1, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
		0x00, 0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00,
		0x30, 0x06, 0x0f, 0x0f, 0x0f, 0x0f, 0x02, 0x02,
		0x00, 0x40, 0x40, 0x20, 0x00, 0x00, 0x40, 0x00,
		0x00, 0x00, 0x20, 0x00, 0x00, 0x5f, 0x00, 0xdb,
		0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x20, 0x00,
		0x00, 0x40, 0x30, 0x20, 0x0f, 0x0f, 0xcb, 0x1f,
		0x26, 0x0f, 0x00, 0x20, 0x40, 0x02, 0x0f, 0x65,
		0x0f, 0x30, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x40,
		0x00, 0x67, 0x0f, 0x0f, 0x0f, 0x67, 0x00, 0x00,
		0x00, 0x36, 0x99, 0x0f, 0x0f, 0x02, 0x42, 0x30,
		0x20, 0x20, 0x00, 0x00, 0x00, 0x67, 0x65, 0x0f,
		0x02, 0x0f, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x40,
		0x20, 0x06, 0x06, 0x99, 0x0f, 0x00, 0x30, 0x40,
		0x00, 0xb3, 0x30, 0x40, 0x00, 0x0c, 0x0f, 0x0f,
		0x06, 0x0f, 0x99, 0x00, 0x40, 0x40, 0x99, 0x02,
		0x67, 0x00, 0x00, 0x20, 0x67, 0x0f, 0x0f, 0x00,
		0x00, 0x0f, 0x0f, 0x5f, 0x02, 0x00, 0x30, 0x00,
		0x0c, 0x0f, 0x0f, 0xdb, 0x12, 0x02, 0x12, 0x00,
		0x00, 0x00, 0xad, 0x06, 0x06, 0x06, 0x0f, 0xcc,
		0x65, 0x0f, 0x02, 0x00, 0x00, 0x0f, 0x40, 0x00,
		0x0f, 0x6b, 0x0f, 0x1f, 0x02, 0x65, 0x99, 0x5f,
		0x00, 0x20, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x02,
		0x1f, 0xbc, 0x1f, 0x40, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x1f, 0x00,
		0x0f, 0x65, 0x1f, 0x0f, 0x02, 0x00, 0x20, 0x30,
		0x0f, 0x0f, 0x5f, 0x0f, 0x0f, 0x02, 0x02, 0x00,
		0x00, 0x02, 0x65, 0x0f, 0x65, 0x06, 0x56, 0x67,
		0x0f, 0x0f, 0x06, 0x16, 0x0f, 0x0f, 0x0f, 0x99,
		0x65, 0x1f, 0x0f, 0x0c, 0x06, 0x0f, 0x65, 0xcc,
		0x67, 0x00, 0x00, 0x20, 0x06, 0x0f, 0x12, 0x67,
		0x5f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x30, 0x00,
		0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x00, 0x00,
		0x0f, 0x0f, 0x0f, 0x0f, 0x40, 0x20, 0x00, 0x99,
		0x0f, 0x0f, 0x02, 0x02, 0x02, 0x0f, 0x02, 0x00,
		0x00, 0x0f, 0x06, 0xa3, 0x0f, 0x65, 0x0f, 0xcb,
		0x36, 0x06, 0x67, 0x0f, 0x0f, 0x61, 0x1f, 0x0f,
		0x06, 0x1f, 0x06, 0x02, 0x0f, 0x0b, 0x0f, 0x0f,
		0x99, 0x0f, 0x06, 0x5f, 0xbc, 0x1f, 0x0f, 0x0f,
		0x0a, 0x99, 0x0f, 0x0f, 0x67, 0x5f, 0x0f, 0x0f,
		0x1f, 0xcb, 0x0f, 0x0f, 0x0f, 0x0f, 0x67, 0x6b,
		0x0f, 0x0f, 0x1f, 0xab, 0x6b, 0x00, 0x00, 0x67,
		0x0f, 0x12, 0x12, 0x42, 0x12, 0x02, 0x0f, 0x1f,
		0x0f, 0x67, 0x0f, 0x1f, 0x74, 0x5f, 0x99, 0x0f,
		0x0f, 0xdc, 0x46, 0x0f, 0x65, 0x0f, 0x0f, 0x02,
		0x0f, 0x0f, 0x0f, 0x0f, 0x65, 0x0f, 0x06, 0x0f,
		0x06, 0x0f, 0x0f, 0x67, 0x40, 0x1f, 0x65, 0x0f,
	};
	
	/**
	 * Area and domain and special flags data extracted from the game source.
	 * Starts in the north-west corner of the map, by rows. 64x61
	 */
	private static final int referenceDescriptionMap[] = {
		0x00, 0x00, 0x42, 0x02, 0x42, 0x02, 0x02, 0x02,
		0x02, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,
		0x07, 0x07, 0x07, 0x07, 0x47, 0x07, 0x07, 0x07,
		0x00, 0x00, 0x47, 0x07, 0x07, 0x07, 0x08, 0x07,
		0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x00, 0x00,
		0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x6c, 0x2c, 0x2c,
		0x2c, 0x00, 0x2d, 0x6d, 0x2d, 0x2d, 0x2d, 0x6d,
		0x2d, 0x2d, 0x00, 0x00, 0x00, 0x2d, 0x2d, 0x2d,
		0x00, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
		0x02, 0x02, 0x07, 0x07, 0x07, 0x47, 0x07, 0x01,
		0x01, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x00,
		0x00, 0x00, 0x00, 0x0b, 0x0b, 0x07, 0x07, 0x07,
		0x2c, 0x2c, 0x6c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c,
		0x2c, 0x2c, 0x2c, 0x2d, 0x2d, 0x2d, 0x2e, 0x2d,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2e, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x00, 0x00, 0x2d, 0x2d, 0x6d,
		0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
		0x02, 0x02, 0x02, 0x07, 0x07, 0x07, 0x01, 0x01,
		0x01, 0x01, 0x07, 0x07, 0x47, 0x07, 0x00, 0x00,
		0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
		0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c,
		0x2c, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x6d, 0x2d, 0x2d, 0x2e, 0x2d,
		0x2d, 0x2d, 0x2d, 0x00, 0x00, 0x6d, 0x2d, 0x00,
		0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
		0x02, 0x02, 0x01, 0x01, 0x01, 0x07, 0x01, 0x01,
		0x01, 0x01, 0x09, 0x07, 0x07, 0x0b, 0x0b, 0x0b,
		0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x4b, 0x0b, 0x0b,
		0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x6c, 0x2c, 0x2c,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x2d, 0x00, 0x2d, 0x00, 0x00,
		0x01, 0x01, 0x01, 0x02, 0x01, 0x01, 0x02, 0x02,
		0x02, 0x02, 0x01, 0x01, 0x01, 0x07, 0x07, 0x01,
		0x01, 0x07, 0x07, 0x0d, 0x0d, 0x0b, 0x0b, 0x0b,
		0x0d, 0x0c, 0x0c, 0x0c, 0x4d, 0x0b, 0x0b, 0x0b,
		0x0b, 0x6c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c,
		0x2c, 0x2c, 0x2d, 0x6d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2e, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2f, 0x00, 0x00, 0x00,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x07, 0x07, 0x07,
		0x47, 0x07, 0x07, 0x0a, 0x0a, 0x0d, 0x0e, 0x0c,
		0x0c, 0x0c, 0x0c, 0x0c, 0x0d, 0x0d, 0x4d, 0x0b,
		0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x2c, 0x2c,
		0x2c, 0x6c, 0x2c, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x00, 0x00,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x07, 0x07,
		0x07, 0x07, 0x07, 0x0a, 0x0a, 0x0a, 0x4d, 0x4d,
		0x4d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d,
		0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
		0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2d, 0x2d, 0x2d,
		0x2d, 0x2d, 0x2d, 0x30, 0x2d, 0x2d, 0x2d, 0x30,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x00,
		0x03, 0x43, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
		0x07, 0x07, 0x07, 0x07, 0x4a, 0x0a, 0x0a, 0x4d,
		0x4d, 0x4d, 0x0d, 0x4d, 0x4d, 0x0f, 0x0d, 0x0d,
		0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x31, 0x31, 0x0b,
		0x0b, 0x71, 0x31, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c,
		0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x30,
		0x30, 0x30, 0x2d, 0x30, 0x2d, 0x2f, 0x2f, 0x2d,
		0x03, 0x03, 0x01, 0x01, 0x41, 0x01, 0x01, 0x01,
		0x01, 0x01, 0x01, 0x41, 0x41, 0x01, 0x01, 0x01,
		0x01, 0x07, 0x07, 0x07, 0x07, 0x0a, 0x0a, 0x0a,
		0x0a, 0x0a, 0x4d, 0x4d, 0x0d, 0x0d, 0x10, 0x10,
		0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x31, 0x31,
		0x71, 0x31, 0x31, 0x31, 0x31, 0x32, 0x59, 0x19,
		0x59, 0x19, 0x30, 0x19, 0x19, 0x59, 0x30, 0x30,
		0x30, 0x2f, 0x30, 0x30, 0x30, 0x30, 0x2d, 0x2d,
		0x03, 0x03, 0x43, 0x43, 0x43, 0x43, 0x43, 0x01,
		0x01, 0x01, 0x41, 0x41, 0x01, 0x01, 0x41, 0x01,
		0x01, 0x07, 0x07, 0x07, 0x07, 0x07, 0x0a, 0x0a,
		0x0a, 0x0a, 0x0a, 0x0a, 0x4d, 0x0d, 0x10, 0x10,
		0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x71,
		0x31, 0x31, 0x31, 0x31, 0x32, 0x32, 0x32, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x59, 0x19, 0x19, 0x30,
		0x30, 0x30, 0x30, 0x70, 0x30, 0x30, 0x6d, 0x2d,
		0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x41, 0x41, 0x01,
		0x01, 0x05, 0x47, 0x07, 0x07, 0x07, 0x11, 0x4a,
		0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0d, 0x10, 0x10,
		0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10,
		0x71, 0x31, 0x31, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x19, 0x59, 0x19, 0x59, 0x19, 0x19, 0x32, 0x19,
		0x30, 0x30, 0x30, 0x30, 0x30, 0x59, 0x19, 0x19,
		0x03, 0x03, 0x43, 0x43, 0x43, 0x43, 0x43, 0x03,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x41, 0x01, 0x01,
		0x01, 0x05, 0x05, 0x05, 0x07, 0x47, 0x11, 0x11,
		0x11, 0x4a, 0x0a, 0x0a, 0x0a, 0x0d, 0x10, 0x10,
		0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10,
		0x10, 0x10, 0x31, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x19, 0x19, 0x32, 0x19, 0x59, 0x19,
		0x30, 0x30, 0x30, 0x30, 0x19, 0x59, 0x19, 0x19,
		0x03, 0x03, 0x03, 0x03, 0x04, 0x04, 0x03, 0x01,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
		0x05, 0x05, 0x05, 0x05, 0x05, 0x06, 0x06, 0x11,
		0x11, 0x0a, 0x0a, 0x0a, 0x0a, 0x11, 0x10, 0x10,
		0x11, 0x11, 0x11, 0x10, 0x51, 0x10, 0x10, 0x10,
		0x10, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x30, 0x30, 0x30, 0x19, 0x19, 0x19, 0x30, 0x19,
		0x03, 0x03, 0x03, 0x04, 0x04, 0x04, 0x04, 0x04,
		0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x05, 0x05,
		0x45, 0x05, 0x05, 0x06, 0x06, 0x06, 0x06, 0x06,
		0x06, 0x11, 0x0a, 0x0a, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x51, 0x11, 0x11, 0x11, 0x51, 0x11,
		0x32, 0x32, 0x32, 0x19, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x19, 0x19, 0x19, 0x19, 0x59,
		0x59, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x03, 0x03, 0x03, 0x04, 0x04, 0x04, 0x44, 0x04,
		0x04, 0x05, 0x05, 0x05, 0x05, 0x05, 0x45, 0x05,
		0x05, 0x05, 0x06, 0x06, 0x06, 0x06, 0x06, 0x46,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x19,
		0x59, 0x32, 0x32, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x59, 0x59, 0x19, 0x19,
		0x13, 0x13, 0x04, 0x44, 0x04, 0x04, 0x04, 0x04,
		0x04, 0x05, 0x45, 0x05, 0x45, 0x05, 0x05, 0x05,
		0x05, 0x06, 0x06, 0x06, 0x46, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x32, 0x32, 0x32, 0x72, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x32, 0x72, 0x32, 0x32, 0x59,
		0x19, 0x19, 0x19, 0x19, 0x32, 0x19, 0x19, 0x34,
		0x19, 0x33, 0x19, 0x19, 0x19, 0x19, 0x19, 0x33,
		0x13, 0x13, 0x04, 0x04, 0x04, 0x12, 0x04, 0x44,
		0x04, 0x04, 0x05, 0x05, 0x05, 0x05, 0x45, 0x05,
		0x06, 0x06, 0x06, 0x11, 0x11, 0x51, 0x11, 0x11,
		0x11, 0x03, 0x11, 0x51, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x51, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x19, 0x59, 0x19, 0x19, 0x59, 0x59, 0x19, 0x19,
		0x19, 0x19, 0x59, 0x59, 0x34, 0x34, 0x34, 0x19,
		0x12, 0x13, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
		0x04, 0x04, 0x05, 0x05, 0x05, 0x05, 0x05, 0x06,
		0x06, 0x06, 0x06, 0x51, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x59, 0x19, 0x19, 0x19, 0x19, 0x59, 0x59, 0x19,
		0x33, 0x19, 0x19, 0x34, 0x34, 0x34, 0x34, 0x19,
		0x13, 0x13, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
		0x04, 0x04, 0x04, 0x05, 0x05, 0x05, 0x05, 0x05,
		0x06, 0x06, 0x06, 0x11, 0x11, 0x11, 0x51, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x51, 0x11, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32, 0x32,
		0x32, 0x19, 0x59, 0x59, 0x59, 0x19, 0x59, 0x59,
		0x34, 0x34, 0x34, 0x33, 0x34, 0x34, 0x34, 0x34,
		0x13, 0x12, 0x12, 0x04, 0x04, 0x13, 0x13, 0x04,
		0x04, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x06,
		0x06, 0x06, 0x06, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x10, 0x11, 0x11,
		0x51, 0x11, 0x51, 0x11, 0x11, 0x11, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x32, 0x59, 0x59, 0x32, 0x32,
		0x32, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x34, 0x34, 0x34, 0x34, 0x34, 0x34, 0x34,
		0x13, 0x12, 0x12, 0x12, 0x04, 0x13, 0x13, 0x13,
		0x13, 0x13, 0x05, 0x05, 0x14, 0x14, 0x46, 0x06,
		0x06, 0x06, 0x06, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x51, 0x11, 0x51, 0x11, 0x51, 0x32, 0x32,
		0x32, 0x32, 0x32, 0x11, 0x11, 0x59, 0x19, 0x19,
		0x19, 0x59, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x34, 0x34, 0x34, 0x34, 0x34, 0x34, 0x34, 0x34,
		0x13, 0x53, 0x12, 0x12, 0x12, 0x13, 0x13, 0x53,
		0x13, 0x13, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14,
		0x06, 0x06, 0x06, 0x06, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x51, 0x11, 0x51, 0x11, 0x11, 0x32,
		0x32, 0x32, 0x51, 0x11, 0x11, 0x11, 0x11, 0x59,
		0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x34,
		0x34, 0x34, 0x34, 0x34, 0x34, 0x34, 0x34, 0x34,
		0x13, 0x13, 0x12, 0x12, 0x12, 0x13, 0x13, 0x13,
		0x13, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14,
		0x14, 0x06, 0x06, 0x06, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x51, 0x11, 0x11, 0x51, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x59, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x19, 0x36, 0x3f, 0x34,
		0x34, 0x34, 0x34, 0x34, 0x34, 0x19, 0x34, 0x34,
		0x13, 0x13, 0x04, 0x12, 0x12, 0x13, 0x13, 0x13,
		0x13, 0x14, 0x14, 0x14, 0x17, 0x14, 0x14, 0x14,
		0x14, 0x14, 0x06, 0x15, 0x06, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x11, 0x11, 0x51, 0x11, 0x11, 0x51,
		0x11, 0x11, 0x11, 0x51, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x19, 0x19, 0x36, 0x36, 0x36, 0x36, 0x36,
		0x34, 0x34, 0x34, 0x34, 0x34, 0x34, 0x37, 0x77,
		0x13, 0x13, 0x12, 0x12, 0x53, 0x13, 0x13, 0x13,
		0x13, 0x14, 0x14, 0x14, 0x17, 0x14, 0x14, 0x14,
		0x14, 0x06, 0x06, 0x06, 0x06, 0x15, 0x15, 0x15,
		0x15, 0x15, 0x15, 0x15, 0x11, 0x11, 0x11, 0x11,
		0x11, 0x11, 0x51, 0x11, 0x51, 0x11, 0x11, 0x11,
		0x11, 0x51, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x3f, 0x36,
		0x76, 0x34, 0x34, 0x34, 0x77, 0x37, 0x77, 0x37,
		0x13, 0x53, 0x12, 0x12, 0x12, 0x13, 0x13, 0x18,
		0x13, 0x13, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14,
		0x06, 0x06, 0x06, 0x46, 0x15, 0x15, 0x15, 0x15,
		0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x11, 0x11,
		0x51, 0x15, 0x15, 0x11, 0x51, 0x51, 0x11, 0x11,
		0x51, 0x19, 0x19, 0x19, 0x19, 0x19, 0x36, 0x36,
		0x36, 0x36, 0x36, 0x76, 0x36, 0x36, 0x36, 0x76,
		0x36, 0x36, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37,
		0x13, 0x12, 0x12, 0x12, 0x13, 0x13, 0x53, 0x18,
		0x18, 0x13, 0x13, 0x14, 0x14, 0x14, 0x14, 0x46,
		0x17, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15,
		0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15,
		0x15, 0x15, 0x15, 0x11, 0x11, 0x11, 0x11, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x38, 0x36, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x36, 0x36, 0x76, 0x36,
		0x76, 0x36, 0x37, 0x37, 0x37, 0x37, 0x77, 0x34,
		0x13, 0x12, 0x12, 0x12, 0x13, 0x13, 0x18, 0x18,
		0x18, 0x18, 0x13, 0x14, 0x14, 0x14, 0x14, 0x14,
		0x15, 0x16, 0x15, 0x15, 0x15, 0x15, 0x56, 0x16,
		0x15, 0x16, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15,
		0x15, 0x15, 0x56, 0x16, 0x16, 0x11, 0x19, 0x19,
		0x19, 0x19, 0x19, 0x38, 0x78, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x77, 0x37, 0x37, 0x37, 0x34, 0x34,
		0x13, 0x13, 0x12, 0x12, 0x13, 0x13, 0x18, 0x18,
		0x18, 0x18, 0x13, 0x14, 0x14, 0x14, 0x14, 0x56,
		0x16, 0x16, 0x16, 0x16, 0x16, 0x56, 0x16, 0x16,
		0x16, 0x16, 0x16, 0x16, 0x15, 0x15, 0x15, 0x16,
		0x16, 0x16, 0x16, 0x16, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x19, 0x78, 0x38, 0x38, 0x37, 0x37, 0x37,
		0x37, 0x77, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x77, 0x37, 0x77, 0x37, 0x37, 0x34,
		0x13, 0x53, 0x53, 0x53, 0x13, 0x13, 0x18, 0x18,
		0x18, 0x18, 0x13, 0x14, 0x14, 0x14, 0x14, 0x16,
		0x16, 0x16, 0x14, 0x14, 0x16, 0x16, 0x16, 0x16,
		0x15, 0x16, 0x16, 0x16, 0x16, 0x16, 0x56, 0x16,
		0x16, 0x16, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x78, 0x38, 0x38, 0x38, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x39,
		0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x18, 0x13,
		0x18, 0x18, 0x13, 0x13, 0x14, 0x14, 0x14, 0x56,
		0x16, 0x14, 0x14, 0x14, 0x14, 0x14, 0x56, 0x16,
		0x16, 0x56, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x38,
		0x38, 0x78, 0x38, 0x37, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x37, 0x3b, 0x37, 0x37,
		0x13, 0x13, 0x13, 0x18, 0x13, 0x13, 0x18, 0x18,
		0x18, 0x18, 0x13, 0x13, 0x14, 0x14, 0x14, 0x14,
		0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x16, 0x16,
		0x16, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x78, 0x38,
		0x38, 0x38, 0x38, 0x78, 0x37, 0x37, 0x37, 0x37,
		0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x79,
		0x39, 0x39, 0x79, 0x39, 0x39, 0x39, 0x3a, 0x79,
		0x13, 0x13, 0x13, 0x18, 0x18, 0x13, 0x18, 0x18,
		0x18, 0x18, 0x13, 0x13, 0x13, 0x14, 0x14, 0x14,
		0x14, 0x14, 0x14, 0x14, 0x14, 0x56, 0x16, 0x56,
		0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19,
		0x19, 0x19, 0x19, 0x19, 0x78, 0x38, 0x38, 0x38,
		0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38,
		0x38, 0x38, 0x78, 0x39, 0x39, 0x39, 0x39, 0x39,
		0x39, 0x39, 0x39, 0x79, 0x79, 0x39, 0x79, 0x39,
		0x13, 0x13, 0x53, 0x18, 0x18, 0x13, 0x18, 0x18,
		0x18, 0x18, 0x18, 0x13, 0x13, 0x14, 0x14, 0x14,
		0x14, 0x14, 0x14, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x19, 0x19, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x38, 0x38, 0x38, 0x38, 0x38, 0x1c,
		0x38, 0x38, 0x38, 0x78, 0x38, 0x38, 0x38, 0x78,
		0x38, 0x3b, 0x39, 0x39, 0x79, 0x39, 0x39, 0x79,
		0x39, 0x79, 0x39, 0x39, 0x39, 0x39, 0x79, 0x39,
		0x18, 0x18, 0x18, 0x18, 0x18, 0x13, 0x13, 0x18,
		0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x14,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x5b, 0x1b,
		0x1b, 0x1b, 0x38, 0x78, 0x78, 0x38, 0x38, 0x38,
		0x38, 0x38, 0x38, 0x38, 0x3b, 0x3b, 0x38, 0x1c,
		0x38, 0x38, 0x39, 0x39, 0x39, 0x39, 0x39, 0x39,
		0x39, 0x39, 0x39, 0x3a, 0x39, 0x39, 0x39, 0x39,
		0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x13, 0x18,
		0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1c, 0x1c, 0x1c, 0x38, 0x38, 0x38, 0x78,
		0x38, 0x78, 0x38, 0x38, 0x7b, 0x3b, 0x3b, 0x3b,
		0x3b, 0x3b, 0x39, 0x79, 0x39, 0x39, 0x3a, 0x39,
		0x39, 0x39, 0x39, 0x39, 0x39, 0x39, 0x79, 0x39,
		0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
		0x18, 0x18, 0x1a, 0x1a, 0x1a, 0x1a, 0x1b, 0x1a,
		0x1a, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1c,
		0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c,
		0x1c, 0x38, 0x78, 0x38, 0x3b, 0x3b, 0x3b, 0x3b,
		0x3b, 0x3b, 0x7b, 0x3b, 0x3b, 0x79, 0x39, 0x39,
		0x39, 0x39, 0x39, 0x39, 0x39, 0x39, 0x39, 0x39,
		0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
		0x18, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1c, 0x1c,
		0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c,
		0x1c, 0x38, 0x38, 0x38, 0x3b, 0x3b, 0x3b, 0x3b,
		0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b,
		0x39, 0x39, 0x39, 0x39, 0x39, 0x39, 0x39, 0x39,
		0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
		0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a,
		0x1a, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x5c, 0x1c, 0x1c,
		0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x3d,
		0x3d, 0x3d, 0x3d, 0x3d, 0x3b, 0x3b, 0x3b, 0x3b,
		0x3b, 0x7b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b,
		0x3b, 0x3b, 0x39, 0x39, 0x39, 0x79, 0x3c, 0x3c,
		0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x1a,
		0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a,
		0x1a, 0x1b, 0x1b, 0x5b, 0x1b, 0x1b, 0x1b, 0x1b,
		0x1b, 0x1b, 0x1b, 0x5b, 0x1b, 0x1c, 0x1c, 0x1c,
		0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x3d, 0x3d,
		0x1c, 0x3d, 0x1c, 0x3d, 0x3d, 0x3d, 0x3b, 0x7b,
		0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x7b, 0x7b, 0x7b,
		0x7b, 0x3b, 0x7b, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x58,
		0x1a, 0x1a, 0x1a, 0x1a, 0x20, 0x20, 0x20, 0x1a,
		0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x5b, 0x1b,
		0x1b, 0x5b, 0x21, 0x21, 0x21, 0x1c, 0x1c, 0x1c,
		0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x7d, 0x3d, 0x7d,
		0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d,
		0x7d, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b, 0x3b,
		0x3b, 0x3b, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x18, 0x18, 0x18, 0x1d, 0x1d, 0x18, 0x18, 0x18,
		0x1a, 0x1a, 0x1a, 0x20, 0x1a, 0x1a, 0x1a, 0x1a,
		0x1a, 0x1b, 0x20, 0x1b, 0x5f, 0x1f, 0x1f, 0x1f,
		0x21, 0x21, 0x21, 0x21, 0x21, 0x1c, 0x1c, 0x1c,
		0x1c, 0x1c, 0x1c, 0x5c, 0x1c, 0x3d, 0x3d, 0x3d,
		0x3d, 0x3d, 0x7d, 0x3d, 0x7d, 0x7d, 0x3d, 0x3d,
		0x3d, 0x3d, 0x3d, 0x3b, 0x7b, 0x3b, 0x3b, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x18, 0x18, 0x1d, 0x1d, 0x1d, 0x1d, 0x18, 0x18,
		0x18, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a,
		0x1b, 0x20, 0x1b, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
		0x1f, 0x1f, 0x1f, 0x21, 0x22, 0x21, 0x21, 0x21,
		0x1c, 0x1c, 0x1c, 0x1c, 0x3d, 0x3d, 0x3d, 0x3d,
		0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d,
		0x3d, 0x7d, 0x3d, 0x3d, 0x3b, 0x3b, 0x3e, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x18, 0x18,
		0x18, 0x18, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a,
		0x1b, 0x1f, 0x1f, 0x5f, 0x1f, 0x1f, 0x1f, 0x1f,
		0x1f, 0x1f, 0x1f, 0x1f, 0x61, 0x21, 0x61, 0x21,
		0x21, 0x61, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d,
		0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x7d, 0x3d,
		0x7d, 0x3d, 0x3d, 0x3d, 0x3e, 0x3e, 0x3e, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x18,
		0x18, 0x18, 0x18, 0x1a, 0x1a, 0x1a, 0x1e, 0x1e,
		0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
		0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x22, 0x21, 0x21,
		0x21, 0x21, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d,
		0x3d, 0x7d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d,
		0x3d, 0x3d, 0x7d, 0x3e, 0x3e, 0x3c, 0x3c, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x18, 0x18, 0x18, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e,
		0x1e, 0x1e, 0x1e, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
		0x1f, 0x1f, 0x5f, 0x1f, 0x1f, 0x21, 0x22, 0x61,
		0x21, 0x21, 0x7d, 0x3d, 0x3d, 0x3d, 0x3d, 0x7d,
		0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d, 0x7d, 0x3d,
		0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3c, 0x3c, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x5d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1e, 0x1e, 0x5e, 0x1e, 0x1f, 0x1e,
		0x1e, 0x24, 0x24, 0x23, 0x25, 0x25, 0x5f, 0x1f,
		0x1f, 0x1f, 0x1f, 0x1f, 0x21, 0x1f, 0x23, 0x23,
		0x21, 0x21, 0x21, 0x3d, 0x7d, 0x3d, 0x3d, 0x3d,
		0x29, 0x3d, 0x7d, 0x3d, 0x3d, 0x3d, 0x3d, 0x3d,
		0x3d, 0x3e, 0x3e, 0x3e, 0x7e, 0x3e, 0x7e, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e,
		0x1e, 0x24, 0x24, 0x25, 0x25, 0x25, 0x25, 0x25,
		0x1f, 0x1f, 0x1f, 0x25, 0x25, 0x23, 0x23, 0x23,
		0x28, 0x68, 0x28, 0x28, 0x28, 0x68, 0x68, 0x29,
		0x29, 0x29, 0x2b, 0x2b, 0x6b, 0x3d, 0x2b, 0x3d,
		0x7e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3c, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3f, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e, 0x5e, 0x1e,
		0x1e, 0x24, 0x24, 0x24, 0x25, 0x25, 0x25, 0x25,
		0x25, 0x24, 0x25, 0x25, 0x23, 0x23, 0x23, 0x23,
		0x28, 0x28, 0x28, 0x68, 0x68, 0x29, 0x29, 0x29,
		0x29, 0x29, 0x2b, 0x2b, 0x2b, 0x3d, 0x6b, 0x2b,
		0x7e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3c, 0x3c, 0x3c,
		0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x5d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e, 0x1e,
		0x5e, 0x24, 0x24, 0x24, 0x24, 0x25, 0x25, 0x25,
		0x25, 0x25, 0x25, 0x23, 0x23, 0x23, 0x23, 0x28,
		0x28, 0x28, 0x68, 0x29, 0x29, 0x29, 0x29, 0x29,
		0x29, 0x29, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x7e, 0x3e, 0x3e, 0x7e, 0x3c, 0x3c, 0x3c,
		0x3e, 0x3e, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x5d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x5e, 0x1e,
		0x24, 0x24, 0x24, 0x25, 0x25, 0x25, 0x25, 0x25,
		0x25, 0x25, 0x23, 0x23, 0x23, 0x28, 0x28, 0x28,
		0x28, 0x29, 0x29, 0x29, 0x29, 0x29, 0x29, 0x29,
		0x69, 0x29, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b,
		0x2b, 0x2b, 0x3e, 0x3e, 0x7e, 0x3e, 0x3e, 0x3e,
		0x3e, 0x3e, 0x3e, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e,
		0x24, 0x24, 0x24, 0x25, 0x25, 0x65, 0x25, 0x25,
		0x25, 0x25, 0x23, 0x23, 0x28, 0x28, 0x23, 0x28,
		0x29, 0x29, 0x29, 0x29, 0x29, 0x29, 0x29, 0x29,
		0x29, 0x29, 0x69, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x7e, 0x7e, 0x7e, 0x3e, 0x3e, 0x3e,
		0x7e, 0x3e, 0x3c, 0x3c, 0x3c, 0x3e, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x26, 0x26, 0x66,
		0x26, 0x26, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e,
		0x24, 0x24, 0x24, 0x25, 0x25, 0x25, 0x25, 0x25,
		0x65, 0x23, 0x23, 0x23, 0x28, 0x28, 0x28, 0x29,
		0x29, 0x29, 0x29, 0x69, 0x29, 0x69, 0x69, 0x69,
		0x29, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x3e, 0x3e, 0x3e,
		0x3e, 0x3e, 0x3c, 0x3c, 0x3e, 0x3c, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x66, 0x26, 0x27,
		0x26, 0x26, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e,
		0x24, 0x24, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25,
		0x23, 0x23, 0x23, 0x28, 0x28, 0x28, 0x28, 0x28,
		0x28, 0x28, 0x29, 0x69, 0x29, 0x29, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x6b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x3e, 0x3e, 0x7e, 0x3e, 0x3e, 0x3c, 0x3c,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x26, 0x27, 0x26,
		0x26, 0x66, 0x1d, 0x1d, 0x5e, 0x1e, 0x5e, 0x1e,
		0x24, 0x24, 0x25, 0x25, 0x25, 0x25, 0x25, 0x23,
		0x23, 0x23, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28,
		0x28, 0x28, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x6b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x3e, 0x7e, 0x3e, 0x3e,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x66, 0x26, 0x26,
		0x26, 0x26, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e,
		0x24, 0x24, 0x25, 0x25, 0x25, 0x25, 0x23, 0x23,
		0x23, 0x28, 0x28, 0x28, 0x28, 0x28, 0x2a, 0x2a,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x3e, 0x3e, 0x7e, 0x3e,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x26, 0x26, 0x66,
		0x26, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e, 0x24,
		0x24, 0x24, 0x25, 0x25, 0x25, 0x65, 0x23, 0x23,
		0x23, 0x28, 0x23, 0x28, 0x28, 0x2a, 0x2a, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b, 0x6b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x6b, 0x2b, 0x2b, 0x2b, 0x7e, 0x3e, 0x3e,
		0x1d, 0x1d, 0x5d, 0x1d, 0x1d, 0x1d, 0x66, 0x26,
		0x66, 0x1d, 0x1d, 0x1d, 0x5e, 0x1e, 0x1e, 0x24,
		0x24, 0x25, 0x25, 0x25, 0x25, 0x23, 0x23, 0x23,
		0x2a, 0x28, 0x28, 0x28, 0x2a, 0x2a, 0x2a, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b,
		0x6b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x6b, 0x2b, 0x2b, 0x2b, 0x6b, 0x6b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
		0x1d, 0x5d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e, 0x1e, 0x24,
		0x25, 0x65, 0x25, 0x25, 0x25, 0x23, 0x23, 0x23,
		0x28, 0x28, 0x28, 0x28, 0x28, 0x2a, 0x2a, 0x2b,
		0x2b, 0x2b, 0x6b, 0x2b, 0x6b, 0x2b, 0x2b, 0x6b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b,
		0x6b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x6b, 0x6b,
		0x6b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b,
		0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d,
		0x1d, 0x1d, 0x1d, 0x1e, 0x1e, 0x1e, 0x24, 0x24,
		0x25, 0x25, 0x25, 0x25, 0x23, 0x23, 0x23, 0x28,
		0x28, 0x28, 0x2a, 0x2a, 0x2a, 0x28, 0x2a, 0x2b,
		0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x6b, 0x2b, 0x6b,
		0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x35, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b,
		0x6b, 0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x2b,
		0x00, 0x5d, 0x1d, 0x1d, 0x5d, 0x1e, 0x1e, 0x1e,
		0x1e, 0x5e, 0x1e, 0x1e, 0x1e, 0x1e, 0x65, 0x24,
		0x25, 0x25, 0x25, 0x25, 0x23, 0x23, 0x23, 0x28,
		0x28, 0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x2b, 0x2b,
		0x2b, 0x2a, 0x2b, 0x2b, 0x2b, 0x2b, 0x6b, 0x2b,
		0x2b, 0x6b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x2b,
		0x2b, 0x2b, 0x2b, 0x6b, 0x2b, 0x2b, 0x6b, 0x2b,
	};

	private int [][] routes = {
		{8, 0, 0, 0},
		{46, 3, 1, 16},
		{28, 4, 2, 6},
		{22, 5, 3, 104},
		{32, 6, 4, 6},
		{23, 7, 5, 3},
		{29, 7, 6, 14},
		{37, 7, 8, 10},
		{40, 8, 1, 103},
		{57, 8, 9, 9},
		{39, 9, 10, 103},
		{11, 10, 11, 11},
		{21, 11, 13, 22},
		{25, 11, 26, 22},
		{29, 12, 22, 23},
		{36, 12, 22, 23},
		{51, 12, 19, 25},
		{62, 12, 20, 20},
		{16, 13, 21, 21},
		{55, 13, 17, 20},
		{57, 15, 31, 110},
		{14, 16, 105, 29},
		{27, 16, 27, 34},
		{34, 16, 34, 41},
		{42, 16, 24, 24},
		{52, 16, 20, 20},
		{19, 17, 32, 32},
		{22, 18, 26, 32},
		{54, 18, 20, 31},
		{14, 20, 32, 32},
		{49, 20, 25, 28},
		{57, 20, 37, 37},
		{18, 21, 42, 42},
		{42, 21, 33, 35},
		{31, 22, 38, 36},
		{46, 22, 35, 30},
		{39, 23, 33, 39},
		{56, 24, 47, 55},
		{32, 25, 41, 41},
		{45, 26, 40, 46},
		{54, 26, 37, 47},
		{34, 27, 45, 45},
		{17, 28, 44, 50},
		{42, 28, 46, 39},
		{24, 29, 49, 50},
		{30, 29, 44, 50},
		{51, 29, 47, 48},
		{57, 29, 48, 55},
		{55, 31, 55, 57},
		{21, 32, 52, 56},
		{23, 32, 56, 54},
		{43, 32, 61, 62},
		{13, 33, 106, 64},
		{34, 33, 43, 51},
		{30, 34, 53, 58},
		{59, 34, 57, 63},
		{21, 36, 59, 60},
		{54, 38, 61, 63},
		{27, 39, 65, 66},
		{22, 40, 64, 65},
		{25, 40, 65, 69},
		{48, 40, 73, 76},
		{42, 41, 66, 68},
		{55, 41, 68, 76},
		{17, 42, 72, 74},
		{28, 42, 111, 66},
		{37, 43, 70, 102},
		{59, 43, 67, 67},
		{44, 45, 70, 73},
		{29, 46, 70, 74},
		{42, 46, 78, 75},
		{7, 47, 86, 85},
		{10, 47, 81, 81},
		{48, 48, 75, 79},
		{21, 49, 82, 88},
		{45, 49, 80, 83},
		{54, 50, 84, 94},
		{39, 51, 82, 89},
		{42, 51, 77, 83},
		{50, 51, 76, 94},
		{46, 52, 83, 96},
		{12, 54, 86, 91},
		{25, 54, 102, 102},
		{44, 54, 96, 96},
		{55, 54, 93, 96},
		{7, 55, 90, 90},
		{10, 55, 85, 91},
		{17, 56, 88, 109},
		{21, 56, 109, 82},
		{37, 56, 92, 92},
		{8, 57, 91, 101},
		{12, 57, 101, 87},
		{39, 58, 95, 95},
		{56, 58, 96, 96},
		{63, 58, 93, 100},
		{42, 59, 96, 96},
		{45, 59, 96, 96},
		{4, 60, 91, 101},
		{33, 60, 92, 95},
		{23, 60, 82, 102},
		{59, 60, 96, 96},
		{14, 60, 99, 99},
		{27, 57, 98, 98},
		{42, 11, 15, 15},
		{18, 3, 12, 18},
		{7, 21, 52, 106},
		{6, 37, 107, 72},
		{2, 39, 108, 108},
		{0, 47, 71, 97},
		{20, 60, 99, 99},
		{62, 24, 47, 55},
		{29, 44, 68, 69},
	};
}
