/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;

/**
 * Class representing a game in progress.
 */
public class Midnight {
	/** Game map */
	protected Map map;
	
	/** Game characters */
	private  ArrayList characters;

	/** Armies guarding the keeps */
	private ArrayList armies;
	
	/** Doomdark's roaming regiments */
	private ArrayList doomguard;
	
	/** Game day */
	protected int day;
	
	/** Whether the Free control the moon ring */
	private boolean moonRingControlled;
	
	/** Whether the Ice Crown has been destroyed */
	private boolean iceCrownDestroyed;

	/** Whether the conditions for ending the game have been met */
	protected boolean gameOver;
	
	/** Game-ending status */
	protected Status status;
	
	/** Battles that took place last night */
	protected HashMap battles;
	
	/** The force multiplyer for DoomDark's ownership of citadels and keeps */
	private int doomDarksCitadels;
	
	/**
	 * Private constructor.
	 */
	private Midnight() {
		this(new Random());
	}
	
	/**
	 * Constructor that sets the random generator to use.
	 * 
	 * NOTE: This constructor should be used only by the testing classes.
	 */
	protected Midnight(Random random) {
		randomGenerator = random;

		map = new Map(this);
		
		day = 0;
		moonRingControlled = true;
		battles = new HashMap();
		initializeCharacters();
		initializeArmies();
		initializeDoomguard();
	}
	
	/**
	 * Returns the current game map.
	 */
	public Map getMap() {
		return map;
	}
	
	/**
	 * Sets the current game map.
	 */
	protected void setMap(Map map) {
		this.map = map;
	}
	
	/**
	 * Iterator on all the game characters.
	 */
	public Iterator characters() {
		return characters.iterator();
	}
	
	/**
	 * Iterator on all the armies guarding places.
	 */
	public Iterator armies() {
		return armies.iterator();
	}
	
	/**
	 * Removes a unit from the list of active doomguard regiments.
	 */
	protected void removeDoomguard(Doomguard army) {
		doomguard.remove(army);
	}
	
	/**
	 * Iterator on all the Doomdark's roaming regiments
	 */
	public Iterator doomguard() {
		return doomguard.iterator();
	}
	
	/**
	 * Returns the current day of the game.
	 */
	public int getDay() {
		return day;
	}
	
	/**
	 * Returns <code>true</code> if the character can be controlled (is recruited
	 * and the player has the Moon Ring, except for Luxor and Morkin).
	 */
	public boolean isControllable(Character character) {
		if (character != LUXOR && character != MORKIN) {
			return character.isRecruited() && isMoonRingControlled();
		}
		return true;
	}
	
	/**
	 * Returns true if Luxor or Morkin have the Moon Ring (and are alive).
	 */
	public boolean isMoonRingControlled() {
		return moonRingControlled;
	}
	
	/**
	 * Starts the nighttime activity: Doomguard movement and battles.
	 */
	public void night() {
		checkSpecialConditions();
		if (!gameOver) {
			day++;
			calcDoomDarksCitadels();
			calcNightActivity();
		}
	}
	
	/**
	 * Starts the day for all the characters and units.
	 */
	public void dawn() {
		for (Iterator it = characters(); it.hasNext();) {
			Character character = (Character) it.next();
			character.dawn();
		}
		for (Iterator it = armies(); it.hasNext();) {
			Army army = (Army) it.next();
			army.dawn();
		}
		for (Iterator it = doomguard(); it.hasNext();) {
			Doomguard army = (Doomguard) it.next();
			army.dawn();
		}
	}
	
	/**
	 * Checks whether the Moon Ring is still under player's control, and
	 * whether victory conditions have been met.
	 */
	private void checkSpecialConditions() {
		if (!LUXOR.isAlive() && 
			LUXOR.getObject() == Location.Object.MOON_RING) {
			
			LUXOR.dropObject();
			moonRingControlled = false;
		}
		
		if (MORKIN.isAlive()) {
			if (MORKIN.getObject() == Location.Object.MOON_RING) {
				moonRingControlled = true;
			}
			else if (MORKIN.getObject() == Location.Object.ICE_CROWN) {
				Location morkinLocation = MORKIN.getLocation();
				
				if (morkinLocation.equals(map.LAKE_MIRROW) ||
					morkinLocation.equals(FAWKRIN.getLocation()) ||
					morkinLocation.equals(LORGRIM.getLocation()) ||
					morkinLocation.equals(FARFLAME.getLocation())) {
					
					iceCrownDestroyed = true;
				}
			}
		}
		checkGameOver();
	}
	
	/**
	 * Calculates DoomDark's army force multiplier thanks to the number of his
	 * keeps and citadels. Citadels are worth five and keeps two points.
	 */
	private void calcDoomDarksCitadels() {
		doomDarksCitadels = 0;
		for (Iterator it = armies(); it.hasNext();) {
			Army army = (Army) it.next();
			if (army.getRace() == Race.FOUL) {
				if (army.getLocation().getFeature() == Location.Feature.CITADEL) {
					doomDarksCitadels += 5;
				}
				else {
					doomDarksCitadels += 2;
				}
			}
		}
	}
	
	/**
	 * Calculates night-time activity: DoomDark's armies are moved and battles
	 * calculated.
	 */
	private void calcNightActivity() {
		battles.clear();
		
		for (Iterator it = characters(); it.hasNext();) {
			Character character = (Character) it.next();
			character.incrementEnergy(character.getTime().getTime() / 2);
			if (character.isAlive() && !character.isHidden()) {
				character.getLocation().setSpecial(true);
				character.setBattle(null);
				character.setEnemyKilled(0);
				character.getRiders().setCasualties(0);
				character.getRiders().setEnemyKilled(0);
				character.getWarriors().setCasualties(0);
				character.getWarriors().setEnemyKilled(0);
			}
		}
		for (Iterator it = armies(); it.hasNext();) {
			Army army = (Army) it.next();
			if (army.getRace() != Race.FOUL) {
				army.getLocation().setSpecial(true);
			}
		}
		
		for (Iterator it = doomguard(); it.hasNext();) {
			Doomguard doomguard = (Doomguard) it.next();
			while (doomguard.getMoveCount() < Doomguard.MAX_MOVE_COUNT) {
				doomguard.executeMove();
			}
			doomguard.resetMoveCount();
		}		
		
		for (Iterator it = characters(); it.hasNext();) {
			Character character = (Character) it.next();
			Location location = character.getLocation();
			character.getLocation().setSpecial(false);
			if ((!location.getArmies().isEmpty() ||
					(location.getGuard() != null && location.getGuard().getRace() == Race.FOUL)) &&
				!battles.containsKey(location)) {
				battles.put(location, new Battle(location));
			}
		}
		for (Iterator it = armies(); it.hasNext();) {
			Army army = (Army) it.next();
			if (army.getRace() != Race.FOUL) {
				army.getLocation().setSpecial(false);
				if (!army.getLocation().getArmies().isEmpty() &&
					!battles.containsKey(army.getLocation())) {
					battles.put(army.getLocation(), new Battle(army.getLocation()));
				}
			}
			else {
				System.out.println(army.getLocation());
			}
		}
		
		for (Iterator it = battles.values().iterator(); it.hasNext();) {
			Battle battle = (Battle) it.next();
			battle.run();
		}
	}
	
	/**
	 * Checks whether the game is over: if both Luxor and Morkin are dead, or if
	 * Morkin is dead and Xajorkith has fallen, Doomdark wins. If Ushgarak has
	 * fallen or if the Ice Crown has been destroyed, the Free win.
	 */
	private void checkGameOver() {
		if (!MORKIN.isAlive()) {
			if (!LUXOR.isAlive()) {
				gameOver = true;
				status = Status.LUXOR_MORKIN_DEAD;
			}
			else if (map.XAJORKITH.getGuard().getRace() == Race.FOUL) {
				gameOver = true;
				status = Status.MORKIN_XAJORKITH;
			}
		}
		if (map.USHGARAK.getGuard().getRace() == Race.FREE) {
			gameOver = true;
			status = Status.USHGARAK;
		}
		else if (iceCrownDestroyed) {
			gameOver = true;
			status = Status.ICE_CROWN;
		}
	}
	
	/** Singleton game instance */
	private static Midnight instance;
	
	/**
	 * Singleton game instance getter.
	 */
	public synchronized static Midnight getInstance() {
		if (instance == null) {
			instance = new Midnight();
		}
		return instance;
	}

	/**
	 * Enum class for describing game ending.
	 */	
	public static class Status {
		public static final Status LUXOR_MORKIN_DEAD = new Status(Race.FOUL,
			"Luxor is dead and Morkin is dead.");
		public static final Status MORKIN_XAJORKITH = new Status (Race.FOUL,
			"Xajorkith has fallend and Morkin is dead.");
		public static final Status USHGARAK = new Status(Race.FREE,
			"Ushgarak has fallen.");
		public static final Status ICE_CROWN = new Status(Race.FREE,
			"The Ice Crown has been destroyed.");
			
		/** Game's winner */
		private Race winner;
		
		/** description of the game-ending condition */
		private String description;
		
		/**
		 * Private constructor
		 */
		private Status(Race winner, String description) {
			this.winner = winner;
			this.description = description;
		}
		
		/**
		 * Returns game's winner.
		 */
		public Race getWinner() {
			return winner;
		}
		
		/**
		 * Returns description of the game-winning condition.
		 */
		public String toString() {
			return description;
		}

		private static Status [] values = {LUXOR_MORKIN_DEAD, MORKIN_XAJORKITH, USHGARAK, ICE_CROWN};
		
		/**
		 * Returns status with the given index.
		 */
		public static Status getStatus(int index) {
			return values[index];
		}
		
		/**
		 * Returns the status's index in the array of existing status types.
		 */
		public int getIndex() {
			for (int i = 0; i < values.length; i++) {
				Status status = values[i];
				if (status == this) return i;
			}
			return -1;
		}
	}		

	/**
	 * Returns the battles that took place last night.
	 */
	public Collection getBattles() {
		return battles.values();
	}

	/**
	 * Returns the list of domains where battles took place last night.
	 */
	public Collection getBattleDomains() {
		HashSet domains = new HashSet();
		for (Iterator it = getBattles().iterator(); it.hasNext();) {
			Battle battle = (Battle) it.next();
			Map.Area domain = battle.getLocation().getDomain();
			domains.add(domain);
		}
		return domains;
	}
	
	/**
	 * Returns the value by which DoomGuard's attack chances are improved
	 * because of the number of citadels and keeps DoomDarks controls.
	 */
	public int getDoomDarksCitadels() {
		return doomDarksCitadels;
	}

	/**
	 * Returns true if the game is over.
	 */
	public boolean isGameOver() {
		return gameOver;
	}

	/**
	 * Returns the game-winning status.
	 */
	public Status getStatus() {
		return status;
	}

	private static final String saveGameHeader = "jLOM save game v0.1";
	public void load(DataInputStream in) throws IOException {
		if (!saveGameHeader.equals(in.readUTF())) throw new IOException("Bad save game header format");
		day = in.readShort();
		map.load(in);
		loadCharacters(in);
		loadGarrisons(in);
		loadDoomguard(in);
	}
	
	/**
	 * Saves the game into an output stream.
	 */
	public void save(DataOutputStream out) throws IOException {
		out.writeUTF(saveGameHeader);
		out.writeShort(day);
		map.save(out);
		saveCharacters(out);
		saveGarrisons(out);
		saveDoomguard(out);
	}
	
	protected void loadCharacters(DataInputStream in) throws IOException {
		characters.clear();
		int n = in.readUnsignedByte();
		for (int i = 0; i < n; i++) {
			Character character = Character.load(in, this, i);
			characters.add(character);
		}

		LUXOR = (Character) characters.get(0);
		MORKIN = (Character) characters.get(1);
		CORLETH = (Character) characters.get(2);
		ROTHRON = (Character) characters.get(3);
		GARD = (Character) characters.get(4);
		MARAKITH = (Character) characters.get(5);
		XAJORKITH = (Character) characters.get(6);
		GLOOM = (Character) characters.get(7);
		SHIMERIL = (Character) characters.get(8);
		KUMAR = (Character) characters.get(9);
		ITHRORN = (Character) characters.get(10);
		DAWN = (Character) characters.get(11);
		DREAMS = (Character) characters.get(12);
		DREGRIM = (Character) characters.get(13);
		THIMRATH = (Character) characters.get(14);
		WHISPERS = (Character) characters.get(15);
		SHADOWS = (Character) characters.get(16);
		LOTHORIL = (Character) characters.get(17);
		KORINEL = (Character) characters.get(18);
		THRALL = (Character) characters.get(19);
		BRITH = (Character) characters.get(20);
		RORATH = (Character) characters.get(21);
		TRORN = (Character) characters.get(22);
		MORNING = (Character) characters.get(23);
		ATHORIL = (Character) characters.get(24);
		BLOOD = (Character) characters.get(25);
		HERATH = (Character) characters.get(26);
		MITHARG = (Character) characters.get(27);
		UTARG = (Character) characters.get(28);
		FAWKRIN = (Character) characters.get(29);
		LORGRIM = (Character) characters.get(30);
		FARFLAME = (Character) characters.get(31);
	}
	
	protected void saveCharacters(DataOutputStream out) throws IOException {
		out.writeByte(characters.size());
		for (Iterator it = characters.iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			character.save(out);
		}
	}
	
	/**
	 * Returns game character with the given id. 
	 */
	protected Character getCharacter(int id) {
		return (Character) characters.get(id);
	}
	
	protected void loadGarrisons(DataInputStream in) throws IOException {
		int n = in.readUnsignedByte();
		armies.clear();
		for (int i = 0; i < n; i++) {
			Army garrison = Army.load(in, this);
			Location location = map.getRouteNode(i);
			garrison.guard(location);
			armies.add(garrison);
		}
	}
	
	protected void saveGarrisons(DataOutputStream out) throws IOException {
		out.writeByte(armies.size());
		for (Iterator it = armies.iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			army.save(out);
		}
	}
	
	protected void loadDoomguard(DataInputStream in) throws IOException {
		int n = in.readUnsignedByte();
		doomguard.clear();
		for (int i = 0; i < n; i++) {
			Doomguard army = (Doomguard) Doomguard.load(in, this);
			doomguard.add(army);
		}
	}
	
	protected void saveDoomguard(DataOutputStream out) throws IOException {
		out.writeByte(doomguard.size());
		for (Iterator it = doomguard.iterator(); it.hasNext();) {
			Doomguard army = (Doomguard) it.next();
			army.save(out);
		}
	}
	
	public Character LUXOR;
	public Character MORKIN;
	public Character CORLETH;
	public Character ROTHRON;
	public Character GARD;
	public Character MARAKITH;
	public Character XAJORKITH;
	public Character GLOOM;
	public Character SHIMERIL;
	public Character KUMAR;
	public Character ITHRORN;
	public Character DAWN;
	public Character DREAMS;
	public Character DREGRIM;
	public Character THIMRATH;
	public Character WHISPERS;
	public Character SHADOWS;
	public Character LOTHORIL;
	public Character KORINEL;
	public Character THRALL;
	public Character BRITH;
	public Character RORATH;
	public Character TRORN;
	public Character MORNING;
	public Character ATHORIL;
	public Character BLOOD;
	public Character HERATH;
	public Character MITHARG;
	public Character UTARG;
	public Character FAWKRIN;
	public Character LORGRIM;
	public Character FARFLAME;
	
	/**
	 * Initializes game characters.
	 */
	private void initializeCharacters() {
		characters = new ArrayList();

		int i = 0;
		
		LUXOR = new Character(this, i++, "Luxor", "Luxor the Moonprince", Race.FREE,
			12, 40, 180, 127, 25, 80, 0x17, 0x00, 0, 0);
		LUXOR.setDirection(Map.Direction.SOUTHEAST);
	
		MORKIN = new Character(this, i++, "Morkin", "Morkin", Race.MORKIN,
			12, 40, 200, 127, 5, 127, 0x7e, 0x00, 0, 0);
		MORKIN.setDirection(Map.Direction.SOUTHEAST);
			
		CORLETH = new Character(this, i++, "Corleth", "Corleth the Fey", Race.FEY,
			12, 40, 180, 127, 20, 96, 0x6b, 0x00, 0, 0);
		CORLETH.setDirection(Map.Direction.EAST);
			
		ROTHRON = new Character(this, i++, "Rothron", "Rothron the Wise", Race.WISE,
			12, 40, 220, 127, 40, 80, 0x7f, 0x00, 0, 0);
		ROTHRON.setDirection(Map.Direction.NORTHEAST);
					
		GARD = new Character(this, i++, "Gard", "the Lord of Gard", Race.FREE,
			10, 55, 150, 64, 10, 64, 0x01, 0x01, 500, 1000);
		GARD.setDirection(Map.Direction.EAST);
			
		MARAKITH = new Character(this, i++, "Marakith", "the Lord of Marakith", Race.FREE,
			43, 32, 150, 64, 10, 64, 0x01, 0x01, 500, 1000);
		MARAKITH.setDirection(Map.Direction.WEST);
			
		XAJORKITH = new Character(this, i++, "Xajorkith", "the Lord of Xajorkith", Race.FREE,
			45, 59, 150, 64, 15, 64, 0x01, 0x01, 800, 1200);
		XAJORKITH.setDirection(Map.Direction.NORTH);
			
		GLOOM = new Character(this, i++, "Gloom", "the Lord of Gloom", Race.FREE,
			8, 0, 150, 64, 15, 56, 0x01, 0x01, 500, 1000);
		GLOOM.setDirection(Map.Direction.EAST);
			
		SHIMERIL = new Character(this, i++, "Shimeril", "the Lord of Shimeril", Race.FREE,
			28, 42, 150, 64, 15, 64, 0x01, 0x01, 800, 1000);
		SHIMERIL.setDirection(Map.Direction.NORTHWEST);
			
		KUMAR = new Character(this, i++, "Kumar", "the Lord of Kumar", Race.FREE,
			57, 29, 150, 64, 10, 64, 0x01, 0x01, 700, 1000);
		KUMAR.setDirection(Map.Direction.NORTH);
			
		ITHRORN = new Character(this, i++, "Ithrorn", "the Lord of Ithrorn", Race.FREE,
			57, 15, 150, 64, 15, 64, 0x09, 0x01, 1000, 1200);
		ITHRORN.setDirection(Map.Direction.NORTHWEST);
			
		DAWN = new Character(this, i++, "Dawn", "the Lord of Dawn", Race.FREE,
			44, 45, 150, 64, 8, 48, 0x01, 0x01, 500, 800);
		DAWN.setDirection(Map.Direction.NORTH);
		
		DREAMS = new Character(this, i++, "Dreams", "the Lord Of Dreams", Race.FEY,
			42, 16, 180 , 64,  20,  90,  0x1f, 0x08, 800,  1200);
		DREAMS.setDirection(Map.Direction.NORTH);
			
		DREGRIM = new Character(this, i++, "Dregrim", "the Lord Of Dregrim", Race.FEY,
			59, 43, 150,  64,  15,  80,  0x1f, 0x08, 400,  1000);
		DREGRIM.setDirection(Map.Direction.NORTH);
			
		THIMRATH = new Character(this, i++, "Thimrath", "Thimrath the Fey",  Race.FEY,
			33, 60, 130,  64,  12,  90,  0x1a, 0x02, 600,  400);
		THIMRATH.setDirection(Map.Direction.WEST);
			
		WHISPERS = new Character(this, i++, "Whispers", "the Lord Of Whispers",  Race.FEY,
			57, 20, 150,  64,  12,  80,  0x1a, 0x02, 300,  600);
		WHISPERS.setDirection(Map.Direction.NORTHWEST);
			
		SHADOWS = new Character(this, i++, "Shadows", "the Lord Of Shadows", Race.FEY,
			11, 37, 130,  64,  12,  70,  0x1a, 0x02, 0,  1000);
		SHADOWS.setDirection(Map.Direction.NORTH);
		SHADOWS.setOnHorse(false);
			
		LOTHORIL = new Character(this, i++, "Lothoril", "the Lord Of Lothoril",  Race.FEY,
			11, 10,  100,  64,  8,  60,  0x1a, 0x02, 200,  500);
		LOTHORIL.setDirection(Map.Direction.EAST);
	
		KORINEL = new Character(this, i++, "Korinel", "Korinel the Fey", Race.FEY,
			23, 21,  120,  64,  12,  60,  0x1a, 0x02, 0,  1000);
		KORINEL.setDirection(Map.Direction.NORTH);
		KORINEL.setOnHorse(false);
			
		THRALL = new Character(this, i++, "Thrall", "the Lord Of Thrall",  Race.FEY,
			33, 38,  150,  64,  10,  70,  0x1a, 0x02, 300,  600);
		THRALL.setDirection(Map.Direction.NORTHWEST);
			
		BRITH = new Character(this, i++, "Brith", "Lord Brith", Race.FREE,
			21, 49,  100,  64,  8,  40,  0x01, 0x01, 500,  300);
		BRITH.setDirection(Map.Direction.NORTHEAST);
			
		RORATH = new Character(this, i++, "Rorath", "Lord Rorath",  Race.FREE,
			23, 60,  100,  64,  8,  50,  0x01, 0x01, 800,  400);
		RORATH.setDirection(Map.Direction.NORTH);
			
		TRORN = new Character(this, i++, "Trorn", "Lord Trorn", Race.FREE,
			54, 50,  100,  64,  8,  35,  0x01, 0x01, 400,  800);
		TRORN.setDirection(Map.Direction.NORTHWEST);
			
		MORNING = new Character(this, i++, "Morning", "the Lord Of Morning",  Race.FREE,
			39, 51,  120,  64,  8,  40,  0x01, 0x01, 300,  800);
		MORNING.setDirection(Map.Direction.NORTH);
			
		ATHORIL = new Character(this, i++, "Athoril", "Lord Athoril", Race.FREE,
			54, 38,  120,  64,  8,  50,  0x01, 0x01, 800,  300);
		ATHORIL.setDirection(Map.Direction.NORTH);
			
		BLOOD = new Character(this, i++, "Blood", "Lord Blood",  Race.FREE,
			21, 36,  150,  64,  15,  80,  0x01, 0x01, 1200,  0);
		BLOOD.setDirection(Map.Direction.NORTH);
			
		HERATH = new Character(this, i++, "Herath", "Lord Herath",  Race.FREE,
			45, 26,  130,  64,  8,  40,  0x01, 0x01, 500,  600);
		HERATH.setDirection(Map.Direction.NORTHEAST);
			
		MITHARG = new Character(this, i++, "Mitharg", "Lord Mitharg",  Race.FREE,
			29, 46,  130,  64,  8,  50,  0x01, 0x01, 500,  600);
		MITHARG.setDirection(Map.Direction.NORTH);
			
		UTARG = new Character(this, i++, "Utarg", "the Utarg Of Utarg",  Race.TARG,
			59, 34,  180,  64,  20,  80,  0x00, 0x04, 1000,  0);
		UTARG.setDirection(Map.Direction.WEST);
			
		FAWKRIN = new Character(this, i++, "Fawkrin", "Fawkrin the Skulkrin",  Race.SKULKRIN,
			1, 10,  200,  64,  1,  30,  0x00, 0x20, 0,  0);
		FAWKRIN.setDirection(Map.Direction.EAST);
			
		LORGRIM = new Character(this, i++, "Lorgrim", "Lorgrim the Wise",  Race.WISE,
			62, 0,  200,  64,  20,  70,  0x7f, 0x10, 0,  0);
		LORGRIM.setDirection(Map.Direction.SOUTH);
			
		FARFLAME = new Character(this, i++, "Farflame", "Farflame the Dragonlord", Race.DRAGON, 
			12, 23, 200, 64, 100, 127, 0x00, 0x40, 0, 0);
		FARFLAME.setDirection(Map.Direction.SOUTHEAST);

		LUXOR.setRecruited(true);
		LUXOR.setObject(Location.Object.MOON_RING);
		MORKIN.setRecruited(true);
		CORLETH.setRecruited(true);
		ROTHRON.setRecruited(true);
		characters.add(LUXOR);
		characters.add(MORKIN);
		characters.add(CORLETH);
		characters.add(ROTHRON);
		characters.add(GARD);
		characters.add(MARAKITH);
		characters.add(XAJORKITH);
		characters.add(GLOOM);
		characters.add(SHIMERIL);
		characters.add(KUMAR);
		characters.add(ITHRORN);
		characters.add(DAWN);
		characters.add(DREAMS);
		characters.add(DREGRIM);
		characters.add(THIMRATH);
		characters.add(WHISPERS);
		characters.add(SHADOWS);
		characters.add(LOTHORIL);
		characters.add(KORINEL);
		characters.add(THRALL);
		characters.add(BRITH);
		characters.add(RORATH);
		characters.add(TRORN);
		characters.add(MORNING);
		characters.add(ATHORIL);
		characters.add(BLOOD);
		characters.add(HERATH);
		characters.add(MITHARG);
		characters.add(UTARG);
		characters.add(FAWKRIN);
		characters.add(LORGRIM);
		characters.add(FARFLAME);
	}
	
	/**
	 * Initializes the positions of all Doomdark's armies.
	 */
	private void initializeArmies() {
		armies = new ArrayList();
		
		Army army = new Army(this, Race.FREE, 600, Army.Type.WARRIORS);
		army.guard(8, 0); /* the Citadel of Gloom */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.RIDERS);
		army.guard(46, 3); /* the Keep of Lorgrim */
		armies.add(army);

		army = new Army(this, Race.FOUL, 400, Army.Type.WARRIORS);
		army.guard(28, 4); /* a keep in the Domain of Despair */
		armies.add(army);

		army = new Army(this, Race.FOUL, 1000, Army.Type.WARRIORS);
		army.guard(22, 5); /* the Citadel of Vorgath */
		armies.add(army);

		army = new Army(this, Race.FOUL, 300, Army.Type.RIDERS);
		army.guard(32, 6); /* the Keep of Death */
		armies.add(army);

		army = new Army(this, Race.FOUL, 500, Army.Type.WARRIORS);
		army.guard(23, 7); /* a keep in the Domain of Despair */
		armies.add(army);

		army = new Army(this, Race.FOUL, 1200, Army.Type.RIDERS);
		army.guard(29, 7); /* the Citadel of Ushgarak */
		armies.add(army);

		army = new Army(this, Race.FOUL, 1100, Army.Type.WARRIORS);
		army.guard(37, 7); /* the Citadel of Grarg */
		armies.add(army);

		army = new Army(this, Race.FOUL, 400, Army.Type.RIDERS);
		army.guard(40, 8); /* a keep in the Domain of Grarg */
		armies.add(army);

		army = new Army(this, Race.FREE, 300, Army.Type.WARRIORS);
		army.guard(57, 8); /* the Keep of Fadrath */
		armies.add(army);

		army = new Army(this, Race.FOUL, 500, Army.Type.WARRIORS);
		army.guard(39, 9); /* a keep in the Domain of Grarg */
		armies.add(army);

		army = new Army(this, Race.FEY, 200, Army.Type.WARRIORS);
		army.guard(11, 10); /* the Keep of Lothoril */
		armies.add(army);

		army = new Army(this, Race.FOUL, 300, Army.Type.WARRIORS);
		army.guard(21, 11); /* a keep in the Domain of the Lost */
		armies.add(army);

		army = new Army(this, Race.FOUL, 250, Army.Type.WARRIORS);
		army.guard(25, 11); /* a keep in the Domain of Ugrak */
		armies.add(army);

		army = new Army(this, Race.FOUL, 1000, Army.Type.RIDERS);
		army.guard(29, 12); /* the Citadel of Kor */
		armies.add(army);

		army = new Army(this, Race.FOUL, 300, Army.Type.RIDERS);
		army.guard(36, 12); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.RIDERS);
		army.guard(51, 12); /* the Keep of Ithril */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(62, 12); /* the Keep of Droon */
		armies.add(army);

		army = new Army(this, Race.FOUL, 200, Army.Type.WARRIORS);
		army.guard(16, 13); /* a keep in the Domain of Glorim */
		armies.add(army);

		army = new Army(this, Race.FREE, 300, Army.Type.WARRIORS);
		army.guard(55, 13); /* a keep in the Domain of Ithril */
		armies.add(army);

		army = new Army(this, Race.FREE, 700, Army.Type.WARRIORS);
		army.guard(57, 15); /* the Citadel of Ithrorn */
		armies.add(army);

		army = new Army(this, Race.FOUL, 250, Army.Type.WARRIORS);
		army.guard(14, 16); /* a keep in the Domain of Glorim */
		armies.add(army);

		army = new Army(this, Race.FOUL, 500, Army.Type.WARRIORS);
		army.guard(27, 16); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FOUL, 200, Army.Type.WARRIORS);
		army.guard(34, 16); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FEY, 550, Army.Type.WARRIORS);
		army.guard(42, 16); /* the Citadel of Dreams */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.RIDERS);
		army.guard(52, 16); /* a keep in the Domain of Ithril */
		armies.add(army);

		army = new Army(this, Race.FOUL, 250, Army.Type.WARRIORS);
		army.guard(19, 17); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FOUL, 150, Army.Type.WARRIORS);
		army.guard(22, 18); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(54, 18); /* a keep in the Domain of Ithril */
		armies.add(army);

		army = new Army(this, Race.FOUL, 100, Army.Type.WARRIORS);
		army.guard(14, 20); /* a keep in the Domain of Korkith */
		armies.add(army);

		army = new Army(this, Race.FREE, 300, Army.Type.WARRIORS);
		army.guard(49, 20); /* a keep in the Domain of Ithril */
		armies.add(army);

		army = new Army(this, Race.FEY, 150, Army.Type.WARRIORS);
		army.guard(57, 20); /* the Keep of Whispers */
		armies.add(army);

		army = new Army(this, Race.FOUL, 900, Army.Type.WARRIORS);
		army.guard(18, 21); /* the Citadel of Korkith */
		armies.add(army);

		army = new Army(this, Race.FOUL, 100, Army.Type.WARRIORS);
		army.guard(42, 21); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FOUL, 350, Army.Type.WARRIORS);
		army.guard(31, 22); /* the Keep of Kor */
		armies.add(army);

		army = new Army(this, Race.FREE, 400, Army.Type.RIDERS);
		army.guard(46, 22); /* a keep in the Domain of Ithril */
		armies.add(army);

		army = new Army(this, Race.FOUL, 250, Army.Type.WARRIORS);
		army.guard(39, 23); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(56, 24); /* a keep in the Domain of Herath */
		armies.add(army);

		army = new Army(this, Race.FOUL, 200, Army.Type.WARRIORS);
		army.guard(32, 25); /* a keep in the Domain of Kor */
		armies.add(army);

		army = new Army(this, Race.FREE, 300, Army.Type.WARRIORS);
		army.guard(45, 26); /* the Keep of Herath */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.RIDERS);
		army.guard(54, 26); /* a keep in the Domain of Herath */
		armies.add(army);

		army = new Army(this, Race.FOUL, 200, Army.Type.RIDERS);
		army.guard(34, 27); /* a keep in the Domain of Valethor */
		armies.add(army);

		army = new Army(this, Race.FOUL, 250, Army.Type.WARRIORS);
		army.guard(17, 28); /* the Keep of Valethor */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(42, 28); /* a keep in the Domain of Marakith */
		armies.add(army);

		army = new Army(this, Race.FOUL, 1000, Army.Type.WARRIORS);
		army.guard(24, 29); /* the Citadel of Gorgrath */
		armies.add(army);

		army = new Army(this, Race.FOUL, 150, Army.Type.WARRIORS);
		army.guard(30, 29); /* a keep in the Domain of Valethor */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.RIDERS);
		army.guard(51, 29); /* the Keep of Kumar */
		armies.add(army);

		army = new Army(this, Race.FREE, 600, Army.Type.RIDERS);
		army.guard(57, 29); /* the Citadel of Kumar */
		armies.add(army);

		army = new Army(this, Race.TARG, 200, Army.Type.RIDERS);
		army.guard(55, 31); /* a keep in the Domain of the Targ */
		armies.add(army);

		army = new Army(this, Race.FOUL, 300, Army.Type.WARRIORS);
		army.guard(21, 32); /* a keep in the Domain of Valethor */
		armies.add(army);

		army = new Army(this, Race.FOUL, 300, Army.Type.WARRIORS);
		army.guard(23, 32); /* a keep in the Domain of Valethor */
		armies.add(army);

		army = new Army(this, Race.FREE, 700, Army.Type.WARRIORS);
		army.guard(43, 32); /* the Citadel of Marakith */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(13, 33); /* the Keep of Dodrak */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(34, 33); /* the Keep of Marakith */
		armies.add(army);

		army = new Army(this, Race.FREE, 100, Army.Type.RIDERS);
		army.guard(30, 34); /* a keep in the Domain of Blood */
		armies.add(army);

		army = new Army(this, Race.TARG, 350, Army.Type.RIDERS);
		army.guard(59, 34); /* the Keep of Utarg */
		armies.add(army);

		army = new Army(this, Race.FREE, 400, Army.Type.WARRIORS);
		army.guard(21, 36); /* the Keep of Blood */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(54, 38); /* the Keep of Athoril */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(27, 39); /* a keep in the Domain of Blood */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(22, 40); /* a keep in the Domain of Blood */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(25, 40); /* a keep in the Domain of Blood */
		armies.add(army);

		army = new Army(this, Race.FREE, 100, Army.Type.WARRIORS);
		army.guard(48, 40); /* a keep in the Domain of Dawn */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.RIDERS);
		army.guard(42, 41); /* a keep in the Domain of Dawn */
		armies.add(army);

		army = new Army(this, Race.FEY, 100, Army.Type.RIDERS);
		army.guard(55, 41); /* the Keep of Dregrim */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.RIDERS);
		army.guard(17, 42); /* the Keep of the Moon */
		armies.add(army);

		army = new Army(this, Race.FREE, 750, Army.Type.WARRIORS);
		army.guard(28, 42); /* the Citadel of Shimeril */
		armies.add(army);

		army = new Army(this, Race.FREE, 100, Army.Type.RIDERS);
		army.guard(37, 43); /* the Keep of Dawn */
		armies.add(army);

		army = new Army(this, Race.FEY, 500, Army.Type.WARRIORS);
		army.guard(59, 43); /* the Citadel of Dregrim */
		armies.add(army);

		army = new Army(this, Race.FREE, 550, Army.Type.WARRIORS);
		army.guard(44, 45); /* the Citadel of Dawn */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(29, 46); /* the Keep of Mitharg */
		armies.add(army);

		army = new Army(this, Race.FREE, 100, Army.Type.RIDERS);
		army.guard(42, 46); /* a keep in the Domain of Dawn */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(7, 47); /* the Keep of Torkren */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(10, 47); /* the Keep of Gard */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(48, 48); /* a keep in the Domain of Trorn */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.RIDERS);
		army.guard(21, 49); /* the Keep of Brith */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.RIDERS);
		army.guard(45, 49); /* the Keep of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(54, 50); /* the Keep of Trorn */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(39, 51); /* the Keep of Morning */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(42, 51); /* a keep in the Domain of Morning */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(50, 51); /* a keep in the Domain of Trorn */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(46, 52); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(12, 54); /* a keep in the Domain of Gard */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(25, 54); /* the Keep of Odrark */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(44, 54); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(55, 54); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 100, Army.Type.RIDERS);
		army.guard(7, 55); /* the Keep of Silence */
		armies.add(army);

		army = new Army(this, Race.FREE, 600, Army.Type.RIDERS);
		army.guard(10, 55); /* the Citadel of Gard */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(17, 56); /* the Keep of Ishmalay */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(21, 56); /* a keep in the Domain of Brith */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(37, 56); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.WARRIORS);
		army.guard(8, 57); /* a keep in the Domain of Silence */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(12, 57); /* a keep in the Domain of Gard */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(39, 58); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(56, 58); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 150, Army.Type.RIDERS);
		army.guard(63, 58); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 300, Army.Type.WARRIORS);
		army.guard(42, 59); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 750, Army.Type.RIDERS);
		army.guard(45, 59); /* the Citadel of Xajorkith */
		armies.add(army);

		army = new Army(this, Race.FREE, 50, Army.Type.RIDERS);
		army.guard(4, 60); /* a keep in the Domain of Torkren */
		armies.add(army);

		army = new Army(this, Race.FEY, 300, Army.Type.RIDERS);
		army.guard(33, 60); /* the Keep of Thimrath */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.RIDERS);
		army.guard(23, 60); /* the Keep of Rorath */
		armies.add(army);

		army = new Army(this, Race.FREE, 250, Army.Type.WARRIORS);
		army.guard(59, 60); /* a keep in the Domain of Corelay */
		armies.add(army);

		army = new Army(this, Race.FREE, 200, Army.Type.WARRIORS);
		army.guard(14, 60); /* a keep in the Domain of Brith */
		armies.add(army);
	}

	/**
	 * Initializes the positions of all Doomdark's armies.
	 */
	private void initializeDoomguard() {
		doomguard = new ArrayList();
		
		Doomguard army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, LUXOR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, MORKIN);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, CORLETH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, ROTHRON);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, GARD);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, MARAKITH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, XAJORKITH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, SHIMERIL);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, KUMAR);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, ITHRORN);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, DAWN);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, DREGRIM);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, THIMRATH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, SHADOWS);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, THRALL);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, BRITH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, RORATH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, TRORN);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, MORNING);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, ATHORIL);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, BLOOD);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, HERATH);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, MITHARG);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(3));
		army.setLocation(22, 5);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(3));
		army.setLocation(22, 5);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(3));
		army.setLocation(22, 5);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(3));
		army.setLocation(22, 5);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(7));
		army.setLocation(37, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(7));
		army.setLocation(37, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(7));
		army.setLocation(37, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(7));
		army.setLocation(37, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, MORKIN);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.FOLLOW, MORKIN);
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(14));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.RIDERS, Doomguard.Orders.ROUTE, getMap().getRouteNode(32));
		army.setLocation(18, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(44));
		army.setLocation(24, 29);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(44));
		army.setLocation(24, 29);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(44));
		army.setLocation(24, 29);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.ROUTE, getMap().getRouteNode(44));
		army.setLocation(24, 29);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.WANDER, null);
		army.setLocation(7, 21);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.WANDER, null);
		army.setLocation(27, 16);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.WANDER, null);
		army.setLocation(40, 8);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.WANDER, null);
		army.setLocation(39, 23);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.WANDER, null);
		army.setLocation(21, 32);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.WANDER, null);
		army.setLocation(23, 32);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.WARRIORS, Doomguard.Orders.WANDER, null);
		army.setLocation(17, 28);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.WARRIORS, Doomguard.Orders.WANDER, null);
		army.setLocation(18, 3);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.WARRIORS, Doomguard.Orders.WANDER, null);
		army.setLocation(30, 29);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.WARRIORS, Doomguard.Orders.WANDER, null);
		army.setLocation(16, 13);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.WARRIORS, Doomguard.Orders.WANDER, null);
		army.setLocation(31, 22);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.WARRIORS, Doomguard.Orders.WANDER, null);
		army.setLocation(6, 37);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(29, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(29, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(22, 5);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(37, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(23, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(28, 4);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(14));
		army.setLocation(25, 11);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(7));
		army.setLocation(36, 12);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(7));
		army.setLocation(40, 8);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(7));
		army.setLocation(39, 9);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(32, 6);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1200, Army.Type.WARRIORS, Doomguard.Orders.GOTO, getMap().getRouteNode(3));
		army.setLocation(21, 11);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(29, 9);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(33, 7);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(30, 6);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(27, 6);
		doomguard.add(army);

		army = new Doomguard(this, 0, 1000, Army.Type.RIDERS, Doomguard.Orders.GOTO, getMap().getRouteNode(6));
		army.setLocation(26, 7);
		doomguard.add(army);

	}
	
	private Random randomGenerator = new Random();
	
	/**
	 * Returns a random number in the range [0-n).
	 * 
	 * @param n upper bound on the range from which the numbers are drawn.
	 */
	protected int random(int n) {
		return randomGenerator.nextInt(n);
	}
}
