/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

import java.util.ArrayList;

/**
 * @author cubranic
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public abstract class Unit {
	public static final int MAX_ENERGY = 127;

	/** game instance that the unit is part of */
	private final Midnight game;

	/** unit's race */
	private Race race;
	
	/** unit's current location */
	private Location location;
	
	/** unit's energy and physical condition (energy >> 4) */
	private int energy;
	private Condition condition;
	
	/** how many enemy killed in the last battle */
	int enemyKilled;
	
	/**
	 * Unit constructor.
	 */
	protected Unit(Midnight game, Race race, int energy) {
		this.game = game;
		this.race = race;
		setEnergy(energy);
	}
	
	/**
	 * Returns unit's game instance.
	 */
	protected Midnight getGame() {
		return game;
	}
	
	/**
	 * Returns unit's race.
	 */
	public Race getRace() {
		return race;
	}
	
	/**
	 * Sets unit's race.
	 */
	protected void setRace(Race race) {
		this.race = race;
	}
	
	/**
	 * Returns unit's location
	 */
	public Location getLocation() {
		return location;
	}
	
	/**
	 * Sets unit's location
	 */
	public void setLocation(Location location) {
		this.location = location;
	}
	
	/**
	 * Returns unit's physical condition.
	 */
	public Condition getCondition() {
		return condition;
	}
	
	/**
	 * Returns the unit's energy.
	 * Energy is represented as a number 0-128, with the
	 * condition corresponding to (energy >> 4).
	 */
	public int getEnergy() {
		return energy;
	}
	
	/**
	 * Sets the unit's energy and, by defition, his physical condition.
	 */
	public void setEnergy(int energy) {
		if (energy < 0) {
			energy = 0;
		}
		else if (energy > MAX_ENERGY) {
			energy = MAX_ENERGY;
		}
		this.energy = energy;
		this.condition = Condition.getCondition(energy >> 4);
	}
	
	/**
	 * Increment's the unit's energy by the given amount.
	 */
	public void incrementEnergy(int increment) {
		setEnergy(getEnergy() + increment);
	}
	
	/**
	 * Decrement's the unit's energy by the give amount.
	 */
	public void decrementEnergy(int decrement) {
		setEnergy(getEnergy() - decrement);
	}

	/**
	 * Enum class for describing character condition.
	 */
	protected static class Condition implements Comparable {
		/** Ordered enumeration instances */
		private static ArrayList values = new ArrayList();
		
		public static final Condition UTTERLY_TIRED = new Condition("utterly tired and cannot continue");
		public static final Condition VERY_TIRED = new Condition("very tired");
		public static final Condition TIRED = new Condition("tired");
		public static final Condition QUITE_TIRED = new Condition("quite tired");
		public static final Condition SLIGHTLY_TIRED = new Condition("slightly tired");

		public static final Condition INVIGORATED = new Condition("invigorated");
		public static final Condition VERY_INVIGORATED = new Condition("very invigorated");
		public static final Condition UTTERLY_INVIGORATED = new Condition("utterly invigorated");

		/** The ordinal value for this condition */
		private final int ordinal;
		
		private String description;

		private Condition(String description) {
			this.description = description;
			ordinal = values.size();
			values.add(this);
		}
		
		public String toString() {
			return description;
		}
		
		public int compareTo(Object o) {
			return ordinal - ((Condition)o).ordinal;
		}
		
		public static Condition getCondition(int index) {
			return (Condition) values.get(index);
		}
	}

	/**
	 * Returns the number of enemy killed in the last battle
	 */
	public int getEnemyKilled() {
		return enemyKilled;
	}

	/**
	 * Sets the number of enemy killed in the last battle.
	 */
	public void setEnemyKilled(int enemyKilled) {
		this.enemyKilled = enemyKilled;
	}
}
