/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;
import java.util.HashMap;

import midnight.Location;

/**
 * Class that's used to draw a feature on the screen.
 */
public class Feature {
	private static HashMap instances = new HashMap();
	
	private int dataOffset;
	
	/**
	 * Private constructor to ensure singletons for each location feature-size
	 * combination.
	 */
	private Feature(int dataOffset) {
		this.dataOffset = dataOffset;
	}
	
	public synchronized static Feature get(Location.Feature feature, int size) {
		if (feature == feature.PLAINS) return null;
		Feature f = (Feature) instances.get(feature.toString() + size);
		if (f == null) {
			f = new Feature(terrainDataIndex[feature.getIndex()][size]);
			instances.put(feature.toString() + size, f);
		}
		return f;
	}

	/**
	 * Draws the feature at a given (x, y) coordinate.
	 */
	public void paint(Graphics g, int x, int y) {
		paint(g, x, y, false);
	}

	/**
	 * Draws the feature at a given (x, y) coordinate.
	 */
	public void paint(Graphics g, int x, int y, boolean isNight) {
		Color fg;
		Color bg;
		if (isNight) {
			fg = Color.BLUE;
			bg = Color.BLACK;
		}
		else {
			fg = Color.BLUE;
			bg = Color.WHITE;
		}
//		System.err.println("Draw at " + x + ", " + y + " using data starting at index " + dataOffset);
		y = 192-16-y;
		x -= 256;
		int i = dataOffset+1;
		for (int height = drawData[dataOffset]; height > 0; height --) {
			while (true) {
				int flags = drawData[i++];
				boolean moreOnThisLine = (flags & 0x80) != 0;
				boolean useInk = (flags & 0x40)  == 0;
				int operationCount = flags & 0x3f;
				
				int x1 = drawData[i++];
				int x2 = drawData[i++];
				DrawLine(g, y, x+x1, x+x2, useInk ? fg : bg);
				
				DrawSet(g, y, x+x1, fg);
				DrawSet(g, y, x+x2, fg);
				
				useInk = !useInk;
				for ( ; operationCount > 0; operationCount--) {
					flags = drawData[i++];
					x1 = flags & 0x7f;
					if ((flags & 0x80) != 0) {
						x2 = drawData[i++];
						DrawLine(g, y, x+x1, x+x2, useInk ? fg : bg);
					}
					else {
						DrawSet(g, y, x+x1, useInk ? fg : bg);
					}
				}
				
				if (!moreOnThisLine) break;
			}
			y--;
		}
	}
	
	private void DrawLine(Graphics g, int y, int x1, int x2, Color color) {
//		System.err.println("Draw line from " + x1 + ", " + y + " to " + x2 + ", " + y + " in " + color);
		g.setColor(color);
		g.drawLine(x1, y, x2, y);
		g.setColor(Color.BLACK);
	}
	
	private void DrawSet(Graphics g, int y, int x, Color color) {
//		System.err.println("Draw point at " + x + ", " + y);
		g.setColor(color);
		g.drawLine(x, y, x, y);
		g.setColor(Color.BLACK);
	}
	
	/**
	 * Index into featureData for a given terrain type and size.
	 */
	private static final short terrainDataIndex[][] = {
		{ 0x0000,0x023B,0x03C5,0x04DD,0x05CA,0x0674,0x06DE,0x072F }, // Mountain
		{ 0x076E,0x0A82,0x0C76,0x0D93,0x0E6E,0x0F00,0x0F5D,0x0FAB }, // Citadel
		{ 0x0FD8,0x120F,0x1384,0x1476,0x153F,0x15D1,0x1632,0x1677 }, // Forest
		{ 0x16AA,0x1819,0x18F7,0x1994,0x1A1C,0x1A6E,0x1AA4,0x1AC9 }, // Henge
		{ 0x1AE1,0x1C42,0x1D2D,0x1DCC,0x1E4E,0x1EB1,0x1EF8,0x1F2F }, // Tower
		{ 0x1F5B,0x204B,0x20E0,0x213E,0x2191,0x21C1,0x21E6,0x21FC }, // Village
		{ 0x220B,0x22D1,0x235C,0x23BF,0x2419,0x245D,0x248D,0x24B2 }, // Downs
		{ 0x24C7,0x25CE,0x2692,0x26F5,0x2744,0x2773,0x2797,0x27AE }, // Keep
		{ 0x27C2,0x2835,0x2884,0x28B7,0x28DD,0x28F5,0x290A,0x291B }, // SnowHall
		{ 0x2926,0x295A,0x2981,0x299E,0x29AE,0x29BB,0x29C5,0x29CC }, // lake
		{ 0x29D0,0x2BEB,0x2D50,0x2E46,0x2F14,0x2FA8,0x3001,0x303F }, // Wastes
		{ 0x3073,0x317F,0x3231,0x32A5,0x330D,0x3359,0x338D,0x33B0 }, // Ruin
		{ 0x33CC,0x3567,0x3688,0x3753,0x3807,0x388B,0x38EC,0x3930 }, // lith
		{ 0x395C,0x39E0,0x3A3E,0x3A83,0x3ABF,0x3AE8,0x3B09,0x3B24 }, // Cavern
		{ 0x3B35,0x3D22,0x3E4A,0x3F11,0x3FB8,0x403E,0x408C,0x40C2 }, // Army
	};

	/**
	 * Instructions describing how to draw a given feature.
	 */
	private static final char drawData[];
	
	static {
		instances = new HashMap();
		drawData = new char[16618];
		initializeDrawData1();
		initializeDrawData2();
	}
	
	/**
	 * Initializes the first half of drawData array to avoid running into
	 * the 65535-byte limit on method/initializer length (?!).
	 */
	private static void initializeDrawData1() {
		char drawData[] = {
		0x4e, 0x41, 0x00, 0x7f, 0xc9, 0x4b, 0x43, 0x01,
		0x7e, 0x02, 0x0f, 0x48, 0x44, 0x03, 0x7d, 0x04,
		0x10, 0x47, 0x7c, 0x42, 0x05, 0x7b, 0x11, 0x46,
		0x43, 0x06, 0x7a, 0x11, 0x46, 0x4a, 0x44, 0x07,
		0x79, 0x12, 0x1e, 0x46, 0x4a, 0x45, 0x08, 0x79,
		0x13, 0x1f, 0x46, 0x4b, 0xf4, 0x75, 0x45, 0x09,
		0x78, 0x13, 0xa0, 0x21, 0xc4, 0x45, 0x4b, 0xf2,
		0x73, 0x45, 0x0a, 0x78, 0x14, 0x22, 0xc2, 0x43,
		0x4b, 0x71, 0x45, 0x0b, 0x77, 0x14, 0x22, 0x41,
		0xcc, 0x4d, 0x71, 0x47, 0x0c, 0x77, 0x15, 0x22,
		0x40, 0x4e, 0x65, 0x70, 0xf5, 0x76, 0x47, 0x0d,
		0x74, 0x96, 0x1a, 0x23, 0x40, 0xc3, 0x44, 0x4e,
		0x65, 0x70, 0x47, 0x0e, 0x73, 0x1b, 0x23, 0x3f,
		0xc5, 0x46, 0x4e, 0x65, 0x70, 0x47, 0x0f, 0x73,
		0x9c, 0x1d, 0x24, 0x3e, 0x47, 0x4e, 0x65, 0x70,
		0x47, 0x0f, 0x72, 0x9e, 0x1f, 0x25, 0x3d, 0x47,
		0x4e, 0x65, 0x70, 0x47, 0x10, 0x72, 0xa0, 0x22,
		0x25, 0x3c, 0x48, 0x4e, 0x65, 0x70, 0x46, 0x10,
		0x72, 0xa3, 0x25, 0x3c, 0x49, 0x4f, 0x65, 0x6f,
		0x47, 0x11, 0x72, 0x92, 0x13, 0x25, 0xba, 0x3b,
		0x4a, 0x4f, 0x65, 0x6f, 0x48, 0x12, 0x72, 0x14,
		0x26, 0x39, 0x4b, 0x4f, 0x63, 0x65, 0xef, 0x71,
		0x47, 0x12, 0x6f, 0x15, 0x26, 0xb6, 0x38, 0x4c,
		0x4f, 0x63, 0x65, 0x47, 0x13, 0x6f, 0x96, 0x17,
		0x26, 0x35, 0x4c, 0x50, 0x63, 0x65, 0x47, 0x13,
		0x6f, 0x18, 0x27, 0x35, 0x4c, 0x51, 0x63, 0x65,
		0x47, 0x14, 0x6f, 0x19, 0x27, 0x34, 0x4c, 0x51,
		0x62, 0x65, 0x47, 0x14, 0x6f, 0x19, 0x28, 0x33,
		0x4c, 0x51, 0x62, 0x65, 0x47, 0x15, 0x6e, 0x1a,
		0xa9, 0x2d, 0x32, 0x4d, 0x51, 0x62, 0x66, 0x47,
		0x15, 0x6e, 0x9b, 0x1c, 0xae, 0x2f, 0x31, 0x4d,
		0x51, 0x62, 0x66, 0x46, 0x16, 0x6e, 0x1d, 0x30,
		0x4d, 0x51, 0x62, 0x66, 0x45, 0x16, 0x6e, 0x1d,
		0x4e, 0x52, 0x62, 0x66, 0x45, 0x16, 0x6e, 0x1d,
		0x4e, 0x52, 0x61, 0x67, 0x45, 0x17, 0x6e, 0x1d,
		0x4e, 0x53, 0x61, 0x67, 0x45, 0x17, 0x6d, 0x1e,
		0x4e, 0x53, 0x60, 0x67, 0x45, 0x18, 0x6d, 0x1e,
		0x4e, 0x54, 0x60, 0x67, 0x45, 0x18, 0x6d, 0x1f,
		0x4f, 0x55, 0x60, 0x67, 0x44, 0x18, 0x6d, 0x1f,
		0x50, 0xd5, 0x5f, 0xe4, 0x6c, 0x43, 0x18, 0x6b,
		0x20, 0xd1, 0x54, 0x63, 0x43, 0x19, 0x6a, 0x20,
		0x51, 0xe0, 0x62, 0x44, 0x19, 0x69, 0x21, 0x50,
		0xde, 0x5f, 0xe6, 0x68, 0x43, 0x1a, 0x65, 0xa2,
		0x23, 0x4f, 0xe3, 0x64, 0x43, 0x1b, 0x62, 0x24,
		0x4f, 0xe0, 0x61, 0x43, 0x1c, 0x61, 0x25, 0x4f,
		0xde, 0x5f, 0x44, 0x1d, 0x61, 0x1e, 0x25, 0x4f,
		0xdc, 0x5d, 0x45, 0x1f, 0x60, 0xa0, 0x22, 0x26,
		0x4e, 0xda, 0x5b, 0x5f, 0x43, 0x23, 0x5e, 0xa4,
		0x26, 0x4e, 0x59, 0x42, 0x26, 0x5e, 0x4d, 0x58,
		0x42, 0x26, 0x5d, 0x4d, 0x57, 0x42, 0x27, 0x5d,
		0x4d, 0x56, 0x42, 0x27, 0x5c, 0x4d, 0x55, 0x43,
		0x27, 0x5c, 0xb5, 0x37, 0x4c, 0x54, 0x43, 0x28,
		0x5c, 0x38, 0x4c, 0x53, 0x43, 0x28, 0x5b, 0x29,
		0xb9, 0x3b, 0x4c, 0x42, 0x2a, 0x5a, 0x3c, 0x4c,
		0x42, 0x2b, 0x5a, 0x3d, 0x59, 0x42, 0x2b, 0x5a,
		0x3d, 0xd7, 0x58, 0x43, 0x2b, 0x59, 0x2c, 0x3e,
		0xd5, 0x56, 0x42, 0x2d, 0x59, 0x3f, 0x54, 0x43,
		0x2e, 0x58, 0xaf, 0x33, 0x40, 0x54, 0x43, 0x31,
		0x57, 0x34, 0x41, 0x53, 0x43, 0x32, 0x57, 0xb5,
		0x36, 0x41, 0x53, 0x43, 0x32, 0x57, 0xb7, 0x3a,
		0x42, 0x52, 0x44, 0x33, 0x57, 0x34, 0x3b, 0x43,
		0x51, 0x43, 0x34, 0x56, 0x3b, 0x44, 0x51, 0x44,
		0x35, 0x56, 0x36, 0x3c, 0x45, 0x50, 0x42, 0x36,
		0x56, 0x3c, 0x46, 0x42, 0x37, 0x55, 0x3c, 0x47,
		0x42, 0x37, 0x54, 0x3d, 0x47, 0x42, 0x38, 0x53,
		0x3e, 0x47, 0x42, 0x38, 0x52, 0x3f, 0x48, 0x43,
		0x39, 0x51, 0x3a, 0x40, 0xc9, 0x4b, 0x42, 0x3b,
		0x50, 0x40, 0xcc, 0x4f, 0x41, 0x3c, 0x4e, 0x3d,
		0x40, 0x3e, 0x4d, 0x41, 0x3f, 0x4c, 0xca, 0x4b,
		0x40, 0x40, 0x49, 0x40, 0x40, 0x49, 0x41, 0x41,
		0x48, 0x42, 0x40, 0x43, 0x47, 0x40, 0x44, 0x46,
		0x40, 0x45, 0x45, 0x37, 0x41, 0x13, 0x6c, 0xc6,
		0x48, 0x44, 0x14, 0x6b, 0x95, 0x16, 0x1e, 0x45,
		0x6a, 0x42, 0x17, 0x69, 0x1f, 0x44, 0x43, 0x17,
		0x68, 0x1f, 0x44, 0x47, 0x45, 0x18, 0x68, 0x20,
		0xa8, 0x29, 0x44, 0x47, 0xe4, 0x65, 0x45, 0x19,
		0x67, 0x20, 0x2a, 0x43, 0x48, 0x63, 0x45, 0x1a,
		0x66, 0x21, 0x2b, 0xc1, 0x42, 0x49, 0x62, 0x46,
		0x1b, 0x65, 0x22, 0x2b, 0x40, 0x4a, 0x5a, 0x61,
		0x48, 0x1c, 0x64, 0x1d, 0xa3, 0x26, 0x2c, 0x3f,
		0xc2, 0x44, 0x4a, 0x5a, 0x61, 0x47, 0x1e, 0x64,
		0x27, 0x2c, 0x3e, 0x45, 0x4a, 0x5a, 0x61, 0x47,
		0x1e, 0x63, 0x28, 0x2d, 0x3e, 0x45, 0x4a, 0x5a,
		0x61, 0x46, 0x1e, 0x63, 0xa9, 0x2d, 0x3d, 0x46,
		0x4a, 0x5a, 0x61, 0x46, 0x1f, 0x63, 0x2d, 0x3c,
		0x47, 0x4a, 0x5a, 0x61, 0x48, 0x20, 0x63, 0x21,
		0x2e, 0xb9, 0x3b, 0x48, 0x4a, 0x58, 0x5a, 0xe1,
		0x62, 0x47, 0x20, 0x61, 0xa2, 0x23, 0x2e, 0x38,
		0x48, 0x4b, 0x58, 0x5a, 0x47, 0x20, 0x61, 0x24,
		0x2e, 0x38, 0x48, 0x4c, 0x58, 0x5a, 0x47, 0x21,
		0x61, 0x25, 0x2f, 0x37, 0x48, 0x4c, 0x58, 0x5a,
		0x47, 0x22, 0x60, 0x25, 0xb0, 0x32, 0x36, 0x49,
		0x4c, 0x58, 0x5a, 0x46, 0x22, 0x60, 0x26, 0xb3,
		0x35, 0x49, 0x4c, 0x58, 0x5a, 0x45, 0x22, 0x60,
		0x27, 0x4a, 0x4c, 0x58, 0x5a, 0x45, 0x22, 0x60,
		0x27, 0x4a, 0x4c, 0x57, 0x5b, 0x45, 0x23, 0x5f,
		0x28, 0x4a, 0x4d, 0x56, 0x5b, 0x45, 0x24, 0x5f,
		0x28, 0x4a, 0x4e, 0x56, 0x5b, 0x44, 0x24, 0x5e,
		0x29, 0x4b, 0xcf, 0x55, 0xd9, 0x5d, 0x43, 0x24,
		0x5e, 0x29, 0xcc, 0x4e, 0x58, 0x44, 0x25, 0x5d,
		0x2a, 0x4b, 0xd5, 0x57, 0xdb, 0x5c, 0x43, 0x25,
		0x5a, 0x2b, 0x4a, 0x59, 0x43, 0x26, 0x58, 0x2c,
		0x4a, 0xd5, 0x57, 0x44, 0x27, 0x57, 0x28, 0x2d,
		0x4a, 0xd3, 0x54, 0x43, 0x29, 0x56, 0xaa, 0x2d,
		0x4a, 0x52, 0x42, 0x2e, 0x55, 0x49, 0x51, 0x42,
		0x2e, 0x54, 0x49, 0x50, 0x42, 0x2e, 0x54, 0x49,
		0x4f, 0x43, 0x2e, 0x53, 0xb8, 0x3a, 0x48, 0x4e,
		0x43, 0x2f, 0x53, 0xbb, 0x3c, 0x48, 0x4d, 0x42,
		0x30, 0x52, 0x3d, 0x48, 0x42, 0x31, 0x51, 0x3e,
		0x50, 0x42, 0x32, 0x51, 0x3e, 0x4f, 0x42, 0x33,
		0x51, 0x3f, 0x4e, 0x43, 0x34, 0x51, 0xb5, 0x37,
		0xc0, 0x41, 0x4d, 0x43, 0x36, 0x50, 0xb8, 0x39,
		0x41, 0x4d, 0x43, 0x36, 0x50, 0xba, 0x3b, 0x42,
		0x4c, 0x43, 0x37, 0x4f, 0x3c, 0x43, 0x4c, 0x44,
		0x38, 0x4f, 0x39, 0x3d, 0x44, 0x4b, 0x42, 0x3a,
		0x4f, 0x3d, 0x45, 0x42, 0x3a, 0x4e, 0x3e, 0x45,
		0x42, 0x3a, 0x4d, 0x3f, 0x45, 0x42, 0x3b, 0x4c,
		0x40, 0xc6, 0x47, 0x44, 0x3c, 0x4b, 0x3d, 0x40,
		0x48, 0x4a, 0x40, 0x3e, 0x49, 0x41, 0x3f, 0x48,
		0x47, 0x40, 0x40, 0x46, 0x40, 0x41, 0x45, 0x40,
		0x42, 0x44, 0x40, 0x43, 0x43, 0x27, 0x43, 0x20,
		0x5f, 0x21, 0x27, 0xc4, 0x45, 0x43, 0x22, 0x5e,
		0x28, 0x43, 0x5d, 0x44, 0x23, 0x5c, 0x28, 0x2f,
		0x43, 0x45, 0x45, 0x24, 0x5c, 0x29, 0x30, 0x42,
		0x45, 0xd9, 0x5a, 0x45, 0x25, 0x5b, 0x2a, 0x31,
		0x41, 0x46, 0x58, 0x46, 0x26, 0x5a, 0xab, 0x2d,
		0x31, 0x40, 0x47, 0x52, 0x58, 0x47, 0x27, 0x5a,
		0x2e, 0x32, 0x3f, 0x42, 0x47, 0x52, 0x58, 0x47,
		0x28, 0x5a, 0xaf, 0x30, 0x32, 0x3e, 0x43, 0x47,
		0x52, 0x58, 0x46, 0x29, 0x59, 0xb1, 0x32, 0x3d,
		0x44, 0x47, 0x52, 0x57, 0x47, 0x29, 0x59, 0x2a,
		0x33, 0xbb, 0x3c, 0x45, 0x47, 0xd1, 0x52, 0xd7,
		0x58, 0x46, 0x29, 0x57, 0x2b, 0x33, 0x3a, 0x46,
		0x48, 0xd1, 0x52, 0x46, 0x2a, 0x57, 0x2c, 0x34,
		0x39, 0x46, 0x48, 0xd1, 0x52, 0x46, 0x2a, 0x57,
		0x2d, 0xb5, 0x38, 0x46, 0x48, 0x51, 0x53, 0x45,
		0x2b, 0x57, 0x2e, 0x46, 0x49, 0x51, 0x53, 0x45,
		0x2b, 0x57, 0x2e, 0x47, 0x49, 0x50, 0x53, 0x45,
		0x2b, 0x56, 0x2f, 0x47, 0xc9, 0x4a, 0x50, 0x53,
		0x44, 0x2c, 0x56, 0x2f, 0x47, 0xca, 0x50, 0xd2,
		0x55, 0x43, 0x2c, 0x55, 0x30, 0xc8, 0x4a, 0x51,
		0x44, 0x2c, 0x54, 0x31, 0x48, 0x50, 0xd2, 0x53,
		0x42, 0x2d, 0x51, 0x32, 0x47, 0x44, 0x2e, 0x50,
		0xaf, 0x30, 0x33, 0x47, 0xcd, 0x4f, 0x43, 0x31,
		0x4f, 0xb2, 0x33, 0x47, 0x4c, 0x42, 0x33, 0x4e,
		0x46, 0x4b, 0x43, 0x33, 0x4e, 0xba, 0x3b, 0x46,
		0x4a, 0x43, 0x34, 0x4e, 0xbc, 0x3d, 0x46, 0x49,
		0x42, 0x35, 0x4d, 0x3e, 0x46, 0x42, 0x35, 0x4d,
		0x3e, 0xcb, 0x4c, 0x43, 0x36, 0x4c, 0x37, 0x3f,
		0x4a, 0x43, 0x38, 0x4b, 0xb9, 0x3b, 0x40, 0x49,
		0x43, 0x39, 0x4b, 0x3c, 0x41, 0x49, 0x43, 0x3a,
		0x4b, 0x3d, 0x42, 0x48, 0x42, 0x3b, 0x4a, 0x3e,
		0x43, 0x42, 0x3c, 0x49, 0x3e, 0x43, 0x42, 0x3d,
		0x48, 0x3f, 0xc4, 0x45, 0x42, 0x3e, 0x47, 0x40,
		0x46, 0x40, 0x3f, 0x45, 0x40, 0x40, 0x44, 0x40,
		0x41, 0x43, 0x40, 0x42, 0x42, 0x23, 0x42, 0x24,
		0x5b, 0x2b, 0xc4, 0x45, 0x43, 0x25, 0x5a, 0x26,
		0x2b, 0x43, 0x44, 0x27, 0x59, 0x2c, 0x31, 0x43,
		0x45, 0x45, 0x28, 0x59, 0x2c, 0x32, 0x42, 0x45,
		0x56, 0x46, 0x29, 0x58, 0x2d, 0x33, 0x41, 0x46,
		0x50, 0x55, 0x47, 0x2a, 0x57, 0xae, 0x30, 0x33,
		0x40, 0x42, 0x46, 0x50, 0x55, 0x47, 0x2b, 0x56,
		0xb1, 0x32, 0x34, 0x3f, 0x43, 0x46, 0x50, 0x55,
		0x45, 0x2c, 0x55, 0xb3, 0x34, 0x3e, 0x44, 0x47,
		0x50, 0x46, 0x2c, 0x55, 0x2d, 0x35, 0xbc, 0x3d,
		0x45, 0x47, 0x50, 0x46, 0x2c, 0x55, 0x2e, 0x35,
		0x3b, 0x45, 0x48, 0x50, 0x46, 0x2d, 0x55, 0x2f,
		0x36, 0x3a, 0x45, 0x48, 0xcf, 0x50, 0x46, 0x2d,
		0x54, 0x30, 0xb7, 0x39, 0x46, 0x48, 0x4f, 0x51,
		0x45, 0x2e, 0x54, 0x30, 0x46, 0x48, 0x4f, 0x51,
		0x45, 0x2e, 0x54, 0x31, 0x46, 0x49, 0xce, 0x4f,
		0x51, 0x45, 0x2f, 0x54, 0x31, 0x47, 0x49, 0x4e,
		0x51, 0x42, 0x2f, 0x54, 0x32, 0xc8, 0x53, 0x42,
		0x2f, 0x50, 0x33, 0x47, 0x42, 0x30, 0x4f, 0x34,
		0x47, 0x43, 0x31, 0x4e, 0xb2, 0x35, 0x47, 0xcc,
		0x4d, 0x42, 0x35, 0x4d, 0x46, 0x4b, 0x42, 0x35,
		0x4d, 0x46, 0x4a, 0x43, 0x35, 0x4c, 0xbb, 0x3c,
		0x45, 0x49, 0x42, 0x36, 0x4c, 0xbd, 0x3e, 0x45,
		0x42, 0x37, 0x4c, 0x3f, 0xca, 0x4b, 0x43, 0x38,
		0x4b, 0x39, 0x40, 0x49, 0x43, 0x3a, 0x4a, 0xbb,
		0x3c, 0x41, 0x48, 0x43, 0x3a, 0x4a, 0x3d, 0x42,
		0x48, 0x43, 0x3b, 0x4a, 0x3e, 0x43, 0x47, 0x42,
		0x3c, 0x49, 0x3f, 0x43, 0x42, 0x3d, 0x48, 0x40,
		0x44, 0x41, 0x3e, 0x47, 0x45, 0x41, 0x3f, 0x46,
		0x45, 0x40, 0x40, 0x44, 0x40, 0x41, 0x43, 0x40,
		0x42, 0x42, 0x1a, 0x42, 0x2b, 0x55, 0x30, 0x43,
		0x43, 0x2c, 0x54, 0x30, 0x42, 0x53, 0x44, 0x2d,
		0x52, 0x31, 0x35, 0x41, 0x51, 0x45, 0x2e, 0x50,
		0x32, 0x36, 0x40, 0x44, 0x4c, 0x46, 0x2f, 0x50,
		0xb3, 0x34, 0x37, 0x3f, 0x42, 0x45, 0x4c, 0x45,
		0x30, 0x50, 0xb5, 0x37, 0x3f, 0x42, 0x45, 0x4c,
		0x45, 0x31, 0x50, 0x38, 0xbd, 0x3e, 0x43, 0x45,
		0x4c, 0x45, 0x32, 0x50, 0x38, 0x3c, 0x44, 0x46,
		0x4c, 0x45, 0x32, 0x4f, 0xb3, 0x34, 0xb9, 0x3b,
		0x44, 0x46, 0xcb, 0x4c, 0x45, 0x33, 0x4f, 0x35,
		0x44, 0x46, 0x4b, 0x4d, 0x45, 0x33, 0x4f, 0x35,
		0x45, 0x47, 0x4b, 0x4d, 0x42, 0x33, 0x4f, 0x35,
		0xc6, 0x4e, 0x42, 0x34, 0x4d, 0x36, 0x46, 0x43,
		0x35, 0x4c, 0x37, 0x45, 0xca, 0x4b, 0x43, 0x36,
		0x4b, 0x37, 0x45, 0xc8, 0x49, 0x42, 0x38, 0x4a,
		0x44, 0x47, 0x43, 0x39, 0x49, 0xbd, 0x3e, 0x44,
		0x46, 0x42, 0x3a, 0x49, 0x3f, 0x44, 0x42, 0x3b,
		0x48, 0x40, 0x47, 0x42, 0x3b, 0x48, 0x41, 0x46,
		0x43, 0x3c, 0x47, 0xbd, 0x3e, 0x42, 0x45, 0x42,
		0x3d, 0x46, 0x3f, 0x42, 0x02, 0x3e, 0x45, 0x3f,
		0xc1, 0x42, 0x40, 0x3f, 0x44, 0x01, 0x40, 0x43,
		0x42, 0x40, 0x42, 0x42, 0x14, 0x41, 0x30, 0x50,
		0x42, 0x43, 0x31, 0x4f, 0x34, 0x37, 0x41, 0x46,
		0x32, 0x4e, 0x35, 0x38, 0x40, 0x43, 0x49, 0x4d,
		0x44, 0x33, 0x4c, 0xb6, 0x38, 0x3f, 0x43, 0x49,
		0x44, 0x34, 0x4c, 0x39, 0x3e, 0x43, 0x49, 0x44,
		0x34, 0x4c, 0x3a, 0x3d, 0x44, 0x49, 0x43, 0x35,
		0x4b, 0xbb, 0x3c, 0x44, 0xc8, 0x49, 0x42, 0x35,
		0x4b, 0x44, 0xc8, 0x49, 0x41, 0x36, 0x4b, 0xc5,
		0x4a, 0x41, 0x36, 0x49, 0x44, 0x42, 0x37, 0x48,
		0x38, 0x44, 0x41, 0x39, 0x47, 0x43, 0x42, 0x3a,
		0x47, 0xbd, 0x3e, 0x43, 0x41, 0x3b, 0x46, 0x3f,
		0x41, 0x3c, 0x46, 0x40, 0x41, 0x3d, 0x46, 0x41,
		0x41, 0x3e, 0x45, 0x42, 0x41, 0x3f, 0x44, 0x43,
		0x40, 0x40, 0x43, 0x00, 0x41, 0x42, 0x10, 0x40,
		0x33, 0x4c, 0x43, 0x34, 0x4c, 0x37, 0x41, 0x43,
		0x44, 0x35, 0x4b, 0x38, 0x40, 0x43, 0x47, 0x44,
		0x36, 0x4a, 0xb9, 0x3a, 0x3f, 0x43, 0x47, 0x44,
		0x37, 0x4a, 0x3b, 0x3e, 0x43, 0x47, 0x43, 0x37,
		0x49, 0xbc, 0x3d, 0x43, 0x47, 0x42, 0x38, 0x49,
		0x43, 0x47, 0x41, 0x39, 0x49, 0xc4, 0x48, 0x41,
		0x3a, 0x47, 0x43, 0x41, 0x3b, 0x46, 0x42, 0x42,
		0x3b, 0x45, 0x3e, 0x42, 0x41, 0x3c, 0x45, 0x3f,
		0x41, 0x3d, 0x44, 0x40, 0x01, 0x3e, 0x43, 0xbf,
		0x40, 0x40, 0x3f, 0x43, 0x00, 0x40, 0x42, 0x0d,
		0x40, 0x36, 0x4a, 0x43, 0x37, 0x4a, 0x3b, 0x41,
		0x43, 0x44, 0x38, 0x49, 0x3c, 0x40, 0x43, 0x46,
		0x44, 0x39, 0x48, 0x3c, 0x3f, 0x43, 0x46, 0x43,
		0x39, 0x48, 0xbd, 0x3e, 0x43, 0x46, 0x41, 0x3a,
		0x48, 0xc3, 0x47, 0x41, 0x3a, 0x46, 0x43, 0x41,
		0x3b, 0x45, 0x42, 0x41, 0x3c, 0x45, 0x40, 0x41,
		0x3d, 0x44, 0x41, 0x41, 0x3e, 0x44, 0x41, 0x41,
		0x3f, 0x43, 0x42, 0x00, 0x40, 0x41, 0x4a, 0x4e,
		0x03, 0x7b, 0x08, 0x0c, 0x33, 0x38, 0x3a, 0x3c,
		0x3e, 0x40, 0x42, 0x44, 0x46, 0x4b, 0x72, 0x76,
		0x49, 0x03, 0x7b, 0x08, 0x8c, 0x0d, 0x33, 0x38,
		0xba, 0x44, 0x46, 0x4b, 0xf1, 0x72, 0x76, 0x4e,
		0x03, 0x7b, 0x08, 0x0d, 0x33, 0x38, 0x3a, 0x3c,
		0x3e, 0x40, 0x42, 0x44, 0x46, 0x4b, 0x71, 0x76,
		0x49, 0x03, 0x7b, 0x08, 0x0d, 0x33, 0x38, 0xba,
		0x44, 0x46, 0x4b, 0x71, 0x76, 0x4f, 0x03, 0x7b,
		0x06, 0x08, 0x0d, 0x33, 0x38, 0x3a, 0x3c, 0x3e,
		0x40, 0x42, 0x44, 0x46, 0x4b, 0x71, 0x76, 0x4e,
		0x03, 0x7b, 0x06, 0x08, 0x0d, 0x33, 0x36, 0x38,
		0xba, 0x44, 0x46, 0x48, 0x4b, 0x6a, 0x71, 0x76,
		0x78, 0x52, 0x03, 0x7b, 0x06, 0x08, 0x0d, 0x2c,
		0x33, 0x36, 0x38, 0xba, 0x3c, 0x3e, 0x40, 0xc2,
		0x44, 0x46, 0x48, 0x4b, 0x6a, 0xf0, 0x71, 0x76,
		0x78, 0x4f, 0x03, 0x7b, 0x08, 0x8d, 0x0e, 0x2c,
		0x33, 0x36, 0x38, 0xbb, 0x43, 0x46, 0x48, 0x4b,
		0x51, 0x6a, 0x70, 0x76, 0x78, 0x4c, 0x03, 0x7b,
		0x08, 0x0e, 0x2c, 0x33, 0x38, 0xbc, 0x42, 0x46,
		0x4b, 0x51, 0x6a, 0x70, 0x76, 0x4a, 0x03, 0x7b,
		0x08, 0x0e, 0x2c, 0x33, 0x38, 0x46, 0x4b, 0x51,
		0x70, 0x76, 0x4b, 0x03, 0x7b, 0x08, 0x0e, 0x15,
		0x26, 0x33, 0x38, 0x46, 0x4b, 0x51, 0x70, 0x76,
		0x4a, 0x03, 0x7b, 0x08, 0x0e, 0x15, 0x26, 0x33,
		0x38, 0x46, 0x4b, 0x70, 0x76, 0x4b, 0x03, 0x7b,
		0x05, 0x88, 0x0e, 0x15, 0x26, 0x33, 0x35, 0xb8,
		0x46, 0x49, 0x4b, 0xf0, 0x76, 0x79, 0x4e, 0x03,
		0x7b, 0x05, 0x08, 0x0e, 0x15, 0x26, 0x33, 0x35,
		0x38, 0x46, 0x49, 0x4b, 0x70, 0x76, 0x79, 0x49,
		0x03, 0x7b, 0x05, 0x88, 0x0f, 0x33, 0x35, 0xb8,
		0x46, 0x49, 0x4b, 0xef, 0x76, 0x79, 0xc1, 0x03,
		0x0b, 0x08, 0xcc, 0x0d, 0x71, 0x0f, 0x33, 0x38,
		0x3b, 0x3d, 0x41, 0x43, 0x46, 0x4b, 0x60, 0x66,
		0x6f, 0x41, 0x73, 0x7b, 0x76, 0x80, 0x02, 0x0b,
		0xc7, 0x0d, 0x71, 0x8e, 0x0f, 0xb2, 0x3b, 0xbd,
		0x41, 0xc3, 0x4c, 0x60, 0x66, 0xef, 0x70, 0x00,
		0x73, 0x7c, 0xc0, 0x02, 0x09, 0xc6, 0x0f, 0x6f,
		0x32, 0x39, 0x45, 0x4c, 0x60, 0x66, 0x40, 0x75,
		0x7c, 0x80, 0x01, 0x0a, 0xc4, 0x0f, 0x6f, 0xb1,
		0x3a, 0xc4, 0x4d, 0x60, 0x66, 0x00, 0x74, 0x7d,
		0xc0, 0x01, 0x0a, 0xc4, 0x0f, 0x6f, 0x31, 0x3a,
		0x44, 0x4d, 0x40, 0x74, 0x7d, 0x80, 0x01, 0x0a,
		0xc2, 0x0f, 0x6f, 0xb1, 0x3a, 0xc4, 0x4d, 0x00,
		0x74, 0x7d, 0x44, 0x0f, 0x6f, 0x10, 0x22, 0x58,
		0x6e, 0x42, 0x10, 0x6e, 0x22, 0x58, 0x43, 0x10,
		0x6e, 0x14, 0x22, 0x58, 0x43, 0x10, 0x6e, 0x14,
		0x22, 0x58, 0x41, 0x10, 0x6e, 0x14, 0x41, 0x10,
		0x6e, 0x14, 0x41, 0x10, 0x6e, 0x3e, 0x42, 0x10,
		0x6e, 0x11, 0x3e, 0x43, 0x11, 0x6e, 0x3e, 0x68,
		0x6d, 0x42, 0x11, 0x6d, 0x3e, 0x68, 0x41, 0x11,
		0x6d, 0x68, 0x41, 0x11, 0x6d, 0x68, 0x41, 0x11,
		0x6d, 0x1b, 0x42, 0x11, 0x6d, 0x1b, 0x4c, 0x44,
		0x11, 0x6d, 0x12, 0x1b, 0x4c, 0x61, 0x44, 0x12,
		0x6d, 0x1b, 0x4c, 0x61, 0x6c, 0x42, 0x12, 0x6c,
		0x4c, 0x61, 0x42, 0x12, 0x6c, 0x2a, 0x61, 0x41,
		0x12, 0x6c, 0x2a, 0x42, 0x12, 0x6c, 0x2a, 0x32,
		0x42, 0x12, 0x6c, 0x2a, 0x32, 0x41, 0x12, 0x6c,
		0x32, 0x43, 0x12, 0x6c, 0x13, 0x32, 0x6b, 0x40,
		0x13, 0x6b, 0x40, 0x13, 0x6b, 0x40, 0x13, 0x6b,
		0x51, 0x13, 0x6b, 0x18, 0x1b, 0x22, 0x25, 0x2c,
		0x2f, 0x36, 0x39, 0x40, 0x43, 0x4a, 0x4d, 0x54,
		0x57, 0x5e, 0x61, 0x68, 0x51, 0x13, 0x6b, 0x18,
		0x1b, 0x22, 0x25, 0x2c, 0x2f, 0x36, 0x39, 0x40,
		0x43, 0x4a, 0x4d, 0x54, 0x57, 0x5e, 0x61, 0x68,
		0x51, 0x13, 0x6b, 0x97, 0x18, 0x9b, 0x1c, 0xa1,
		0x22, 0xa5, 0x26, 0xab, 0x2c, 0xaf, 0x30, 0xb5,
		0x36, 0xb9, 0x3a, 0xbf, 0x40, 0xc3, 0x44, 0xc9,
		0x4a, 0xcd, 0x4e, 0xd3, 0x54, 0xd7, 0x58, 0xdd,
		0x5e, 0xe1, 0x62, 0xe7, 0x68, 0x00, 0x11, 0x6d,
		0x40, 0x11, 0x6d, 0x40, 0x11, 0x6d, 0x13, 0x11,
		0x6d, 0x12, 0x96, 0x17, 0x9b, 0x1c, 0xa0, 0x21,
		0xa5, 0x26, 0xaa, 0x2b, 0xaf, 0x30, 0xb4, 0x35,
		0xb9, 0x3a, 0xbe, 0x3f, 0xc3, 0x44, 0xc8, 0x49,
		0xcd, 0x4e, 0xd2, 0x53, 0xd7, 0x58, 0xdc, 0x5d,
		0xe1, 0x62, 0xe6, 0x67, 0xeb, 0x6c, 0x80, 0x11,
		0x13, 0x80, 0x15, 0x18, 0x80, 0x1a, 0x1d, 0x80,
		0x1f, 0x22, 0x80, 0x24, 0x27, 0x80, 0x29, 0x2c,
		0x86, 0x2e, 0x4f, 0x32, 0x37, 0x3c, 0x41, 0x46,
		0x4b, 0x80, 0x51, 0x54, 0x80, 0x56, 0x59, 0x80,
		0x5b, 0x5e, 0x80, 0x60, 0x63, 0x80, 0x65, 0x68,
		0x00, 0x6a, 0x6d, 0x40, 0x2f, 0x4d, 0x40, 0x2f,
		0x4d, 0x4e, 0x2f, 0x4d, 0x31, 0x33, 0x35, 0x37,
		0x39, 0x3b, 0x3d, 0x3f, 0x41, 0x43, 0x45, 0x47,
		0x49, 0x4b, 0x08, 0x2f, 0x4d, 0x30, 0x34, 0x38,
		0x3c, 0x40, 0x44, 0x48, 0x4c, 0x08, 0x2e, 0x4e,
		0x30, 0x34, 0x38, 0x3c, 0x40, 0x44, 0x48, 0x4c,
		0x40, 0x2e, 0x4e, 0x00, 0x2e, 0x4e, 0x40, 0x2f,
		0x4d, 0x43, 0x2f, 0x4d, 0xb4, 0x36, 0xbd, 0x3f,
		0xc6, 0x48, 0x43, 0x2f, 0x4d, 0xb4, 0x36, 0xbd,
		0x3f, 0xc6, 0x48, 0x43, 0x2f, 0x4d, 0xb4, 0x36,
		0xbd, 0x3f, 0xc6, 0x48, 0x43, 0x2f, 0x4d, 0xb4,
		0x36, 0xbd, 0x3f, 0xc6, 0x48, 0x40, 0x2f, 0x4d,
		0x40, 0x2f, 0x4d, 0x00, 0x2d, 0x4f, 0x40, 0x2d,
		0x4f, 0x40, 0x2d, 0x4f, 0x07, 0x2d, 0x4f, 0xae,
		0x2f, 0xb3, 0x34, 0xb8, 0x39, 0xbd, 0x3e, 0xc2,
		0x43, 0xc7, 0x48, 0xcc, 0x4e, 0x80, 0x2d, 0x30,
		0x80, 0x32, 0x35, 0x80, 0x37, 0x3a, 0x80, 0x3c,
		0x3f, 0x80, 0x41, 0x44, 0x80, 0x46, 0x49, 0x00,
		0x4b, 0x4f, 0x34, 0x49, 0x15, 0x69, 0x18, 0x1b,
		0x37, 0x3a, 0xbc, 0x42, 0x44, 0x47, 0x63, 0x66,
		0x49, 0x15, 0x69, 0x18, 0x9b, 0x1c, 0x37, 0x3a,
		0xbc, 0x42, 0x44, 0x47, 0xe2, 0x63, 0x66, 0x49,
		0x15, 0x69, 0x18, 0x1c, 0x37, 0x3a, 0xbc, 0x42,
		0x44, 0x47, 0x62, 0x66, 0x49, 0x15, 0x69, 0x18,
		0x1c, 0x37, 0x3a, 0xbc, 0x42, 0x44, 0x47, 0x62,
		0x66, 0x4b, 0x15, 0x69, 0x18, 0x1c, 0x32, 0x37,
		0x3a, 0xbc, 0x42, 0x44, 0x47, 0x5d, 0xe1, 0x62,
		0x66, 0x4c, 0x15, 0x69, 0x18, 0x9c, 0x1d, 0x32,
		0x37, 0x3a, 0xbc, 0x42, 0x44, 0x47, 0x4c, 0x5d,
		0x61, 0x66, 0x4c, 0x15, 0x69, 0x18, 0x1d, 0x32,
		0x37, 0x3a, 0xbd, 0x41, 0x44, 0x47, 0x4c, 0x5d,
		0x61, 0x66, 0x4b, 0x15, 0x69, 0x18, 0x1d, 0x22,
		0x2e, 0x37, 0x3a, 0x44, 0x47, 0x4c, 0x61, 0x66,
		0x47, 0x15, 0x69, 0x98, 0x1d, 0x22, 0x2e, 0x37,
		0xba, 0x44, 0x47, 0xe1, 0x66, 0x4a, 0x15, 0x69,
		0x18, 0x1d, 0x22, 0x2e, 0x37, 0x3a, 0x44, 0x47,
		0x61, 0x66, 0x4c, 0x15, 0x69, 0x18, 0x9a, 0x1b,
		0x9d, 0x1e, 0x37, 0x3a, 0xbc, 0x3e, 0xc0, 0x42,
		0x44, 0x47, 0x61, 0xe3, 0x64, 0x66, 0x80, 0x14,
		0x1a, 0xc7, 0x1c, 0x62, 0x9d, 0x1e, 0xb6, 0x3c,
		0xbe, 0x40, 0xc2, 0x48, 0x56, 0x5a, 0x61, 0x00,
		0x64, 0x6a, 0xc0, 0x14, 0x19, 0xc6, 0x1e, 0x61,
		0x36, 0x3b, 0x44, 0x48, 0x56, 0x5a, 0x40, 0x65,
		0x6a, 0xc0, 0x13, 0x1a, 0xc6, 0x1e, 0x61, 0x35,
		0x3c, 0x43, 0x49, 0x56, 0x5a, 0x40, 0x64, 0x6b,
		0x80, 0x13, 0x1a, 0xc2, 0x1e, 0x61, 0xb5, 0x3c,
		0xc3, 0x49, 0x00, 0x64, 0x6b, 0x43, 0x1e, 0x61,
		0x2b, 0x51, 0x60, 0x43, 0x1e, 0x60, 0x21, 0x2b,
		0x51, 0x43, 0x1e, 0x60, 0x21, 0x2b, 0x51, 0x41,
		0x1e, 0x60, 0x21, 0x41, 0x1e, 0x60, 0x3e, 0x44,
		0x1e, 0x60, 0x1f, 0x3e, 0x5c, 0x5f, 0x42, 0x1f,
		0x5f, 0x3e, 0x5c, 0x41, 0x1f, 0x5f, 0x5c, 0x41,
		0x1f, 0x5f, 0x26, 0x42, 0x1f, 0x5f, 0x26, 0x48,
		0x44, 0x1f, 0x5f, 0x20, 0x26, 0x48, 0x57, 0x42,
		0x20, 0x5f, 0x48, 0x57, 0x42, 0x20, 0x5f, 0x30,
		0x57, 0x42, 0x20, 0x5f, 0x30, 0x36, 0x42, 0x20,
		0x5f, 0x30, 0x36, 0x42, 0x20, 0x5f, 0x36, 0x5e,
		0x40, 0x20, 0x5e, 0x40, 0x20, 0x5e, 0x51, 0x20,
		0x5e, 0x24, 0x26, 0x2b, 0x2d, 0x32, 0x34, 0x39,
		0x3b, 0x40, 0x42, 0x47, 0x49, 0x4e, 0x50, 0x55,
		0x57, 0x5c, 0x12, 0x20, 0x5e, 0xa1, 0x22, 0x25,
		0xa8, 0x29, 0x2c, 0xaf, 0x30, 0x33, 0xb6, 0x37,
		0x3a, 0xbd, 0x3e, 0x41, 0xc4, 0x45, 0x48, 0xcb,
		0x4c, 0x4f, 0xd2, 0x53, 0x56, 0xd9, 0x5a, 0x5d,
		0x00, 0x1f, 0x5f, 0x40, 0x1f, 0x5f, 0x12, 0x1f,
		0x5f, 0x23, 0x26, 0x2a, 0x2d, 0x31, 0x34, 0x38,
		0x3b, 0x3f, 0x42, 0x46, 0x49, 0x4d, 0x50, 0x54,
		0x57, 0x5b, 0x5e, 0x80, 0x1f, 0x20, 0x80, 0x22,
		0x27, 0x80, 0x29, 0x2e, 0x83, 0x30, 0x4a, 0x36,
		0x3d, 0x44, 0x80, 0x4c, 0x51, 0x80, 0x53, 0x58,
		0x00, 0x5a, 0x5f, 0x40, 0x34, 0x49, 0x07, 0x34,
		0x49, 0x35, 0x38, 0x3b, 0x3e, 0x41, 0x44, 0xc7,
		0x48, 0x07, 0x33, 0x4a, 0x35, 0x38, 0x3b, 0x3e,
		0x41, 0x44, 0xc7, 0x48, 0x40, 0x33, 0x4a, 0x00,
		0x33, 0x4a, 0x40, 0x34, 0x49, 0x43, 0x34, 0x49,
		0xb8, 0x39, 0xbe, 0x3f, 0xc4, 0x45, 0x43, 0x34,
		0x49, 0xb8, 0x39, 0xbe, 0x3f, 0xc4, 0x45, 0x40,
		0x34, 0x49, 0x00, 0x33, 0x4a, 0x40, 0x33, 0x4a,
		0x07, 0x33, 0x4a, 0x34, 0x37, 0x3b, 0x3e, 0x42,
		0x45, 0x49, 0x80, 0x33, 0x38, 0x80, 0x3a, 0x3f,
		0x80, 0x41, 0x46, 0x00, 0x48, 0x4a, 0x26, 0x49,
		0x21, 0x5d, 0x23, 0x25, 0x39, 0x3b, 0xbd, 0x41,
		0x43, 0x45, 0x59, 0x5b, 0x49, 0x21, 0x5d, 0x23,
		0x25, 0x39, 0x3b, 0xbd, 0x41, 0x43, 0x45, 0x59,
		0x5b, 0x49, 0x21, 0x5d, 0x23, 0x26, 0x39, 0x3b,
		0xbd, 0x41, 0x43, 0x45, 0x58, 0x5b, 0x4b, 0x21,
		0x5d, 0x23, 0x26, 0x36, 0x39, 0x3b, 0xbd, 0x41,
		0x43, 0x45, 0x54, 0x58, 0x5b, 0x4c, 0x21, 0x5d,
		0x23, 0x26, 0x36, 0x39, 0x3b, 0xbd, 0x41, 0x43,
		0x45, 0x48, 0x54, 0x58, 0x5b, 0x4b, 0x21, 0x5d,
		0x23, 0x27, 0x2a, 0x33, 0x39, 0x3b, 0x43, 0x45,
		0x48, 0x57, 0x5b, 0x47, 0x21, 0x5d, 0xa3, 0x27,
		0x2a, 0x33, 0x39, 0xbb, 0x43, 0x45, 0xd7, 0x5b,
		0x48, 0x21, 0x5d, 0x23, 0x27, 0x39, 0x3b, 0x43,
		0x45, 0x57, 0x5b, 0x45, 0x21, 0x5d, 0xa2, 0x27,
		0xb9, 0x45, 0x4f, 0x52, 0xd7, 0x5c, 0xc0, 0x20,
		0x24, 0xc6, 0x27, 0x57, 0x38, 0x3c, 0x42, 0x46,
		0x4f, 0x52, 0x40, 0x5a, 0x5e, 0x80, 0x20, 0x24,
		0xc2, 0x27, 0x57, 0xb8, 0x3c, 0xc2, 0x46, 0x00,
		0x5a, 0x5e, 0x43, 0x27, 0x57, 0x2a, 0x31, 0x4c,
		0x43, 0x27, 0x57, 0x2a, 0x31, 0x4c, 0x40, 0x27,
		0x57, 0x42, 0x27, 0x57, 0x3e, 0x54, 0x42, 0x28,
		0x56, 0x3e, 0x54, 0x41, 0x28, 0x56, 0x2d, 0x42,
		0x28, 0x56, 0x2d, 0x45, 0x42, 0x28, 0x56, 0x45,
		0x50, 0x42, 0x28, 0x56, 0x34, 0x50, 0x42, 0x28,
		0x56, 0x34, 0x39, 0x41, 0x28, 0x56, 0x39, 0x40,
		0x29, 0x55, 0x40, 0x29, 0x55, 0x51, 0x29, 0x55,
		0x2b, 0x2d, 0x30, 0x32, 0x35, 0x37, 0x3a, 0x3c,
		0x3f, 0x41, 0x44, 0x46, 0x49, 0x4b, 0x4e, 0x50,
		0x53, 0x00, 0x28, 0x56, 0x40, 0x28, 0x56, 0x00,
		0x28, 0x56, 0x40, 0x37, 0x46, 0x07, 0x36, 0x47,
		0x38, 0x3a, 0x3c, 0x3e, 0x40, 0x42, 0x44, 0x40,
		0x36, 0x47, 0x00, 0x36, 0x47, 0x43, 0x37, 0x46,
		0xba, 0x3b, 0xbe, 0x3f, 0xc2, 0x43, 0x43, 0x37,
		0x46, 0xba, 0x3b, 0xbe, 0x3f, 0xc2, 0x43, 0x40,
		0x37, 0x46, 0x00, 0x36, 0x47, 0x40, 0x36, 0x47,
		0x00, 0x36, 0x47, 0x21, 0x49, 0x24, 0x59, 0x26,
		0x28, 0x39, 0x3b, 0xbd, 0x40, 0x42, 0x44, 0x55,
		0x57, 0x49, 0x24, 0x59, 0x26, 0x28, 0x39, 0x3b,
		0xbd, 0x40, 0x42, 0x44, 0x55, 0x57, 0x49, 0x24,
		0x59, 0x26, 0x28, 0x39, 0x3b, 0xbd, 0x40, 0x42,
		0x44, 0x55, 0x57, 0x4b, 0x24, 0x59, 0x26, 0x28,
		0x37, 0x39, 0x3b, 0xbd, 0x40, 0x42, 0x44, 0x51,
		0x55, 0x57, 0x4b, 0x24, 0x59, 0x26, 0x28, 0x2c,
		0x34, 0x39, 0x3b, 0x42, 0x44, 0x46, 0x55, 0x57,
		0x45, 0x24, 0x59, 0xa6, 0x28, 0x39, 0xbb, 0x42,
		0x44, 0xd5, 0x57, 0x48, 0x24, 0x59, 0x26, 0x29,
		0x39, 0x3b, 0x42, 0x44, 0x54, 0x57, 0x45, 0x24,
		0x59, 0xa5, 0x29, 0xb9, 0x44, 0x4d, 0x4f, 0xd4,
		0x58, 0xc0, 0x23, 0x27, 0xc4, 0x29, 0x54, 0x38,
		0x3c, 0x41, 0x45, 0x40, 0x56, 0x5a, 0x80, 0x23,
		0x27, 0xc2, 0x29, 0x54, 0xb8, 0x3c, 0xc1, 0x45,
		0x00, 0x56, 0x5a, 0x43, 0x29, 0x54, 0x2c, 0x32,
		0x4a, 0x40, 0x29, 0x54, 0x42, 0x29, 0x54, 0x3e,
		0x51, 0x40, 0x2a, 0x53, 0x41, 0x2a, 0x53, 0x2f,
		0x41, 0x2a, 0x53, 0x44, 0x41, 0x2a, 0x53, 0x4d,
		0x41, 0x2a, 0x53, 0x35, 0x41, 0x2a, 0x53, 0x39,
		0x40, 0x2b, 0x52, 0x40, 0x2b, 0x52, 0x40, 0x2b,
		0x52, 0x00, 0x2a, 0x53, 0x40, 0x2a, 0x53, 0x00,
		0x2a, 0x53, 0x40, 0x38, 0x45, 0x40, 0x38, 0x45,
		0x04, 0x38, 0x45, 0x39, 0xbc, 0x3d, 0xc0, 0x41,
		0x44, 0x04, 0x38, 0x45, 0x39, 0xbc, 0x3d, 0xc0,
		0x41, 0x44, 0x40, 0x38, 0x45, 0x00, 0x38, 0x45,
		0x40, 0x38, 0x45, 0x00, 0x38, 0x45, 0x18, 0x47,
		0x2b, 0x53, 0x2d, 0x2f, 0x3b, 0xbd, 0x41, 0x43,
		0x4f, 0x51, 0x47, 0x2b, 0x53, 0x2d, 0x2f, 0x3b,
		0xbd, 0x41, 0x43, 0x4f, 0x51, 0x47, 0x2b, 0x53,
		0x2d, 0x2f, 0x3b, 0xbd, 0x41, 0x43, 0x4f, 0x51,
		0x48, 0x2b, 0x53, 0x2d, 0x2f, 0x3b, 0x3d, 0x41,
		0x43, 0x4f, 0x51, 0x45, 0x2b, 0x53, 0xad, 0x2f,
		0x3b, 0xbd, 0x41, 0x43, 0xcf, 0x51, 0x80, 0x2b,
		0x2d, 0xc2, 0x2f, 0x4f, 0xbb, 0x3c, 0xc2, 0x43,
		0x00, 0x51, 0x53, 0xc0, 0x2a, 0x2d, 0xc4, 0x2f,
		0x4f, 0x3a, 0x3d, 0x41, 0x44, 0x40, 0x51, 0x54,
		0x80, 0x2a, 0x2d, 0xc2, 0x2f, 0x4f, 0xba, 0x3d,
		0xc1, 0x44, 0x00, 0x51, 0x54, 0x40, 0x2f, 0x4f,
		0x40, 0x2f, 0x4f, 0x40, 0x30, 0x4e, 0x40, 0x30,
		0x4e, 0x40, 0x30, 0x4e, 0x40, 0x30, 0x4e, 0x40,
		0x30, 0x4e, 0x00, 0x30, 0x4e, 0x40, 0x30, 0x4e,
		0x00, 0x30, 0x4e, 0x40, 0x3a, 0x44, 0x43, 0x3a,
		0x44, 0x3c, 0x3f, 0x42, 0x40, 0x3a, 0x44, 0x00,
		0x3a, 0x44, 0x40, 0x3a, 0x44, 0x00, 0x3a, 0x44,
		0x12, 0x45, 0x30, 0x4e, 0x32, 0x3c, 0xbe, 0x40,
		0x42, 0x4c, 0x45, 0x30, 0x4e, 0x32, 0x3c, 0xbe,
		0x40, 0x42, 0x4c, 0x45, 0x30, 0x4e, 0x32, 0x3c,
		0xbe, 0x40, 0x42, 0x4c, 0x45, 0x30, 0x4e, 0x32,
		0x3c, 0xbe, 0x40, 0x42, 0x4c, 0x46, 0x30, 0x4e,
		0x32, 0x3c, 0x3e, 0x40, 0x42, 0x4c, 0x44, 0x30,
		0x4e, 0xb1, 0x33, 0xbc, 0x3e, 0xc0, 0x42, 0xcb,
		0x4d, 0x40, 0x33, 0x4b, 0x40, 0x33, 0x4b, 0x40,
		0x33, 0x4b, 0x40, 0x33, 0x4b, 0x40, 0x33, 0x4b,
		0x00, 0x33, 0x4b, 0x40, 0x33, 0x4b, 0x00, 0x33,
		0x4b, 0x40, 0x3b, 0x43, 0x40, 0x3b, 0x43, 0x40,
		0x3b, 0x43, 0x00, 0x3b, 0x43, 0x0f, 0x45, 0x34,
		0x4b, 0x36, 0x3d, 0xbf, 0x40, 0x42, 0x49, 0x45,
		0x34, 0x4b, 0x36, 0x3d, 0xbf, 0x40, 0x42, 0x49,
		0x45, 0x34, 0x4b, 0x36, 0x3d, 0xbf, 0x40, 0x42,
		0x49, 0x45, 0x34, 0x4b, 0x36, 0x3d, 0xbf, 0x40,
		0x42, 0x49, 0x44, 0x34, 0x4b, 0xb5, 0x36, 0xbd,
		0x3e, 0xc1, 0x42, 0xc9, 0x4a, 0x40, 0x36, 0x49,
		0x40, 0x36, 0x49, 0x40, 0x36, 0x49, 0x40, 0x36,
		0x49, 0x40, 0x36, 0x49, 0x00, 0x36, 0x49, 0x40,
		0x3c, 0x43, 0x40, 0x3c, 0x43, 0x40, 0x3c, 0x43,
		0x00, 0x3c, 0x43, 0x0c, 0x41, 0x37, 0x48, 0xbf,
		0x40, 0x41, 0x37, 0x48, 0xbf, 0x40, 0x41, 0x37,
		0x48, 0xbf, 0x40, 0x42, 0x37, 0x48, 0x38, 0x47,
		0x40, 0x38, 0x47, 0x40, 0x38, 0x47, 0x40, 0x38,
		0x47, 0x40, 0x38, 0x47, 0x00, 0x38, 0x47, 0x40,
		0x3d, 0x43, 0x40, 0x3d, 0x43, 0x00, 0x3d, 0x43,
		0x29, 0x45, 0x09, 0x76, 0x8a, 0x14, 0x96, 0x1a,
		0xb8, 0x3d, 0xdd, 0x60, 0xec, 0x75, 0x0a, 0x09,
		0x76, 0x0a, 0x11, 0x16, 0x9a, 0x2c, 0xb1, 0x38,
		0x3d, 0xd6, 0x5d, 0xe0, 0x66, 0x70, 0x75, 0x0c,
		0x0a, 0x75, 0x0b, 0x10, 0x17, 0x19, 0x2c, 0x31,
		0xb7, 0x39, 0x3c, 0x5d, 0x60, 0x71, 0x74, 0x0e,
		0x0b, 0x75, 0x0c, 0x0f, 0x17, 0x19, 0x2d, 0x30,
		0x39, 0x3c, 0x4e, 0x53, 0x5d, 0x60, 0x71, 0x74,
		0x0d, 0x09, 0x7a, 0x0c, 0x0f, 0x17, 0x19, 0x2d,
		0x30, 0x39, 0x3c, 0x4f, 0x52, 0xdc, 0x64, 0x71,
		0x74, 0x4a, 0x05, 0x7d, 0x86, 0x08, 0x93, 0x16,
		0x18, 0x9a, 0x2b, 0xc8, 0x4e, 0xd0, 0x51, 0xd3,
		0x54, 0xdc, 0x63, 0xe5, 0x69, 0xfb, 0x7c, 0x08,
		0x03, 0x7e, 0x85, 0x08, 0x93, 0x1b, 0x2b, 0x48,
		0xce, 0x52, 0x54, 0xe5, 0x6a, 0xfb, 0x7d, 0x08,
		0x02, 0x7f, 0x83, 0x04, 0x8d, 0x14, 0x9c, 0x24,
		0xa8, 0x2a, 0xc9, 0x4d, 0x53, 0x6b, 0x7e, 0x09,
		0x02, 0x7f, 0x03, 0x0c, 0x25, 0x27, 0x2a, 0x49,
		0x52, 0x6c, 0x7e, 0x08, 0x01, 0x7f, 0x02, 0x0c,
		0x26, 0x2a, 0x49, 0x52, 0x6d, 0x7e, 0x08, 0x00,
		0x7f, 0x01, 0x0b, 0x26, 0x2a, 0x49, 0x51, 0x6d,
		0x7e, 0x08, 0x00, 0x7f, 0x01, 0x0a, 0x26, 0x2a,
		0x49, 0x51, 0x6d, 0x7e, 0x08, 0x00, 0x7f, 0x01,
		0x0a, 0x26, 0x2a, 0x49, 0x51, 0x6d, 0x7e, 0x08,
		0x00, 0x7f, 0x01, 0x0a, 0x26, 0x2a, 0x49, 0x50,
		0x6e, 0x7e, 0x09, 0x00, 0x7f, 0x01, 0x0a, 0x26,
		0x2a, 0x48, 0x50, 0xe9, 0x6b, 0x6e, 0x7e, 0x08,
		0x00, 0x7f, 0x01, 0x0a, 0x26, 0x2a, 0x48, 0x50,
		0xec, 0x6d, 0x7e, 0x08, 0x00, 0x7f, 0x01, 0x0a,
		0x25, 0x2a, 0x47, 0x50, 0x6e, 0x7e, 0x08, 0x00,
		0x7f, 0x01, 0x0a, 0x25, 0x2b, 0x47, 0x50, 0x6e,
		0x7e, 0x08, 0x00, 0x7f, 0x01, 0x0b, 0x25, 0x2b,
		0x47, 0x50, 0x6f, 0x7e, 0x08, 0x00, 0x7f, 0x01,
		0x0b, 0x25, 0x2c, 0x47, 0x50, 0x70, 0x7d, 0x08,
		0x00, 0x7e, 0x01, 0x0b, 0x25, 0x2c, 0x47, 0x50,
		0x70, 0x7d, 0x08, 0x00, 0x7e, 0x01, 0x0b, 0x25,
		0x2c, 0x47, 0x50, 0x70, 0x7d, 0x08, 0x01, 0x7e,
		0x02, 0x0c, 0x25, 0x2c, 0x46, 0x51, 0xee, 0x6f,
		0x7d, 0x08, 0x01, 0x7e, 0x02, 0x0d, 0x24, 0x2c,
		0x46, 0x51, 0x6d, 0x7d, 0x08, 0x02, 0x7d, 0x03,
		0x0e, 0x24, 0x2c, 0x46, 0x51, 0x6c, 0x7c, 0x08,
		0x02, 0x7d, 0x03, 0x0e, 0x24, 0x2d, 0x46, 0x51,
		0x6c, 0x7c, 0x08, 0x02, 0x7d, 0x03, 0x0f, 0x23,
		0x2e, 0x46, 0x51, 0x6b, 0x7c, 0x08, 0x03, 0x7c,
		0x84, 0x06, 0x0f, 0x23, 0x2f, 0x46, 0x52, 0x6a,
		0x7b, 0x08, 0x04, 0x7b, 0x07, 0x0f, 0x22, 0x2f,
		0x45, 0x52, 0x69, 0x7a, 0x08, 0x07, 0x7a, 0x08,
		0x10, 0x22, 0x30, 0x44, 0xd3, 0x54, 0xe9, 0x6a,
		0x79, 0x08, 0x08, 0x7a, 0x09, 0x11, 0xa1, 0x23,
		0x31, 0xd5, 0x56, 0x68, 0xeb, 0x6c, 0x79, 0x0a,
		0x08, 0x79, 0x09, 0x91, 0x14, 0x9f, 0x20, 0x24,
		0x32, 0x43, 0x57, 0x67, 0x6d, 0x78, 0x0b, 0x09,
		0x78, 0x8a, 0x0b, 0x8e, 0x10, 0x95, 0x16, 0x1e,
		0x25, 0x33, 0x43, 0x57, 0x66, 0xee, 0x6f, 0xf6,
		0x77, 0x81, 0x0a, 0x10, 0x8c, 0x0d, 0x08, 0x15,
		0x77, 0x97, 0x18, 0x9c, 0x1d, 0x25, 0xb4, 0x37,
		0xc1, 0x43, 0xd8, 0x5d, 0xe2, 0x65, 0xf0, 0x75,
		0x80, 0x0c, 0x0d, 0x06, 0x17, 0x75, 0x99, 0x1b,
		0x26, 0xb8, 0x40, 0xc4, 0x46, 0xde, 0x61, 0x73,
		0x84, 0x19, 0x40, 0x1a, 0xa4, 0x25, 0xa7, 0x28,
		0x3b, 0x03, 0x44, 0x74, 0xc7, 0x49, 0x60, 0xf1,
		0x73, 0x84, 0x19, 0x3b, 0x1a, 0xa2, 0x23, 0xa9,
		0x2a, 0xb9, 0x3a, 0x03, 0x47, 0x73, 0x4a, 0xde,
		0x5f, 0xef, 0x71, 0x82, 0x1a, 0x23, 0x9b, 0x1c,
		0x21, 0x82, 0x29, 0x3a, 0x2b, 0xb7, 0x38, 0x04,
		0x4a, 0x71, 0x4b, 0xd9, 0x5d, 0x60, 0xed, 0x6e,
		0xc1, 0x1b, 0x21, 0x1c, 0x82, 0x2b, 0x38, 0x2c,
		0xb4, 0x36, 0x82, 0x4b, 0x5d, 0x4c, 0x58, 0x41,
		0x60, 0x6e, 0x6d, 0x80, 0x1d, 0x20, 0xc1, 0x2c,
		0x36, 0xb4, 0x35, 0xc0, 0x4c, 0x58, 0x00, 0x61,
		0x6b, 0x80, 0x2d, 0x33, 0x00, 0x4d, 0x57, 0x1d,
		0x43, 0x21, 0x56, 0xa3, 0x25, 0xbb, 0x3d, 0xd4,
		0x55, 0x49, 0x19, 0x66, 0x9b, 0x1e, 0xa0, 0x21,
		0x24, 0x34, 0x3c, 0xbe, 0x4e, 0x55, 0xdb, 0x60,
		0xe2, 0x64, 0x0e, 0x1a, 0x65, 0x1b, 0x1e, 0x23,
		0x25, 0x33, 0x35, 0x3b, 0x3d, 0x49, 0x4d, 0x54,
		0x56, 0x62, 0x64, 0x0d, 0x17, 0x69, 0x1b, 0x1e,
		0x23, 0x25, 0x33, 0x35, 0x3b, 0x3d, 0x4a, 0x4c,
		0xd3, 0x58, 0x62, 0x64, 0x45, 0x15, 0x6a, 0x16,
		0xa7, 0x30, 0xc6, 0x49, 0x4d, 0xd4, 0x58, 0x08,
		0x14, 0x6b, 0x95, 0x16, 0x1c, 0xa7, 0x2c, 0xaf,
		0x30, 0xc6, 0x49, 0x4d, 0x5e, 0x6a, 0x08, 0x13,
		0x6b, 0x94, 0x15, 0x1b, 0xad, 0x2e, 0x30, 0x46,
		0x4c, 0x5f, 0x6a, 0x08, 0x13, 0x6c, 0x14, 0x1b,
		0x2e, 0x30, 0x46, 0x4c, 0x5f, 0x6b, 0x08, 0x13,
		0x6c, 0x14, 0x1a, 0x2e, 0x30, 0x46, 0x4c, 0x5f,
		0x6b, 0x08, 0x13, 0x6c, 0x14, 0x1a, 0x2e, 0x30,
		0x46, 0x4b, 0x60, 0x6b, 0x09, 0x13, 0x6c, 0x14,
		0x1a, 0x2e, 0x30, 0x45, 0x4b, 0xdd, 0x5e, 0x60,
		0x6b, 0x08, 0x13, 0x6c, 0x14, 0x1a, 0x2d, 0x30,
		0x45, 0x4b, 0xdf, 0x60, 0x6b, 0x08, 0x13, 0x6c,
		0x14, 0x1a, 0x2d, 0x31, 0x45, 0x4b, 0x60, 0x6b,
		0x08, 0x13, 0x6c, 0x14, 0x1b, 0x2d, 0x31, 0x45,
		0x4b, 0x61, 0x6b, 0x08, 0x13, 0x6c, 0x14, 0x1b,
		0x2d, 0x32, 0x45, 0x4b, 0x61, 0x6b, 0x08, 0x13,
		0x6c, 0x14, 0x1b, 0x2d, 0x32, 0x44, 0x4b, 0x60,
		0x6b, 0x08, 0x13, 0x6c, 0x14, 0x1c, 0x2c, 0x32,
		0x44, 0x4c, 0x5f, 0x6b, 0x08, 0x14, 0x6b, 0x15,
		0x1d, 0x2c, 0x32, 0x44, 0x4c, 0x5f, 0x6a, 0x08,
		0x14, 0x6b, 0x15, 0x1d, 0x2c, 0x33, 0x44, 0x4c,
		0x5e, 0x6a, 0x08, 0x15, 0x6a, 0x96, 0x17, 0x1e,
		0x2c, 0x34, 0x44, 0x4c, 0x5d, 0x69, 0x08, 0x16,
		0x69, 0x18, 0x1e, 0x2b, 0x34, 0x43, 0xcd, 0x4e,
		0x5d, 0x68, 0x08, 0x18, 0x68, 0x19, 0x1f, 0xaa,
		0x2b, 0x35, 0x42, 0x4f, 0x5c, 0x5e, 0x0a, 0x19,
		0x68, 0x1a, 0x9d, 0x22, 0xa8, 0x29, 0x2c, 0x36,
		0x42, 0x50, 0x5b, 0xdf, 0x60, 0xe6, 0x67, 0x09,
		0x1a, 0x67, 0x9b, 0x1c, 0xa3, 0x24, 0x27, 0x2d,
		0xb7, 0x39, 0xc1, 0x42, 0xd1, 0x54, 0xd8, 0x5a,
		0xe1, 0x65, 0x80, 0x1b, 0x1c, 0x06, 0x23, 0x65,
		0xa5, 0x26, 0x2e, 0xba, 0x40, 0xc3, 0x44, 0xd5,
		0x57, 0x64, 0x83, 0x25, 0x40, 0x26, 0xab, 0x30,
		0xbb, 0x3c, 0x03, 0x43, 0x64, 0xc5, 0x47, 0x56,
		0xe1, 0x63, 0x83, 0x26, 0x3c, 0xa7, 0x2a, 0x31,
		0x3a, 0x03, 0x45, 0x63, 0x48, 0xd1, 0x56, 0x60,
		0x80, 0x27, 0x2a, 0xc0, 0x31, 0x3a, 0x41, 0x48,
		0x60, 0xd1, 0x56, 0x80, 0x32, 0x39, 0x80, 0x49,
		0x50, 0x00, 0x57, 0x5f, 0x15, 0x41, 0x2c, 0x50,
		0x3d, 0x49, 0x24, 0x5b, 0x26, 0xa8, 0x2a, 0x2c,
		0x37, 0x3d, 0xbf, 0x44, 0x50, 0xd2, 0x57, 0x59,
		0x0d, 0x22, 0x5c, 0x25, 0x27, 0x2b, 0x2d, 0x36,
		0x38, 0x3c, 0x3e, 0x47, 0x49, 0xce, 0x51, 0x58,
		0x5a, 0x44, 0x21, 0x5d, 0xae, 0x34, 0xc4, 0x46,
		0x48, 0xce, 0x51, 0x06, 0x20, 0x5e, 0x21, 0x26,
		0xae, 0x34, 0xc4, 0x49, 0x55, 0x5d, 0x07, 0x1f,
		0x5f, 0x20, 0x25, 0x34, 0x44, 0x48, 0x56, 0x5e,
		0x07, 0x1f, 0x5f, 0x20, 0x25, 0x34, 0x44, 0x48,
		0x57, 0x5e, 0x07, 0x1f, 0x5f, 0x20, 0x25, 0x34,
		0x43, 0x48, 0x57, 0x5e, 0x07, 0x1f, 0x5f, 0x20,
		0x25, 0x34, 0x43, 0x48, 0x57, 0x5e, 0x08, 0x1f,
		0x5f, 0x20, 0x25, 0x33, 0x35, 0x43, 0x48, 0x57,
		0x5e, 0x08, 0x1f, 0x5f, 0x20, 0x25, 0x32, 0x36,
		0x43, 0x48, 0x57, 0x5e, 0x08, 0x1f, 0x5f, 0x20,
		0x26, 0x32, 0x36, 0x43, 0x48, 0x57, 0x5e, 0x08,
		0x20, 0x5f, 0x21, 0x27, 0x31, 0x36, 0x43, 0x48,
		0x56, 0x5e, 0x08, 0x20, 0x5f, 0x21, 0x27, 0x31,
		0x36, 0x43, 0x49, 0x55, 0x5e, 0x08, 0x21, 0x5e,
		0xa2, 0x23, 0x27, 0x31, 0x37, 0x42, 0x49, 0x54,
		0x5d, 0x08, 0x23, 0x5d, 0x24, 0x28, 0xb0, 0x31,
		0x38, 0x41, 0x4a, 0xd4, 0x55, 0x5c, 0x08, 0x24,
		0x5c, 0xa5, 0x2c, 0xae, 0x2f, 0x32, 0xb9, 0x3a,
		0x41, 0xcb, 0x4e, 0xd1, 0x53, 0xd6, 0x5b, 0x06,
		0x25, 0x5b, 0xac, 0x2d, 0x33, 0xbb, 0x40, 0x42,
		0xcf, 0x50, 0x58, 0x83, 0x2c, 0x40, 0x2d, 0xb2,
		0x34, 0xbc, 0x3d, 0x03, 0x42, 0x58, 0xc3, 0x44,
		0x4e, 0x57, 0xc2, 0x2d, 0x3d, 0xb2, 0x34, 0x3c,
		0x42, 0x43, 0x57, 0x44, 0x4e, 0x80, 0x2e, 0x31,
		0x80, 0x35, 0x3b, 0x00, 0x45, 0x56, 0x13, 0x43,
		0x2d, 0x4d, 0x2e, 0xbc, 0x3d, 0x4c, 0x45, 0x27,
		0x55, 0x28, 0xad, 0x2e, 0xbc, 0x3d, 0xcc, 0x4d,
		0x54, 0x00, 0x25, 0x58, 0x43, 0x24, 0x59, 0xb0,
		0x35, 0x43, 0x47, 0x07, 0x23, 0x5a, 0x24, 0x28,
		0xb0, 0x35, 0x43, 0x47, 0xd2, 0x53, 0x59, 0x07,
		0x22, 0x5b, 0x23, 0x27, 0x35, 0x43, 0x46, 0x54,
		0x5a, 0x07, 0x22, 0x5b, 0x23, 0x27, 0x35, 0x42,
		0x46, 0x54, 0x5a, 0x07, 0x22, 0x5b, 0x23, 0x27,
		0x35, 0x42, 0x46, 0x54, 0x5a, 0x08, 0x22, 0x5b,
		0x23, 0x28, 0x34, 0x36, 0x42, 0x46, 0x54, 0x5a,
		0x08, 0x22, 0x5b, 0x23, 0x28, 0x33, 0x37, 0x42,
		0x46, 0x54, 0x5a, 0x08, 0x23, 0x5b, 0x24, 0x28,
		0x33, 0x37, 0x42, 0x46, 0x54, 0x5a, 0x08, 0x23,
		0x5b, 0x24, 0x29, 0x32, 0x37, 0x42, 0x47, 0xd2,
		0x53, 0x5a, 0x08, 0x24, 0x5a, 0xa5, 0x26, 0x29,
		0x32, 0x37, 0x41, 0x47, 0x51, 0x59, 0x07, 0x26,
		0x59, 0xa7, 0x2a, 0xb1, 0x32, 0x38, 0x40, 0x48,
		0xd1, 0x52, 0x58, 0x07, 0x27, 0x58, 0xab, 0x30,
		0x33, 0xb9, 0x3a, 0x40, 0xc9, 0x4c, 0xce, 0x50,
		0xd3, 0x57, 0x07, 0x2b, 0x57, 0x2e, 0x34, 0x3b,
		0xbe, 0x3f, 0x41, 0x4d, 0x54, 0x83, 0x2e, 0x3f,
		0x2f, 0xb3, 0x35, 0xbc, 0x3d, 0x02, 0x41, 0x54,
		0xc2, 0x43, 0x4c, 0xc2, 0x2f, 0x3d, 0xb3, 0x35,
		0x3c, 0x42, 0x42, 0x54, 0x43, 0x4c, 0x80, 0x30,
		0x32, 0x80, 0x36, 0x3b, 0x00, 0x44, 0x53, 0x0f,
		0x42, 0x32, 0x4a, 0xbd, 0x3e, 0x49, 0x45, 0x2d,
		0x50, 0x2e, 0x32, 0xbd, 0x3e, 0xc9, 0x4a, 0x4f,
		0x06, 0x2d, 0x53, 0x2f, 0x39, 0x41, 0x46, 0x4d,
		0x52, 0x07, 0x2b, 0x54, 0x2c, 0x2e, 0x38, 0x42,
		0x45, 0x4e, 0x53, 0x07, 0x2a, 0x54, 0x2b, 0x2d,
		0x38, 0x42, 0x44, 0x4f, 0x53, 0x07, 0x2a, 0x54,
		0x2b, 0x2d, 0x38, 0x42, 0x44, 0x4f, 0x53, 0x08,
		0x2a, 0x54, 0x2b, 0x2e, 0x37, 0x39, 0x42, 0x44,
		0x4f, 0x53, 0x08, 0x2a, 0x54, 0x2b, 0x2e, 0x36,
		0x39, 0x42, 0x44, 0x4f, 0x53, 0x08, 0x2b, 0x54,
		0x2c, 0xae, 0x2f, 0x36, 0x39, 0x42, 0x45, 0x4e,
		0x53, 0x08, 0x2c, 0x53, 0x2d, 0x2f, 0x36, 0x39,
		0x41, 0x45, 0x4d, 0x52, 0x06, 0x2d, 0x52, 0xae,
		0x30, 0xb5, 0x36, 0x3a, 0x40, 0x46, 0xcd, 0x51,
		0x05, 0x2e, 0x51, 0xb0, 0x34, 0x37, 0xbb, 0x3c,
		0xbf, 0x41, 0xc7, 0x4c, 0x04, 0x30, 0x4c, 0x38,
		0xbd, 0x3e, 0x42, 0x49, 0xc1, 0x38, 0x3e, 0x3d,
		0x40, 0x42, 0x49, 0x80, 0x39, 0x3c, 0x00, 0x43,
		0x48, 0x0c, 0x42, 0x35, 0x4a, 0x3d, 0x47, 0x02,
		0x32, 0x4e, 0x40, 0x4d, 0x06, 0x31, 0x4f, 0x32,
		0x3a, 0x41, 0x44, 0x4a, 0x4e, 0x06, 0x30, 0x4f,
		0x31, 0x39, 0x41, 0x43, 0x4b, 0x4e, 0x06, 0x30,
		0x4f, 0x31, 0x39, 0x41, 0x43, 0x4b, 0x4e, 0x06,
		0x31, 0x4f, 0x32, 0x39, 0x41, 0x43, 0x4b, 0x4e,
		0x07, 0x31, 0x4f, 0x32, 0x38, 0x3a, 0x41, 0x43,
		0x4a, 0x4e, 0x07, 0x32, 0x4e, 0x33, 0x38, 0x3a,
		0x40, 0x44, 0x49, 0x4d, 0x44, 0x33, 0x4d, 0x38,
		0x3a, 0xbd, 0x3e, 0xc1, 0x44, 0x04, 0x33, 0x4c,
		0x39, 0xbd, 0x3e, 0x41, 0x46, 0xc1, 0x39, 0x3e,
		0x3d, 0x40, 0x41, 0x46, 0x80, 0x3a, 0x3c, 0x00,
		0x42, 0x45, 0x0a, 0x41, 0x37, 0x47, 0x3f, 0x00,
		0x34, 0x4b, 0x04, 0x33, 0x4b, 0x34, 0x3b, 0x43,
		0x4a, 0x04, 0x33, 0x4b, 0x34, 0x3b, 0x43, 0x4a,
		0x04, 0x34, 0x4a, 0x35, 0x3b, 0x43, 0x49, 0x06,
		0x34, 0x4a, 0x35, 0x3a, 0x3c, 0xc1, 0x42, 0x44,
		0x49, 0x43, 0x35, 0x49, 0xba, 0x3c, 0xbe, 0x3f,
		0xc1, 0x44, 0x04, 0x36, 0x48, 0x3b, 0xbe, 0x3f,
		0x41, 0x45, 0xc1, 0x3b, 0x3f, 0x3e, 0x40, 0x41,
		0x45, 0x80, 0x3c, 0x3d, 0x00, 0x42, 0x44, 0x08,
		0x41, 0x39, 0x46, 0x40, 0x00, 0x37, 0x48, 0x04,
		0x36, 0x49, 0x37, 0x3c, 0x43, 0x48, 0x04, 0x36,
		0x48, 0x37, 0x3c, 0x43, 0x47, 0x06, 0x36, 0x48,
		0x37, 0x3b, 0x3d, 0x42, 0x44, 0x47, 0x43, 0x37,
		0x47, 0xbc, 0x3d, 0xbf, 0x40, 0xc2, 0x44, 0x44,
		0x38, 0x46, 0xb9, 0x3b, 0x3e, 0x41, 0x45, 0x00,
		0x3c, 0x44, 0x17, 0x47, 0x1a, 0x60, 0x9e, 0x1f,
		0xa5, 0x28, 0xb0, 0x33, 0xbc, 0x3f, 0xc7, 0x4a,
		0xd2, 0x55, 0xdb, 0x5c, 0x47, 0x1a, 0x60, 0x9e,
		0x20, 0xa5, 0x28, 0xb0, 0x33, 0xbc, 0x3f, 0xc7,
		0x4a, 0xd2, 0x55, 0xdb, 0x5c, 0x47, 0x1a, 0x60,
		0x9e, 0x20, 0xa5, 0x28, 0xaf, 0x33, 0xbc, 0x3f,
		0xc7, 0x4a, 0xd2, 0x55, 0xdb, 0x5c, 0x47, 0x1a,
		0x60, 0x9e, 0x20, 0xa4, 0x29, 0xaf, 0x33, 0xbc,
		0x40, 0xc7, 0x4a, 0xd2, 0x56, 0xdb, 0x5d, 0x47,
		0x1a, 0x60, 0x9e, 0x20, 0xa5, 0x29, 0xb0, 0x34,
		0xbb, 0x3f, 0xc6, 0x4a, 0xd2, 0x55, 0xdb, 0x5c,
		0x47, 0x1a, 0x60, 0x9e, 0x1f, 0xa5, 0x29, 0xb0,
		0x33, 0xbb, 0x3f, 0xc6, 0x4b, 0xd1, 0x55, 0xdb,
		0x5d, 0x47, 0x1a, 0x60, 0x9d, 0x1f, 0xa5, 0x28,
		0xb0, 0x33, 0xbb, 0x3f, 0xc7, 0x4a, 0xd1, 0x55,
		0xdb, 0x5c, 0x47, 0x1a, 0x60, 0x9d, 0x1f, 0xa5,
		0x28, 0xb0, 0x33, 0xbb, 0x3f, 0xc7, 0x4a, 0xd1,
		0x55, 0xdb, 0x5c, 0x47, 0x1a, 0x60, 0x9d, 0x1f,
		0xa5, 0x28, 0xb0, 0x33, 0xba, 0x40, 0xc7, 0x4a,
		0xd1, 0x56, 0xdb, 0x5c, 0x47, 0x1a, 0x60, 0x9d,
		0x1f, 0xa5, 0x28, 0xb0, 0x33, 0xbb, 0x40, 0xc7,
		0x4a, 0xd2, 0x56, 0xdb, 0x5c, 0x47, 0x1a, 0x60,
		0x9d, 0x1f, 0xa5, 0x28, 0xb0, 0x34, 0xbb, 0x40,
		0xc7, 0x4b, 0xd2, 0x56, 0xdb, 0x5c, 0x47, 0x1a,
		0x60, 0x9e, 0x1f, 0xa5, 0x28, 0xb0, 0x34, 0xbb,
		0x3f, 0xc7, 0x4a, 0xd2, 0x56, 0xdb, 0x5c, 0x47,
		0x1a, 0x60, 0x9e, 0x1f, 0xa4, 0x28, 0xb0, 0x34,
		0xbb, 0x3f, 0xc7, 0x4b, 0xd2, 0x56, 0xdb, 0x5c,
		0x47, 0x1a, 0x60, 0x9e, 0x1f, 0xa5, 0x28, 0xaf,
		0x33, 0xbb, 0x3f, 0xc6, 0x4a, 0xd2, 0x55, 0xdb,
		0x5d, 0x47, 0x1a, 0x60, 0x9e, 0x1f, 0xa5, 0x28,
		0xb0, 0x33, 0xbb, 0x3f, 0xc7, 0x4a, 0xd2, 0x55,
		0xdb, 0x5c, 0x47, 0x1a, 0x60, 0x9e, 0x1f, 0xa5,
		0x28, 0xb0, 0x33, 0xbb, 0x3f, 0xc7, 0x4a, 0xd2,
		0x55, 0xdb, 0x5c, 0x08, 0x1a, 0x60, 0x9b, 0x1c,
		0xa1, 0x23, 0xaa, 0x2e, 0xb5, 0x39, 0xc1, 0x45,
		0xcc, 0x50, 0xd7, 0x59, 0xde, 0x5f, 0x00, 0x19,
		0x61, 0x4c, 0x19, 0x61, 0x9b, 0x1d, 0xa1, 0x23,
		0xab, 0x2d, 0x30, 0x33, 0xb6, 0x38, 0x3c, 0xc2,
		0x44, 0xcd, 0x4f, 0x54, 0xd7, 0x59, 0xdd, 0x5f,
		0x48, 0x19, 0x61, 0x1c, 0x22, 0x2c, 0x37, 0x43,
		0x4e, 0x58, 0x5e, 0x48, 0x19, 0x61, 0x1c, 0x22,
		0x2c, 0x37, 0x43, 0x4e, 0x58, 0x5e, 0x4c, 0x19,
		0x61, 0x9b, 0x1d, 0xa1, 0x23, 0x26, 0xab, 0x2d,
		0xb6, 0x38, 0xbd, 0x3e, 0xc2, 0x44, 0x47, 0xcd,
		0x4f, 0x52, 0xd7, 0x59, 0xdd, 0x5f, 0x00, 0x19,
		0x61, 0x10, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbd, 0x3f, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbd, 0x3f, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x30, 0xb5, 0x37, 0xbd, 0x3f, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x30, 0xb5, 0x37, 0xbc, 0x3f, 0xc4, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa7, 0x29, 0xad,
		0x30, 0xb5, 0x37, 0xbc, 0x3f, 0xc4, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa7, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbc, 0x3f, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa7, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbc, 0x40, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa7, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbc, 0x40, 0xc5, 0x48, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbc, 0x3f, 0xc5, 0x48, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbc, 0x3f, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x47, 0x25, 0x56, 0xa8, 0x29, 0xad,
		0x2f, 0xb5, 0x37, 0xbc, 0x3f, 0xc5, 0x47, 0xcc,
		0x4f, 0x53, 0x08, 0x25, 0x56, 0x26, 0xaa, 0x2b,
		0xb1, 0x33, 0xb8, 0x3b, 0xc1, 0x43, 0xc9, 0x4b,
		0xd0, 0x52, 0x55, 0x00, 0x25, 0x57, 0x48, 0x25,
		0x57, 0x27, 0x2b, 0x32, 0x3a, 0x42, 0x4a, 0x51,
		0x55, 0x48, 0x25, 0x57, 0x27, 0x2b, 0x32, 0x3a,
		0x42, 0x4a, 0x51, 0x55, 0x00, 0x25, 0x57, 0x0c,
		0x47, 0x2c, 0x4f, 0x2f, 0xb2, 0x34, 0xb8, 0x39,
		0xbe, 0x3f, 0xc3, 0x45, 0xc9, 0x4a, 0x4d, 0x47,
		0x2c, 0x4f, 0x2f, 0xb2, 0x34, 0xb8, 0x39, 0xbe,
		0x3f, 0xc3, 0x45, 0xc9, 0x4a, 0x4d, 0x47, 0x2c,
		0x4f, 0x2f, 0xb2, 0x34, 0xb8, 0x39, 0xbd, 0x3f,
		0xc3, 0x45, 0xc9, 0x4a, 0x4d, 0x47, 0x2c, 0x4f,
		0xae, 0x2f, 0xb2, 0x34, 0xb8, 0x39, 0xbd, 0x3f,
		0xc3, 0x45, 0xc9, 0x4a, 0x4d, 0x47, 0x2c, 0x4f,
		0xae, 0x2f, 0xb2, 0x34, 0xb8, 0x39, 0xbd, 0x3f,
		0xc3, 0x45, 0xc9, 0x4a, 0x4d, 0x47, 0x2c, 0x4f,
		0xae, 0x2f, 0xb2, 0x34, 0xb8, 0x39, 0xbd, 0x3f,
		0xc3, 0x45, 0xc9, 0x4a, 0x4d, 0x47, 0x2c, 0x4f,
		0x2f, 0xb2, 0x34, 0xb8, 0x39, 0xbd, 0x3f, 0xc3,
		0x45, 0xc9, 0x4a, 0x4d, 0x47, 0x2c, 0x4f, 0x2f,
		0xb2, 0x34, 0xb8, 0x39, 0xbd, 0x3f, 0xc3, 0x45,
		0xc9, 0x4a, 0x4d, 0x08, 0x2c, 0x4f, 0x2d, 0xb0,
		0x31, 0xb5, 0x36, 0xba, 0x3c, 0xc0, 0x42, 0xc6,
		0x48, 0xcb, 0x4c, 0x4e, 0x00, 0x2c, 0x4f, 0x47,
		0x2c, 0x4f, 0x2e, 0x31, 0x36, 0x3b, 0x41, 0x47,
		0x4c, 0x00, 0x2c, 0x4f, 0x0b, 0x47, 0x2e, 0x4d,
		0x30, 0xb3, 0x34, 0xb8, 0x39, 0xbd, 0x3e, 0xc2,
		0x43, 0xc7, 0x48, 0x4b, 0x47, 0x2e, 0x4d, 0x30,
		0xb3, 0x34, 0xb8, 0x39, 0xbd, 0x3e, 0xc2, 0x43,
		0xc7, 0x48, 0x4b, 0x47, 0x2e, 0x4d, 0x30, 0xb3,
		0x34, 0xb8, 0x39, 0xbd, 0x3e, 0xc2, 0x43, 0xc7,
		0x48, 0x4b, 0x47, 0x2e, 0x4d, 0x30, 0xb3, 0x34,
		0xb8, 0x39, 0xbd, 0x3e, 0xc2, 0x43, 0xc7, 0x48,
		0x4b, 0x47, 0x2e, 0x4d, 0x30, 0xb3, 0x34, 0xb8,
		0x39, 0xbd, 0x3e, 0xc2, 0x43, 0xc7, 0x48, 0x4b,
		0x47, 0x2e, 0x4d, 0x30, 0xb3, 0x34, 0xb8, 0x39,
		0xbd, 0x3e, 0xc2, 0x43, 0xc7, 0x48, 0x4b, 0x47,
		0x2e, 0x4d, 0x30, 0xb3, 0x34, 0xb8, 0x39, 0xbd,
		0x3e, 0xc2, 0x43, 0xc7, 0x48, 0x4b, 0x47, 0x2e,
		0x4d, 0x30, 0xb3, 0x34, 0xb8, 0x39, 0xbd, 0x3e,
		0xc2, 0x43, 0xc7, 0x48, 0x4b, 0x00, 0x2e, 0x4d,
		0x46, 0x2e, 0x4d, 0x32, 0x37, 0x3b, 0x40, 0x45,
		0x4a, 0x00, 0x2e, 0x4d, 0x09, 0x47, 0x33, 0x4a,
		0x35, 0x38, 0x3b, 0xbe, 0x3f, 0x42, 0x45, 0x48,
		0x47, 0x33, 0x4a, 0x35, 0x38, 0x3b, 0xbe, 0x3f,
		0x42, 0x45, 0x48, 0x47, 0x33, 0x4a, 0x35, 0x38,
		0x3b, 0xbe, 0x3f, 0x42, 0x45, 0x48, 0x47, 0x33,
		0x4a, 0x35, 0x38, 0x3b, 0xbe, 0x3f, 0x42, 0x45,
		0x48, 0x47, 0x33, 0x4a, 0x35, 0x38, 0x3b, 0xbe,
		0x3f, 0x42, 0x45, 0x48, 0x47, 0x33, 0x4a, 0x35,
		0x38, 0x3b, 0xbe, 0x3f, 0x42, 0x45, 0x48, 0x00,
		0x33, 0x4a, 0x46, 0x33, 0x4a, 0x36, 0x39, 0x3c,
		0x40, 0x44, 0x48, 0x00, 0x33, 0x4a, 0x07, 0x08,
		0x36, 0x47, 0x37, 0x39, 0x3b, 0x3d, 0x40, 0x42,
		0x44, 0x46, 0x08, 0x36, 0x47, 0x37, 0x39, 0x3b,
		0x3d, 0x40, 0x42, 0x44, 0x46, 0x08, 0x36, 0x47,
		0x37, 0x39, 0x3b, 0x3d, 0x40, 0x42, 0x44, 0x46,
		0x08, 0x36, 0x47, 0x37, 0x39, 0x3b, 0x3d, 0x40,
		0x42, 0x44, 0x46, 0x00, 0x36, 0x47, 0x40, 0x36,
		0x47, 0x00, 0x36, 0x47, 0x06, 0x46, 0x38, 0x46,
		0x3a, 0x3c, 0x3e, 0x40, 0x42, 0x44, 0x46, 0x38,
		0x46, 0x3a, 0x3c, 0x3e, 0x40, 0x42, 0x44, 0x46,
		0x38, 0x46, 0x3a, 0x3c, 0x3e, 0x40, 0x42, 0x44,
		0x00, 0x38, 0x46, 0x40, 0x38, 0x46, 0x00, 0x38,
		0x46, 0x05, 0x44, 0x3a, 0x44, 0x3c, 0x3e, 0x40,
		0x42, 0x44, 0x3a, 0x44, 0x3c, 0x3e, 0x40, 0x42,
		0x00, 0x3a, 0x44, 0x40, 0x3a, 0x44, 0x00, 0x3a,
		0x44, 0x4e, 0x40, 0x2c, 0x60, 0x42, 0x2d, 0x5f,
		0xc2, 0x44, 0xdd, 0x5e, 0x41, 0x2e, 0x5c, 0x48,
		0x43, 0x2e, 0x5b, 0xc0, 0x42, 0x49, 0xd9, 0x5a,
		0x41, 0x2f, 0x58, 0xca, 0x4c, 0x44, 0x30, 0x57,
		0x31, 0x38, 0xbe, 0x40, 0xcd, 0x52, 0xc2, 0x32,
		0x4f, 0x33, 0x37, 0x00, 0x53, 0x56, 0x42, 0x34,
		0x4e, 0xb5, 0x36, 0xbc, 0x3e, 0x40, 0x37, 0x4d,
		0x41, 0x38, 0x4d, 0xbb, 0x3d, 0x40, 0x39, 0x4c,
		0x43, 0x3a, 0x4b, 0xbc, 0x3e, 0x42, 0x4a, 0x41,
		0x3b, 0x49, 0xc3, 0x44, 0x02, 0x3b, 0x48, 0x3c,
		0xc0, 0x44, 0x41, 0x3b, 0x46, 0xbd, 0x3f, 0x41,
		0x3b, 0x46, 0xbd, 0x3f, 0x41, 0x3b, 0x46, 0xbd,
		0x3f, 0x41, 0x3b, 0x46, 0xbd, 0x3f, 0x41, 0x3b,
		0x46, 0xbd, 0x3f, 0x40, 0x3b, 0x46, 0x40, 0x3b,
		0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40,
		0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46,
		0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b,
		0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40,
		0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46,
		0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b,
		0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40,
		0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46,
		0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b,
		0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40,
		0x3b, 0x46, 0x40, 0x3b, 0x46, 0x40, 0x3b, 0x46,
		0x40, 0x3b, 0x46, 0x44, 0x3b, 0x46, 0x3d, 0x3f,
		0x42, 0x44, 0x02, 0x3b, 0x46, 0x3e, 0x43, 0x00,
		0x38, 0x49, 0x40, 0x38, 0x49, 0x40, 0x38, 0x49,
		0x40, 0x38, 0x49, 0x00, 0x35, 0x4c, 0x40, 0x35,
		0x4c, 0x00, 0x35, 0x4c, 0x46, 0x35, 0x4c, 0x37,
		0x3c, 0x3e, 0x43, 0x45, 0x4a, 0x46, 0x35, 0x4c,
		0x37, 0x3c, 0x3e, 0x43, 0x45, 0x4a, 0x46, 0x35,
		0x4c, 0x37, 0x3c, 0x3e, 0x43, 0x45, 0x4a, 0x46,
		0x35, 0x4c, 0x37, 0x3c, 0x3e, 0x43, 0x45, 0x4a,
		0x46, 0x35, 0x4c, 0x37, 0x3c, 0x3e, 0x43, 0x45,
		0x4a, 0x46, 0x35, 0x4c, 0x37, 0x3c, 0x3e, 0x43,
		0x45, 0x4a, 0x46, 0x35, 0x4c, 0x37, 0x3c, 0x3e,
		0x43, 0x45, 0x4a, 0x46, 0x35, 0x4c, 0x37, 0x3c,
		0x3e, 0x43, 0x45, 0x4a, 0x03, 0x35, 0x4c, 0xb9,
		0x3a, 0xc0, 0x41, 0xc7, 0x48, 0x04, 0x35, 0x4c,
		0xb6, 0x37, 0xbc, 0x3e, 0xc3, 0x45, 0xca, 0x4b,
		0x40, 0x35, 0x4c, 0x40, 0x35, 0x4c, 0x00, 0x35,
		0x4c, 0x40, 0x3a, 0x47, 0x40, 0x3b, 0x46, 0x40,
		0x3c, 0x45, 0x42, 0x3d, 0x44, 0x3e, 0x43, 0x00,
		0x3f, 0x42, 0x37, 0x40, 0x32, 0x56, 0x43, 0x33,
		0x55, 0xc1, 0x43, 0x45, 0xd3, 0x54, 0x42, 0x33,
		0x52, 0xc0, 0x41, 0x46, 0x41, 0x34, 0x51, 0xc7,
		0x48, 0x44, 0x35, 0x50, 0x36, 0x3a, 0xbe, 0x40,
		0xc9, 0x4f, 0x42, 0x37, 0x4a, 0xb8, 0x39, 0xbd,
		0x3e, 0x41, 0x3a, 0x49, 0xbc, 0x3e, 0x40, 0x3b,
		0x48, 0x02, 0x3c, 0x47, 0xbf, 0x40, 0xc3, 0x45,
		0x02, 0x3c, 0x45, 0x3d, 0xc0, 0x42, 0x41, 0x3c,
		0x44, 0xbe, 0x3f, 0x41, 0x3c, 0x44, 0xbe, 0x3f,
		0x41, 0x3c, 0x44, 0xbe, 0x3f, 0x41, 0x3c, 0x44,
		0xbe, 0x3f, 0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44,
		0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c,
		0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40,
		0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44,
		0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c,
		0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40,
		0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44,
		0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c,
		0x44, 0x40, 0x3c, 0x44, 0x40, 0x3c, 0x44, 0x43,
		0x3c, 0x44, 0x3e, 0x40, 0x42, 0x00, 0x3a, 0x46,
		0x40, 0x3a, 0x46, 0x40, 0x3a, 0x46, 0x00, 0x39,
		0x47, 0x44, 0x39, 0x47, 0x3c, 0x3e, 0x42, 0x44,
		0x44, 0x39, 0x47, 0x3c, 0x3e, 0x42, 0x44, 0x44,
		0x39, 0x47, 0x3c, 0x3e, 0x42, 0x44, 0x44, 0x39,
		0x47, 0x3c, 0x3e, 0x42, 0x44, 0x44, 0x39, 0x47,
		0x3c, 0x3e, 0x42, 0x44, 0x03, 0x39, 0x47, 0x3a,
		0x40, 0x46, 0x02, 0x39, 0x47, 0xbc, 0x3e, 0xc2,
		0x44, 0x40, 0x39, 0x47, 0x00, 0x39, 0x47, 0x40,
		0x3b, 0x45, 0x40, 0x3c, 0x44, 0x42, 0x3d, 0x43,
		0x3e, 0x42, 0x00, 0x3f, 0x41, 0x28, 0x40, 0x36,
		0x4f, 0x43, 0x36, 0x4e, 0xc0, 0x41, 0x44, 0x4d,
		0x41, 0x37, 0x4c, 0xc5, 0x47, 0x42, 0x38, 0x4b,
		0xbe, 0x40, 0xc8, 0x4a, 0x43, 0x39, 0x47, 0xba,
		0x3b, 0xbd, 0x3e, 0x46, 0x41, 0x3c, 0x45, 0x41,
		0x02, 0x3d, 0x44, 0x3e, 0xc0, 0x41, 0x41, 0x3d,
		0x43, 0x3f, 0x41, 0x3d, 0x43, 0x3f, 0x41, 0x3d,
		0x43, 0x3f, 0x40, 0x3d, 0x43, 0x40, 0x3d, 0x43,
		0x40, 0x3d, 0x43, 0x40, 0x3d, 0x43, 0x40, 0x3d,
		0x43, 0x40, 0x3d, 0x43, 0x40, 0x3d, 0x43, 0x40,
		0x3d, 0x43, 0x40, 0x3d, 0x43, 0x40, 0x3d, 0x43,
		0x40, 0x3d, 0x43, 0x40, 0x3d, 0x43, 0x40, 0x3d,
		0x43, 0x40, 0x3d, 0x43, 0x40, 0x3d, 0x43, 0x40,
		0x3d, 0x43, 0x40, 0x3d, 0x43, 0x00, 0x3c, 0x44,
		0x40, 0x3c, 0x44, 0x00, 0x3b, 0x45, 0x44, 0x3b,
		0x45, 0x3d, 0x3f, 0x41, 0x43, 0x44, 0x3b, 0x45,
		0x3d, 0x3f, 0x41, 0x43, 0x44, 0x3b, 0x45, 0x3d,
		0x3f, 0x41, 0x43, 0x44, 0x3b, 0x45, 0x3d, 0x3f,
		0x41, 0x43, 0x00, 0x3b, 0x45, 0x40, 0x3b, 0x45,
		0x00, 0x3b, 0x45, 0x40, 0x3d, 0x43, 0x40, 0x3e,
		0x42, 0x00, 0x3f, 0x41, 0x23, 0x40, 0x37, 0x4d,
		0x43, 0x38, 0x4c, 0xbf, 0x40, 0x43, 0x4b, 0x41,
		0x39, 0x4a, 0xc4, 0x45, 0x42, 0x3a, 0x49, 0xbe,
		0x3f, 0xc6, 0x48, 0x43, 0x3b, 0x45, 0x3c, 0x40,
		0x44, 0x01, 0x3d, 0x43, 0xbf, 0x40, 0x41, 0x3d,
		0x42, 0x3e, 0x41, 0x3d, 0x42, 0x3e, 0x41, 0x3d,
		0x42, 0x3e, 0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42,
		0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42, 0x40, 0x3d,
		0x42, 0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42, 0x40,
		0x3d, 0x42, 0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42,
		0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42, 0x40, 0x3d,
		0x42, 0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42, 0x00,
		0x3c, 0x43, 0x40, 0x3c, 0x43, 0x00, 0x3b, 0x44,
		0x42, 0x3b, 0x44, 0x3e, 0x41, 0x42, 0x3b, 0x44,
		0x3e, 0x41, 0x42, 0x3b, 0x44, 0x3e, 0x41, 0x00,
		0x3b, 0x44, 0x40, 0x3b, 0x44, 0x00, 0x3b, 0x44,
		0x40, 0x3d, 0x42, 0x00, 0x3e, 0x41, 0x1b, 0x40,
		0x39, 0x49, 0x43, 0x3a, 0x48, 0xbf, 0x40, 0x42,
		0x47, 0x41, 0x3b, 0x46, 0xc3, 0x45, 0x42, 0x3c,
		0x44, 0x3d, 0x40, 0x01, 0x3e, 0x43, 0x40, 0x41,
		0x3e, 0x42, 0x3f, 0x41, 0x3e, 0x42, 0x3f, 0x40,
		0x3e, 0x42, 0x40, 0x3e, 0x42, 0x40, 0x3e, 0x42,
		0x40, 0x3e, 0x42, 0x40, 0x3e, 0x42, 0x40, 0x3e,
		0x42, 0x40, 0x3e, 0x42, 0x40, 0x3e, 0x42, 0x40,
		0x3e, 0x42, 0x40, 0x3e, 0x42, 0x40, 0x3e, 0x42,
		0x00, 0x3e, 0x42, 0x40, 0x3e, 0x42, 0x00, 0x3d,
		0x43, 0x42, 0x3d, 0x43, 0x3f, 0x41, 0x42, 0x3d,
		0x43, 0x3f, 0x41, 0x42, 0x3d, 0x43, 0x3f, 0x41,
		0x00, 0x3d, 0x43, 0x40, 0x3e, 0x42, 0x00, 0x3f,
		0x41, 0x15, 0x40, 0x3a, 0x46, 0x41, 0x3b, 0x45,
		0x41, 0x42, 0x3c, 0x44, 0x3d, 0xc2, 0x43, 0x01,
		0x3e, 0x41, 0x40, 0x01, 0x3e, 0x41, 0x40, 0x01,
		0x3e, 0x41, 0x40, 0x40, 0x3e, 0x41, 0x40, 0x3e,
		0x41, 0x40, 0x3e, 0x41, 0x40, 0x3e, 0x41, 0x40,
		0x3e, 0x41, 0x40, 0x3e, 0x41, 0x40, 0x3e, 0x41,
		0x40, 0x3e, 0x41, 0x40, 0x3e, 0x41, 0x00, 0x3d,
		0x42, 0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42, 0x00,
		0x3d, 0x42, 0x40, 0x3e, 0x41, 0x00, 0x3f, 0x40,
		0x11, 0x40, 0x3c, 0x45, 0x42, 0x3d, 0x44, 0x40,
		0x43, 0x41, 0x3e, 0x42, 0x41, 0x40, 0x3f, 0x41,
		0x40, 0x3f, 0x41, 0x40, 0x3f, 0x41, 0x40, 0x3f,
		0x41, 0x40, 0x3f, 0x41, 0x40, 0x3f, 0x41, 0x40,
		0x3f, 0x41, 0x40, 0x3f, 0x41, 0x40, 0x3f, 0x41,
		0x00, 0x3e, 0x42, 0x40, 0x3e, 0x42, 0x00, 0x3e,
		0x42, 0x40, 0x3f, 0x41, 0x40, 0x40, 0x40, 0x0e,
		0x40, 0x3d, 0x44, 0x40, 0x3e, 0x43, 0x01, 0x3f,
		0x42, 0x40, 0x40, 0x3f, 0x41, 0x40, 0x3f, 0x41,
		0x40, 0x3f, 0x41, 0x40, 0x3f, 0x41, 0x40, 0x3f,
		0x41, 0x40, 0x3f, 0x41, 0x00, 0x3e, 0x42, 0x40,
		0x3e, 0x42, 0x00, 0x3e, 0x42, 0x40, 0x3f, 0x41,
		0x40, 0x40, 0x40, 0x13, 0x46, 0x0f, 0x4c, 0x12,
		0x9f, 0x20, 0x2f, 0x39, 0xbd, 0x3e, 0x42, 0x46,
		0x0f, 0x4c, 0x12, 0x9f, 0x20, 0x2f, 0x39, 0xbd,
		0x3e, 0x42, 0x4b, 0x0f, 0x60, 0x12, 0x14, 0x9f,
		0x20, 0x25, 0x2a, 0x2f, 0x39, 0xbd, 0x3e, 0x42,
		0x49, 0xcc, 0x5f, 0x4b, 0x0f, 0x60, 0x12, 0x14,
		0x9f, 0x20, 0x25, 0x2a, 0x2f, 0x39, 0x42, 0x49,
		0x4c, 0xd0, 0x52, 0x46, 0x0f, 0x60, 0x12, 0xaf,
		0x39, 0x42, 0x4c, 0xd0, 0x52, 0xd8, 0x59, 0x08,
		0x0f, 0x60, 0x10, 0xb1, 0x38, 0xba, 0x41, 0xc3,
		0x45, 0xc7, 0x4b, 0xcd, 0x4f, 0xd3, 0x57, 0xda,
		0x5f, 0x4b, 0x0f, 0x60, 0x11, 0x30, 0xb3, 0x35,
		0xb9, 0x42, 0x46, 0x4c, 0x4e, 0x54, 0x56, 0x5b,
		0x5d, 0x0b, 0x0f, 0x60, 0x10, 0xb1, 0x32, 0xb6,
		0x39, 0xbb, 0x42, 0xc4, 0x4a, 0x4d, 0xd0, 0x52,
		0x55, 0xd8, 0x59, 0x5c, 0x5f, 0x45, 0x0f, 0x61,
		0x11, 0x30, 0xbb, 0x3e, 0x44, 0xca, 0x60, 0x04,
		0x0f, 0x61, 0x10, 0xbd, 0x42, 0xc6, 0x48, 0xca,
		0x60, 0x4a, 0x0f, 0x61, 0x11, 0x30, 0x34, 0x36,
		0x39, 0x3d, 0x46, 0xc8, 0x4c, 0xd0, 0x5a, 0xdd,
		0x60, 0x06, 0x0f, 0x61, 0x10, 0xb1, 0x33, 0x35,
		0xb7, 0x38, 0xba, 0x3c, 0xc8, 0x60, 0xc0, 0x0f,
		0x12, 0x48, 0x1f, 0x61, 0x28, 0x2e, 0x36, 0x3d,
		0x43, 0xca, 0x4e, 0x57, 0xda, 0x60, 0x80, 0x0f,
		0x12, 0x45, 0x20, 0x61, 0x27, 0xad, 0x2e, 0x36,
		0x43, 0xd0, 0x53, 0xc1, 0x21, 0x2c, 0x26, 0x03,
		0x2e, 0x61, 0xaf, 0x3c, 0xbe, 0x42, 0xd1, 0x52,
		0xc2, 0x22, 0x2c, 0x25, 0x2b, 0xc0, 0x2e, 0x47,
		0x40, 0x50, 0x53, 0x81, 0x23, 0x2c, 0x2b, 0x80,
		0x2e, 0x47, 0x00, 0x50, 0x53, 0x40, 0x29, 0x2c,
		0x00, 0x29, 0x2c, 0x0e, 0x46, 0x1e, 0x48, 0x20,
		0x29, 0x34, 0x3b, 0x3e, 0x41, 0x4b, 0x1e, 0x56,
		0x20, 0x22, 0x29, 0x2d, 0x30, 0x34, 0x3b, 0x3e,
		0x41, 0x46, 0xc8, 0x55, 0x4b, 0x1e, 0x56, 0x20,
		0x22, 0x29, 0x2d, 0x30, 0x34, 0x3b, 0x41, 0x46,
		0x48, 0xcb, 0x4c, 0x46, 0x1e, 0x56, 0x20, 0xb4,
		0x3b, 0x41, 0x48, 0xcb, 0x4c, 0x51, 0x07, 0x1e,
		0x56, 0x1f, 0xb6, 0x3a, 0xc2, 0x43, 0xc5, 0x47,
		0xc9, 0x4a, 0xcd, 0x50, 0xd2, 0x55, 0x46, 0x1e,
		0x56, 0x20, 0x35, 0x38, 0x3c, 0x42, 0x47, 0x03,
		0x1e, 0x57, 0x1f, 0xbe, 0x42, 0xc4, 0x45, 0x47,
		0x1e, 0x57, 0x20, 0x35, 0x37, 0x39, 0x3b, 0x3e,
		0xc4, 0x45, 0x09, 0x1e, 0x57, 0x1f, 0x36, 0x38,
		0x3a, 0xbc, 0x3e, 0xc5, 0x46, 0xcb, 0x4f, 0x52,
		0xd4, 0x56, 0x80, 0x1e, 0x20, 0x45, 0x29, 0x57,
		0x2e, 0xb3, 0x34, 0x39, 0x42, 0xcb, 0x4d, 0xc1,
		0x2a, 0x32, 0x2d, 0x02, 0x34, 0x57, 0xb5, 0x42,
		0x4c, 0x80, 0x2b, 0x32, 0x80, 0x34, 0x45, 0x00,
		0x4b, 0x4d, 0x40, 0x30, 0x32, 0x00, 0x30, 0x32,
		0x0a, 0x46, 0x28, 0x45, 0x2a, 0x2f, 0x37, 0x3c,
		0x3e, 0x40, 0x0a, 0x28, 0x4f, 0x29, 0x2b, 0xad,
		0x2e, 0xb0, 0x31, 0x33, 0xb5, 0x36, 0xb8, 0x3b,
		0x3d, 0x3f, 0xc1, 0x44, 0x45, 0x28, 0x4f, 0x2a,
		0xb7, 0x3c, 0x40, 0x45, 0x48, 0x05, 0x28, 0x4f,
		0x29, 0xb9, 0x3b, 0xc1, 0x44, 0xc6, 0x4b, 0xcd,
		0x4e, 0x45, 0x28, 0x50, 0x2a, 0xb8, 0x3b, 0x3d,
		0x41, 0xc5, 0x4f, 0x45, 0x28, 0x50, 0x2a, 0x38,
		0x3e, 0x42, 0x44, 0x02, 0x28, 0x50, 0xb9, 0x3e,
		0xc4, 0x4f, 0x44, 0x30, 0x50, 0x32, 0x37, 0xc3,
		0x47, 0xca, 0x4f, 0x81, 0x31, 0x43, 0xb5, 0x36,
		0x00, 0x48, 0x49, 0x00, 0x34, 0x36, 0x09, 0x45,
		0x2a, 0x44, 0x2c, 0x30, 0x37, 0x3c, 0x3f, 0x09,
		};
		
		for (int i = 0; i < drawData.length; i++) {
			Feature.drawData[i] = drawData[i];
		}
	}
		
	/**
	 * Initializes the second half of drawData array to avoid running into
	 * the 65535-byte limit on method/initializer length (?!).
	 */
	private static void initializeDrawData2() {
		char drawData[] = {
			0x2a, 0x4d, 0x2b, 0x2d, 0x2f, 0xb1, 0x32, 0x34,
			0x36, 0xb8, 0x3b, 0xbd, 0x3e, 0xc0, 0x43, 0x45,
			0x2a, 0x4d, 0x2c, 0xb7, 0x3c, 0x3f, 0x44, 0x46,
			0x05, 0x2a, 0x4d, 0x2b, 0xb8, 0x3b, 0xc0, 0x43,
			0xc5, 0x49, 0xcb, 0x4c, 0x05, 0x2a, 0x4d, 0x2b,
			0xad, 0x37, 0x3c, 0xbe, 0x3f, 0xc1, 0x42, 0x02,
			0x2a, 0x4d, 0xb9, 0x3d, 0xc3, 0x4c, 0x44, 0x31,
			0x4d, 0x33, 0x37, 0xc2, 0x45, 0xc8, 0x4c, 0x81,
			0x32, 0x42, 0x36, 0x00, 0x46, 0x47, 0x00, 0x35,
			0x37, 0x07, 0x44, 0x30, 0x43, 0x34, 0x39, 0x3d,
			0x3f, 0x44, 0x30, 0x4a, 0x39, 0x3d, 0x3f, 0xc3,
			0x49, 0x02, 0x30, 0x4a, 0xc0, 0x42, 0xc4, 0x49,
			0x44, 0x30, 0x4a, 0x3a, 0x3e, 0x40, 0xc2, 0x49,
			0x44, 0x30, 0x4a, 0xb1, 0x35, 0xb7, 0x3a, 0x3f,
			0x41, 0x01, 0x35, 0x4a, 0xb8, 0x39, 0x00, 0x38,
			0x39, 0x06, 0x44, 0x33, 0x42, 0x36, 0x3a, 0x3d,
			0x3f, 0x03, 0x33, 0x47, 0xbb, 0x3c, 0x3e, 0xc0,
			0x41, 0x42, 0x33, 0x47, 0xba, 0x40, 0x42, 0x44,
			0x33, 0x47, 0xb4, 0x36, 0x3a, 0x3e, 0x41, 0x01,
			0x37, 0x47, 0x3a, 0x00, 0x39, 0x3a, 0x04, 0x42,
			0x36, 0x42, 0x3b, 0x3e, 0x02, 0x36, 0x46, 0xbc,
			0x3d, 0xbf, 0x41, 0x43, 0x36, 0x46, 0x3b, 0x3f,
			0x41, 0x00, 0x36, 0x46, 0x03, 0x42, 0x38, 0x42,
			0x3c, 0x3f, 0x43, 0x38, 0x45, 0x3c, 0x3f, 0x42,
			0x00, 0x38, 0x45, 0x16, 0x42, 0x00, 0x7f, 0xa0,
			0x23, 0xfd, 0x7e, 0x43, 0x01, 0x7c, 0xa4, 0x25,
			0xe1, 0x63, 0xf8, 0x7b, 0x43, 0x02, 0x77, 0xa6,
			0x28, 0xdf, 0x60, 0xf3, 0x76, 0x43, 0x03, 0x72,
			0xa9, 0x2b, 0xdc, 0x5e, 0xef, 0x71, 0x43, 0x04,
			0x6e, 0xac, 0x2e, 0xd9, 0x5b, 0xec, 0x6d, 0x43,
			0x05, 0x6b, 0xaf, 0x30, 0xd7, 0x58, 0xe9, 0x6a,
			0x43, 0x06, 0x68, 0xb1, 0x32, 0xd0, 0x56, 0xe5,
			0x67, 0x44, 0x07, 0x64, 0xb3, 0x35, 0xcd, 0x4f,
			0x54, 0xe2, 0x63, 0x43, 0x08, 0x61, 0xb6, 0x37,
			0x53, 0x60, 0x43, 0x09, 0x5f, 0x0a, 0xb8, 0x39,
			0x52, 0x44, 0x0b, 0x5e, 0x8c, 0x0d, 0x3a, 0x51,
			0x5d, 0x44, 0x0c, 0x5c, 0x8e, 0x10, 0x3b, 0xce,
			0x50, 0xda, 0x5b, 0x44, 0x0d, 0x59, 0x91, 0x14,
			0x3c, 0xcc, 0x4d, 0x58, 0x44, 0x0e, 0x57, 0xb9,
			0x3b, 0x3d, 0xca, 0x4b, 0x56, 0x43, 0x0f, 0x55,
			0xb6, 0x38, 0xbe, 0x3f, 0xc8, 0x49, 0x43, 0x10,
			0x54, 0x11, 0xb3, 0x35, 0xc0, 0x47, 0x42, 0x12,
			0x53, 0xaf, 0x32, 0xd1, 0x52, 0xc2, 0x13, 0x2e,
			0x14, 0x2d, 0x42, 0x33, 0x50, 0xb4, 0x36, 0xce,
			0x4f, 0xc2, 0x15, 0x2c, 0x96, 0x17, 0x2b, 0x41,
			0x37, 0x4d, 0xcb, 0x4c, 0xc1, 0x18, 0x2a, 0x29,
			0x42, 0x38, 0x4a, 0xb9, 0x3a, 0xc8, 0x49, 0xc2,
			0x19, 0x28, 0x9a, 0x1b, 0xa4, 0x27, 0x01, 0x3b,
			0x47, 0xbe, 0x42, 0x80, 0x1c, 0x23, 0x00, 0x3e,
			0x42, 0x10, 0x42, 0x13, 0x6c, 0xa9, 0x2c, 0x6b,
			0x43, 0x14, 0x6a, 0xad, 0x2f, 0xd6, 0x58, 0xe4,
			0x69, 0x43, 0x15, 0x63, 0xb0, 0x31, 0xd3, 0x55,
			0xe1, 0x62, 0x43, 0x16, 0x60, 0xb2, 0x33, 0xd1,
			0x52, 0x5f, 0x43, 0x17, 0x5e, 0xb4, 0x36, 0xcb,
			0x50, 0xda, 0x5d, 0x44, 0x18, 0x59, 0xb7, 0x38,
			0xc9, 0x4a, 0x4e, 0x58, 0x44, 0x19, 0x57, 0x1a,
			0xb9, 0x3b, 0x4d, 0x56, 0x44, 0x1b, 0x55, 0x9c,
			0x1e, 0x3c, 0x4c, 0x54, 0x44, 0x1c, 0x53, 0x9f,
			0x21, 0x3d, 0xc8, 0x4b, 0xd1, 0x52, 0x43, 0x1d,
			0x50, 0xbb, 0x3c, 0x3e, 0x47, 0x43, 0x1e, 0x4f,
			0xb9, 0x3a, 0x3f, 0xc5, 0x46, 0x44, 0x1f, 0x4e,
			0x20, 0xb4, 0x38, 0xc0, 0x44, 0xcc, 0x4d, 0xc0,
			0x21, 0x33, 0x41, 0x39, 0x4b, 0x4a, 0xc2, 0x22,
			0x32, 0xa3, 0x24, 0xb0, 0x31, 0x42, 0x3a, 0x49,
			0x3b, 0xc6, 0x48, 0xc2, 0x25, 0x2f, 0x26, 0xad,
			0x2e, 0x01, 0x3c, 0x45, 0xbe, 0x41, 0x80, 0x27,
			0x2c, 0x00, 0x3e, 0x41, 0x0c, 0x42, 0x20, 0x5f,
			0xaf, 0x31, 0x5e, 0x43, 0x21, 0x5d, 0xb2, 0x35,
			0xce, 0x51, 0xd7, 0x5c, 0x42, 0x22, 0x56, 0x36,
			0xcc, 0x4d, 0x43, 0x22, 0x55, 0xb7, 0x38, 0xc7,
			0x4b, 0xd2, 0x54, 0x45, 0x23, 0x51, 0x24, 0xb9,
			0x3c, 0x46, 0x49, 0xcf, 0x50, 0x42, 0x25, 0x4e,
			0x3d, 0x48, 0x44, 0x26, 0x4d, 0x3c, 0x3e, 0xc5,
			0x47, 0xcb, 0x4c, 0x43, 0x27, 0x4a, 0x3b, 0x3f,
			0xc3, 0x44, 0x44, 0x28, 0x49, 0x29, 0xb7, 0x3a,
			0xc0, 0x42, 0x48, 0xc2, 0x2a, 0x36, 0x2b, 0x35,
			0x42, 0x3b, 0x47, 0x3c, 0xc4, 0x46, 0xc2, 0x2c,
			0x34, 0x2d, 0xb2, 0x33, 0x41, 0x3d, 0x43, 0xc1,
			0x42, 0x80, 0x2e, 0x31, 0x00, 0x3e, 0x40, 0x0b,
			0x42, 0x23, 0x5b, 0xb0, 0x32, 0x5a, 0x43, 0x24,
			0x59, 0xb3, 0x36, 0xcc, 0x4e, 0xd4, 0x58, 0x42,
			0x25, 0x53, 0x37, 0xca, 0x4b, 0x43, 0x26, 0x52,
			0x38, 0xc8, 0x49, 0xcf, 0x51, 0x43, 0x27, 0x4e,
			0xb9, 0x3b, 0xc6, 0x47, 0xcc, 0x4d, 0x43, 0x28,
			0x4b, 0xbc, 0x3d, 0xc4, 0x45, 0xc9, 0x4a, 0x43,
			0x29, 0x48, 0x3b, 0x3e, 0xc2, 0x43, 0x44, 0x2a,
			0x47, 0x2b, 0xb7, 0x3a, 0xbf, 0x41, 0x46, 0xc1,
			0x2c, 0x36, 0x2d, 0x42, 0x3b, 0x45, 0x3c, 0xc3,
			0x44, 0x81, 0x2e, 0x35, 0xb0, 0x32, 0x01, 0x3d,
			0x42, 0xbe, 0x3f, 0x80, 0x30, 0x32, 0x00, 0x3e,
			0x3f, 0x09, 0x41, 0x2a, 0x54, 0xb4, 0x35, 0x43,
			0x2b, 0x53, 0xb6, 0x38, 0xc9, 0x4a, 0xcf, 0x52,
			0x43, 0x2c, 0x4e, 0xb9, 0x3a, 0xc6, 0x48, 0xcc,
			0x4d, 0x43, 0x2d, 0x4b, 0xbb, 0x3c, 0x45, 0xc9,
			0x4a, 0x43, 0x2e, 0x48, 0xbd, 0x3e, 0xc2, 0x44,
			0x47, 0x44, 0x2f, 0x46, 0x30, 0xb9, 0x3c, 0xbf,
			0x41, 0x45, 0xc1, 0x31, 0x38, 0x32, 0x41, 0x3d,
			0x44, 0xc2, 0x43, 0xc1, 0x33, 0x37, 0x36, 0x00,
			0x3e, 0x41, 0x00, 0x34, 0x35, 0x07, 0x41, 0x2f,
			0x4f, 0xb6, 0x37, 0x43, 0x30, 0x4e, 0xb8, 0x39,
			0xc2, 0x44, 0xcb, 0x4d, 0x43, 0x31, 0x4a, 0xba,
			0x3c, 0xc0, 0x41, 0xc6, 0x49, 0x42, 0x32, 0x45,
			0xbd, 0x3f, 0xc3, 0x44, 0x42, 0x33, 0x42, 0xba,
			0x3c, 0x41, 0x81, 0x34, 0x39, 0xb6, 0x37, 0x00,
			0x3d, 0x40, 0x00, 0x36, 0x37, 0x06, 0x41, 0x33,
			0x4b, 0xb8, 0x39, 0x43, 0x34, 0x4a, 0xba, 0x3b,
			0x41, 0xc8, 0x49, 0x43, 0x35, 0x47, 0x3c, 0xbe,
			0x40, 0xc3, 0x46, 0x42, 0x36, 0x42, 0xbb, 0x3d,
			0x41, 0xc0, 0x37, 0x3a, 0x00, 0x3e, 0x40, 0x00,
			0x38, 0x39, 0x04, 0x42, 0x36, 0x4a, 0x3a, 0xc7,
			0x49, 0x42, 0x37, 0x46, 0x3b, 0xc1, 0x45, 0x01,
			0x38, 0x40, 0xb9, 0x3b, 0x00, 0x39, 0x3b, 0x22,
			0x43, 0x27, 0x5b, 0x2e, 0x3c, 0x48, 0x44, 0x28,
			0x5b, 0x2f, 0x32, 0xbc, 0x48, 0x54, 0x46, 0x29,
			0x5a, 0xad, 0x2e, 0xb0, 0x31, 0x3d, 0x47, 0xd2,
			0x53, 0x59, 0x44, 0x2a, 0x58, 0xab, 0x2c, 0xbd,
			0x47, 0x51, 0xd4, 0x57, 0x43, 0x2c, 0x56, 0x3d,
			0xc0, 0x44, 0x47, 0x43, 0x2c, 0x56, 0x3d, 0xc0,
			0x44, 0x47, 0x43, 0x2c, 0x56, 0x3d, 0xc0, 0x44,
			0x47, 0x43, 0x2c, 0x56, 0x3d, 0xc0, 0x44, 0x47,
			0x43, 0x2c, 0x56, 0x3d, 0xc0, 0x44, 0x47, 0x43,
			0x2c, 0x56, 0x3d, 0xc0, 0x44, 0x47, 0x42, 0x2c,
			0x56, 0xbc, 0x48, 0x4b, 0x43, 0x2c, 0x56, 0x3c,
			0x48, 0x4b, 0x45, 0x2c, 0x56, 0x31, 0x3c, 0x48,
			0x4b, 0x51, 0x45, 0x2c, 0x56, 0x31, 0x38, 0xbc,
			0x48, 0x4b, 0x51, 0x43, 0x2c, 0x56, 0x31, 0x38,
			0x51, 0x43, 0x2c, 0x56, 0x31, 0x38, 0x51, 0x41,
			0x2c, 0x56, 0x38, 0x40, 0x2c, 0x56, 0x40, 0x2c,
			0x56, 0x42, 0x2c, 0x56, 0xbd, 0x3e, 0xc6, 0x47,
			0x42, 0x2c, 0x56, 0xbd, 0x3e, 0xc6, 0x47, 0x42,
			0x2c, 0x56, 0xbd, 0x3e, 0xc6, 0x47, 0x42, 0x2c,
			0x56, 0xbd, 0x3e, 0xc6, 0x47, 0x40, 0x2c, 0x56,
			0x40, 0x2c, 0x56, 0x4a, 0x2c, 0x56, 0x2f, 0x31,
			0x36, 0x38, 0x40, 0x43, 0x4b, 0x4d, 0x52, 0x54,
			0x4a, 0x2c, 0x56, 0x2f, 0x31, 0x36, 0x38, 0x40,
			0x43, 0x4b, 0x4d, 0x52, 0x54, 0x4a, 0x2c, 0x56,
			0xae, 0x2f, 0xb1, 0x32, 0xb5, 0x36, 0xb8, 0x39,
			0xbf, 0x40, 0xc3, 0x44, 0xca, 0x4b, 0xcd, 0x4e,
			0xd1, 0x52, 0xd4, 0x55, 0x00, 0x2a, 0x58, 0x40,
			0x2a, 0x58, 0x40, 0x2a, 0x58, 0x40, 0x2a, 0x58,
			0x07, 0x2a, 0x58, 0xab, 0x2d, 0xb2, 0x34, 0xb9,
			0x3b, 0xc0, 0x42, 0xc7, 0x49, 0xce, 0x50, 0xd5,
			0x57, 0x80, 0x2a, 0x2e, 0x80, 0x31, 0x35, 0x80,
			0x38, 0x3c, 0x80, 0x3f, 0x43, 0x80, 0x46, 0x4a,
			0x80, 0x4d, 0x51, 0x00, 0x54, 0x58, 0x18, 0x43,
			0x2e, 0x53, 0x32, 0x3d, 0x46, 0x46, 0x2f, 0x53,
			0x30, 0xb3, 0x34, 0xbd, 0x3e, 0xc5, 0x46, 0x4d,
			0xd1, 0x52, 0x44, 0x30, 0x51, 0xb1, 0x32, 0xbe,
			0x45, 0x4c, 0xce, 0x50, 0x43, 0x32, 0x4f, 0x3e,
			0xc0, 0x43, 0x45, 0x43, 0x32, 0x4f, 0x3e, 0xc0,
			0x43, 0x45, 0x43, 0x32, 0x4f, 0x3e, 0xc0, 0x43,
			0x45, 0x43, 0x32, 0x4f, 0x3e, 0xc0, 0x43, 0x45,
			0x42, 0x32, 0x4f, 0xbd, 0x46, 0x48, 0x44, 0x32,
			0x4f, 0x35, 0x3d, 0x46, 0x48, 0x06, 0x32, 0x4f,
			0xb3, 0x34, 0xb6, 0x39, 0xbb, 0x3c, 0x47, 0xc9,
			0x4b, 0xcd, 0x4e, 0x43, 0x32, 0x4f, 0x35, 0x3a,
			0x4c, 0x42, 0x32, 0x4f, 0x3a, 0x4c, 0x40, 0x32,
			0x4f, 0x42, 0x32, 0x4f, 0xbd, 0x3e, 0xc4, 0x45,
			0x42, 0x32, 0x4f, 0xbd, 0x3e, 0xc4, 0x45, 0x42,
			0x32, 0x4f, 0xbd, 0x3e, 0xc4, 0x45, 0x40, 0x32,
			0x4f, 0x40, 0x32, 0x4f, 0x4a, 0x32, 0x4f, 0x34,
			0x36, 0x39, 0x3b, 0x40, 0x42, 0x46, 0x48, 0x4b,
			0x4d, 0x4a, 0x32, 0x4f, 0x34, 0x36, 0x39, 0x3b,
			0x40, 0x42, 0x46, 0x48, 0x4b, 0x4d, 0x00, 0x30,
			0x51, 0x40, 0x30, 0x51, 0x07, 0x30, 0x51, 0xb1,
			0x32, 0xb6, 0x37, 0xbb, 0x3c, 0xc0, 0x41, 0xc5,
			0x46, 0xca, 0x4b, 0xcf, 0x50, 0x80, 0x30, 0x33,
			0x80, 0x35, 0x38, 0x80, 0x3a, 0x3d, 0x80, 0x3f,
			0x42, 0x80, 0x44, 0x47, 0x80, 0x49, 0x4c, 0x00,
			0x4e, 0x51, 0x11, 0x44, 0x33, 0x4d, 0x36, 0x3e,
			0xc0, 0x41, 0x43, 0x45, 0x34, 0x4c, 0x35, 0x3e,
			0xc0, 0x41, 0x43, 0xc9, 0x4b, 0x43, 0x36, 0x4a,
			0x3e, 0xc0, 0x41, 0x43, 0x43, 0x36, 0x4a, 0x3e,
			0xc0, 0x41, 0x43, 0x41, 0x36, 0x4a, 0xbe, 0x43,
			0x43, 0x36, 0x4a, 0x3e, 0x43, 0x45, 0x43, 0x36,
			0x4a, 0x38, 0xbe, 0x43, 0x45, 0x43, 0x36, 0x4a,
			0x38, 0x3b, 0x48, 0x42, 0x36, 0x4a, 0x3b, 0x48,
			0x40, 0x36, 0x4a, 0x42, 0x36, 0x4a, 0x3e, 0x43,
			0x42, 0x36, 0x4a, 0x3e, 0x43, 0x40, 0x36, 0x4a,
			0x49, 0x36, 0x4a, 0x38, 0x3a, 0x3c, 0x3e, 0x40,
			0x42, 0x44, 0x46, 0x48, 0x00, 0x34, 0x4c, 0x40,
			0x34, 0x4c, 0x00, 0x34, 0x4c, 0x0f, 0x43, 0x34,
			0x4b, 0x37, 0xbe, 0x40, 0x42, 0x44, 0x35, 0x4a,
			0x36, 0xbe, 0x40, 0x42, 0xc7, 0x49, 0x42, 0x37,
			0x48, 0xbe, 0x40, 0x42, 0x42, 0x37, 0x48, 0xbe,
			0x40, 0x42, 0x43, 0x37, 0x48, 0xbe, 0x40, 0x42,
			0x44, 0x43, 0x37, 0x48, 0x39, 0xbe, 0x42, 0x44,
			0x43, 0x37, 0x48, 0x39, 0x3b, 0x46, 0x42, 0x37,
			0x48, 0x3b, 0x46, 0x40, 0x37, 0x48, 0x42, 0x37,
			0x48, 0x3e, 0x42, 0x42, 0x37, 0x48, 0x3e, 0x42,
			0x40, 0x37, 0x48, 0x00, 0x36, 0x49, 0x40, 0x36,
			0x49, 0x00, 0x36, 0x49, 0x0c, 0x41, 0x39, 0x48,
			0xbf, 0x41, 0x42, 0x3a, 0x47, 0xbf, 0x41, 0xc5,
			0x46, 0x41, 0x3a, 0x46, 0xbf, 0x41, 0x41, 0x3a,
			0x46, 0xbf, 0x41, 0x40, 0x3a, 0x46, 0x40, 0x3a,
			0x46, 0x40, 0x3a, 0x46, 0x40, 0x3a, 0x46, 0x40,
			0x3a, 0x46, 0x00, 0x39, 0x47, 0x40, 0x39, 0x47,
			0x00, 0x39, 0x47, 0x09, 0x41, 0x3a, 0x46, 0xbf,
			0x40, 0x42, 0x3b, 0x45, 0xbf, 0x40, 0xc3, 0x44,
			0x41, 0x3b, 0x44, 0xbf, 0x40, 0x40, 0x3b, 0x44,
			0x40, 0x3b, 0x44, 0x40, 0x3b, 0x44, 0x00, 0x3b,
			0x44, 0x40, 0x3a, 0x45, 0x00, 0x3a, 0x45, 0x06,
			0x41, 0x3c, 0x44, 0xbf, 0x40, 0x41, 0x3c, 0x43,
			0xbf, 0x40, 0x40, 0x3c, 0x43, 0x40, 0x3c, 0x43,
			0x40, 0x3c, 0x43, 0x00, 0x3c, 0x43, 0x05, 0x02,
			0x3d, 0x42, 0x3e, 0x41, 0x02, 0x3d, 0x42, 0x3e,
			0x41, 0x40, 0x3d, 0x42, 0x40, 0x3d, 0x42, 0x00,
			0x3d, 0x42, 0x12, 0x45, 0x2c, 0x4c, 0x33, 0x39,
			0xbb, 0x3d, 0x3f, 0x46, 0x45, 0x2d, 0x4b, 0x33,
			0x39, 0xbb, 0x3d, 0x3f, 0x46, 0x02, 0x2d, 0x4b,
			0x3a, 0x3e, 0x47, 0x2d, 0x4b, 0x30, 0x36, 0x39,
			0x3c, 0x3f, 0x43, 0x49, 0x46, 0x2d, 0x4b, 0x30,
			0x36, 0x3a, 0x3e, 0x43, 0x49, 0x00, 0x2d, 0x4b,
			0x44, 0x2e, 0x4a, 0x33, 0x39, 0x3f, 0x46, 0x44,
			0x2e, 0x4a, 0x33, 0x39, 0x3f, 0x46, 0x00, 0x2f,
			0x49, 0x45, 0x2f, 0x49, 0x31, 0x36, 0x3c, 0x42,
			0x47, 0x45, 0x2f, 0x49, 0x31, 0x36, 0x3c, 0x42,
			0x47, 0x00, 0x30, 0x48, 0x44, 0x31, 0x47, 0x34,
			0x39, 0x3f, 0x44, 0x44, 0x32, 0x46, 0x35, 0x39,
			0x3f, 0x43, 0x00, 0x33, 0x45, 0x42, 0x35, 0x43,
			0x3b, 0x40, 0x43, 0x36, 0x42, 0xb7, 0x38, 0x3b,
			0xc0, 0x41, 0x00, 0x39, 0x3f, 0x0d, 0x45, 0x33,
			0x47, 0x37, 0x3b, 0xbd, 0x3e, 0x40, 0x44, 0x02,
			0x33, 0x47, 0x3c, 0x3f, 0x45, 0x33, 0x47, 0x35,
			0x39, 0xbb, 0x3c, 0xbf, 0x40, 0x42, 0x44, 0x33,
			0x47, 0x35, 0x39, 0xbc, 0x3f, 0x42, 0x01, 0x33,
			0x47, 0xbc, 0x3e, 0x44, 0x33, 0x47, 0x37, 0x3b,
			0x3f, 0x44, 0x00, 0x34, 0x46, 0x43, 0x34, 0x46,
			0x39, 0x3d, 0x41, 0x00, 0x35, 0x45, 0x42, 0x36,
			0x44, 0x3b, 0x3f, 0x43, 0x37, 0x43, 0x39, 0xbb,
			0x3d, 0x40, 0x02, 0x38, 0x42, 0xbb, 0x3c, 0xbe,
			0x3f, 0x00, 0x3b, 0x3f, 0x0a, 0x43, 0x36, 0x45,
			0x3c, 0x3e, 0x40, 0x43, 0x36, 0x45, 0x3c, 0x3e,
			0x40, 0x44, 0x36, 0x45, 0xb7, 0x38, 0x3c, 0x40,
			0xc3, 0x44, 0x41, 0x37, 0x44, 0xbd, 0x3f, 0x40,
			0x37, 0x44, 0x01, 0x37, 0x44, 0xbd, 0x40, 0x40,
			0x38, 0x43, 0x41, 0x39, 0x42, 0xbd, 0x3f, 0x42,
			0x3a, 0x41, 0x3b, 0x40, 0x00, 0x3c, 0x3f, 0x09,
			0x42, 0x37, 0x44, 0x3c, 0x3f, 0x42, 0x37, 0x44,
			0x3c, 0x3f, 0x42, 0x37, 0x44, 0x3c, 0x3f, 0x41,
			0x38, 0x43, 0xbd, 0x3e, 0x40, 0x38, 0x43, 0x40,
			0x39, 0x42, 0x40, 0x3a, 0x41, 0x01, 0x3b, 0x40,
			0xbd, 0x3e, 0x00, 0x3d, 0x3e, 0x06, 0x42, 0x39,
			0x43, 0x3d, 0x3f, 0x42, 0x39, 0x43, 0x3d, 0x3f,
			0x41, 0x3a, 0x42, 0x3e, 0x40, 0x3a, 0x42, 0x40,
			0x3b, 0x41, 0x00, 0x3c, 0x40, 0x05, 0x42, 0x3a,
			0x42, 0x3d, 0x3f, 0x42, 0x3a, 0x42, 0x3d, 0x3f,
			0x41, 0x3b, 0x41, 0x3e, 0x40, 0x3c, 0x40, 0x00,
			0x3d, 0x3f, 0x04, 0x41, 0x3b, 0x42, 0xbe, 0x3f,
			0x41, 0x3b, 0x42, 0xbe, 0x3f, 0x40, 0x3c, 0x41,
			0x00, 0x3d, 0x40, 0x03, 0x41, 0x3c, 0x42, 0x3f,
			0x40, 0x3d, 0x41, 0x00, 0x3e, 0x40, 0x0b, 0x00,
			0x27, 0x56, 0x00, 0x18, 0x62, 0x00, 0x0e, 0x6b,
			0x00, 0x09, 0x70, 0x00, 0x07, 0x73, 0x01, 0x07,
			0x74, 0x88, 0x0a, 0x02, 0x08, 0x74, 0x8b, 0x10,
			0xef, 0x73, 0x02, 0x0b, 0x73, 0x91, 0x17, 0xe9,
			0x6e, 0x02, 0x11, 0x6e, 0x98, 0x25, 0xde, 0x68,
			0x42, 0x18, 0x68, 0x99, 0x25, 0xde, 0x67, 0x00,
			0x26, 0x5d, 0x08, 0x00, 0x2e, 0x4f, 0x00, 0x1d,
			0x5e, 0x00, 0x19, 0x61, 0x01, 0x18, 0x64, 0x99,
			0x1a, 0x02, 0x18, 0x64, 0x9b, 0x1e, 0xe1, 0x63,
			0x02, 0x1b, 0x64, 0x9f, 0x2d, 0xd6, 0x60, 0x42,
			0x1f, 0x60, 0xa0, 0x2d, 0xd6, 0x5f, 0x00, 0x2e,
			0x55, 0x06, 0x00, 0x2c, 0x53, 0x00, 0x24, 0x57,
			0x01, 0x23, 0x59, 0xa4, 0x27, 0x02, 0x24, 0x59,
			0xa8, 0x32, 0xd0, 0x58, 0x42, 0x28, 0x58, 0xa9,
			0x32, 0xd0, 0x57, 0x00, 0x33, 0x4f, 0x05, 0x00,
			0x2e, 0x50, 0x00, 0x27, 0x54, 0x00, 0x26, 0x55,
			0x00, 0x27, 0x54, 0x00, 0x31, 0x4b, 0x04, 0x00,
			0x33, 0x4c, 0x00, 0x2e, 0x4f, 0x00, 0x2d, 0x50,
			0x00, 0x35, 0x48, 0x03, 0x00, 0x36, 0x49, 0x00,
			0x32, 0x4b, 0x00, 0x39, 0x47, 0x02, 0x00, 0x38,
			0x47, 0x00, 0x36, 0x48, 0x01, 0x00, 0x37, 0x46,
			0x23, 0x45, 0x00, 0x7f, 0x05, 0x0f, 0x1e, 0xcb,
			0x4c, 0x6b, 0x45, 0x00, 0x7f, 0x05, 0x0f, 0x1f,
			0xcd, 0x4e, 0x6a, 0x46, 0x00, 0x7f, 0x06, 0x10,
			0x1f, 0xcf, 0x50, 0x5a, 0x69, 0x46, 0x00, 0x7f,
			0x06, 0x10, 0x1f, 0x51, 0x5a, 0x69, 0x47, 0x01,
			0x7e, 0x07, 0x10, 0x1f, 0x34, 0x51, 0x5b, 0x69,
			0x48, 0x01, 0x7d, 0x08, 0x11, 0x1f, 0x28, 0x34,
			0x52, 0x5b, 0x69, 0x49, 0x01, 0x7c, 0x09, 0x11,
			0x1e, 0x20, 0x28, 0x33, 0x53, 0x5b, 0x69, 0x49,
			0x01, 0x7b, 0x0a, 0x11, 0x1d, 0x20, 0x28, 0x33,
			0x53, 0x5c, 0x69, 0x49, 0x02, 0x7b, 0x0b, 0x11,
			0x1c, 0x20, 0x28, 0x33, 0x53, 0x5c, 0x69, 0x4a,
			0x02, 0x7b, 0x0b, 0x12, 0x1b, 0x20, 0x27, 0x32,
			0x43, 0xd4, 0x55, 0x5c, 0x69, 0x4c, 0x03, 0x7a,
			0x0c, 0x12, 0x1b, 0x20, 0x27, 0x2e, 0x31, 0x43,
			0x56, 0x5d, 0x69, 0x6f, 0x4c, 0x03, 0x7a, 0x0c,
			0x12, 0x1b, 0x20, 0x27, 0x2e, 0x31, 0x43, 0x56,
			0x5e, 0x69, 0x70, 0x4c, 0x03, 0x79, 0x0c, 0x12,
			0x1b, 0x21, 0x27, 0x2e, 0x31, 0x42, 0x57, 0x5e,
			0x69, 0x71, 0xc0, 0x03, 0x0b, 0x4a, 0x0d, 0x79,
			0x12, 0x1a, 0x21, 0x27, 0xae, 0x30, 0x42, 0xd7,
			0x58, 0x5e, 0xe9, 0x6a, 0x71, 0xc0, 0x03, 0x0b,
			0xc7, 0x0d, 0x56, 0x11, 0x19, 0x21, 0x27, 0x2e,
			0x42, 0xd4, 0x55, 0xc1, 0x59, 0x69, 0x5e, 0x41,
			0x6b, 0x78, 0x72, 0xc0, 0x03, 0x0a, 0xc7, 0x0d,
			0x53, 0x11, 0x19, 0x21, 0x27, 0x2e, 0x41, 0xd1,
			0x52, 0xc1, 0x59, 0x69, 0x5f, 0x41, 0x6b, 0x77,
			0x72, 0xc0, 0x03, 0x0a, 0xc7, 0x0e, 0x50, 0x11,
			0x98, 0x1b, 0x21, 0x27, 0x2e, 0x41, 0xce, 0x4f,
			0xc1, 0x5a, 0x69, 0x60, 0x41, 0x6c, 0x77, 0x73,
			0xc0, 0x04, 0x0a, 0xc2, 0x0e, 0x17, 0x10, 0x16,
			0xc3, 0x1c, 0x4d, 0xa2, 0x26, 0xae, 0x30, 0x41,
			0xc2, 0x5a, 0x68, 0x60, 0x67, 0x41, 0x6c, 0x77,
			0xf3, 0x76, 0xc0, 0x04, 0x09, 0xc1, 0x0e, 0x15,
			0x10, 0xc2, 0x1d, 0x2e, 0x22, 0x27, 0xc2, 0x31,
			0x4c, 0x40, 0x4b, 0xc1, 0x5b, 0x67, 0x60, 0x41,
			0x6c, 0x76, 0x72, 0xc0, 0x05, 0x08, 0xc0, 0x0f,
			0x15, 0xc2, 0x1d, 0x2e, 0x23, 0x28, 0xc1, 0x32,
			0x4a, 0x40, 0x81, 0x5b, 0x67, 0xdc, 0x60, 0x41,
			0x6d, 0x75, 0x71, 0xc0, 0x05, 0x08, 0xc0, 0x0f,
			0x14, 0xc1, 0x1e, 0x2d, 0x24, 0xc1, 0x33, 0x49,
			0x40, 0xc1, 0x5c, 0x66, 0x60, 0x40, 0x6d, 0x74,
			0xc0, 0x05, 0x08, 0xc0, 0x0f, 0x13, 0xc0, 0x1f,
			0x23, 0xc0, 0x25, 0x2d, 0xc1, 0x33, 0x48, 0x3f,
			0xc1, 0x5c, 0x65, 0x60, 0x40, 0x6d, 0x74, 0xc0,
			0x05, 0x08, 0xc0, 0x0f, 0x12, 0xc0, 0x20, 0x22,
			0xc0, 0x26, 0x2c, 0xc2, 0x34, 0x47, 0x3f, 0x46,
			0xc1, 0x5d, 0x64, 0x5f, 0x40, 0x6e, 0x73, 0xc0,
			0x06, 0x08, 0x80, 0x0f, 0x11, 0xc0, 0x21, 0x21,
			0xc0, 0x26, 0x2b, 0xc1, 0x34, 0x45, 0x3e, 0xc0,
			0x5d, 0x63, 0x40, 0x6e, 0x73, 0xc0, 0x06, 0x08,
			0xc0, 0x26, 0x2a, 0xc1, 0x35, 0x44, 0x3e, 0xc0,
			0x5d, 0x62, 0x40, 0x6e, 0x72, 0xc0, 0x06, 0x08,
			0xc0, 0x26, 0x2a, 0xc1, 0x35, 0x43, 0x3e, 0xc1,
			0x5d, 0x62, 0x5f, 0x40, 0x6f, 0x72, 0xc0, 0x06,
			0x08, 0xc0, 0x26, 0x2a, 0xc2, 0x36, 0x42, 0x3d,
			0x41, 0xc1, 0x5d, 0x61, 0x5f, 0x40, 0x6f, 0x72,
			0xc0, 0x07, 0x07, 0xc0, 0x26, 0x2a, 0xc1, 0x36,
			0x40, 0x3d, 0x81, 0x5e, 0x61, 0x60, 0x40, 0x70,
			0x72, 0xc0, 0x26, 0x29, 0xc1, 0x37, 0x40, 0x3d,
			0xc0, 0x5f, 0x61, 0x40, 0x71, 0x71, 0xc0, 0x26,
			0x29, 0xc1, 0x37, 0x3f, 0x3c, 0x40, 0x5f, 0x61,
			0xc0, 0x26, 0x29, 0xc1, 0x37, 0x3f, 0x3c, 0x40,
			0x5f, 0x61, 0xc0, 0x26, 0x28, 0xc1, 0x37, 0x3e,
			0x3c, 0x40, 0x5f, 0x61, 0xc0, 0x26, 0x28, 0xc1,
			0x38, 0x3d, 0x3b, 0x40, 0x60, 0x60, 0xc0, 0x26,
			0x28, 0x01, 0x38, 0x3c, 0x39, 0x80, 0x26, 0x28,
			0x40, 0x39, 0x39, 0x19, 0x45, 0x13, 0x6c, 0x17,
			0x1e, 0x28, 0x48, 0x5e, 0x46, 0x13, 0x6c, 0x17,
			0x1e, 0x29, 0xc9, 0x4b, 0x52, 0x5d, 0x46, 0x13,
			0x6c, 0x17, 0x1e, 0x29, 0x4c, 0x52, 0x5d, 0x47,
			0x14, 0x6b, 0x18, 0x1e, 0x29, 0x37, 0x4c, 0x53,
			0x5d, 0x48, 0x14, 0x6a, 0x19, 0x1f, 0xa8, 0x29,
			0x2f, 0x37, 0x4d, 0x53, 0x5d, 0x49, 0x14, 0x69,
			0x1a, 0x1f, 0x27, 0x29, 0x2f, 0x37, 0x4d, 0x53,
			0x5d, 0x4a, 0x14, 0x69, 0x1b, 0x20, 0x27, 0x29,
			0x2e, 0x36, 0x42, 0x4e, 0x53, 0x5d, 0x4c, 0x15,
			0x68, 0x1b, 0x20, 0x26, 0x29, 0x2e, 0x33, 0x35,
			0x42, 0x4f, 0x54, 0x5d, 0x61, 0x4c, 0x15, 0x68,
			0x1b, 0x20, 0x26, 0x2a, 0x2e, 0x33, 0x35, 0x41,
			0x50, 0x55, 0x5d, 0x62, 0x4b, 0x15, 0x68, 0x1b,
			0x20, 0x25, 0x2a, 0x2e, 0xb3, 0x34, 0x41, 0x50,
			0x55, 0x5d, 0x62, 0xc8, 0x15, 0x4f, 0x9b, 0x1c,
			0x1f, 0x25, 0x2a, 0x2e, 0x33, 0x41, 0x4e, 0x43,
			0x51, 0x67, 0x55, 0x5d, 0x63, 0xc0, 0x15, 0x1a,
			0xc7, 0x1d, 0x4d, 0x1f, 0xa4, 0x26, 0x2a, 0x2e,
			0x33, 0x41, 0xca, 0x4c, 0x43, 0x51, 0x66, 0x56,
			0xdd, 0x5e, 0x63, 0xc0, 0x16, 0x1a, 0xc0, 0x1e,
			0x23, 0xc3, 0x27, 0x49, 0xab, 0x2e, 0xb3, 0x34,
			0x41, 0xc2, 0x52, 0x5c, 0x56, 0x5b, 0x41, 0x5f,
			0x66, 0x64, 0xc0, 0x17, 0x19, 0xc0, 0x1e, 0x22,
			0xc2, 0x27, 0x33, 0x2b, 0x2f, 0xc3, 0x35, 0x48,
			0x36, 0x40, 0x47, 0x81, 0x53, 0x5a, 0xd4, 0x56,
			0x41, 0x5f, 0x66, 0xe4, 0x65, 0xc0, 0x17, 0x19,
			0xc0, 0x1e, 0x21, 0xc1, 0x28, 0x33, 0x2c, 0xc1,
			0x37, 0x46, 0x40, 0xc1, 0x53, 0x5a, 0x56, 0x40,
			0x5f, 0x65, 0xc0, 0x17, 0x19, 0xc0, 0x1e, 0x20,
			0xc1, 0x29, 0x32, 0xab, 0x2d, 0xc2, 0x37, 0x45,
			0x3f, 0x44, 0xc1, 0x53, 0x59, 0x56, 0x40, 0x60,
			0x64, 0xc0, 0x17, 0x19, 0xc0, 0x1f, 0x1f, 0xc0,
			0x2a, 0x2a, 0xc1, 0x2d, 0x31, 0x2e, 0xc1, 0x37,
			0x43, 0x3e, 0xc0, 0x54, 0x58, 0x40, 0x60, 0x64,
			0xc0, 0x17, 0x19, 0xc0, 0x2e, 0x30, 0xc1, 0x38,
			0x43, 0x3e, 0xc0, 0x54, 0x58, 0x40, 0x60, 0x63,
			0xc0, 0x17, 0x19, 0xc0, 0x2e, 0x30, 0xc2, 0x38,
			0x42, 0x3e, 0x41, 0x81, 0x54, 0x57, 0x55, 0x40,
			0x61, 0x63, 0xc0, 0x18, 0x18, 0xc0, 0x2e, 0x30,
			0xc1, 0x39, 0x40, 0x3e, 0xc0, 0x55, 0x55, 0x40,
			0x61, 0x63, 0xc0, 0x2e, 0x30, 0xc1, 0x3a, 0x3f,
			0x3d, 0x40, 0x62, 0x62, 0xc0, 0x2e, 0x30, 0x41,
			0x3a, 0x3f, 0x3d, 0xc0, 0x2f, 0x2f, 0x01, 0x3a,
			0x3e, 0x3b, 0x40, 0x3a, 0x3c, 0x40, 0x3b, 0x3b,
			0x12, 0x45, 0x20, 0x5f, 0x22, 0x27, 0x2f, 0x45,
			0x55, 0x46, 0x20, 0x5f, 0x22, 0x27, 0x2f, 0xc6,
			0x47, 0x4d, 0x54, 0x47, 0x20, 0x5e, 0x23, 0x27,
			0x2f, 0x39, 0x48, 0x4d, 0x54, 0x48, 0x20, 0x5e,
			0x24, 0x28, 0x2f, 0x34, 0x39, 0x49, 0x4d, 0x54,
			0x49, 0x20, 0x5d, 0x25, 0x28, 0xae, 0x2f, 0x33,
			0x39, 0x41, 0x49, 0x4d, 0x54, 0x4c, 0x21, 0x5c,
			0x25, 0x29, 0x2d, 0x2f, 0x33, 0x36, 0x38, 0x41,
			0x4a, 0x4e, 0x54, 0x57, 0x4b, 0x21, 0x5c, 0x25,
			0x29, 0x2d, 0x30, 0x33, 0xb6, 0x37, 0x40, 0x4b,
			0x4f, 0x54, 0x58, 0xc7, 0x21, 0x4a, 0xa5, 0x26,
			0x28, 0x2c, 0x30, 0x33, 0x36, 0x40, 0x43, 0x4c,
			0x5c, 0x4f, 0x54, 0x59, 0xc0, 0x21, 0x25, 0xc6,
			0x27, 0x49, 0xac, 0x2d, 0x30, 0x33, 0x36, 0x40,
			0xc7, 0x48, 0x43, 0x4c, 0x5b, 0x4f, 0xd4, 0x55,
			0x59, 0xc0, 0x22, 0x24, 0xc0, 0x27, 0x2b, 0xc4,
			0x2e, 0x46, 0xb1, 0x33, 0xb6, 0x38, 0x40, 0x45,
			0x81, 0x4d, 0x53, 0xce, 0x4f, 0x40, 0x56, 0x5a,
			0xc0, 0x22, 0x24, 0xc0, 0x27, 0x2a, 0xc1, 0x2f,
			0x36, 0x31, 0xc1, 0x39, 0x44, 0x40, 0xc0, 0x4d,
			0x52, 0x40, 0x56, 0x5a, 0xc0, 0x22, 0x24, 0x80,
			0x28, 0x29, 0xc1, 0x30, 0x36, 0xb1, 0x32, 0xc1,
			0x39, 0x43, 0x3f, 0xc0, 0x4d, 0x52, 0x40, 0x57,
			0x59, 0xc0, 0x22, 0x24, 0xc0, 0x33, 0x35, 0xc1,
			0x3a, 0x42, 0x3e, 0xc0, 0x4e, 0x51, 0x40, 0x57,
			0x59, 0xc0, 0x22, 0x24, 0xc0, 0x33, 0x35, 0xc2,
			0x3a, 0x41, 0x3e, 0x40, 0xc0, 0x4e, 0x50, 0x40,
			0x57, 0x59, 0xc0, 0x23, 0x23, 0xc0, 0x33, 0x35,
			0xc1, 0x3b, 0x3f, 0x3e, 0xc0, 0x4f, 0x4f, 0x40,
			0x58, 0x58, 0xc0, 0x34, 0x34, 0x40, 0x3b, 0x3e,
			0x40, 0x3b, 0x3d, 0x40, 0x3c, 0x3c, 0x10, 0x45,
			0x23, 0x5b, 0x25, 0x29, 0x30, 0x44, 0x52, 0x46,
			0x23, 0x5b, 0x25, 0x29, 0x30, 0x45, 0x4b, 0x51,
			0x47, 0x23, 0x5a, 0x26, 0x29, 0x30, 0x39, 0x46,
			0x4b, 0x51, 0x48, 0x23, 0x5a, 0x27, 0x2a, 0x30,
			0x35, 0x39, 0x47, 0x4b, 0x51, 0x4a, 0x23, 0x59,
			0x28, 0x2b, 0x30, 0x34, 0x38, 0x40, 0x48, 0x4c,
			0x51, 0x54, 0x4b, 0x24, 0x58, 0x28, 0x2b, 0x2f,
			0x31, 0x34, 0x37, 0x3f, 0x49, 0x4d, 0x51, 0x54,
			0xc7, 0x24, 0x48, 0x28, 0x2a, 0x2e, 0x31, 0x34,
			0x37, 0x3f, 0x43, 0x4a, 0x58, 0x4d, 0x51, 0x55,
			0xc7, 0x24, 0x47, 0xa8, 0x29, 0xae, 0x2f, 0x31,
			0x34, 0x37, 0x3f, 0xc5, 0x46, 0x43, 0x4a, 0x57,
			0x4d, 0xd1, 0x52, 0x55, 0xc0, 0x25, 0x27, 0xc0,
			0x29, 0x2d, 0xc3, 0x2f, 0x44, 0xb2, 0x33, 0xb7,
			0x38, 0x3f, 0x81, 0x4b, 0x50, 0xcc, 0x4d, 0x40,
			0x53, 0x56, 0xc0, 0x25, 0x27, 0xc0, 0x29, 0x2c,
			0xc1, 0x30, 0x37, 0x32, 0xc1, 0x39, 0x43, 0x3f,
			0xc0, 0x4b, 0x4f, 0x40, 0x53, 0x56, 0xc0, 0x25,
			0x27, 0x80, 0x2a, 0x2b, 0xc1, 0x31, 0x37, 0xb2,
			0x33, 0xc1, 0x39, 0x42, 0x3e, 0xc0, 0x4b, 0x4f,
			0x40, 0x54, 0x56, 0xc0, 0x25, 0x27, 0xc0, 0x34,
			0x36, 0x81, 0x3a, 0x41, 0xbb, 0x3d, 0xc0, 0x4c,
			0x4e, 0x40, 0x55, 0x55, 0xc0, 0x26, 0x26, 0xc0,
			0x34, 0x36, 0xc0, 0x3b, 0x3e, 0x40, 0x4d, 0x4d,
			0xc0, 0x35, 0x35, 0x40, 0x3b, 0x3e, 0x40, 0x3b,
			0x3d, 0x40, 0x3c, 0x3c, 0x0c, 0x45, 0x2a, 0x54,
			0x2c, 0x2f, 0x34, 0x43, 0x4e, 0x46, 0x2a, 0x54,
			0x2c, 0x2f, 0x34, 0x44, 0x48, 0x4d, 0x48, 0x2a,
			0x54, 0x2d, 0x30, 0x34, 0x38, 0x3b, 0x45, 0x48,
			0x4d, 0x4a, 0x2b, 0x53, 0x2e, 0x30, 0x34, 0x37,
			0x3a, 0x40, 0x46, 0x49, 0x4d, 0x4f, 0x4b, 0x2b,
			0x52, 0x2e, 0x30, 0x33, 0x35, 0x37, 0x39, 0x3f,
			0x47, 0x4a, 0x4d, 0x4f, 0xc8, 0x2c, 0x46, 0x2e,
			0x30, 0x33, 0x35, 0x37, 0x39, 0x3f, 0xc4, 0x45,
			0x43, 0x48, 0x51, 0x4a, 0x4d, 0x50, 0xc5, 0x2c,
			0x43, 0xae, 0x2f, 0xb2, 0x33, 0x36, 0xb9, 0x3a,
			0x3f, 0xc1, 0x48, 0x4c, 0x4b, 0x40, 0x4e, 0x51,
			0xc0, 0x2d, 0x2d, 0xc0, 0x2f, 0x31, 0xc1, 0x34,
			0x39, 0x36, 0xc1, 0x3b, 0x42, 0x3f, 0xc0, 0x48,
			0x4b, 0x40, 0x4f, 0x51, 0xc0, 0x30, 0x30, 0x81,
			0x35, 0x39, 0x38, 0x81, 0x3c, 0x41, 0xbd, 0x3e,
			0xc0, 0x49, 0x4b, 0x40, 0x50, 0x50, 0xc0, 0x37,
			0x39, 0xc0, 0x3c, 0x3e, 0x40, 0x4a, 0x4a, 0xc0,
			0x38, 0x38, 0x40, 0x3c, 0x3e, 0x40, 0x3d, 0x3d,
			0x09, 0x44, 0x2f, 0x4f, 0x33, 0x36, 0x42, 0x4a,
			0x44, 0x2f, 0x4f, 0x33, 0x36, 0x43, 0x49, 0x46,
			0x2f, 0x4e, 0x33, 0x36, 0x3c, 0x3f, 0x44, 0x49,
			0x46, 0x30, 0x4d, 0x33, 0x35, 0x3b, 0x3f, 0x45,
			0x49, 0xc5, 0x30, 0x44, 0xb2, 0x33, 0x35, 0x3a,
			0x3f, 0x43, 0x02, 0x46, 0x4d, 0xc7, 0x48, 0x4c,
			0xc0, 0x31, 0x31, 0xc4, 0x34, 0x42, 0xb5, 0x36,
			0xba, 0x3b, 0x3f, 0x41, 0xc0, 0x46, 0x49, 0x40,
			0x4c, 0x4c, 0x81, 0x37, 0x3a, 0x39, 0xc1, 0x3c,
			0x40, 0x3f, 0x40, 0x46, 0x48, 0xc0, 0x39, 0x39,
			0xc0, 0x3c, 0x3e, 0x40, 0x47, 0x47, 0x40, 0x3d,
			0x3d, 0x07, 0x43, 0x33, 0x4c, 0x36, 0x42, 0x48,
			0x43, 0x33, 0x4c, 0x36, 0x43, 0x47, 0x44, 0x33,
			0x4b, 0x36, 0x3c, 0x44, 0x47, 0xc2, 0x33, 0x43,
			0xb5, 0x37, 0x3b, 0x02, 0x45, 0x4b, 0x46, 0x4a,
			0xc0, 0x34, 0x34, 0xc2, 0x38, 0x42, 0xbb, 0x3c,
			0x41, 0xc0, 0x45, 0x47, 0x40, 0x4a, 0x4a, 0x80,
			0x39, 0x3a, 0x81, 0x3d, 0x40, 0x3e, 0x40, 0x45,
			0x47, 0xc0, 0x3e, 0x3e, 0x40, 0x46, 0x46, 0x06,
			0x42, 0x36, 0x4a, 0x42, 0x47, 0x43, 0x36, 0x4a,
			0x3e, 0x43, 0x46, 0x44, 0x36, 0x49, 0xb8, 0x39,
			0x3d, 0xc3, 0x44, 0xc6, 0x47, 0xc0, 0x37, 0x37,
			0xc2, 0x3a, 0x42, 0xbc, 0x3d, 0x41, 0xc0, 0x44,
			0x46, 0x40, 0x48, 0x48, 0xc0, 0x3b, 0x3b, 0xc0,
			0x3e, 0x40, 0x40, 0x44, 0x46, 0xc0, 0x3f, 0x3f,
			0x40, 0x45, 0x45, 0x22, 0x47, 0x18, 0x6e, 0x2e,
			0x3a, 0x3d, 0xc3, 0x44, 0x46, 0x48, 0xd1, 0x53,
			0x49, 0x18, 0x6d, 0x2d, 0xb9, 0x3a, 0x3d, 0xc3,
			0x44, 0x46, 0x48, 0xd4, 0x56, 0x5c, 0xea, 0x6c,
			0x47, 0x19, 0x69, 0x2c, 0xb5, 0x39, 0x3d, 0xc3,
			0x44, 0x46, 0x48, 0xd7, 0x5b, 0x48, 0x1a, 0x68,
			0x9b, 0x1c, 0x2b, 0x35, 0x3d, 0xc3, 0x44, 0x46,
			0x48, 0x5a, 0x48, 0x1d, 0x67, 0x9e, 0x20, 0xa8,
			0x2a, 0xb4, 0x35, 0x3d, 0x46, 0xc8, 0x4b, 0x5a,
			0xe2, 0x66, 0x47, 0x1e, 0x65, 0x27, 0x29, 0x34,
			0x3d, 0x46, 0xcb, 0x4d, 0x5a, 0x46, 0x1e, 0x65,
			0x29, 0x34, 0x3d, 0xc6, 0x49, 0x4d, 0x5a, 0xc2,
			0x1e, 0x46, 0x29, 0xb4, 0x3d, 0x42, 0x49, 0x65,
			0xca, 0x4d, 0x5a, 0xc1, 0x1e, 0x34, 0x29, 0xc0,
			0x3d, 0x46, 0x42, 0x4d, 0x65, 0xce, 0x51, 0x5a,
			0xc1, 0x1e, 0x34, 0x29, 0x81, 0x3d, 0x46, 0xc3,
			0x45, 0x41, 0x51, 0x65, 0x5a, 0xc1, 0x1e, 0x34,
			0x29, 0x80, 0x42, 0x46, 0x41, 0x51, 0x65, 0xd2,
			0x5a, 0xc2, 0x1e, 0x34, 0x29, 0x33, 0x40, 0x5a,
			0x65, 0xc2, 0x1e, 0x33, 0x29, 0xb0, 0x32, 0x41,
			0x5a, 0x65, 0xdb, 0x5c, 0xc2, 0x1e, 0x31, 0x29,
			0x30, 0x40, 0x5d, 0x65, 0xc1, 0x1e, 0x30, 0x29,
			0x40, 0x5d, 0x65, 0xc1, 0x1e, 0x30, 0x29, 0x41,
			0x5e, 0x65, 0x64, 0xc1, 0x1e, 0x30, 0x29, 0x00,
			0x5e, 0x63, 0x42, 0x1e, 0x30, 0x29, 0xad, 0x2f,
			0x43, 0x1e, 0x2d, 0xa1, 0x22, 0x29, 0x2c, 0x42,
			0x1e, 0x2c, 0xa1, 0x22, 0x29, 0x42, 0x1e, 0x2c,
			0xa1, 0x22, 0xa9, 0x2b, 0x41, 0x1e, 0x29, 0xa1,
			0x22, 0x41, 0x1e, 0x29, 0xa1, 0x22, 0x40, 0x1e,
			0x29, 0x40, 0x1e, 0x29, 0x41, 0x1e, 0x29, 0x28,
			0x41, 0x1e, 0x28, 0x27, 0x42, 0x1e, 0x27, 0x21,
			0x23, 0x03, 0x1e, 0x27, 0x1f, 0x22, 0xa5, 0x26,
			0x00, 0x1d, 0x27, 0x40, 0x1d, 0x27, 0x40, 0x1d,
			0x26, 0x40, 0x1d, 0x25, 0x00, 0x1d, 0x25, 0x18,
			0x47, 0x24, 0x60, 0x33, 0x3c, 0x3e, 0x42, 0x44,
			0x46, 0xcc, 0x4d, 0xc2, 0x25, 0x3b, 0x32, 0xb9,
			0x3a, 0x45, 0x3e, 0x5f, 0x42, 0x44, 0x46, 0xce,
			0x53, 0xdd, 0x5e, 0x47, 0x26, 0x5c, 0x31, 0x38,
			0x3e, 0x42, 0x44, 0x46, 0x52, 0x48, 0x27, 0x5b,
			0xa8, 0x29, 0xaf, 0x30, 0x38, 0x3e, 0x44, 0xc6,
			0x48, 0x52, 0xd8, 0x5a, 0x47, 0x28, 0x5a, 0x2e,
			0x30, 0x37, 0x3e, 0x44, 0x49, 0x52, 0x44, 0x28,
			0x5a, 0x30, 0xb7, 0x3e, 0xc4, 0x49, 0x52, 0xc1,
			0x28, 0x37, 0x30, 0x81, 0x3e, 0x44, 0xc2, 0x43,
			0x42, 0x49, 0x5a, 0xca, 0x4c, 0x52, 0xc1, 0x28,
			0x37, 0x30, 0x80, 0x41, 0x44, 0x41, 0x4c, 0x5a,
			0xcd, 0x52, 0xc2, 0x28, 0x36, 0x30, 0x35, 0x41,
			0x52, 0x5a, 0x53, 0xc1, 0x28, 0x35, 0x30, 0x40,
			0x54, 0x5a, 0xc1, 0x28, 0x35, 0x30, 0x40, 0x54,
			0x5a, 0xc1, 0x28, 0x35, 0x30, 0x00, 0x55, 0x5a,
			0x42, 0x28, 0x34, 0x30, 0x33, 0x42, 0x28, 0x32,
			0xaa, 0x2b, 0x30, 0x42, 0x28, 0x31, 0xaa, 0x2b,
			0x30, 0x41, 0x28, 0x30, 0xaa, 0x2b, 0x40, 0x28,
			0x30, 0x40, 0x28, 0x30, 0x41, 0x28, 0x2f, 0x2e,
			0x42, 0x28, 0x2e, 0x2a, 0x2c, 0x00, 0x27, 0x2e,
			0x40, 0x27, 0x2e, 0x40, 0x27, 0x2d, 0x00, 0x27,
			0x2d, 0x11, 0x45, 0x2c, 0x57, 0x36, 0xbd, 0x3e,
			0x41, 0x43, 0xc8, 0x49, 0x47, 0x2d, 0x56, 0x35,
			0xbb, 0x3c, 0x3e, 0x41, 0x43, 0xca, 0x4d, 0xd4,
			0x55, 0x46, 0x2e, 0x53, 0x34, 0x3a, 0x3e, 0xc3,
			0x45, 0x4d, 0xd1, 0x52, 0x46, 0x2f, 0x52, 0x34,
			0x39, 0x3e, 0x43, 0x46, 0x4d, 0x44, 0x2f, 0x52,
			0x34, 0xb9, 0x40, 0xc3, 0x48, 0x4d, 0xc1, 0x2f,
			0x39, 0x34, 0x80, 0x40, 0x43, 0x01, 0x48, 0x52,
			0xce, 0x51, 0xc1, 0x2f, 0x39, 0x34, 0x40, 0x4d,
			0x52, 0xc1, 0x2f, 0x38, 0x34, 0x40, 0x4e, 0x52,
			0xc1, 0x2f, 0x38, 0x34, 0x00, 0x4f, 0x52, 0x43,
			0x2f, 0x37, 0x31, 0x34, 0x36, 0x42, 0x2f, 0x35,
			0x31, 0x34, 0x41, 0x2f, 0x34, 0x31, 0x40, 0x2f,
			0x34, 0x40, 0x2f, 0x33, 0x00, 0x2e, 0x33, 0x40,
			0x2e, 0x33, 0x00, 0x2e, 0x32, 0x0f, 0x45, 0x2e,
			0x54, 0x37, 0xbd, 0x3e, 0x40, 0x42, 0xc6, 0x47,
			0x47, 0x2f, 0x53, 0x36, 0xbb, 0x3c, 0x3e, 0x40,
			0x42, 0xc8, 0x4b, 0xd1, 0x52, 0x46, 0x2f, 0x50,
			0x35, 0x3a, 0x3e, 0xc2, 0x44, 0x4b, 0xce, 0x4f,
			0xc3, 0x30, 0x42, 0x35, 0x39, 0x3e, 0x41, 0x45,
			0x4f, 0x4b, 0x82, 0x30, 0x42, 0xb1, 0x34, 0xb6,
			0x38, 0x01, 0x45, 0x4f, 0xcc, 0x4e, 0xc1, 0x30,
			0x39, 0x35, 0x40, 0x4b, 0x4f, 0xc1, 0x30, 0x38,
			0x35, 0x40, 0x4c, 0x4f, 0xc1, 0x30, 0x38, 0x35,
			0x00, 0x4d, 0x4f, 0x42, 0x30, 0x37, 0x32, 0x35,
			0x42, 0x30, 0x36, 0x32, 0x35, 0x41, 0x30, 0x35,
			0x32, 0x40, 0x30, 0x34, 0x00, 0x2f, 0x34, 0x40,
			0x2f, 0x34, 0x00, 0x2f, 0x33, 0x0c, 0x44, 0x33,
			0x4f, 0x39, 0x3f, 0x42, 0x45, 0x46, 0x33, 0x4e,
			0x39, 0x3d, 0x3f, 0x42, 0xc6, 0x48, 0x4d, 0x47,
			0x33, 0x4c, 0x34, 0x38, 0x3c, 0x3f, 0xc2, 0x44,
			0x48, 0x4b, 0x82, 0x34, 0x42, 0xb5, 0x37, 0xb9,
			0x3b, 0x01, 0x44, 0x4b, 0xc9, 0x4a, 0xc1, 0x34,
			0x3b, 0x38, 0x40, 0x48, 0x4b, 0xc1, 0x34, 0x3a,
			0x38, 0x00, 0x49, 0x4b, 0x02, 0x34, 0x39, 0x35,
			0x37, 0x41, 0x34, 0x38, 0x36, 0x40, 0x34, 0x38,
			0x00, 0x33, 0x37, 0x40, 0x33, 0x37, 0x00, 0x33,
			0x36, 0x09, 0x44, 0x36, 0x4b, 0x3a, 0x3f, 0x41,
			0x43, 0x45, 0x36, 0x4a, 0x3a, 0x3f, 0x41, 0x44,
			0x46, 0x03, 0x36, 0x49, 0xb7, 0x38, 0xba, 0x3b,
			0xc7, 0x48, 0xc1, 0x36, 0x3b, 0x39, 0x40, 0x46,
			0x48, 0xc1, 0x36, 0x3b, 0x39, 0x00, 0x46, 0x48,
			0x41, 0x36, 0x3a, 0x39, 0x40, 0x36, 0x39, 0x40,
			0x36, 0x38, 0x00, 0x36, 0x38, 0x07, 0x43, 0x38,
			0x49, 0x3b, 0x3f, 0x41, 0x44, 0x38, 0x48, 0x3b,
			0x3f, 0x41, 0x45, 0x02, 0x38, 0x47, 0xb9, 0x3a,
			0x46, 0xc0, 0x38, 0x3b, 0x00, 0x45, 0x47, 0x40,
			0x38, 0x3b, 0x40, 0x38, 0x3b, 0x00, 0x38, 0x3a,
			0x06, 0x43, 0x3a, 0x47, 0x3c, 0x3f, 0x41, 0x02,
			0x3a, 0x47, 0x3b, 0xc5, 0x46, 0xc0, 0x3a, 0x3c,
			0x00, 0x44, 0x46, 0x40, 0x3a, 0x3c, 0x40, 0x3a,
			0x3c, 0x00, 0x3a, 0x3b, 0x31, 0x46, 0x06, 0x7e,
			0x07, 0x99, 0x1e, 0xca, 0x4e, 0xda, 0x5c, 0xf0,
			0x78, 0xfc, 0x7d, 0x47, 0x08, 0x7b, 0x9f, 0x20,
			0xc7, 0x49, 0x5d, 0xe4, 0x66, 0xee, 0x6f, 0x73,
			0xf9, 0x7a, 0x48, 0x09, 0x7a, 0x8a, 0x0b, 0xa1,
			0x22, 0x46, 0x54, 0x5e, 0xe7, 0x68, 0xec, 0x6d,
			0x73, 0x47, 0x0c, 0x7b, 0x8d, 0x0e, 0xa3, 0x25,
			0x45, 0x54, 0x5e, 0xe9, 0x6b, 0x73, 0x46, 0x0f,
			0x7c, 0xa6, 0x29, 0x45, 0x54, 0x5f, 0x6b, 0x73,
			0x47, 0x10, 0x7c, 0x91, 0x17, 0xaa, 0x2d, 0x45,
			0x54, 0x5f, 0xe7, 0x6a, 0x73, 0xc6, 0x18, 0x66,
			0x19, 0x2e, 0x44, 0x54, 0x5f, 0xe3, 0x65, 0x41,
			0x6a, 0x7c, 0x73, 0xc4, 0x1a, 0x62, 0xaf, 0x32,
			0x44, 0x55, 0x5e, 0x41, 0x6a, 0x7c, 0x73, 0xc5,
			0x1b, 0x61, 0xb3, 0x34, 0x44, 0x48, 0x55, 0x5d,
			0x41, 0x6a, 0x7c, 0x73, 0xc5, 0x1c, 0x60, 0x1d,
			0xb5, 0x37, 0xc5, 0x47, 0x55, 0xdd, 0x5f, 0xc0,
			0x6a, 0x72, 0x40, 0x74, 0x7c, 0xc3, 0x1e, 0x5d,
			0xb8, 0x3f, 0x45, 0x55, 0x42, 0x6a, 0x7c, 0x71,
			0x75, 0xc0, 0x1f, 0x37, 0xc2, 0x40, 0x5d, 0xc1,
			0x45, 0x55, 0x42, 0x6a, 0x7c, 0x70, 0x75, 0xc1,
			0x20, 0x36, 0xb3, 0x35, 0xc1, 0x45, 0x5d, 0x55,
			0x42, 0x6b, 0x7b, 0x70, 0x75, 0x81, 0x21, 0x32,
			0xa3, 0x28, 0xc0, 0x35, 0x35, 0xc1, 0x45, 0x5d,
			0xd4, 0x55, 0x42, 0x6c, 0x7a, 0x70, 0x75, 0xc2,
			0x23, 0x35, 0xa4, 0x28, 0x30, 0xc2, 0x45, 0x5c,
			0x53, 0x56, 0x02, 0x6c, 0x7a, 0xed, 0x6e, 0xf8,
			0x79, 0xc1, 0x24, 0x35, 0x30, 0xc2, 0x45, 0x5c,
			0x52, 0xd7, 0x58, 0x42, 0x6d, 0x79, 0x6e, 0x78,
			0xc1, 0x24, 0x35, 0x30, 0xc2, 0x46, 0x5b, 0x51,
			0xd9, 0x5a, 0x40, 0x6d, 0x77, 0xc1, 0x24, 0x35,
			0x30, 0xc1, 0x47, 0x5b, 0xcf, 0x50, 0x01, 0x6e,
			0x76, 0xf3, 0x74, 0xc1, 0x24, 0x35, 0x30, 0xc1,
			0x47, 0x5b, 0x4e, 0x00, 0x73, 0x74, 0xc1, 0x24,
			0x35, 0x30, 0x41, 0x47, 0x5b, 0x4d, 0xc1, 0x24,
			0x35, 0x30, 0x41, 0x48, 0x5b, 0x4c, 0xc1, 0x24,
			0x35, 0x30, 0x41, 0x49, 0x5a, 0xca, 0x4b, 0xc1,
			0x24, 0x35, 0x30, 0x41, 0x4c, 0x59, 0x58, 0xc1,
			0x24, 0x34, 0x30, 0x41, 0x4d, 0x57, 0x4e, 0xc1,
			0x24, 0x34, 0x30, 0x00, 0x4f, 0x56, 0x41, 0x24,
			0x34, 0x30, 0x41, 0x25, 0x34, 0x30, 0x41, 0x25,
			0x34, 0x30, 0x41, 0x25, 0x34, 0x30, 0x41, 0x25,
			0x34, 0x30, 0x41, 0x25, 0x34, 0x30, 0x41, 0x25,
			0x34, 0x30, 0x41, 0x25, 0x34, 0x30, 0x41, 0x25,
			0x34, 0x30, 0x41, 0x25, 0x34, 0x30, 0x41, 0x25,
			0x34, 0x2f, 0x41, 0x25, 0x33, 0x2f, 0x41, 0x25,
			0x33, 0x2f, 0x41, 0x25, 0x33, 0x2f, 0x41, 0x25,
			0x33, 0x2f, 0x41, 0x25, 0x33, 0x2f, 0x41, 0x25,
			0x33, 0x2f, 0x41, 0x25, 0x33, 0x2f, 0x41, 0x25,
			0x33, 0x2f, 0x41, 0x25, 0x33, 0x2e, 0x41, 0x25,
			0x32, 0x2e, 0x42, 0x25, 0x32, 0xa6, 0x27, 0x2e,
			0x41, 0x27, 0x32, 0x2e, 0x00, 0x27, 0x32, 0x23,
			0x46, 0x17, 0x6b, 0x18, 0xa5, 0x28, 0xc7, 0x4a,
			0xd2, 0x53, 0xe1, 0x67, 0x6a, 0x49, 0x19, 0x69,
			0x1a, 0xa9, 0x2b, 0xc4, 0x46, 0x4e, 0x54, 0xd9,
			0x5c, 0xdf, 0x60, 0x64, 0x68, 0x47, 0x1b, 0x69,
			0x9c, 0x1d, 0xac, 0x2d, 0x43, 0x4e, 0x55, 0xdd,
			0x5e, 0x64, 0x46, 0x1e, 0x6a, 0xae, 0x2f, 0x43,
			0x4e, 0x56, 0x5e, 0x64, 0x47, 0x1f, 0x6a, 0xa0,
			0x24, 0xb0, 0x33, 0x43, 0x4e, 0x56, 0xd9, 0x5d,
			0x64, 0xc4, 0x25, 0x58, 0xb4, 0x36, 0x43, 0x4f,
			0x55, 0x41, 0x5d, 0x6a, 0x64, 0xc5, 0x26, 0x57,
			0x27, 0xb7, 0x39, 0x44, 0x4f, 0xd5, 0x56, 0x41,
			0x5d, 0x6a, 0xe3, 0x64, 0xc3, 0x28, 0x54, 0xba,
			0x3f, 0x43, 0x4f, 0x42, 0x5d, 0x6a, 0x62, 0x65,
			0xc1, 0x29, 0x39, 0xb7, 0x38, 0xc2, 0x40, 0x54,
			0xc1, 0x43, 0x4f, 0x42, 0x5e, 0x69, 0x61, 0x65,
			0x82, 0x2a, 0x38, 0xac, 0x2f, 0x37, 0xc1, 0x43,
			0x54, 0xce, 0x4f, 0x42, 0x5f, 0x68, 0x61, 0x65,
			0xc2, 0x2c, 0x38, 0xad, 0x2f, 0x35, 0xc2, 0x43,
			0x53, 0x4d, 0x4f, 0x02, 0x5f, 0x68, 0x60, 0xe6,
			0x67, 0xc1, 0x2c, 0x38, 0x35, 0xc3, 0x43, 0x53,
			0x44, 0x4c, 0xd0, 0x52, 0x41, 0x60, 0x67, 0x66,
			0xc1, 0x2c, 0x38, 0x35, 0xc1, 0x45, 0x53, 0x4b,
			0x00, 0x61, 0x65, 0xc1, 0x2c, 0x38, 0x35, 0x41,
			0x45, 0x53, 0xc9, 0x4a, 0xc1, 0x2c, 0x38, 0x35,
			0x41, 0x45, 0x52, 0x48, 0xc1, 0x2c, 0x38, 0x35,
			0x41, 0x46, 0x51, 0xc7, 0x48, 0xc1, 0x2c, 0x37,
			0x35, 0x41, 0x49, 0x50, 0x4a, 0xc1, 0x2c, 0x37,
			0x35, 0x00, 0x4b, 0x4f, 0x41, 0x2c, 0x37, 0x35,
			0x41, 0x2d, 0x37, 0x35, 0x41, 0x2d, 0x37, 0x35,
			0x41, 0x2d, 0x37, 0x35, 0x41, 0x2d, 0x37, 0x35,
			0x41, 0x2d, 0x37, 0x35, 0x41, 0x2d, 0x37, 0x35,
			0x41, 0x2d, 0x36, 0x34, 0x41, 0x2d, 0x36, 0x34,
			0x41, 0x2d, 0x36, 0x34, 0x41, 0x2d, 0x36, 0x34,
			0x41, 0x2d, 0x36, 0x34, 0x41, 0x2d, 0x36, 0x34,
			0x41, 0x2d, 0x35, 0x33, 0x42, 0x2d, 0x35, 0x2e,
			0x33, 0x41, 0x2e, 0x35, 0x33, 0x00, 0x2e, 0x35,
			0x19, 0x45, 0x22, 0x5e, 0x23, 0xac, 0x2e, 0xc5,
			0x47, 0x4d, 0xd7, 0x5c, 0x47, 0x24, 0x5d, 0xa5,
			0x26, 0xaf, 0x32, 0xc2, 0x44, 0x4a, 0x4e, 0x56,
			0x59, 0x46, 0x27, 0x5e, 0x33, 0x42, 0x4a, 0x4f,
			0x55, 0x59, 0x47, 0x28, 0x5e, 0xa9, 0x2b, 0xb4,
			0x36, 0x42, 0x4a, 0x4f, 0xd2, 0x54, 0x59, 0xc5,
			0x2c, 0x51, 0xad, 0x2e, 0xb7, 0x3a, 0x42, 0x4a,
			0xcf, 0x50, 0x41, 0x54, 0x5e, 0x59, 0xc3, 0x2f,
			0x4e, 0xbb, 0x3f, 0x42, 0x4a, 0x42, 0x54, 0x5e,
			0x58, 0x5a, 0x81, 0x30, 0x3b, 0xb1, 0x34, 0xc2,
			0x40, 0x4e, 0xc1, 0x42, 0x4a, 0x42, 0x55, 0x5d,
			0x57, 0x5a, 0x82, 0x31, 0x3a, 0xb5, 0x37, 0x39,
			0xc1, 0x42, 0x4d, 0xc9, 0x4a, 0x01, 0x56, 0x5c,
			0x5b, 0xc1, 0x31, 0x3a, 0x38, 0xc3, 0x42, 0x4d,
			0x43, 0x48, 0xcb, 0x4c, 0x40, 0x57, 0x5b, 0xc1,
			0x31, 0x3a, 0x38, 0xc1, 0x43, 0x4d, 0xc6, 0x47,
			0x00, 0x58, 0x5a, 0xc1, 0x31, 0x3a, 0x38, 0x41,
			0x43, 0x4d, 0x45, 0xc1, 0x31, 0x3a, 0x38, 0x01,
			0x44, 0x4c, 0xc8, 0x4a, 0xc1, 0x31, 0x3a, 0x38,
			0x00, 0x48, 0x4a, 0x41, 0x31, 0x3a, 0x38, 0x41,
			0x32, 0x3a, 0x38, 0x41, 0x32, 0x3a, 0x38, 0x41,
			0x32, 0x3a, 0x38, 0x41, 0x32, 0x3a, 0x38, 0x41,
			0x32, 0x39, 0x37, 0x41, 0x32, 0x39, 0x37, 0x41,
			0x32, 0x39, 0x37, 0x41, 0x32, 0x39, 0x37, 0x41,
			0x32, 0x38, 0x36, 0x42, 0x32, 0x38, 0x33, 0x36,
			0x00, 0x33, 0x38, 0x16, 0x45, 0x25, 0x5a, 0x26,
			0xae, 0x2f, 0xc4, 0x45, 0x4b, 0xd4, 0x58, 0x47,
			0x27, 0x59, 0x28, 0xb0, 0x33, 0xc1, 0x43, 0x48,
			0x4c, 0x53, 0x55, 0x46, 0x29, 0x5a, 0x34, 0x41,
			0x48, 0x4d, 0x52, 0x55, 0x47, 0x2a, 0x5a, 0xab,
			0x2d, 0xb5, 0x36, 0x41, 0x48, 0x4d, 0xcf, 0x51,
			0x55, 0xc6, 0x2e, 0x4e, 0xaf, 0x30, 0x37, 0xbc,
			0x3e, 0x41, 0x48, 0x4d, 0x41, 0x51, 0x5a, 0x55,
			0x81, 0x31, 0x3b, 0xb2, 0x35, 0xc2, 0x3f, 0x4c,
			0xc0, 0x41, 0x48, 0x42, 0x52, 0x59, 0x54, 0x56,
			0x82, 0x32, 0x3a, 0xb6, 0x37, 0x39, 0xc1, 0x41,
			0x4b, 0xc7, 0x48, 0x01, 0x53, 0x58, 0x57, 0xc1,
			0x32, 0x3a, 0x38, 0xc2, 0x41, 0x4b, 0x46, 0xc9,
			0x4a, 0x40, 0x53, 0x57, 0xc1, 0x32, 0x3a, 0x38,
			0xc1, 0x42, 0x4b, 0x45, 0x00, 0x54, 0x56, 0xc1,
			0x32, 0x3a, 0x38, 0x41, 0x42, 0x4b, 0x44, 0xc1,
			0x32, 0x3a, 0x38, 0x01, 0x43, 0x4a, 0xc6, 0x48,
			0xc1, 0x32, 0x3a, 0x38, 0x00, 0x46, 0x48, 0x41,
			0x33, 0x3a, 0x38, 0x41, 0x33, 0x3a, 0x38, 0x41,
			0x33, 0x3a, 0x38, 0x41, 0x33, 0x3a, 0x38, 0x41,
			0x33, 0x39, 0x37, 0x41, 0x33, 0x39, 0x37, 0x41,
			0x33, 0x39, 0x37, 0x41, 0x33, 0x39, 0x37, 0x42,
			0x33, 0x39, 0x34, 0x37, 0x00, 0x34, 0x39, 0x11,
			0x44, 0x2c, 0x54, 0x33, 0xc2, 0x44, 0x48, 0xce,
			0x52, 0x47, 0x2d, 0x53, 0x2e, 0xb4, 0x36, 0x41,
			0x46, 0x49, 0x4d, 0x50, 0x46, 0x2f, 0x54, 0xb0,
			0x32, 0xb7, 0x38, 0x41, 0x46, 0xca, 0x4d, 0x50,
			0xc5, 0x33, 0x4a, 0x34, 0x39, 0xbd, 0x3e, 0x41,
			0x46, 0x41, 0x4d, 0x54, 0x50, 0x81, 0x35, 0x3c,
			0xb6, 0x38, 0xc2, 0x3f, 0x49, 0xc0, 0x41, 0x46,
			0x02, 0x4e, 0x53, 0x50, 0x52, 0x82, 0x36, 0x3c,
			0x39, 0x3b, 0x81, 0x41, 0x48, 0xc2, 0x44, 0x40,
			0x4e, 0x52, 0xc1, 0x36, 0x3c, 0x3a, 0xc1, 0x42,
			0x48, 0x44, 0x00, 0x4f, 0x51, 0xc1, 0x36, 0x3c,
			0x3a, 0x40, 0x43, 0x47, 0xc1, 0x36, 0x3c, 0x3a,
			0x00, 0x44, 0x46, 0x41, 0x36, 0x3c, 0x3a, 0x41,
			0x36, 0x3c, 0x3a, 0x41, 0x36, 0x3c, 0x3a, 0x41,
			0x36, 0x3b, 0x39, 0x41, 0x36, 0x3b, 0x39, 0x41,
			0x36, 0x3b, 0x39, 0x02, 0x36, 0x3b, 0x38, 0x3a,
			0x00, 0x37, 0x3b, 0x0d, 0x44, 0x30, 0x4d, 0x36,
			0x41, 0x46, 0xca, 0x4c, 0x47, 0x31, 0x4e, 0x32,
			0xb7, 0x38, 0x40, 0x44, 0x46, 0x49, 0x4c, 0x46,
			0x33, 0x4e, 0xb4, 0x36, 0x39, 0x40, 0x44, 0xc7,
			0x49, 0x4c, 0x83, 0x37, 0x46, 0xb8, 0x39, 0xc1,
			0x43, 0x45, 0x01, 0x49, 0x4e, 0x4d, 0x82, 0x38,
			0x3d, 0x3a, 0x3c, 0x81, 0x40, 0x45, 0xc1, 0x42,
			0x40, 0x4a, 0x4d, 0xc1, 0x38, 0x3d, 0x3b, 0xc1,
			0x41, 0x45, 0x42, 0x00, 0x4b, 0x4c, 0xc1, 0x38,
			0x3d, 0x3b, 0x00, 0x43, 0x44, 0x41, 0x38, 0x3d,
			0x3b, 0x41, 0x38, 0x3d, 0x3b, 0x41, 0x38, 0x3c,
			0x3a, 0x41, 0x38, 0x3c, 0x3a, 0x41, 0x38, 0x3c,
			0x3a, 0x00, 0x39, 0x3c, 0x0b, 0x44, 0x33, 0x4a,
			0xb8, 0x39, 0x41, 0x45, 0x49, 0x46, 0x34, 0x4b,
			0x35, 0x3a, 0x40, 0x43, 0x45, 0x48, 0x04, 0x36,
			0x4b, 0x3a, 0xc1, 0x42, 0x44, 0xc9, 0x4a, 0xc0,
			0x3a, 0x3e, 0xc1, 0x40, 0x44, 0x43, 0x40, 0x48,
			0x4b, 0xc0, 0x3a, 0x3e, 0x81, 0x41, 0x44, 0x43,
			0x00, 0x49, 0x4a, 0xc0, 0x3a, 0x3e, 0x40, 0x43,
			0x43, 0x40, 0x3a, 0x3e, 0x40, 0x3a, 0x3d, 0x40,
			0x3a, 0x3d, 0x40, 0x3a, 0x3d, 0x00, 0x3b, 0x3d,
			0x09, 0x43, 0x36, 0x48, 0x41, 0x44, 0x47, 0x44,
			0x37, 0x49, 0x3a, 0x40, 0x44, 0x47, 0x02, 0x38,
			0x49, 0xc1, 0x43, 0x48, 0xc0, 0x3b, 0x3e, 0x80,
			0x41, 0x43, 0x40, 0x48, 0x48, 0x40, 0x3b, 0x3e,
			0x40, 0x3b, 0x3e, 0x40, 0x3b, 0x3e, 0x40, 0x3b,
			0x3e, 0x00, 0x3c, 0x3d, 0x14, 0x44, 0x13, 0x6e,
			0x94, 0x17, 0x2f, 0xb3, 0x38, 0xcb, 0x4f, 0x45,
			0x18, 0x6d, 0x99, 0x1c, 0x2f, 0xb3, 0x38, 0xc8,
			0x4a, 0xeb, 0x6c, 0x44, 0x1d, 0x6a, 0x9e, 0x22,
			0x2f, 0xb3, 0x38, 0xc5, 0x47, 0x44, 0x23, 0x69,
			0xa4, 0x26, 0x2f, 0xb3, 0x37, 0xc3, 0x44, 0x44,
			0x27, 0x68, 0x30, 0xb3, 0x37, 0xc1, 0x42, 0xe5,
			0x67, 0x43, 0x28, 0x64, 0x30, 0xb4, 0x37, 0x40,
			0x44, 0x29, 0x63, 0x31, 0xb5, 0x36, 0x3f, 0xe1,
			0x62, 0x41, 0x2a, 0x60, 0xb2, 0x35, 0x40, 0x2b,
			0x60, 0x41, 0x2c, 0x5f, 0xdc, 0x5e, 0x41, 0x2d,
			0x5b, 0xc6, 0x4a, 0x42, 0x2d, 0x5a, 0xc4, 0x45,
			0x59, 0x41, 0x2e, 0x58, 0x43, 0x41, 0x2f, 0x57,
			0xbf, 0x42, 0x43, 0x30, 0x56, 0xb1, 0x32, 0xbc,
			0x3e, 0x55, 0x41, 0x33, 0x54, 0xb4, 0x3b, 0x40,
			0x3c, 0x53, 0x41, 0x3d, 0x52, 0xcd, 0x51, 0x42,
			0x3e, 0x4c, 0x3f, 0xc7, 0x4b, 0x00, 0x40, 0x46,
			0x0e, 0x44, 0x20, 0x60, 0xa1, 0x23, 0x34, 0xb7,
			0x3a, 0xc8, 0x4a, 0x45, 0x24, 0x5f, 0xa5, 0x2b,
			0x34, 0xb7, 0x3a, 0xc4, 0x47, 0x5e, 0x44, 0x2c,
			0x5d, 0x2d, 0x34, 0xb7, 0x3a, 0xc2, 0x43, 0x44,
			0x2e, 0x5c, 0x35, 0xb7, 0x3a, 0x41, 0xda, 0x5b,
			0x44, 0x2f, 0x59, 0x35, 0xb8, 0x3a, 0xbf, 0x40,
			0xd7, 0x58, 0x41, 0x30, 0x56, 0xb6, 0x39, 0x41,
			0x31, 0x55, 0x54, 0x41, 0x32, 0x53, 0xc4, 0x47,
			0x42, 0x33, 0x52, 0xc2, 0x43, 0x51, 0x41, 0x34,
			0x50, 0xbf, 0x41, 0x42, 0x35, 0x4f, 0x36, 0x3e,
			0x42, 0x37, 0x4e, 0xb8, 0x3d, 0x4d, 0x41, 0x3e,
			0x4c, 0xc9, 0x4b, 0x00, 0x3f, 0x48, 0x0a, 0x44,
			0x2b, 0x57, 0xac, 0x2e, 0x37, 0xb9, 0x3b, 0xc5,
			0x47, 0x45, 0x2f, 0x56, 0xb0, 0x32, 0x37, 0xb9,
			0x3b, 0xc1, 0x44, 0x55, 0x44, 0x33, 0x54, 0x38,
			0xba, 0x3b, 0x40, 0x53, 0x44, 0x34, 0x52, 0x38,
			0xba, 0x3b, 0x3f, 0xd0, 0x51, 0x42, 0x35, 0x4f,
			0xb9, 0x3a, 0x4e, 0x41, 0x36, 0x4d, 0xc3, 0x45,
			0x43, 0x37, 0x4c, 0x38, 0xc0, 0x42, 0x4b, 0x41,
			0x39, 0x4a, 0x3f, 0x01, 0x3a, 0x49, 0xbf, 0x44,
			0x00, 0x3f, 0x44, 0x09, 0x43, 0x2d, 0x54, 0xae,
			0x30, 0xb8, 0x3b, 0xc4, 0x45, 0x44, 0x31, 0x53,
			0xb2, 0x33, 0xb8, 0x3b, 0xc0, 0x43, 0x52, 0x43,
			0x34, 0x51, 0xb8, 0x3b, 0x3f, 0x50, 0x43, 0x35,
			0x4f, 0xb9, 0x3a, 0x3e, 0xcd, 0x4e, 0x42, 0x36,
			0x4c, 0xc2, 0x44, 0x4b, 0x43, 0x37, 0x4a, 0x38,
			0xc0, 0x41, 0x49, 0x41, 0x39, 0x48, 0x3f, 0x01,
			0x3a, 0x47, 0xbf, 0x43, 0x00, 0x3f, 0x43, 0x07,
			0x43, 0x32, 0x4f, 0xb3, 0x34, 0xba, 0x3c, 0x4e,
			0x43, 0x35, 0x4d, 0x36, 0xba, 0x3c, 0x4c, 0x43,
			0x37, 0x4b, 0x38, 0x3b, 0x4a, 0x42, 0x39, 0x49,
			0xc2, 0x43, 0x48, 0x41, 0x3a, 0x47, 0xc0, 0x41,
			0x01, 0x3b, 0x46, 0xc0, 0x42, 0x00, 0x40, 0x42,
			0x06, 0x43, 0x36, 0x4b, 0x37, 0xbb, 0x3c, 0x4a,
			0x41, 0x38, 0x49, 0xbb, 0x3c, 0x43, 0x39, 0x48,
			0x3a, 0xc1, 0x42, 0xc6, 0x47, 0x41, 0x3b, 0x45,
			0x40, 0x01, 0x3c, 0x44, 0xc0, 0x41, 0x00, 0x40,
			0x41, 0x05, 0x43, 0x38, 0x49, 0x39, 0xbc, 0x3d,
			0x48, 0x41, 0x3a, 0x47, 0xbc, 0x3d, 0x42, 0x3b,
			0x46, 0x40, 0xc4, 0x45, 0x01, 0x3c, 0x43, 0xc0,
			0x41, 0x00, 0x40, 0x41, 0x03, 0x41, 0x3a, 0x47,
			0x3c, 0x42, 0x3b, 0x46, 0x3f, 0xc4, 0x45, 0x80,
			0x3c, 0x3e, 0x00, 0x40, 0x43, 0x23, 0x00, 0x0e,
			0x74, 0x00, 0x0e, 0x74, 0x61, 0x0e, 0x74, 0x11,
			0x14, 0x17, 0x1a, 0x1d, 0x20, 0x23, 0x26, 0x29,
			0x2c, 0x2f, 0x32, 0x35, 0x38, 0x3b, 0x3e, 0x41,
			0x44, 0x47, 0x4a, 0x4d, 0x50, 0x53, 0x56, 0x59,
			0x5c, 0x5f, 0x62, 0x65, 0x68, 0x6b, 0x6e, 0x71,
			0x61, 0x0e, 0x74, 0x11, 0x14, 0x17, 0x1a, 0x1d,
			0x20, 0x23, 0x26, 0x29, 0x2c, 0x2f, 0x32, 0x35,
			0x38, 0x3b, 0x3e, 0x41, 0x44, 0x47, 0x4a, 0x4d,
			0x50, 0x53, 0x56, 0x59, 0x5c, 0x5f, 0x62, 0x65,
			0x68, 0x6b, 0x6e, 0x71, 0x00, 0x0e, 0x74, 0x22,
			0x0e, 0x73, 0x0f, 0x12, 0x15, 0x18, 0x1b, 0x1e,
			0x21, 0x24, 0x27, 0x2a, 0x2d, 0x30, 0x33, 0x36,
			0x39, 0x3c, 0x3f, 0x42, 0x45, 0x48, 0x4b, 0x4e,
			0x51, 0x54, 0x57, 0x5a, 0x5d, 0x60, 0x63, 0x66,
			0x69, 0x6c, 0x6f, 0x72, 0x00, 0x0e, 0x73, 0xc0,
			0x10, 0x10, 0x1d, 0x15, 0x6b, 0x16, 0x19, 0x1c,
			0x1f, 0x22, 0x25, 0x28, 0x2b, 0x2e, 0x31, 0x34,
			0x37, 0x3a, 0x3d, 0x40, 0x43, 0x46, 0x49, 0x4c,
			0x4f, 0x52, 0x55, 0x58, 0x5b, 0x5e, 0x61, 0x64,
			0x67, 0x6a, 0xc0, 0x10, 0x10, 0x00, 0x15, 0x6b,
			0xc0, 0x10, 0x10, 0xc0, 0x1e, 0x1e, 0xc0, 0x29,
			0x29, 0xc0, 0x3a, 0x3a, 0xc0, 0x4b, 0x4b, 0x40,
			0x57, 0x57, 0xc0, 0x10, 0x10, 0xc0, 0x1e, 0x1e,
			0xc0, 0x29, 0x29, 0xc0, 0x3a, 0x3a, 0xc0, 0x4b,
			0x4b, 0x40, 0x57, 0x57, 0xc0, 0x10, 0x10, 0xc0,
			0x1e, 0x1e, 0xc0, 0x29, 0x29, 0xc0, 0x3a, 0x3a,
			0xc0, 0x4b, 0x4b, 0x40, 0x57, 0x57, 0x80, 0x10,
			0x17, 0xc0, 0x1e, 0x1e, 0x80, 0x29, 0x41, 0xc0,
			0x4b, 0x4b, 0x40, 0x57, 0x57, 0xc0, 0x10, 0x17,
			0xc0, 0x1e, 0x1e, 0xc0, 0x29, 0x41, 0xc0, 0x4b,
			0x4b, 0x40, 0x57, 0x57, 0xc0, 0x10, 0x17, 0xc0,
			0x1e, 0x1e, 0xc0, 0x29, 0x41, 0x00, 0x4b, 0x5f,
			0xc0, 0x10, 0x17, 0xc0, 0x1e, 0x1e, 0xc0, 0x29,
			0x41, 0x41, 0x4b, 0x5f, 0x57, 0xc0, 0x10, 0x17,
			0xc0, 0x1e, 0x1e, 0xc0, 0x29, 0x41, 0x41, 0x4b,
			0x5f, 0x57, 0xc0, 0x10, 0x17, 0xc0, 0x1e, 0x1e,
			0xc0, 0x29, 0x41, 0xc1, 0x4b, 0x5f, 0x57, 0x00,
			0x6a, 0x72, 0xc0, 0x10, 0x17, 0x81, 0x1e, 0x41,
			0xb7, 0x40, 0x02, 0x4b, 0x72, 0xcc, 0x56, 0xeb,
			0x71, 0xc0, 0x10, 0x17, 0xc1, 0x1e, 0x41, 0x36,
			0x41, 0x4b, 0x72, 0x57, 0xc0, 0x10, 0x17, 0xc1,
			0x1e, 0x41, 0x35, 0x41, 0x4b, 0x72, 0x57, 0xc0,
			0x10, 0x17, 0xc1, 0x1e, 0x41, 0x34, 0x41, 0x4b,
			0x72, 0x57, 0xc0, 0x10, 0x17, 0xc1, 0x1e, 0x41,
			0x33, 0x41, 0x4b, 0x72, 0x57, 0xc0, 0x10, 0x17,
			0xc1, 0x1e, 0x41, 0x32, 0x41, 0x4b, 0x72, 0xcc,
			0x57, 0xc0, 0x10, 0x17, 0xc2, 0x1e, 0x49, 0x31,
			0xc1, 0x48, 0x40, 0x57, 0x72, 0xc0, 0x10, 0x17,
			0xc3, 0x1e, 0x52, 0x30, 0x41, 0xc9, 0x51, 0x40,
			0x57, 0x72, 0xc0, 0x10, 0x17, 0x42, 0x1e, 0x72,
			0xaf, 0x41, 0xd2, 0x5b, 0xc0, 0x10, 0x17, 0xc0,
			0x1e, 0x30, 0x41, 0x3a, 0x72, 0xdb, 0x63, 0xc0,
			0x10, 0x17, 0xc0, 0x1e, 0x31, 0x41, 0x3a, 0x72,
			0x63, 0xc0, 0x10, 0x17, 0xc0, 0x1e, 0x32, 0x41,
			0x3a, 0x72, 0xdb, 0x63, 0xc0, 0x10, 0x17, 0xc0,
			0x1e, 0x33, 0x41, 0x3a, 0x72, 0xd2, 0x5b, 0xc0,
			0x10, 0x17, 0xc0, 0x1e, 0x34, 0xc1, 0x3a, 0x52,
			0xc9, 0x51, 0x40, 0x57, 0x72, 0xc0, 0x10, 0x17,
			0xc0, 0x1e, 0x35, 0x80, 0x3a, 0x49, 0x41, 0x57,
			0x72, 0xea, 0x71, 0xc0, 0x10, 0x17, 0xc0, 0x1e,
			0x36, 0x00, 0x57, 0x6a, 0x80, 0x10, 0x17, 0x00,
			0x1e, 0x36, 0x19, 0x00, 0x1d, 0x64, 0x00, 0x1d,
			0x64, 0x61, 0x1d, 0x64, 0x1f, 0x21, 0x23, 0x25,
			0x27, 0x29, 0x2c, 0x2e, 0x30, 0x32, 0x34, 0x36,
			0x38, 0x3a, 0x3c, 0x3e, 0x41, 0x43, 0x45, 0x47,
			0x49, 0x4b, 0x4d, 0x4f, 0x51, 0x53, 0x56, 0x58,
			0x5a, 0x5c, 0x5e, 0x60, 0x62, 0x00, 0x1d, 0x64,
			0x00, 0x1d, 0x64, 0xc0, 0x1e, 0x1e, 0x0c, 0x22,
			0x5e, 0x2b, 0x2d, 0x2f, 0x31, 0x40, 0x42, 0x44,
			0x46, 0x55, 0x57, 0x59, 0x5b, 0xc0, 0x1e, 0x1e,
			0x00, 0x22, 0x5e, 0xc0, 0x1e, 0x1e, 0xc0, 0x28,
			0x28, 0xc0, 0x30, 0x30, 0xc0, 0x3c, 0x3c, 0xc0,
			0x48, 0x48, 0x40, 0x50, 0x50, 0x80, 0x1e, 0x23,
			0xc0, 0x28, 0x28, 0x80, 0x30, 0x41, 0xc0, 0x48,
			0x48, 0x40, 0x50, 0x50, 0xc0, 0x1e, 0x23, 0xc0,
			0x28, 0x28, 0xc0, 0x30, 0x41, 0xc0, 0x48, 0x48,
			0x40, 0x50, 0x50, 0xc0, 0x1e, 0x23, 0xc0, 0x28,
			0x28, 0xc0, 0x30, 0x41, 0x00, 0x48, 0x56, 0xc0,
			0x1e, 0x23, 0xc0, 0x28, 0x28, 0xc0, 0x30, 0x41,
			0x41, 0x48, 0x56, 0x50, 0xc0, 0x1e, 0x23, 0xc0,
			0x28, 0x28, 0xc0, 0x30, 0x41, 0xc1, 0x48, 0x56,
			0x50, 0x00, 0x5d, 0x63, 0xc0, 0x1e, 0x23, 0x81,
			0x28, 0x41, 0xba, 0x40, 0x02, 0x48, 0x63, 0xc9,
			0x4f, 0xde, 0x62, 0xc0, 0x1e, 0x23, 0xc1, 0x28,
			0x41, 0x38, 0x41, 0x48, 0x63, 0x50, 0xc0, 0x1e,
			0x23, 0xc1, 0x28, 0x41, 0x37, 0x41, 0x48, 0x63,
			0x50, 0xc0, 0x1e, 0x23, 0xc1, 0x28, 0x41, 0x36,
			0x41, 0x48, 0x63, 0xc9, 0x50, 0xc0, 0x1e, 0x23,
			0xc2, 0x28, 0x46, 0x35, 0xc1, 0x45, 0x40, 0x50,
			0x63, 0xc0, 0x1e, 0x23, 0x03, 0x28, 0x63, 0xa9,
			0x33, 0xc2, 0x45, 0xd9, 0x62, 0xc0, 0x1e, 0x23,
			0xc0, 0x28, 0x35, 0x41, 0x3c, 0x63, 0x58, 0xc0,
			0x1e, 0x23, 0xc0, 0x28, 0x36, 0x41, 0x3c, 0x63,
			0xd3, 0x58, 0xc0, 0x1e, 0x23, 0xc0, 0x28, 0x37,
			0x41, 0x3c, 0x63, 0xcc, 0x53, 0xc0, 0x1e, 0x23,
			0xc0, 0x28, 0x38, 0x80, 0x3c, 0x4c, 0x41, 0x50,
			0x63, 0xdd, 0x62, 0xc0, 0x1e, 0x23, 0xc0, 0x28,
			0x39, 0x00, 0x50, 0x5d, 0x80, 0x1e, 0x23, 0x00,
			0x28, 0x39, 0x12, 0x00, 0x27, 0x59, 0x00, 0x27,
			0x59, 0x58, 0x27, 0x59, 0x29, 0x2b, 0x2d, 0x2f,
			0x31, 0x33, 0x35, 0x37, 0x39, 0x3b, 0x3d, 0x3f,
			0x41, 0x43, 0x45, 0x47, 0x49, 0x4b, 0x4d, 0x4f,
			0x51, 0x53, 0x55, 0x57, 0x00, 0x27, 0x59, 0xc0,
			0x27, 0x27, 0x00, 0x2a, 0x55, 0xc0, 0x27, 0x27,
			0xc0, 0x2f, 0x2f, 0xc0, 0x34, 0x34, 0xc0, 0x3d,
			0x3d, 0xc0, 0x45, 0x45, 0x40, 0x4b, 0x4b, 0x80,
			0x27, 0x2b, 0xc0, 0x2f, 0x2f, 0x80, 0x34, 0x40,
			0xc0, 0x45, 0x45, 0x40, 0x4b, 0x4b, 0xc0, 0x27,
			0x2b, 0xc0, 0x2f, 0x2f, 0xc0, 0x34, 0x40, 0x00,
			0x45, 0x4f, 0xc0, 0x27, 0x2b, 0xc0, 0x2f, 0x2f,
			0xc0, 0x34, 0x40, 0x41, 0x45, 0x4f, 0x4b, 0xc0,
			0x27, 0x2b, 0x81, 0x2f, 0x40, 0xbc, 0x3f, 0x01,
			0x45, 0x59, 0xc6, 0x4a, 0xc0, 0x27, 0x2b, 0xc1,
			0x2f, 0x40, 0x3a, 0x41, 0x45, 0x59, 0x4b, 0xc0,
			0x27, 0x2b, 0xc1, 0x2f, 0x40, 0x39, 0x41, 0x45,
			0x59, 0xc6, 0x4b, 0xc0, 0x27, 0x2b, 0xc2, 0x2f,
			0x44, 0x38, 0xc0, 0x43, 0x40, 0x4b, 0x59, 0xc0,
			0x27, 0x2b, 0x42, 0x2f, 0x59, 0xb7, 0x40, 0xc4,
			0x4c, 0xc0, 0x27, 0x2b, 0xc0, 0x2f, 0x38, 0x41,
			0x3d, 0x59, 0xcc, 0x50, 0xc0, 0x27, 0x2b, 0xc0,
			0x2f, 0x39, 0x41, 0x3d, 0x59, 0xc8, 0x4b, 0xc0,
			0x27, 0x2b, 0xc0, 0x2f, 0x3a, 0x80, 0x3d, 0x48,
			0x00, 0x4b, 0x59, 0x80, 0x27, 0x2b, 0x00, 0x2f,
			0x3b, 0x10, 0x00, 0x29, 0x55, 0x00, 0x29, 0x55,
			0x55, 0x29, 0x55, 0x2b, 0x2d, 0x2f, 0x31, 0x33,
			0x35, 0x37, 0x39, 0x3b, 0x3d, 0x3f, 0x41, 0x43,
			0x45, 0x47, 0x49, 0x4b, 0x4d, 0x4f, 0x51, 0x53,
			0x00, 0x29, 0x55, 0xc0, 0x29, 0x29, 0x80, 0x2f,
			0x35, 0x00, 0x40, 0x52, 0x80, 0x29, 0x2d, 0xc0,
			0x30, 0x30, 0x80, 0x35, 0x3f, 0xc0, 0x44, 0x44,
			0x40, 0x49, 0x49, 0xc0, 0x29, 0x2d, 0xc0, 0x30,
			0x30, 0xc0, 0x35, 0x3f, 0x00, 0x44, 0x4d, 0xc0,
			0x29, 0x2d, 0xc0, 0x30, 0x30, 0xc0, 0x35, 0x3f,
			0x41, 0x44, 0x4d, 0x49, 0xc0, 0x29, 0x2d, 0x81,
			0x30, 0x3f, 0xbc, 0x3e, 0x01, 0x44, 0x55, 0xc5,
			0x48, 0xc0, 0x29, 0x2d, 0xc1, 0x30, 0x3f, 0x3a,
			0x41, 0x44, 0x55, 0x49, 0xc0, 0x29, 0x2d, 0xc1,
			0x30, 0x3f, 0x39, 0x41, 0x44, 0x55, 0x49, 0xc0,
			0x29, 0x2d, 0x02, 0x30, 0x55, 0xb1, 0x37, 0xcb,
			0x54, 0xc0, 0x29, 0x2d, 0xc0, 0x30, 0x38, 0x41,
			0x3d, 0x55, 0x4a, 0xc0, 0x29, 0x2d, 0xc0, 0x30,
			0x39, 0x41, 0x3d, 0x55, 0xc6, 0x49, 0xc0, 0x29,
			0x2d, 0xc0, 0x30, 0x3a, 0x80, 0x3d, 0x46, 0x00,
			0x49, 0x55, 0x80, 0x29, 0x2d, 0x00, 0x30, 0x3b,
			0x0c, 0x00, 0x2f, 0x50, 0x4f, 0x2f, 0x50, 0x32,
			0x34, 0x36, 0x38, 0x3a, 0x3c, 0x3e, 0x40, 0x42,
			0x44, 0x46, 0x48, 0x4a, 0x4c, 0x4e, 0x00, 0x2f,
			0x50, 0xc0, 0x2f, 0x2f, 0xc0, 0x34, 0x34, 0xc0,
			0x38, 0x38, 0xc0, 0x43, 0x43, 0x40, 0x47, 0x47,
			0x80, 0x2f, 0x32, 0xc0, 0x34, 0x34, 0x80, 0x38,
			0x3f, 0xc0, 0x43, 0x43, 0x40, 0x47, 0x47, 0xc0,
			0x2f, 0x32, 0xc0, 0x34, 0x34, 0xc0, 0x38, 0x3f,
			0x00, 0x43, 0x47, 0xc0, 0x2f, 0x32, 0x81, 0x34,
			0x3f, 0xbd, 0x3e, 0x01, 0x43, 0x50, 0xc4, 0x46,
			0xc0, 0x2f, 0x32, 0xc1, 0x34, 0x3f, 0x3c, 0x41,
			0x43, 0x50, 0x47, 0xc0, 0x2f, 0x32, 0x02, 0x34,
			0x50, 0xb5, 0x3a, 0xc9, 0x4f, 0xc0, 0x2f, 0x32,
			0xc0, 0x34, 0x3a, 0x41, 0x3e, 0x50, 0x48, 0xc0,
			0x2f, 0x32, 0xc0, 0x34, 0x3b, 0x41, 0x3e, 0x50,
			0xc5, 0x47, 0x80, 0x2f, 0x32, 0x80, 0x34, 0x3c,
			0x80, 0x3e, 0x45, 0x00, 0x47, 0x50, 0x09, 0x00,
			0x33, 0x4c, 0x0c, 0x33, 0x4c, 0x34, 0x37, 0x39,
			0x3b, 0x3d, 0x3f, 0x41, 0x43, 0x45, 0x47, 0x49,
			0x4b, 0x00, 0x33, 0x4c, 0xc0, 0x33, 0x36, 0xc0,
			0x39, 0x3f, 0xc0, 0x42, 0x42, 0x40, 0x45, 0x45,
			0xc0, 0x33, 0x36, 0xc0, 0x39, 0x3f, 0x00, 0x42,
			0x45, 0x82, 0x33, 0x3f, 0xb4, 0x35, 0xbd, 0x3e,
			0x01, 0x42, 0x4c, 0xc3, 0x44, 0x03, 0x33, 0x4c,
			0xb4, 0x35, 0xb7, 0x3b, 0xc7, 0x4b, 0xc1, 0x33,
			0x3c, 0x36, 0x41, 0x3e, 0x4c, 0x46, 0x80, 0x33,
			0x3c, 0x00, 0x3e, 0x4c, 0x07, 0x00, 0x36, 0x4a,
			0x49, 0x36, 0x4a, 0x38, 0x3a, 0x3c, 0x3e, 0x40,
			0x42, 0x44, 0x46, 0x48, 0x00, 0x36, 0x4a, 0xc0,
			0x36, 0x38, 0xc0, 0x3b, 0x3f, 0x40, 0x42, 0x44,
			0x82, 0x36, 0x3f, 0x37, 0x3e, 0x01, 0x42, 0x4a,
			0x43, 0xc2, 0x36, 0x3f, 0x38, 0xbd, 0x3e, 0x41,
			0x42, 0x4a, 0xc3, 0x44, 0x80, 0x36, 0x3d, 0x00,
			0x44, 0x4a, 0x06, 0x00, 0x38, 0x48, 0x47, 0x38,
			0x48, 0x3a, 0x3c, 0x3e, 0x40, 0x42, 0x44, 0x46,
			0x00, 0x38, 0x48, 0xc1, 0x38, 0x3e, 0x3a, 0x01,
			0x41, 0x48, 0x42, 0xc1, 0x38, 0x3e, 0x3a, 0x41,
			0x41, 0x48, 0xc2, 0x43, 0x80, 0x38, 0x3e, 0x00,
			0x43, 0x48,
		};

		int count = 1;
		for (int i = drawData.length - 1; i >= 0; i--, count++) {
			Feature.drawData[Feature.drawData.length - count] = drawData[i];
		}
	}
}
